%function plot_output()
%PLOT_OUTPUT Plots model output
  
% RSW 04/15/2004.  

fprintf('Program plot_output.m is plotting data to eps file ...\n');fflush(stdout);  

CISM_DX_GLOBALS;  % Load the global parameter data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set some local variables
NDAYS       = 40;        % Number of days to plot
LEAD_TIME   = 7;         % Max lead time
threat(1)   =  20;       %     Low watch level on plot
threat(2)   =  30;       %  Medium watch level on plot
threat(3)   =  50;       %   heigh watch level
threat(4)   = 100;       % highest watch level
MINY        = 0;         % Min y-value on plot
MINX        = 0;         % Min x-value on plot
MAXY        = 100;       % Max y-value on plot
YSTEP       = 20;        % Horizontal grid spacing in y-direction
Nright      = 0;         % Number of days to the right to leave empty.
MAXX        = NDAYS+LEAD_TIME;
CISM_DX_FLAG        = 99999;     % Global flag value
NAME        = 'Ap';      % Name of quantity plotted
FILTER      = 'ARX';
% Name of eps file to write.
EPS_NAME    = sprintf('./web_page/%s_current.eps',NAME);
% Text to put on plot
NOTE_HIGHEST= 'Highest watch level';
NOTE_HIGH   = 'High watch level';
NOTE_MEDIUM = 'Moderate watch level';
NOTE_LOW    = 'Low watch level';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get ensemble prediction
T_full       = load('./Persistence/T_full.dat');

clear ensemble_prediction;
T_ensemble   = ensemble_prediction();
L            = size(T_full,1);   % Number of rows
Xa           = T_full;           % a = actual ('full' means with time stamp)
Xf           = T_ensemble;       % f = forecasted
Xf           = unique(Xf,'rows');% Remove duplicate rows
Xa           = unique(Xa,'rows');% Remove duplicate rows
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dump last 10 data points to screen
fprintf('   Last 10 data points of forecast:\n');
temp         = Xf;
Lw           = size(Xf,2);
temp(:,7:Lw) = floor(temp(:,7:Lw)); % (So integer formatting of listing)
fprintf('   %4d %2d %2d %2d %2d %2d %5d %5d %5d %5d %5d %5d %5d\n',...
	                                   temp(size(Xf,1)-9:size(Xf,1),:)')

fprintf('   Last 10 data points of measured:\n');
temp         = Xa;
Lw           = size(Xa,2);
temp(:,7:Lw) = floor(temp(:,7:Lw)); % (So integer formatting of listing)
fprintf('   %4d %2d %2d %2d %2d %2d %5d\n',temp(size(Xa,1)-15:size(Xa,1)-7,:)')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set the axis labels and scale
[Vf,Va,T]   = time_union(Xf,Xa,CISM_DX_FLAG,'day');

L           = length(Va)+LEAD_TIME;
Days        = day_number(T);
I           = find(Days == floor(now));
tempXLabels = date_labels([1:L]',[1:L]',T(1,:));

Iplot       = [I-NDAYS+1 : I + LEAD_TIME];

Vf          = Vf(Iplot,:);
Va          = Va(Iplot,:);
tempXLabels = tempXLabels(Iplot,:);

XLabels     = repmat(blanks(size(tempXLabels,2)),size(tempXLabels,1),1);

K            = [length(Iplot):-7:1];
K            = sort(K);
XLabels(K,:) = tempXLabels(K,:);

if (max(reflag(Va,CISM_DX_FLAG,-1)) > MAXY)
  MAXY = 10+10*floor(max(reflag(Va,CISM_DX_FLAG,-1)/10));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create eps plots using epsTK
eglobpar2;        % get access to global parameters
einit2;

eWinWidth   = 195.9; % mm
eWinHeight  = 125;

eopen2(EPS_NAME); % open eps-file and write eps-head

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Gray strip
ePlotAreaPos         = [20 28];
ePlotAreaWidth       = 170;
ePlotAreaHeight      = 80;
L  = size(Va,1);
xi = 0.5+L-LEAD_TIME-27-1-Nright;
xf = 0.5+L-LEAD_TIME-1-Nright;
xb = [xi xi xf xf];
yb = [0 30 30 0];
eXAxisNorthVisible   = 0;
eYAxisEastVisible    = 0;
eXAxisSouthVisible   = 0;
eYAxisWestVisible    = 0;

eAxes2([MINX 1 MAXX],XLabels,[MINY YSTEP MAXY],[]);
eplot2(xb,yb,'27 days of most recent solar rotation',-1,[0.75 0.75 0.75]);
eplot2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ePlotAreaPos         = [20 40];
eXAxisSouthVisible   = 1;
eYAxisWestVisible    = 1;

% Set up the Axes
eAxes2([MINX 1 MAXX],XLabels,[MINY YSTEP MAXY],[]);
 
% Plot green horizontal line
xb = [MINX MAXX];
yb = [threat(1) threat(1)];
eplot2(xb,yb,'',0,[0 0.8 0])

% Plot yellow horizontal line
xb = [MINX MAXX];
yb = [threat(2) threat(2)];
eplot2(xb,yb,'',0,[0.8 0.8 0])

% Plot red horizontal line
xb = [MINX MAXX];
yb = [threat(3) threat(3)];
eplot2(xb,yb,'',0,[0.8 0.8 0])

L  = size(Va,1);
Tf = L-LEAD_TIME;

% Plot right bounding line
eplot2([MAXX MAXX],[MINY MAXY],'',0,[0 0 0]);
% Plot left bounding box
eplot2([MINX MINX],[MINY MAXY],'',0,[0 0 0]);
% Plot top bounding box
eplot2([MINX MAXX],[MAXY MAXY],'',0,[0 0 0]); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot blue bars of actual measurements
I         = find(abs(Va)>=0.99*CISM_DX_FLAG); % Find flagged values
Va(I)     = -1;                       % Put off scale
[xb yb]   = ebar2(Va(1:L),1,1,1,0.5+[0:L-1]); 
eplot2(xb,yb,sprintf('Measured %s',NAME),-1,[0 0 1])
eplot2(xb,yb,'',0,[0 0 0]) % Plot black outline around bars
clear I;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot black predition bars
I         = find(abs(Vf)>=CISM_DX_FLAG*0.99);
Vf(I)     = -1;
[xb yb]   = ebar2(Vf(1:L),0.5,1,1,0.5+[0:L-1]); 
eplot2(xb,yb,sprintf('Predicted %s with %s',NAME,FILTER),-1,[0 0 0])
clear I;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot black predition bars for 2-day to LEAD_TIME-day predictions
for i = 1:LEAD_TIME
 I        = [Tf+i-1];
 if (abs(Vf(I,i))>CISM_DX_FLAG*0.99)
   Vf(I,i)= -1;
 end
 [xb yb]   = ebar2(Vf(I,i),0.8,1,1,-0.5+I); 
 eplot2(xb,yb,'',-1,[0 0 0])
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

eplot2; % Close the current plot

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute probabilities
I1 = find(Va > threat(1) & abs(Va) < CISM_DX_FLAG & Vf(:,1) > 0.9*Vf(Tf,1) & Vf(:,1) < 1.1*Vf(Tf,1));
I1 = find(Va > threat(1)); % & Vf(:,1) > 0.5*Vf(Tf,1));
I2 = find(Va > threat(2) & abs(Va) < CISM_DX_FLAG & Vf(:,1) > 0.9*Vf(Tf,1) & Vf(:,1) < 1.1*Vf(Tf,1));
I2 = find(Va > threat(2) & Vf(:,1) > 0.5*Vf(Tf,1));
I3 = find(Va > threat(3) & abs(Va) < CISM_DX_FLAG & Vf(:,1) > 0.9*Vf(Tf,1) & Vf(:,1) < 1.1*Vf(Tf,1));
I3 = find(Va > threat(3) & Vf(:,1) > 0.5*Vf(Tf,1));
I4 = find(Va > threat(4) & abs(Va) < CISM_DX_FLAG & Vf(:,1) > 0.9*Vf(Tf,1) & Vf(:,1) < 1.1*Vf(Tf,1));
I4 = find(Va > threat(4) & Vf(:,1) > 0.5*Vf(Tf,1));
L  = find(abs(Va) < CISM_DX_FLAG);

if  (~isempty(I1))
  p1 = length(I1)/length(L);
else
  p1 = 0;
end
if  (~isempty(I2))
  p2 = length(I2)/length(L);
else
  p2 = 0;
end
if  (~isempty(I3))
  p3 = length(I3)/length(L);
else
  p3 = 0;
end
if  (~isempty(I4))
  p4 = length(I4)/length(L);
else
  p4 = 0;
end

date = datestr(floor(now)+1,6);
str1 = sprintf('Probability of %s > %2d is %.2f on %s',NAME,threat(1),p1,date);
str2 = sprintf('Probability of %s > %2d is %.2f on %s',NAME,threat(2),p2,date);
str3 = sprintf('Probability of %s > %2d is %.2f on %s',NAME,threat(3),p3,date);
str4 = sprintf('Probability of %s > %2d is %.2f on %s',NAME,threat(4),p4,date);
 eplotlg2(eFile,250,30,[1 1 1],0,0,str1,eFonts(ePlotLegendTextFont,:),10);
 eplotlg2(eFile,250,40,[1 1 1],0,0,str2,eFonts(ePlotLegendTextFont,:),10);
 eplotlg2(eFile,250,50,[1 1 1],0,0,str3,eFonts(ePlotLegendTextFont,:),10);
 eplotlg2(eFile,250,60,[1 1 1],0,0,str4,eFonts(ePlotLegendTextFont,:),10);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot note on dark bar
if (0)
x1   = 1;
y1   = threat(4)+1;
xpos = ePlotAreaPos(1)+(x1-ePlotAreaXValueStart)*ePlotAreaXFac/eFac;
ypos = ePlotAreaPos(2)+(y1-ePlotAreaYValueStart)*ePlotAreaYFac/eFac;
etext2(NOTE_HIGHEST,xpos,ypos,3,1,1,0,[0.5 0 0])
end

% Plot note on red bar
x1   = 1;
y1   = threat(3)+1;
xpos = ePlotAreaPos(1)+(x1-ePlotAreaXValueStart)*ePlotAreaXFac/eFac;
ypos = ePlotAreaPos(2)+(y1-ePlotAreaYValueStart)*ePlotAreaYFac/eFac;
etext2(NOTE_HIGH,xpos,ypos,3,1,1,0,[1 0 0])

% Plot note on yellow bar
x1   = 1;
y1   = threat(2)+1;
xpos = ePlotAreaPos(1)+(x1-ePlotAreaXValueStart)*ePlotAreaXFac/eFac;
ypos = ePlotAreaPos(2)+(y1-ePlotAreaYValueStart)*ePlotAreaYFac/eFac;
etext2(NOTE_MEDIUM,xpos,ypos,3,1,1,0,[0.8 0.8 0])

% Plot note on green bar
x1   = 1;
y1   = threat(1)+1;
xpos = ePlotAreaPos(1)+(x1-ePlotAreaXValueStart)*ePlotAreaXFac/eFac;
ypos = ePlotAreaPos(2)+(y1-ePlotAreaYValueStart)*ePlotAreaYFac/eFac;
etext2(NOTE_LOW,xpos,ypos,3,1,1,0,[0 0.8 0])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot vertical grey bars
yh      = [threat(1) MAXY];
xh      = [(xf-0.5) (xf-0.5)];
eplot2(xh,yh,'',0,[0.5 0.5 0.5]);
yh      = [threat(1) MAXY];
xh      = [(xf+0.5) (xf+0.5)];
eplot2(xh,yh,'',0,[0.5 0.5 0.5]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eplot2; % Close the axis
eclose2; % Close the file

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('... done plotting data.\n');fflush(stdout);
