function plot_output2()
%PLOT_OUTPUT2 Plots detailed model output
  
% Original by R.S. Weigel 04/15/2004.  
% Modified by M.  Gehmeyr 07/12/2004.  

fprintf('Program plot_output2.m is plotting data to eps file ...\n');fflush(stdout);  

CISM_DX_GLOBALS;  % Load the global parameter data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NAME        = 'Ap';      % Name of quantity plotted
EPS_NAME    = sprintf('./web_page/%s_current2.eps',NAME);
MINY        = 0;         % Min y-value on plot
MINX        = 0;         % Min x-value on plot
MAXY        = 100;       % Max y-value on plot
m           = 4+2;       % Column to plot (m=5 is 1 day lead time)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get all predictions
Ta = load('./Persistence/T_full.dat');
Ta = [Ta,repmat(Ta(:,7),1,6)];
T1 = load('./Persistence/T_combined.dat');
T2 = load('./Autoregressive/T_combined.dat');
T3 = load('./ARX/T_combined.dat');
Te = ensemble_prediction();

Tu  = time_union({T1,T2,T3,Te},CISM_DX_FLAG,'day');
T1  = Tu{1};
T2  = Tu{2};
T3  = Tu{3};
Te  = Tu{4};
%Ta  = Tu{5};

M  = size(Ta,2);
N  = size(Ta,1);

%arv1 = arv(Ta,T1,CISM_DX_FLAG)
%arv2 = arv(Ta,T2,CISM_DX_FLAG)
%arv3 = arv(Ta,T3,CISM_DX_FLAG)
%arve = arv(Ta,Te,CISM_DX_FLAG)

Ta   = reflag(Ta,CISM_DX_FLAG,-1);
T1   = reflag(T1,CISM_DX_FLAG,-1);
T2   = reflag(T2,CISM_DX_FLAG,-1);
T3   = reflag(T3,CISM_DX_FLAG,-1);
Te   = reflag(Te,CISM_DX_FLAG,-1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DAYN         = datenum(Ta(1,1),Ta(1,2),Ta(1,3));
DAYN         = [DAYN:DAYN + N - 1];
XLabels_o    = datestr(DAYN,2);

I            = find    (str2num(XLabels_o(:,4:5))    == 1);
J            = find(mod(str2num(XLabels_o(:,1:2)),1) == 0);
II           = intersect(I,J);
XLabels      = datestr(DAYN(II),32); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create eps plots using epsTK
eglobpar2;
einit2;
%eopen2(EPS_NAME); % open eps-file and write eps-head

%ePlotAreaHeight = 215; %160;  % for YMAX = 35
%ePlotAreaWidth  = 180;
%ePlotAreaPos    = [40 40];
%ePageHeight     = 160;  % mm Letter=279.4 A3=420 A4=297 A5=210
%ePageHeight     = 215;  % mm Letter=279.4 A3=420 A4=297 A5=210
%ePageWidth      = 260;  % mm Letter=215.9 A3=297 A4=210 A5=148
%ePageWidth      = 300;  % mm Letter=215.9 A3=297 A4=210 A5=148
%PageWidth  = 215.9; % mm Letter=215.9 A3=297 A4=210 A5=148 
%PageHeight = 279.4; % mm Letter=279.4 A3=420 A4=297 A5=210
PageOrientation = 0;
WinHeight = 210;
WinWidth  = 240;
WinOrigin = [0 0];
XScaleFac = 0.7;
YScaleFac = 0.7;
eopen2(EPS_NAME,PageOrientation,WinWidth,WinHeight,WinOrigin,XScaleFac,YScaleFac); % open eps-file and write eps-head

n=size(XLabels,1);% number of date labels on plot x axis
XSTEP = N/n;      % Vertical   grid spacing for dates
YSTEP = 20;       % Horizontal grid spacing in y-direction
ePlotAreaHeight = 38;
II = [0 XSTEP N-1]; 
JJ = [MINY YSTEP MAXY]; 

% Bottom Plot
EndY            = 30;
ePlotAreaPos    = [15 EndY];
eAxes2(II,XLabels,JJ,'');
eplot2([0:N-1],T1(1:N,m),'Ap Persistence',0,[0 0 0])
eplot2([0:N-1],Ta(1:N,M),'Ap Measured',0,[0 0 1])
eplot2;

% Next Plot
EndY            = (ePlotAreaPos(2)+ePlotAreaHeight+30);
ePlotAreaPos    = [15 EndY];
eAxes2(II,XLabels,JJ,'');
eplot2([0:N-1],T2(1:N,m),'Ap Predicted AR',0,[0 0 0])
eplot2([0:N-1],Ta(1:N,M),'Ap Measured',0,[0 0 1])
eplot2;

% Next Plot
EndY            = (ePlotAreaPos(2)+ePlotAreaHeight+30);
ePlotAreaPos    = [15 EndY];
eAxes2(II,XLabels,JJ,'');
eplot2([0:N-1],T3(1:N,m),'Ap Predicted ARX',0,[0 0 0])
eplot2([0:N-1],Ta(1:N,M),'Ap Measured',0,[0 0 1])
eplot2;
etitle2('                                 Summary of two day lead time predictions',380)

% Top plot
%EndY            = (ePlotAreaPos(2)+ePlotAreaHeight+30);
%ePlotAreaPos    = [15 EndY];
%eAxes2([0 XSTEP N-1],XLabels,[MINY YSTEP MAXY],'');
%eplot2([0:N-1]',Te(1:N,m),'Ap Predicted Ensemble',0,[0 0 0])
%eplot2([0:N-1]',Ta(1:N,M),'Ap Measured',0,[0 0 1])
eplot2;

eclose2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('... done plotting data.\n');fflush(stdout);
