pro ef_val1_d, printout = printout


; default parameters
if n_elements(printout) eq 0 then printout = 0


; define constants
sc = 'sampex'
output_path = 'Output/'


print, 'Choose year:'
s_arr = ['1993', '1994', '1995', '1996', $
	'1997', '1998', '1999', '2000', '2001']
year = s_arr(query_menu(s_arr))

print, 'Choice of storm (or entire year). 
print, 'Consult documentation (or see IDL code) about which storms are 
print, 'available. In each year only a number of storms are available
print, 'for validation.
print, 'Please choose:
s_arr = ['All_year', $
	'January', 'March', 'May', 'June', $
	'July', 'August', 'September', 'October']
interval = s_arr(query_menu(s_arr))


; graphics constants
printprep, printout, 1.2
pmulti, 1, 2
!p.color=1
!p.background=255
loadct1, 45


; derived variables
ef_path = 'Data/sampex/'+year+'/'
vsw_path = 'Data/omniweb/'+year+'/'
ly = leapyear(fix(year))
n_days_year = 365+ly


; read filter
path = 'Models/fir/'
file = 'je_fir_0.1.dat
ldim_start1 = 0	& ldim1 = 0	& n_lshells = 0
openr, 1, path+file
	readf, 1, ldim_start1
	readf, 1, ldim1, n_lshells
	h_fir = fltarr(ldim1, n_lshells)
	readf, 1, h_fir
close, 1
np1 = n_days_year-(ldim1+ldim_start1)
l_shell = findgen(n_lshells)*0.1+1.0


; read ARMA filter
path = 'Models/arma/'
file = 'arma.Vsw.1993-2000.None.dat
mdim = 0	& ml = 0	& n_lshells = 0
openr, 1, path+file
	readf, 1, mdim
	readf, 1, ml, n_lshells
	h_arma = fltarr(ml, n_lshells)
	readf, 1, h_arma
close, 1
ldim2 = ml-mdim
np2 = n_days_year-ldim2


; read flux
readmtsu, ef, n_lshells, n_days_year, ef_path+'FluxVsL.u', 0, 1
ef = transpose(ef)

; take log
min_ef = min(abs1(ef))
eflux = alog10(ef+min_ef)


; read input
readtsu, vsw, n_days_year, vsw_path+'vsw.u', 0, 1


; Validation

case interval of 
	'All_year': begin
		index1 = findgen(np1)
		index2 = findgen(np2)
	end
	'January': begin
		case year of 
			'1997': begin
				index1 = 5+findgen(15)-(ldim1+ldim_start1)
				index2 = 5+findgen(15)-ldim2
			end
		endcase
	end
	'March': begin
		case year of 
			'1998': begin
				index1 = 31+28+ly+5+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+5+findgen(15)-ldim2
			end
		endcase
	end
	'May': begin
		case year of 
			'1997': begin
				index1 = 31+28+ly+31+30+10+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+10+findgen(15)-ldim2
			end
			'1998': begin
				index1 = 31+28+ly+31+30+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+findgen(15)-ldim2
			end
		endcase
	end
	'June': begin
		case year of 
			'1998': begin
				index1 = 31+28+ly+31+30+31+20+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+20+findgen(15)-ldim2
			end
		endcase
	end
	'July': begin
		case year of 
			'2000': begin
				index1 = 31+28+ly+31+30+31+30+10+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+30+10+findgen(15)-ldim2
			end
		endcase
	end
	'August': begin
		case year of 
			'1998': begin	; only August 26. Not August 6
				index1 = 31+28+ly+31+30+31+30+31+20+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+30+31+20+findgen(15)-ldim2
			end
		endcase
	end
	'September': begin
		case year of 
			'1995': begin
				index1 = 31+28+ly+31+30+31+30+31+31+0+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+30+31+31+0+findgen(15)-ldim2
			end
			'1998': begin
				index1 = 31+28+ly+31+30+31+30+31+31+20+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+30+31+31+20+findgen(15)-ldim2
			end
		endcase
	end
	'October': begin
		case year of 
			'1998': begin
				index1 = 31+28+ly+31+30+31+30+31+31+30+15+findgen(15)-(ldim1+ldim_start1)
				index2 = 31+28+ly+31+30+31+30+31+31+30+15+findgen(15)-ldim2
			end
		endcase
	end
endcase


; run FIR
h_fir1 = h_fir(-ldim_start1:*,*)
eflux1 = eflux(ldim1+ldim_start1:*,*)
efluxp1 = fltarr(np1,n_lshells)
for i = 0, np1-1 do begin
	vsw1 = vsw(i:i+ldim1-1+ldim_start1)
	efluxp1(i,*) = h_fir1##rotate(vsw1,2)
endfor


; run ARMA
eflux2 = eflux(ldim2:*,*)
efluxp2 = fltarr(np2,n_lshells)
state_input = fltarr(ml, n_lshells)
state_update = fltarr(n_lshells)
; initialize
for i = 0, mdim-1  do state_input(i,*)      = eflux(ldim2-1-i,*)
for i = 0, ldim2-1 do state_input(mdim+i,*) = vsw  (ldim2-1-i)
; run
for i = 0, np2-1 do begin
	new_state = (h_arma*state_input)##n_ones(ml)
	efluxp2(i,*) = new_state
	for j = mdim-1, 1,-1 do state_input(j,*) = state_input(j-1,*)
	state_input(0,*) = new_state
	for j = mdim+ldim2-1,mdim+1,-1 do state_input(j,*) = state_input(j-1,*)
	state_input(mdim,*) = vsw(ldim2+i)
endfor


; FIR: Data-model correlation and prediction error
corre1 = fltarr(n_lshells)
rmse1  = fltarr(n_lshells)
for i = 0, n_lshells-1 do begin
	t1 = eflux1(index1,i)
	t2 = efluxp1(index1,i)
	corre1(i) = corr0(t1,t2)
	rmse1(i) = average(abs(t2-t1))/average(abs(t1))
endfor

; ARMA: Data-model correlation and prediction error
corre2 = fltarr(n_lshells)
rmse2  = fltarr(n_lshells)
for i = 0, n_lshells-1 do begin
	t1 = eflux2(index2,i)
	t2 = efluxp2(index2,i)
	corre2(i) = corr0(t1,t2)
	rmse2(i)  = average(abs(t2-t1))/average(abs(t1))
endfor


; plot
; Data-model correlation profiles
plot, l_shell, corre1, $
	yrange = [-50, 100], $
	xtit = 'L SHELL [R!DE!N]', ytit = 'CORRELATION [%]', title = interval+' '+year
oplot, l_shell, corre2, color = 250
oplot, !x.crange, 0.*[1,1]
oplot, !x.crange, 70.*[1,1], line=2
oplot, 3*[1,1], !y.crange, line=1
oplot, 4*[1,1], !y.crange, line=1
oplot, 7.5*[1,1], !y.crange, line=1

; prediction error profiles
plot, l_shell, rmse1, $
	yrange = [0, 1.2], $
	xtit = 'L SHELL [R!DE!N]', ytit = 'CORRELATION [%]', title = interval+' '+year
oplot, l_shell, rmse2, color = 250
oplot, !x.crange, 1.*[1,1]
oplot, 3*[1,1], !y.crange, line=1
oplot, 4*[1,1], !y.crange, line=1
oplot, 7.5*[1,1], !y.crange, line=1


if printout eq 1 then begin
	output_plot, printout, output_path+'val1.'+interval+'.'+year+'.ps
endif 


print, 'Average RMSE from FIR:  ', average(rmse1)
print, 'Average RMSE from ARMA: ', average(rmse2)


; save profiles
openw, 1, output_path+'corre_FIR.'+interval+'.'+year+'.dat'
	printf, 1, corre1
close, 1
openw, 1, output_path+'corre_ARMA.'+interval+'.'+year+'.dat'
	printf, 1, corre2
close, 1
openw, 1, output_path+'rmse_FIR.'+interval+'.'+year+'.dat'
	printf, 1, rmse1
close, 1
openw, 1, output_path+'rmse_ARMA.'+interval+'.'+year+'.dat'
	printf, 1, rmse2
close, 1


stop
return
end
