% Main function to execute Solar Wind model

clear; % Clear memory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the startup file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cwd = pwd;
CISM_DX_GLOBALS;

fprintf('__________________________________________________________________\n');
fprintf('Entering main program for running Solar Wind Velocity models.\n');
fprintf('__________________________________________________________________\n');
fprintf('\n');fflush(stdout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get latest ACE, synoptic data, and ss_maps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (1)
fprintf('--Start getting new data.\n');fflush(stdout);
clear get_real_time_data
      get_real_time_data('ace');
%      get_real_time_data('synoptic');
%      get_real_time_data('ss_maps');
cd(cwd)
fprintf('--Finished getting new data.\n');fflush(stdout);
fprintf('\n');fflush(stdout);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run the models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('--Start running models.\n');fflush(stdout);
if (1)
clear run_* pre_* post_* create_*
cd('./Persistence');
  pre_persistence([CISM_DX_DATA_DIR_ACE,'YYYY_MM_DD_HH_MM_SS_V.daily.dat'])
  run_persistence
 post_persistence
cd('..');
end

if (1)
clear run_* pre_* post_* create_*
cd('./Autoregressive');
  pre_autoregressive([CISM_DX_DATA_DIR_ACE,'YYYY_MM_DD_HH_MM_SS_V.daily.dat'])
  run_autoregressive
 post_autoregressive
cd('..');
end

if (1) % WSA predictions don't work yet
clear run_* pre_* post_* create_*
cd('./wsa_wso')
  pre_wsa_wso
  run_wsa_wso
 post_wsa_wso
cd('..')
end

if (1)
clear run_* pre_* post_* create_*
cd('./wsa_mwo')
  pre_wsa_mwo
  run_wsa_mwo
 post_wsa_mwo
cd('..')
end
fprintf('--Finished running models.\n');fflush(stdout);
fprintf('\n');fflush(stdout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the results to .eps files using Octave epsTK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('--Start plotting data.\n');fflush(stdout);
clear plot_output;
      plot_output;
fprintf('--Finished plotting data.\n');fflush(stdout);
fprintf('\n');fflush(stdout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert the .eps files to .jpg using Imagemagik
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('--Start converting .eps file to .jpg ...\n');fflush(stdout);
cd('./web_page');
system([CISM_DX_CONVERT_PATH,...
	' -density  72 -quality 100 Vsw_current.eps    Vsw_current.jpg']);
system([CISM_DX_CONVERT_PATH,...
	' -density 144 -quality 100 Vsw_current.eps Vsw_current_lg.jpg']);
cd('..');
fprintf('--Done converting plots.\n');fflush(stdout);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('___________________________________________________________________\n');
fprintf('Exiting main program for running Solar Wind Velocity models.\n');
fprintf('__________________________________________________________________\n');
    


