#!/bin/bash
##########################################################################
# UpdateDXNetworks.sh 
#
# Peter Schmitt schmitt(at)ucar.edu
# Time-stamp: <2007-11-29 15:10:17 schmitt>
#
# This script will
#  1.  convert exsisting DX networks to work with "new" cismDX 0.59.
#  2.  downgrade networks to "old" cismDX 0.58.
# Run ./UpdateDXNetworks.sh help for more information.
#
# This script requires the "find_replace.pl" script.
##########################################################################

function find_replace() {

    BASE_DIR=$(dirname $(readlink -f "$0" 2>/dev/null||echo "$0"))
    if [ "$BASE_DIR" = '.' ]; then
	BASE_DIR=$(pwd)
    fi
    echo "the current dir is"
    echo $PWD
    echo "The base dir is " 
    echo $BASE_DIR

    echo 
    echo "Changing   \"$1\"   with   \"$2\""
    CHANGE=$(perl $BASE_DIR/find_replace.pl "$1" "$2" 1 1 $3)
    return 0
}

if [ $# -ne 2 ]; then
    echo "Usage: "
    echo "  sh UpdateDXNetworks.sh [path] new"
    echo "    Search [path] and convert cismDX <=0.58 networks to work with cismDX >0.59"
    echo
    echo "  sh UpdateDXNetworks.sh [path] old"
    echo "    Search [path] and convert cismDX >=0.59 networks to work with cismDX <0.59"
    echo
    exit 0
fi

if ! [ -d $1 ]; then
    echo "ERROR:  Need valid directory to search for DX files."
    echo
    exit 1
fi

if [ $2 == "new" ]; then
    echo "Converting Networks from cismdDX <= 0.58 to cismDX >= 0.59"
    echo "Finding files which end in .animate, .cfg, .m, or .net, and replacing" 
    find_replace "yrdayhr" "YrDayHr" $1
    find_replace "animatefile" "AnimateFile" $1
    find_replace "axes" "Axes" $1
    find_replace "fileloop" "FileLoop" $1
    find_replace "readHelio" "ReadHelio" $1
    find_replace "readmhdhdf106" "ReadMHDHDF106" $1
    find_replace "readmhdhdf53" "ReadMHDHDF53" $1
    find_replace "readmhdhdfdbl" "ReadMHDHDFDbl" $1
    find_replace "readmhdhdf" "ReadMHDHDF" $1
    find_replace "smgse" "SMGSE" $1
    find_replace "yrdayhr" "YrDayHr" $1

    echo "Converted DX files (.animate, .cfg, .m, .net) from cismdDX <= 0.58 to cismDX >= 0.59"
    echo "To undo changes, simply execute "
    echo "  UpdateDXNetworks.sh $1 old"
    exit 0

else 
    if [ $2 == "old" ]; then
	echo "Converting DX files from cismdDX >= 0.59 to cismDX <= 0.58"
	echo "Finding files which end in .animate, .cfg, .m, or .net, and replacing" 
	find_replace "YrDayHr" "yrdayhr" $1
	find_replace "AnimateFile" "animatefile" $1
	find_replace "Axes" "axes" $1
	find_replace "FileLoop" "fileloop" $1
	find_replace "ReadHelio" "readHelio" $1
	find_replace "ReadMHDHDF106" "readmhdhdf106" $1
	find_replace "ReadMHDHDF53" "readmhdhdf53" $1
	find_replace "ReadMHDHDFDbl" "readmhdhdfdbl" $1
	find_replace "ReadMHDHDF" "readmhdhdf" $1
	find_replace "SMGSE" "smgse" $1
	find_replace "YrDayHr" "yrdayhr" $1
	
	echo "Converted DX files (.animate, .cfg, .m, .net) from cismdDX >= 0.59 to cismDX <= 0.58"
	echo "To undo changes, simply execute "
	echo "  UpdateDXNetworks.sh $1 new"
	exit 0
    else
	echo "ERROR:  Unsure if converting to new or old networks"
	echo "Try running  \"UpdateDXNetworks.sh help\""
	echo 
	exit 1
    fi
fi
