#include <stdio.h>
#include <math.h>
#include <dx/dx.h>

#define MAX_LENGTH 256
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

struct NextNState {
  int iStart;
  int iStop;
  int iDelta;
  int iNext;
};

Error NextNState_delete(Pointer in) {
  if (in) DXFree(in);
  return OK;
}

Error m_AnimateFile(Object *in, Object *out)
{
  int i, iNumFiles, iStart, iStop, iDelta, iLen, iBaseLen, iTiffLen;
  int *ipStart, *ipStop, *ipDelta;
  int iInputExtLen, iOutputExtLen;
  char *cpAnimateFile, caBaseDir[MAX_LENGTH], caTiffDir[MAX_LENGTH];
  char *cpInputExt, *cpOutputExt;
  char caFileName[MAX_LENGTH], **cppInFiles, **cppOutFiles,  *cpPtr;
  Array AInFiles, AOutFiles;
  Object OInFiles, OOutFiles, ONNSFiles;
  char *cpModId;
  int iFirst;
  FILE *FpAnimateFile;
  struct NextNState *NNSpFiles=NULL;

  out[0] = NULL;
  out[1] = NULL;
  out[2] = NULL;
  cpModId = DXGetModuleId();
  iFirst = DXLoopFirst();

  DXMessage("First: %d",iFirst);
  if(iFirst) {
    DXMessage("Inside First: %d",iFirst);
    /* Extract animate.txt file name */
    if(!in[0]) {
      DXSetError(ERROR_BAD_PARAMETER,"Missing Input Filename");
      goto error;
    } 
    if (!DXExtractString(in[0], &cpAnimateFile)) {
      DXSetError(ERROR_BAD_PARAMETER, "Animate File must be a string");
      goto error;
    }
    if(!in[1]) {
      cpInputExt = ".hdf";
    } else if (!DXExtractString(in[1], &cpInputExt)) {
      DXSetError(ERROR_BAD_PARAMETER, "Input Ext must be a string");
      goto error;
    }
      iInputExtLen = strlen(cpInputExt);
    if(!in[2]) {
     cpOutputExt = ".tiff";
    } else  if (!DXExtractString(in[2], &cpOutputExt)) {
      DXSetError(ERROR_BAD_PARAMETER, "Output Ext must be a string");
      goto error;
    }
      iOutputExtLen = strlen(cpOutputExt);
    /* Now we open the animate.txt file and read the files to be processed */
    FpAnimateFile = fopen(cpAnimateFile,"r");
    if(!FpAnimateFile) {
      DXSetError(ERROR_DATA_INVALID,"Cannot open animte file");
      goto error;
    }
    fscanf(FpAnimateFile,"%d",&iNumFiles);
    fscanf(FpAnimateFile,"%s",caBaseDir);
    iBaseLen = strlen(caBaseDir);
    fscanf(FpAnimateFile,"%s",caTiffDir);
    iTiffLen = strlen(caTiffDir);
    DXMessage("%d %s %s",iNumFiles,caBaseDir,caTiffDir);
    cppInFiles = (char *)malloc(iNumFiles*sizeof(char*));
    cppOutFiles = (char *)malloc(iNumFiles*sizeof(char*));
     for(i=0;i<iNumFiles;i++) {
      fscanf(FpAnimateFile,"%s %d %d %d",caFileName,&iStart,&iStop,&iDelta);
      /*DXMessage("%s %d %d %d",caFileName,iStart,iStop,iDelta);*/
      /*fscanf(FpAnimateFile,"%s",caFileName,&iStart,&iStop,&iDelta);*/
      iLen = strlen(caFileName);
      /*DXMessage("%d %d %d %s",iTiffLen,iLen,iInputExtLen,cpInputExt);*/
      cpPtr = (char *) malloc((iBaseLen+iLen+iInputExtLen+1)*sizeof(char));
      strcpy(cpPtr,caBaseDir);
      strcat(cpPtr,caFileName);
      strcat(cpPtr,cpInputExt);
      cppInFiles[i] = cpPtr;
      /*DXMessage("%d %d %d %s",iTiffLen,iLen,iOutputExtLen,cpOutputExt);*/
      cpPtr = (char *) malloc((iTiffLen+iLen+iOutputExtLen+1)*sizeof(char));
      strcpy(cpPtr,caTiffDir);
      strcat(cpPtr,caFileName);
      strcat(cpPtr,cpOutputExt);
      cppOutFiles[i] = cpPtr;
     }
     /* Now we create the output array and put the string list into it*/
     AInFiles = DXMakeStringListV(iNumFiles,cppInFiles);
     AOutFiles = DXMakeStringListV(iNumFiles,cppOutFiles);
     OInFiles = (Object)AInFiles;
     OOutFiles = (Object)AOutFiles;
     /* Next we get the NextNState object up */
     NNSpFiles = DXAllocateZero(sizeof(struct NextNState));
     if(NNSpFiles == NULL) {
      DXSetError(ERROR_INTERNAL,"Lost state information for loop");
       goto error;
     }
     NNSpFiles->iStart = 0 ; /* Start at beginning*/
     NNSpFiles->iStop = iNumFiles-1; /* Using 0 to N-1 for C indexing*/
     NNSpFiles->iDelta = 1; /* Want each file*/
     NNSpFiles->iNext = 0; /* First time is first file*/
     DXLoopDone(FALSE);  /* First time thourgh so we are not done*/

     /*Finally we cache the state information*/
     ONNSFiles = (Object)DXNewPrivate((Pointer)NNSpFiles,NextNState_delete);
     DXSetCacheEntryV(ONNSFiles,CACHE_PERMANENT,cpModId,0,0,NULL);
     /*DXSetCacheEntryV(OInFiles,CACHE_PERMANENT,cpModId,0,0,NULL);*/
     /*DXSetCacheEntryV(OOutFiles,CACHE_PERMANENT,cpModId,0,0,NULL);*/
  } else {
    DXMessage("Inside else");
    ONNSFiles = DXGetCacheEntryV(cpModId,0,0,NULL);
    DXDelete(ONNSFiles);
    if(ONNSFiles==NULL) {
      DXSetError(ERROR_INTERNAL,"Lost state information for loop");
      goto error;
    }
    /*OInFiles = DXGetCacheEntryV(cpModId,0,0,NULL);*/
    /*OOutFiles = DXGetCacheEntryV(cpModId,0,0,NULL);*/
    NNSpFiles = (struct NextNState*)DXGetPrivateData((Private)ONNSFiles);
    DXMessage("%d %d %d %d",NNSpFiles->iStart,NNSpFiles->iStop,
		            NNSpFiles->iDelta,NNSpFiles->iNext);
    NNSpFiles->iNext += NNSpFiles->iDelta;
  }
   out[0] = OInFiles;
   out[1] = OOutFiles;
   out[2] = (Object)DXMakeInteger(NNSpFiles->iNext);

  /* Check and see if we are done*/
  if(NNSpFiles->iNext>(NNSpFiles->iStop-NNSpFiles->iDelta)) {
    DXLoopDone(TRUE);
  }

  /* if first time save modid so that program can clean up when it terminates*/
  if(iFirst)
    DXSaveForId(cpModId);
  else
    DXFreeModuleId(cpModId);
  
  return OK;

  error:
    out[0] = NULL;
    out[1] = NULL;
    out[2] = NULL;
    if(NNSpFiles)
      DXFree(NNSpFiles);
    DXFreeModuleId(cpModId);
    DXLoopDone(TRUE);
    return ERROR;
}
