/* R.S. Weigel, 11/15/2004 */

#include "dx/dx.h"

#include "OctaveOp.h"

extern "C" int OctaveOp(char *command, OctaveOp_structure *OctaveOpStruct,
			int verbosity);

extern "C" Error m_OctaveOp(Object *in, Object *out)
{

  static OctaveOp_structure OctOpStrct;
  int OctaveOpError;

  int nIn     = 0; /* Number of items */
  int rIn     = 0; /* rank  */
  int sIn     = 0; /* shape */
  int nIn_pos = 0; /* Number of items */
  int rIn_pos = 0; /* rank  */
  int sIn_pos = 0; /* shape */
  int nIn_con = 0; /* Number of items */
  int rIn_con = 0; /* rank  */
  int sIn_con = 0; /* shape */
  int nOut    = 0;
  int rOut    = 2;
  int sOut    = 0;

  Type   tIn;
  Type   tIn_pos;
  Type   tIn_con;
  Category cIn;
  Category cIn_pos;
  Category cIn_con;
  Array  aIn;        /* Input 0 */
  Array  aIn_pos;    /* Input 0 */
  Array  aIn_con;    /* Input 0 */
  Array  aOut;       /* Output 0 */
  Array  aOut_pos;   /* Output 0 */
  Array  aOut_con;   /* Output 0 */
  Field  FOut = NULL;

  int   i, j;
  int   N, Mitems;
  int   N_pos, Mitems_pos;
  int   N_con, Mitems_con;
  char *OctaveCommand;  /* Input 1 */
  int   verbosity = 0;  /* Input 2 */

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/

  /******************************************************************/
  /******************************************************************/
  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"Input 0 is required.");
    return ERROR;
  }

  /* Input 0 can be a Field or Array */
  /* Case Field */
  if (DXGetObjectClass(in[0]) == CLASS_FIELD) {
    OctOpStrct.isvaluelist = 0;
    OctOpStrct.isfield     = 1;

    /* Data compononent */
    aIn   = (Array) DXGetComponentValue((Field)in[0],"data");
    DXGetArrayInfo(aIn, &nIn, &tIn, &cIn, &rIn, &sIn);

    /* Allow case where Field has no data component (aIn = NULL) */
    if (aIn != NULL) {
      /* If data component exists, demand it to be float and real */
      if (tIn != TYPE_FLOAT) {
	DXSetError(ERROR_DATA_INVALID, 
		   "Type of input 1 data component should be float.");
	return ERROR;
      }
      if (cIn != CATEGORY_REAL) {
	DXSetError(ERROR_DATA_INVALID, 
		   "Category of input 1 data component should be real.");
	return ERROR;
      }	
    }

    N      = sIn; /* Number of columns */
    Mitems = nIn; /* Number of rows */

    /* Position compononent */
    aIn_pos = (Array) DXGetComponentValue((Field)in[0],"positions");
    DXGetArrayInfo(aIn_pos, &nIn_pos, &tIn_pos, &cIn_pos, &rIn_pos, &sIn_pos);

    /* If field has no position component, ERROR */
    if (aIn_pos == NULL) {
      DXSetError(ERROR_DATA_INVALID,"Input 1 must have a position component.");
      return ERROR;
    }

    /* Demand position component to be float and real */
    if (tIn_pos != TYPE_FLOAT) {
      DXSetError(ERROR_DATA_INVALID, 
		 "Type of input 1 position component should be float.");
      return ERROR;
    }
    if (cIn_pos != CATEGORY_REAL) {
      DXSetError(ERROR_DATA_INVALID, 
		 "Category of input 1 position component should be real.");
      return ERROR;
    }	

    N_pos      = sIn_pos; /* Number of columns */
    Mitems_pos = nIn_pos; /* Number of rows */


  /* Error if rank is greater than 1 */
  if (rIn_pos > 1) {
    DXSetError(ERROR_DATA_INVALID,
	       "Input position component has a rank of %d. Only vectors or scalars are allowed.",
	       rIn_pos);
    return ERROR;
  }

    /* Connection compononent */
    /* Allow case where Field has no connection component (aIn_con = NULL) */
    if (aIn_con != NULL) {
      aIn_con = (Array) DXGetComponentValue((Field)in[0],"connections");
      DXGetArrayInfo(aIn_con, &nIn_con, &tIn_con, &cIn_con, &rIn_con, &sIn_con);

      /* In some cases there can be a connections component when there */
      /* are no connections.  In this case, force the connections component  */
      /* to be an empty array in Octave. */

      if (sIn_con == 0) {
	nIn_con = 0; 
      } else {
	/* Demand connection component to be float and real */
	if (tIn_con != TYPE_INT) {
	  DXSetError(ERROR_DATA_INVALID, 
	   "Type of input 1 connection component should be integer.");
	  return ERROR;
	}
	if (cIn_con != CATEGORY_REAL) {
	  DXSetError(ERROR_DATA_INVALID, 
		     "Category of input 1 connection component should be real.");
	  return ERROR;
	}	
      }

      N_con      = sIn_con; /* Number of columns */
      Mitems_con = nIn_con; /* Number of rows */
    }


  } /* end of class field case */


  /******************************************************************/
  /******************************************************************/

  /******************************************************************/
  /******************************************************************/
  /* Input 0 can be a Field or Array */
  /* Case Array */
  if (DXGetObjectClass(in[0]) == CLASS_ARRAY) {
    aIn  = (Array) in[0];
    if (aIn == NULL) {
      DXSetError(ERROR_DATA_INVALID,"Input 1 is not a valid array.");
      return ERROR;
    }
    DXGetArrayInfo(aIn, &nIn, &tIn, &cIn, &rIn, &sIn);
    OctOpStrct.isvaluelist = 1;
    OctOpStrct.isfield     = 0;
    if (rIn == 0) {
      sIn = 1; /* In OpenDX, shape is not returned for a 1-vector */
               /* Define a scalar as a 1-vector for convienence   */
    }

    /* Demand array to be float and real */
    if (tIn != TYPE_FLOAT) {
      DXSetError(ERROR_DATA_INVALID, 
		 "Type of input 1 should be float.");
      return ERROR;
    }
    if (cIn != CATEGORY_REAL) {
      DXSetError(ERROR_DATA_INVALID, 
		 "Category of input 1 position component should be real.");
      return ERROR;
    }	

    N      = sIn; /* Number of columns */
    Mitems = nIn; /* Number of rows */
  } 
  /******************************************************************/
  /******************************************************************/

  /******************************************************************/
  /* Get the dummy string.
  /******************************************************************/
  if (!in[1]){
    OctaveCommand = "";
  } else {
    DXExtractString(in[1],  &OctaveCommand);
  }
  /******************************************************************/
  /******************************************************************/
 
  /******************************************************************/
  /******************************************************************/
  if (!in[2]){
    verbosity = 0;
  } else {
    DXExtractInteger(in[2], &verbosity);
  }
  /******************************************************************/
  /******************************************************************/


  /******************************************************************/
  /* Done getting inputs                                            */
  /******************************************************************/

  if (verbosity > 0)
    printf("\nOctaveOp.c was passed the command %s from OpenDX.\n",
	   OctaveCommand);fflush(stdout);

    OctOpStrct.rank         = rIn;
    OctOpStrct.dim_sizes    = (int*)malloc(sizeof(int) * 2);
    OctOpStrct.dim_sizes[0] = Mitems;
    OctOpStrct.dim_sizes[1] = N;
    OctOpStrct.array_1D     = (float*)DXGetArrayData(aIn); 
    
    if (verbosity > 0) {
    if ( OctOpStrct.isvaluelist == 1 )
      printf("A %d item, %d-vector list passed to OctaveOp.c from OpenDX:\n",
	     Mitems,N);
    if ( OctOpStrct.isfield == 1 )
      printf("Field passed to OctaveOp.c from OpenDX.\nData Component:\n");

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){
	printf ("%f ", OctOpStrct.array_1D[j + N*i]);
      }
      printf ("\n"); fflush(stdout);
    }

  }
  if ( OctOpStrct.isfield == 1 ) {

    OctOpStrct.rank_pos         = rIn_pos;
    OctOpStrct.dim_sizes_pos    = (int*)malloc(sizeof(int) * 2);
    OctOpStrct.dim_sizes_pos[0] = Mitems_pos;
    OctOpStrct.dim_sizes_pos[1] = N_pos;
    OctOpStrct.array_1D_pos     = (float*)DXGetArrayData(aIn_pos);

    if (verbosity > 0) {
      printf("Position Component:\n",Mitems_pos,N_pos);
      for (i = 0; i < Mitems_pos; i++){
	for (j = 0; j < N_pos; j++){
	  printf ("%f ", OctOpStrct.array_1D_pos[j + N_pos*i]);
	}
	printf ("\n"); fflush(stdout);
      }
    }

    OctOpStrct.rank_con         = rIn_con;
    OctOpStrct.dim_sizes_con    = (int*)malloc(sizeof(int) * 2);
    OctOpStrct.dim_sizes_con[0] = Mitems_con;
    OctOpStrct.dim_sizes_con[1] = N_con;
    OctOpStrct.array_1D_con     = (int*)DXGetArrayData(aIn_con);

    if (verbosity > 0) {
      printf("Connection Component:\n",Mitems_con,N_con);
      for (i = 0; i < Mitems_con; i++){
	for (j = 0; j < N_con; j++){
	  printf ("%d ", OctOpStrct.array_1D_con[j + N_con*i]);
	}
	printf ("\n"); fflush(stdout);
      }
    }

  }


  /*******************************************************************/
  /*******************************************************************/
  /*******************************************************************/
  /*******************************************************************/
  /*******************************************************************/
  /* Call the CPP function *******************************************/

  /* Without this line, and with verbosity = 0, program crashes */
  /* on second execution.  Need to find out why. */ 
  printf ("\n"); fflush(stdout); 

  OctaveOpError = OctaveOp(OctaveCommand , &OctOpStrct, verbosity);
  if (OctaveOpError == -1)
    DXSetError(ERROR_INTERNAL, "OctaveOp returned error.  Aborting.");
  /*******************************************************************/
  /*******************************************************************/
  /*******************************************************************/
  /*******************************************************************/

  Mitems    = OctOpStrct.dim_sizes[0]; /* Same for position vector */
  N         = OctOpStrct.dim_sizes[1];

  if ( OctOpStrct.isfield == 1 ) {
    N_pos     = OctOpStrct.dim_sizes_pos[1];
    N_con     = OctOpStrct.dim_sizes_con[1];
  }

  if (verbosity > 0) {

    if ( OctOpStrct.isvaluelist == 1 ) {
      printf("A %d item,%d-vector list returned to OctaveOp.c",Mitems,N);
      printf("from OctaveOp.cpp:\n");
    }
    if ( OctOpStrct.isfield == 1 ) {
      printf("Field returned to OctaveOp.c from OctaveOp.cpp.");
      printf("\nData Component:\n");
    }

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){
	  printf ("%f ", OctOpStrct.array_1D[j + N*i]);
      }
	printf ("\n"); fflush(stdout);
    }

    if ( OctOpStrct.isfield == 1 ) {
      printf("Position Component:\n");
      for (i = 0; i < Mitems; i++){
	for (j = 0; j < N_pos; j++){
	  printf ("%f ", OctOpStrct.array_1D_pos[j + N_pos*i]);
	}
	printf ("\n"); fflush(stdout);
      }

      printf("Connection Component:\n");
      for (i = 0; i < Mitems; i++){
	for (j = 0; j < N_con; j++){
	  printf ("%d ", OctOpStrct.array_1D_con[j + N_con*i]);
	}
	printf ("\n"); fflush(stdout);
      }


    }

  }
  
  if (DXGetObjectClass(in[0]) == CLASS_ARRAY) {
    aOut   = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, N);
             DXAddArrayData(aOut, 0, Mitems, OctOpStrct.array_1D);
    out[0] = (Object) aOut;

  }

  if (DXGetObjectClass(in[0]) == CLASS_FIELD) {

    FOut  = DXNewField();

    aOut  = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, N);
            DXAddArrayData(aOut, 0, Mitems, OctOpStrct.array_1D);
    DXSetStringAttribute((Object)aOut,"dep", "positions");

    DXSetComponentValue(FOut, "data", (Object)aOut);

    aOut_pos = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, N_pos);
    DXAddArrayData(aOut_pos, 0, Mitems, OctOpStrct.array_1D_pos);
    DXSetComponentValue(FOut, "positions", (Object)aOut_pos);

    /* OctaveOp.cpp returns N_con = 0 (and a warning) if mfile returned */
    /* a field with an invalid connections component. */
    if (N_con > 0) {
      aOut_con = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, N_con);
      DXAddArrayData(aOut_con, 0, Mitems, OctOpStrct.array_1D_con);
      DXSetComponentValue(FOut, "connections", (Object)aOut_con);
    }
    out[0] = (Object) FOut;

    free(OctOpStrct.array_1D_pos);
    free(OctOpStrct.dim_sizes_pos);
    free(OctOpStrct.array_1D_con);
    free(OctOpStrct.dim_sizes_con);

  }

  free(OctOpStrct.dim_sizes);
  free(OctOpStrct.array_1D);
  
  if (verbosity > 0)
    printf("OctaveOp.c was has completed the command: %s\n\n\n",
	   OctaveCommand);fflush(stdout);

  return OK;

}

