/* R.S. Weigel 11/15/2004 */
#include <stdio.h>
#include <iostream>

#include "embed.h"
#include "OctaveOp.h"


extern "C" int OctaveOp(char *command, OctaveOp_structure *OctOpStrct,
			int verbosity)
{

  int   i,j;
  int   Mitems,N;
  int   Mitems_pos,N_pos;
  int   Mitems_con,N_con;
  int   rank = 2;
  int   argc = 3;
  int   parse_status = 1;
  char *argv[10];

  /* Set up Octave command line arguements */
  //argv[0] = (char*)calloc(67,sizeof(char));
  argv[1] = (char*)calloc(17,sizeof(char));
  argv[2] = (char*)calloc(2,sizeof(char));
  //strcpy(argv[0],"/usr/local/CISM_DX/octave");
  strcpy(argv[1],"--traditional");
  strcpy(argv[2],"-q");
  //printf("%s\n",argv[0]);fflush(stdout);

  /////////////////////////////////////////////////////////////////////////////
  // Data component
  /////////////////////////////////////////////////////////////////////////////
  Mitems = OctOpStrct->dim_sizes[0];
  N      = OctOpStrct->dim_sizes[1];

  // Is there a way to declare a float Octave matrix so that
  // (double) cast is not needed in next for loop?
  Matrix Mdatain = Matrix(Mitems,N);

  if ( (verbosity > 1) & (OctOpStrct->isvaluelist == 1) ) {
    printf("A %d item, %d-vector list passed to OctaveOp.cpp\n",Mitems,N);
    printf(" from OctaveOp.c:\n");
  }

  if ( (verbosity > 1) & (OctOpStrct->isfield == 1) )
    printf("Field passed to OctaveOp.cpp from OctaveOp.c.\nData Component:\n");

  for (i = 0; i < Mitems; i++){
    for (j = 0; j < N; j++){
      Mdatain(i, j) = (double) OctOpStrct->array_1D[j + N*i];
      if (verbosity > 1)
	printf ("%f ", OctOpStrct->array_1D[j + N*i]);fflush(stdout);
    }
    if (verbosity > 1)
      printf ("\n"); fflush(stdout);    
  }

  /////////////////////////////////////////////////////////////////////////////
  // Position component
  /////////////////////////////////////////////////////////////////////////////
  if (OctOpStrct->isfield == 1) {
    Mitems = OctOpStrct->dim_sizes_pos[0];
    N      = OctOpStrct->dim_sizes_pos[1];
  } else {
    Mitems = 0;
    N      = 0;
  }

  // Need to put have declaration outside of below if statement
  // otherwise compiler error.  Is there a better way to do this?
  Matrix Mposin  = Matrix(Mitems,N);

  if (OctOpStrct->isfield == 1) {
    if (verbosity > 1)
      printf("Position Component:\n",Mitems,N);

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){
	Mposin(i, j) = (double) OctOpStrct->array_1D_pos[j + N*i];
	if (verbosity > 1)
	  printf ("%f ", OctOpStrct->array_1D_pos[j + N*i]);fflush(stdout);
      }
      if (verbosity > 1)
	printf ("\n"); fflush(stdout);    
    }
  }
  ////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////////
  // Connection component
  /////////////////////////////////////////////////////////////////////////////
  if (OctOpStrct->isfield == 1) {
    Mitems = OctOpStrct->dim_sizes_con[0];
    N      = OctOpStrct->dim_sizes_con[1];
  } else {
    Mitems = 0;
    N      = 0;
  }

  // Need to put have declaration outside of below if statement
  // otherwise compiler error.  Is there a better way to do this?
  Matrix Mconin  = Matrix(Mitems,N);

  if (OctOpStrct->isfield == 1) {
    if (verbosity > 1)
      printf("Connection Component:\n",Mitems,N);

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){
	Mconin(i, j) = (int) OctOpStrct->array_1D_con[j + N*i];
	if (verbosity > 1)
	  printf ("%d ", OctOpStrct->array_1D_con[j + N*i]);fflush(stdout);
      }
      if (verbosity > 1)
	printf ("\n"); fflush(stdout);    
    }
  }
  ////////////////////////////////////////////////////////////////////////////

  if (verbosity > 0) {
    printf("\n\n\n==========================================================");
    printf("========================\n");
    printf("============================================================");
    printf("======================\n");
    printf("OctaveOp.cpp is evaluating command: %s\n",command);
    printf("With command line arguments: %s %s\n\n",argv[1],argv[2]);
    fflush(stdout);
  }

  /////////////////////////////////////////////////////////////////////////////
  // Pass command to the Octave interpreter
  /////////////////////////////////////////////////////////////////////////////
  octave_init(argc, argv);

  /////////////////////////////////////////////////////
  // Install the ValueList into Octave's symbol table
  /////////////////////////////////////////////////////
  if (OctOpStrct->isvaluelist == 1) {
    octave_value ovMdata;
    ovMdata = Mdatain;
    symbol_record *const sr2 = curr_sym_tab->lookup ("ValueList", true);
    if (sr2)
      sr2->define (ovMdata);
    else
      panic_impossible ();
  }
  /////////////////////////////////////////////////////

  /////////////////////////////////////////////////////
  // Install the Field into Octave's symbol table
  /////////////////////////////////////////////////////
  if (OctOpStrct->isfield == 1) {
    octave_value ovMdatain;
    octave_value ovMposin;
    octave_value ovMconin;
    ovMdatain = Mdatain;
    ovMposin  = Mposin;
    ovMconin  = Mconin;
    Octave_map map;
    map.assign ("data",octave_value (ovMdatain));
    map.assign ("positions",octave_value (ovMposin));
    map.assign ("connections",octave_value (ovMconin));
    symbol_record *const sr = curr_sym_tab->lookup ("Field", true);
    if (sr)
      sr->define (map);
    else
      panic_impossible ();
  }
  /////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////
  // Evaluate the command /////////////////////////////////////
  octave_value tmp2 = eval_string (command,false,parse_status);
  /////////////////////////////////////////////////////////////

  octave_exit();
  /////////////////////////////////////////////////////////////////////////////

  if (verbosity > 0) {
    printf("OctaveOp.cpp has evaluated command: %s\n",command);
    printf("With command line arguments: %s %s\n",argv[1],argv[2]);
    printf("================================================================");
    printf("==================\n");
    printf("================================================================");
    printf("==================\n\n\n\n");
    fflush(stdout);
  }
  /////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////////
  // Extract the ValueList from Octave's symbol table
  /////////////////////////////////////////////////////////////////////////////
  if (OctOpStrct->isvaluelist == 1) {
    Matrix Mdataout = tmp2.matrix_value();
    N               = Mdataout.cols ();
    Mitems          = Mdataout.rows ();

    if (Mitems > 1)
      rank = 2;

    // Allocate space based on returned data
    OctOpStrct->rank         = rank;
    OctOpStrct->dim_sizes[0] = Mitems;
    OctOpStrct->dim_sizes[1] = N;
    // Seems like this should be a realloc.
    OctOpStrct->array_1D     = (float*)malloc(sizeof(float)*(int) N*Mitems);
    
    if (verbosity > 1) {
      printf("A %d item, %d-vector list returned to ",Mitems,N);
      printf("OctaveOp.cpp from Octave:\n");
    }

    // recasting
    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){

	OctOpStrct->array_1D[j+N*i] = (float) Mdataout.data ()[i+Mitems*j];

	if (verbosity > 1)
	  printf ("%f ", OctOpStrct->array_1D[j+N*i]);fflush(stdout);
      }
      if (verbosity > 1)
	printf ("\n"); fflush(stdout);
    }
  } // end of isvaluelist
  /////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////////
  // Extract the Field from Octave's symbol table
  /////////////////////////////////////////////////////////////////////////////
  if (OctOpStrct->isfield == 1) {
    Octave_map Om = tmp2.map_value ();

    if (verbosity > 1) {
      printf("Does data component exist?: %d\n",Om.contains("data"));
      fflush(stdout);
      printf("Does a positions component exist?: %d\n",
	     Om.contains("positions"));
      fflush(stdout);
    }

    Octave_map::const_iterator ii = Om.seek ("data");
    // Next line worked in 2.1.50, but not 2.1.71
    // octave_value_list val = Om.contents (ii);
    // use these two lines instead
    Cell val;
    val = Om.contents (ii);
    Matrix Mdataout = val(0).matrix_value ();

    N        = Mdataout.cols ();
    Mitems   = Mdataout.rows ();

    printf("N=%d, Mitems=%d\n",N,Mitems);fflush(stdout);

    Octave_map::const_iterator ii_pos = Om.seek ("positions");
    // Worked in 2.1.50, but not 2.1.71
    // octave_value_list val_pos = Om.contents (ii_pos);
    // use these two lines instead
    Cell val_pos;
    val_pos = Om.contents (ii_pos);
    Matrix Mposout = val_pos(0).matrix_value ();

    N_pos      = Mposout.cols ();
    Mitems_pos = Mposout.rows ();

    if (Mitems != Mitems_pos){
      printf("OctaveOp.cpp Error: Octave script returned position \n");
      printf("component with (%d) items which must be the ",Mposout.rows ());
      printf("same as that for the data component (%d).\n",Mitems);
      return -1;
    }

    if (Mitems > 1)
      rank = 2;

    Octave_map::const_iterator ii_con = Om.seek ("connections");
    // Worked in 2.1.50, but not 2.1.71
    // octave_value_list val_con = Om.contents (ii_con);
    // use these two lines instead
    Cell val_con;
    val_con = Om.contents (ii_con);
    Matrix Mconout = val_con(0).matrix_value ();

    N_con      = Mconout.cols ();
    Mitems_con = Mconout.rows ();

    if (Mitems != Mitems_con){
      printf("\nOctaveOp.cpp Warning: Octave script returned connection \n");
      printf("component with (%d) items which is not the ",Mconout.rows ());
      printf("same as that for the data component (%d).\n",Mitems);
      printf("Returned field will not have a connections component.\n\n");
      fflush(stdout);
      N_con      = 0;
      Mitems_con = 0;
    }

    // Allocate space based on returned data
    OctOpStrct->rank         = rank;
    OctOpStrct->dim_sizes[0] = Mitems;
    OctOpStrct->dim_sizes[1] = N;
    // Seems like this should be a realloc.
    OctOpStrct->array_1D     =  (float*)malloc(sizeof(float)*(int) N*Mitems);

    OctOpStrct->rank_pos         = rank;
    OctOpStrct->dim_sizes_pos[0] = Mitems;
    OctOpStrct->dim_sizes_pos[1] = N_pos;
    // Seems like this should be a realloc.
    OctOpStrct->array_1D_pos=(float*)malloc(sizeof(float)*(int) N_pos*Mitems);

    OctOpStrct->rank_con         = rank;
    OctOpStrct->dim_sizes_con[0] = Mitems;
    OctOpStrct->dim_sizes_con[1] = N_con;
    // Seems like this should be a realloc.
    OctOpStrct->array_1D_con=(int*)malloc(sizeof(int)*(int) N_con*Mitems_con);

    // It would be nice to remove this casting step, but I don't think
    // it is possible.  (Octave uses double for nearly everything, but
    // OpenDX usually uses floats which is usually sufficient for graphics).
    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){
	OctOpStrct->array_1D[j+N*i] = 
	  (float) Mdataout.data ()[i+Mitems*j];
      }
      for (j = 0; j < N_pos; j++){
	OctOpStrct->array_1D_pos[j+N_pos*i] = 
	  (float) Mposout.data ()[i+Mitems*j];
      }
      for (j = 0; j < N_con; j++){
	OctOpStrct->array_1D_con[j+N_con*i] = 
	  (int) Mconout.data ()[i+Mitems*j];
      }
    }

    if (verbosity > 1) {
      printf("\nField returned to OctaveOp.cpp from Octave.\n");
      printf("\nData component:\n");
      fflush(stdout);
    }

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N; j++){

	OctOpStrct->array_1D[j+N*i] = 
	  (float) Mdataout.data ()[i+Mitems*j];

	if (verbosity > 1)
	  printf ("%f ", OctOpStrct->array_1D[j+N*i]);fflush(stdout);

      }
      if (verbosity > 1)
	printf ("\n"); fflush(stdout);
    }

    if (verbosity > 1) {
      printf("Position component:\n",Mitems_pos,N_pos);
      fflush(stdout);
    }

    for (i = 0; i < Mitems; i++){
      for (j = 0; j < N_pos; j++){

	OctOpStrct->array_1D_pos[j+N_pos*i] = 
	  (float) Mposout.data ()[i+Mitems*j];

	if (verbosity > 1)
	  printf ("%f ", OctOpStrct->array_1D_pos[j+N_pos*i]);fflush(stdout);
      }

      if (verbosity > 1)
	printf ("\n"); fflush(stdout);
    }

    for (i = 0; i < Mitems_con; i++){
      for (j = 0; j < N_con; j++){

	OctOpStrct->array_1D_con[j+N_con*i] = 
	  (int) Mconout.data ()[i+Mitems*j];

	if (verbosity > 1)
	  printf ("%d ", OctOpStrct->array_1D_con[j+N_con*i]);fflush(stdout);
      }

      if (verbosity > 1)
	printf ("\n"); fflush(stdout);
    }

  } // end of isfield
  /////////////////////////////////////////////////////////////////////////////

  return 0;

}
