function Field = OctaveOpDemoSombrero(Field,astring)
%OctaveOpDemoSombrero Demo of passing a field to/from OpenDX.
%
%  Field = OctaveOPDemoSombrero(Field, String)
%  
%    Field has components "data", "positions", "connections", and
%    optionally "colors".
%
%    String is an arbitrary string.
%
%    See also OctaveOpDemoValueList.

% Extract the x and y positions from the Field

xx = Field.positions(:,1); 
yy = Field.positions(:,2);

% The sombrero equations
r  = sqrt (xx .^ 2 + yy .^ 2) + eps;
z  = sin (r) ./ r;

% Give the field positions a z component
Field.positions = [Field.positions , 8.0*z(:)];

% Input Field is not a complete OpenDX Field because it lacks
% a data component.  The field needs a component for renedering
% and other operations.

% Make the data proportional to z position.
Field.data      = 8.0*z(:);
