#include <stdio.h>
#include <dx/dx.h>

#define MAX_NAME_SIZE 256
#define BUFFER 256

/* Another implementation of the IGRF model */
/* Based on igrf.c code by C-L. Huang */
/* Eliminated the need to read files because the input data */
/* can be drawn from the CISM_DX uniform time series data base */

/* R.S. Weigel 10/30/2004 */
/* To do: needs more error checking */

Error
m_IGRFDipole(Object *in, Object *out)
{


  /* Input Declarations */
  Array aDataVector;
  Array aPosVector;
  Array aConVector;
  float *fpDataVector;
  float *fpPosVector;
  float *fpConVector;
  Type tDataVector;
  Type tPosVector;
  Type tConVector;
  Category cDataVector;
  Category cPosVector;
  Category cConVector;
  int nDataVector, nPosVector, nConVector; /* Number of items */
  int sDataVector, sPosVector, sConVector; /* Shape of each item */
  int rDataVector, rPosVector, rConVector; /* Rank of each item */

  /* Output Declarations */
  Group gOutGroup = NULL;
  Field fOutField = NULL;
  Array aOutDataVector = NULL;
  Array aOutPosVector = NULL;
  Array aOutConVector = NULL;
  float *fpOutDataVector;
  float *fpOutPosVector;
  float *fpOutConVector;

  int i,j,k;

  int M;     /* Returned field has data component that is a M-vector. */
  int Nitems;/* Returned field has data component w/ N M-vectors (or "items")  */

  int   nUT;
  int   nPS;
  int *iaUT;
  float *faPS;
  int iUT[5];
  float fPS;
  Array aUT = NULL;
  Array aPS = NULL;
  float fX,fY,fZ,fBx,fBy,fBz;
  int iOpt = 0;

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"missing field");
    return ERROR;
  }
  else{

    aDataVector  = (Array) DXGetComponentValue((Field)in[0],"data");
    fpDataVector = DXGetArrayData(aDataVector);
    aPosVector   = (Array) DXGetComponentValue((Field)in[0],"positions");
    fpPosVector  = DXGetArrayData(aPosVector);
    aConVector   = (Array) DXGetComponentValue((Field)in[0],"connections");
    fpConVector  = DXGetArrayData(aConVector);

    DXGetArrayInfo(aDataVector, &nDataVector, &tDataVector, &cDataVector,
		   &rDataVector, &sDataVector);
    DXGetArrayInfo(aPosVector, &nPosVector, &tPosVector, &cPosVector,
		   &rPosVector, &sPosVector);
    DXGetArrayInfo(aConVector, &nConVector, &tConVector, &cConVector,
		   &rConVector, &sConVector);
    
    M      = sPosVector;
    Nitems = nPosVector;

  }

  if (!in[1]){
    iUT[0] =  2000; /* Year   */
    iUT[1] =     1; /* Month  */
    iUT[2] =     1; /* Day    */
    iUT[3] =     0; /* Hour   */
    iUT[4] =     0; /* Min   */
    nUT    =     1; 
  } 
  else {
    aUT  = (Array) in[1];
    iaUT = DXGetArrayData(aUT);
    DXGetArrayInfo(aUT, &nUT, NULL,NULL,NULL,NULL);
  }
  
  if (nUT > 1) {
    DXMessage("Multiple (%d) Universal Times specified.", nUT);
    DXMessage("Output will be a Group of Fields.  Each Field in");
    DXMessage("the Group corresponds to a vector in the UT vector");
    DXMessage("list.");
  }

  /******************************************************************/

  /******************************************************************/  
  /* Fill in the Output arrays with data                            */
  /******************************************************************/  

  if (nUT > 1) {
    gOutGroup = DXNewGroup();
  }

  for (k = 0; k < nUT ; k++) {

    /******************************************************************/
    /* Allocate and declare output OpenDX arrays as M-vectors         */
    /* with Nitems items                                              */
    /******************************************************************/
    aOutDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
    DXAddArrayData(aOutDataVector, 0, Nitems, NULL);
    fpOutDataVector = (float*)DXGetArrayData(aOutDataVector);
    /******************************************************************/

    if (in[1]){
   	iUT[0] = (int) iaUT[0+5*k];
   	iUT[1] = (int) iaUT[1+5*k];
   	iUT[2] = (int) iaUT[2+5*k];
   	iUT[3] = (int) iaUT[3+5*k];
   	iUT[4] = (int) iaUT[4+5*k];
    }

    DXMessage("Year = %d, Month = %d, Day = %d\n",iUT[0],iUT[1],iUT[2]);

    for (i=0; i < Nitems; i++) {
      
      fX = fpPosVector[0+M*i];
      fY = fpPosVector[1+M*i];
      fZ = fpPosVector[2+M*i];

      recalc_(&iUT[0], &iUT[1], &iUT[2], &iUT[3], &iUT[4]);
      dip_(&fX,&fY,&fZ,&fBx,&fBy,&fBz);

      fpOutDataVector[0+M*i] = fBx;
      fpOutDataVector[1+M*i] = fBy;
      fpOutDataVector[2+M*i] = fBz;

    }
    /******************************************************************/  

    /******************************************************************/   
    /* Form output field **********************************************/
    /******************************************************************/   
    fOutField = DXNewField();
    
    DXSetComponentValue(fOutField, "positions", (Object)aPosVector);
    
    DXSetStringAttribute((Object)aOutDataVector,"dep", "positions");
    DXSetComponentValue(fOutField, "data", (Object)aOutDataVector);
    
    DXSetComponentValue(fOutField, "connections", (Object)aConVector);
    
    DXEndField(fOutField);  /* this closes the field */
    /******************************************************************/   
    
    if (nUT > 1)
      DXSetEnumeratedMember((Group) gOutGroup, k , (Object) fOutField);

  } /* end for k loop */
  
  if (nUT > 1) {
    out[0] = (Object)gOutGroup;
  } else {
    out[0] = (Object)fOutField;
  }
  
  return OK;
  
}

