#include <stdio.h>
#include <dx/dx.h>

#define MAX_NAME_SIZE 256
#define BUFFER 256

/* Another implementation of the Tsyganenko models */
/* Based on code by T.B. Guild, C-L. Huang, and M. Wiltberger */
/* Eliminated the need to read files because the input data */
/* can be drawn from the CISM_DX uniform time series data base */

/* R.S. Weigel 10/30/2004 */
/* To do: needs more error checking */

/* Note that t03 expects double inputs where other tXX models */
/* expect float inputs. */

Error
m_Tsyganenko03(Object *in, Object *out)
{


  /* Input Declarations */
  Array aDataVector;
  Array aPosVector;
  Array aConVector;
  float *fpDataVector;
  float *fpPosVector;
  float *fpConVector;
  Type tDataVector;
  Type tPosVector;
  Type tConVector;
  Category cDataVector;
  Category cPosVector;
  Category cConVector;
  int nDataVector, nPosVector, nConVector; /* Number of items */
  int sDataVector, sPosVector, sConVector; /* Shape of each item */
  int rDataVector, rPosVector, rConVector; /* Rank of each item */

  /* Output Declarations */
  Group gOutGroup = NULL;
  Field fOutField = NULL;
  Array aOutDataVector = NULL;
  Array aOutPosVector = NULL;
  Array aOutConVector = NULL;
  float *fpOutDataVector;
  float *fpOutPosVector;
  float *fpOutConVector;

  int i,j,k;

  int M;     /* Returned field has data component that is a M-vector. */
  int Nitems;/* Returned field has data component w/ N M-vectors (or "items")  */

  int   nParmod;
  int   nPS;
  float *faParmod;
  float *faPS;
  double dParmod[6];
  double dPS;
  Array aParmod = NULL;
  Array aPS = NULL;
  double dX,dY,dZ,dBx,dBy,dBz;
  int iOpt = 0;

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"missing field");
    return ERROR;
  }
  else{

    aDataVector  = (Array) DXGetComponentValue((Field)in[0],"data");
    fpDataVector = DXGetArrayData(aDataVector);
    aPosVector   = (Array) DXGetComponentValue((Field)in[0],"positions");
    fpPosVector  = DXGetArrayData(aPosVector);
    aConVector   = (Array) DXGetComponentValue((Field)in[0],"connections");
    fpConVector  = DXGetArrayData(aConVector);

    DXGetArrayInfo(aDataVector, &nDataVector, &tDataVector, &cDataVector,
		   &rDataVector, &sDataVector);
    DXGetArrayInfo(aPosVector, &nPosVector, &tPosVector, &cPosVector,
		   &rPosVector, &sPosVector);
    DXGetArrayInfo(aConVector, &nConVector, &tConVector, &cConVector,
		   &rConVector, &sConVector);
   
    M      = sPosVector;
    Nitems = nPosVector;

  if (!in[1]){
    dParmod[0] =  10.0; /* Pdyn */
    dParmod[1] = -10.0; /* Dst  */
    dParmod[2] =   0.0; /* By   */
    dParmod[3] =   0.0; /* Bz   */
    dParmod[4] =   0.0; /* G2   */
    dParmod[5] =   0.0; /* G3   */
    nParmod    =   1;
  } 
  else {
    aParmod  = (Array) in[1];
    faParmod = DXGetArrayData(aParmod);
    DXGetArrayInfo(aParmod, &nParmod, NULL,NULL,NULL,NULL);
  }
  
  if (nParmod > 1) {
    DXMessage("Multiple (%d) solar wind conditions specified.", nParmod);
    DXMessage("Output will be a Group of Fields.  Each Field in the Group corresponds to");
    DXMessage("a vector in the PS and PARMOD vector lists.");
  }

  if (!in[2]){
    dPS = 0.0; /* Dipole Tilt Angle */
  } 
  else {
    aPS      = (Array) in[2];
    faPS     = DXGetArrayData(aPS);
    DXGetArrayInfo(aPS, &nPS, NULL,NULL,NULL,NULL);
    if (nPS != nParmod)
      DXSetError(ERROR_BAD_PARAMETER, 
		 "Number of items in PS list must be equal to number in PARMOD");
  }


  }
  /******************************************************************/

  /******************************************************************/  
  /* Fill in the Output arrays with data                            */
  /******************************************************************/  

  if (nParmod > 1) {
    gOutGroup = DXNewGroup();
  }

  for (k = 0;k < nParmod;k++) {

    /******************************************************************/
    /* Allocate and declare output OpenDX arrays as M-vectors         */
    /* with Nitems items                                              */
    /******************************************************************/
    aOutDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
    DXAddArrayData(aOutDataVector, 0, Nitems, NULL);
    fpOutDataVector = (float*)DXGetArrayData(aOutDataVector);
    /******************************************************************/

    if (in[1]){
   	dParmod[0] = (double) faParmod[0+6*k];
   	dParmod[1] = (double) faParmod[1+6*k];
   	dParmod[2] = (double) faParmod[2+6*k];
   	dParmod[3] = (double) faParmod[3+6*k];
   	dParmod[4] = (double) faParmod[4+6*k];
   	dParmod[5] = (double) faParmod[5+6*k];
   	dPS        = (double) faPS[k];
    }

    /* DXMessage("%.2f %.2f %.2f %.2f %.2f %.2f %.2f\n", 
     *          dParmod[0],dParmod[1],dParmod[2],       
     *          dParmod[3],dParmod[4],dParmod[5],dPS);
    */
    
    for (i=0; i < Nitems; i++) {
      
      dX = (double) fpPosVector[0+M*i];
      dY = (double) fpPosVector[1+M*i];
      dZ = (double) fpPosVector[2+M*i];
      
      t03_(&iOpt,dParmod,&dPS,&dX,&dY,&dZ, &dBx,&dBy,&dBz);

      fpOutDataVector[0+M*i] = (float) dBx;
      fpOutDataVector[1+M*i] = (float) dBy;
      fpOutDataVector[2+M*i] = (float) dBz;

    }
    /******************************************************************/  

    /******************************************************************/   
    /* Form output field **********************************************/
    /******************************************************************/   
    fOutField = DXNewField();
    
    DXSetComponentValue(fOutField, "positions", (Object)aPosVector);
    
    DXSetStringAttribute((Object)aOutDataVector,"dep", "positions");
    DXSetComponentValue(fOutField, "data", (Object)aOutDataVector);
    
    DXSetComponentValue(fOutField, "connections", (Object)aConVector);
    
    DXEndField(fOutField);  /* this closes the field */
    /******************************************************************/   
    
    if (nParmod > 1)
      DXSetEnumeratedMember((Group) gOutGroup, k , (Object) fOutField);
  }
  
  if (nParmod > 1) {
    out[0] = (Object)gOutGroup;
  } else {
    out[0] = (Object)fOutField;
  }
  
  return OK;
  
}

