#include <stdio.h>
#include <dx/dx.h>

#define MAX_NAME_SIZE 256
#define BUFFER 256

/* Another implementation of the Tsyganenko models */
/* Based on code by T.B. Guild, C-L. Huang, and M. Wiltberger */
/* Eliminated the need to read files because the input data */
/* can be drawn from the CISM_DX uniform time series data base */

/* R.S. Weigel 10/30/2004 */
/* To do: needs more error checking */

Error
m_Tsyganenko89(Object *in, Object *out)
{


  /* Input Declarations */
  Array aDataVector;
  Array aPosVector;
  Array aConVector;
  float *fpDataVector;
  float *fpPosVector;
  float *fpConVector;
  Type tDataVector;
  Type tPosVector;
  Type tConVector;
  Category cDataVector;
  Category cPosVector;
  Category cConVector;
  int nDataVector, nPosVector, nConVector; /* Number of items */
  int sDataVector, sPosVector, sConVector; /* Shape of each item */
  int rDataVector, rPosVector, rConVector; /* Rank of each item */

  /* Output Declarations */
  Group gOutGroup = NULL;
  Field fOutField = NULL;
  Array aOutDataVector = NULL;
  Array aOutPosVector = NULL;
  Array aOutConVector = NULL;
  float *fpOutDataVector;
  float *fpOutPosVector;
  float *fpOutConVector;

  int i,j,k;


  int M;     /* Returned field has data component that is a M-vector. */
  int Nitems;/* Returned field has data component w/ N M-vectors (or "items")  */

  int   niOpt;
  int   nParmod;
  int   nPS;
  int   *iaiOpt;
  float *faParmod;
  float *faPS;
  float fParmod[4];
  float fPS;
  Array aiOpt = NULL;
  Array aParmod = NULL;
  Array aPS = NULL;
  float fX,fY,fZ,fBx,fBy,fBz;
  int iOpt = 1;

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"missing field");
    return ERROR;
  }
  else{

    aDataVector  = (Array) DXGetComponentValue((Field)in[0],"data");
    fpDataVector = DXGetArrayData(aDataVector);
    aPosVector   = (Array) DXGetComponentValue((Field)in[0],"positions");
    fpPosVector  = DXGetArrayData(aPosVector);
    aConVector   = (Array) DXGetComponentValue((Field)in[0],"connections");
    fpConVector  = DXGetArrayData(aConVector);

    DXGetArrayInfo(aDataVector, &nDataVector, &tDataVector, &cDataVector,
		   &rDataVector, &sDataVector);
    DXGetArrayInfo(aPosVector, &nPosVector, &tPosVector, &cPosVector,
		   &rPosVector, &sPosVector);
    DXGetArrayInfo(aConVector, &nConVector, &tConVector, &cConVector,
		   &rConVector, &sConVector);
   

    M      = sPosVector;
    Nitems = nPosVector;

  if (!in[1]){
    iOpt    = 1;
    niOpt   = 1;
  } 
  else {
    aiOpt   = (Array) in[1];
    iaiOpt  = DXGetArrayData(aiOpt);
    DXGetArrayInfo(iaiOpt, &niOpt, NULL,NULL,NULL,NULL);
  }
  if (niOpt > 1) {
    DXMessage("Multiple (%d) Kp conditions specified.", niOpt);
    DXMessage("Output will be a Group of Fields.  Each Field in the Group corresponds to");
    DXMessage("a vector in the PS and iOpt vector lists.");
  }

  if (!in[2]){
    fPS = 0.0; /* Dipole Tilt Angle */
  } 
  else {
    aPS      = (Array) in[2];
    faPS     = DXGetArrayData(aPS);
    DXGetArrayInfo(aPS, &nPS, NULL,NULL,NULL,NULL);
    if (nPS != niOpt)
      DXSetError(ERROR_BAD_PARAMETER, 
		 "Number of items in PS list must be equal to number in iOpt");
  }

    DXMessage("%d\n",nPosVector);  

  }
  /******************************************************************/

  /******************************************************************/  
  /* Fill in the Output arrays with data                            */
  /******************************************************************/  

  if (niOpt > 1) {
    gOutGroup = DXNewGroup();
  }

  for (k = 0;k < niOpt;k++) {

    /******************************************************************/
    /* Allocate and declare output OpenDX arrays as M-vectors         */
    /* with Nitems items                                              */
    /******************************************************************/
    aOutDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
    DXAddArrayData(aOutDataVector, 0, Nitems, NULL);
    fpOutDataVector = (float*)DXGetArrayData(aOutDataVector);
    /******************************************************************/

    if (in[1]){
   	fParmod[0] = 0.0;
   	fParmod[1] = 0.0;
   	fParmod[2] = 0.0;
   	fParmod[3] = 0.0;
   	fPS        = (float) faPS[k];
   	iOpt       = (int) iaiOpt[k];
    }

    DXMessage("%d\n",iOpt);

    for (i=0; i < Nitems; i++) {
      
      fX = fpPosVector[0+M*i];
      fY = fpPosVector[1+M*i];
      fZ = fpPosVector[2+M*i];

      t89c_(&iOpt,fParmod,&fPS,&fX,&fY,&fZ, &fBx,&fBy,&fBz);

      fpOutDataVector[0+M*i] = fBx;
      fpOutDataVector[1+M*i] = fBy;
      fpOutDataVector[2+M*i] = fBz;

    }
    /******************************************************************/  

    /******************************************************************/   
    /* Form output field **********************************************/
    /******************************************************************/   
    fOutField = DXNewField();
    
    DXSetComponentValue(fOutField, "positions", (Object)aPosVector);
    
    DXSetStringAttribute((Object)aOutDataVector,"dep", "positions");
    DXSetComponentValue(fOutField, "data", (Object)aOutDataVector);
    
    DXSetComponentValue(fOutField, "connections", (Object)aConVector);
    
    DXEndField(fOutField);  /* this closes the field */
    /******************************************************************/   
    
    if (niOpt > 1)
      DXSetEnumeratedMember((Group) gOutGroup, k , (Object) fOutField);
  }
  
  if (niOpt > 1) {
    out[0] = (Object)gOutGroup;
  } else {
    out[0] = (Object)fOutField;
  }
  
  return OK;
  
}

