#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"
#include "bzero_pointers.h"

#define SMALL 1.e-15
FILE *FpDebug;

int mfGetMHDData(int32 liFileId, int iStepfix, int ni, int nj, int nk,
	       float *fpMHDData,  float *fpMF_MHDData, float *fpFluxData, float *fpDivBData,
               float *fpTubeData)
{
  int32 iErr, liIndex,liSecId,liStart,liStride,liEdge,liStat,liBzeroSize;
  float triple(), dot();
  void cross3();
  char *cpDum, caSec[MAX_NC_NAME],caStep[10];
  char H_caSec[MAX_NC_NAME],O_caSec[MAX_NC_NAME];
  char PE_caSec[MAX_NC_NAME];
  int i,j,k,l;
  int nip1, njp1, nkp1, njp2, nv, iSize;
  int iAxis;
  int Nbtot;
  float *fpVar, *fpReadVar, fAxisVal, *fpVol, *fpCur;
  float *fpVar2;
  float *fpBzeroVar;
  float *fpBXqnK, *fpBYqnK, *fpBZqnK;
  float x_i, y_i, z_i, e_i, e_j, e_k;
  float x_j, y_j, z_j, x_k, y_k, z_k;
  float cx[3], cy[3], cz[3], et[3];
  float det;
  float Bfmax;
  float mass, massconv;

  /*set some useful parameters */
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  njp2 = nj+2;
  iSize = nip1*njp1*nkp1;
  Nbtot = 8*nip1*nj*nk + 3*nip1*njp1*nk +
          8*ni*njp1*nk + 3*ni*njp1*nkp1 +
          8*ni*nj*nkp1 + 3*nip1*nj*nkp1;

  /*Get the Bzero data*/
  fpBzeroVar = mffpGetData(liFileId,"bzeros");
  fp_bnqfi = fpBzeroVar;
#include "bzero_assign.h"
  fpBXqnK = fp_bxqnk;
  fpBYqnK = fp_byqnk;
  fpBZqnK = fp_bzqnk;
  
  /*Get the data */
  sprintf(caSec,"time_step_%d",iStepfix);
  fpReadVar = mffpGetData(liFileId,caSec);
  DXMessage("fpREADVar 0 and 1000, %g %g\n", fpReadVar[0], fpReadVar[1000]);
  
 /* Get Bfluxes */
  /* First read the Bi,Bj, and Bk */
  sprintf(caSec,"bi_time_step_%d",iStepfix);
  fpBi = mffpGetData(liFileId,caSec);
  sprintf(caSec,"bj_time_step_%d",iStepfix);
  fpBj = mffpGetData(liFileId,caSec);
  sprintf(caSec,"bk_time_step_%d",iStepfix);
  fpBk = mffpGetData(liFileId,caSec);

  /*       Create the Flux Field  */
  iSize = nip1*njp1*nkp1;
  nv = 3;
  /* First initialize the entire field to zero */
  for ( i=0; i<iSize; i++ ) {
    fpFluxData[i] = 0.0;
  }
  /* The i faces have size nip1,nj,nk */
  for (i=0;i<nip1;i++) {
    for (j=0;j<nj;j++) {
      for (k=0;k<nk;k++) {
        fpBi[i1jk(i,j,k)] += fpFace_i[i1jk(i,j,k)]*fp_bnqfi[i1jk(i,j,k)];
        fpBi[i1jk(i,j,k)] = fpBi[i1jk(i,j,k)]/(6.38e8*6.38e8);
        //fpBi[i1jk(i,j,k)] = fpFace_i[i1jk(i,j,k)];
        fpFluxData[vi1j1k(0,i,j,k)] = fpBi[i1jk(i,j,k)];
      }
    }
  }
  /* The j faces have size ni,njp1,nk */
  for (i=0;i<ni;i++) {
    for (j=0;j<njp1;j++) {
      for (k=0;k<nk;k++) {
        fpBj[ij1k(i,j,k)] += fpFace_j[ij1k(i,j,k)]*fp_bnqfj[ij1k(i,j,k)];
        fpBj[ij1k(i,j,k)] = fpBj[ij1k(i,j,k)]/(6.38e8*6.38e8);
        //fpBj[ij1k(i,j,k)] = fpFace_j[ij1k(i,j,k)];
        fpFluxData[vi1j1k(1,i,j,k)] = fpBj[ij1k(i,j,k)];
      }
    }
  }
  /* The k faces have size ni,nj,nkp1 */
  for (i=0;i<ni;i++) {
    for (j=0;j<nj;j++) {
      for (k=0;k<nkp1;k++) {
        fpBk[ijk(i,j,k)] += fpFace_k[ijk(i,j,k)]*fp_bnqfk[ijk(i,j,k)];
        fpBk[ijk(i,j,k)] = fpBk[ijk(i,j,k)]/(6.38e8*6.38e8);
        //fpBk[ijk(i,j,k)] = fpFace_k[ijk(i,j,k)];
        fpFluxData[vi1j1k(2,i,j,k)] = fpBk[ijk(i,j,k)];
      }
    }
  }  

  /*      Create the Div B Field  */
    for (k=0; k<nk; k++) {
    for (j=0; j<nj; j++) {
      for (i=0; i<ni; i++) {
        Bfmax = fabs( fpBi[ i + nip1*(j + njp1*k) ]) +
          fabs( fpBi[ i+1 + nip1*(j + njp1*k) ]) +
          fabs( fpBj[ i + nip1*(j + njp1*k) ]) +
          fabs( fpBj[ i + nip1*(j+1 + njp1*k) ]) +
          fabs( fpBk[ i + nip1*(j + njp1*k) ]) +
          fabs( fpBk[ i + nip1*(j + njp1*(k+1)) ]) +
          SMALL;
        fpDivBData[ijk(i,j,k)] =
          fabs( ( fpBi[ i + nip1*(j + njp1*k) ]) -
              ( fpBi[ i+1 + nip1*(j + njp1*k) ]) +
              ( fpBj[ i + nip1*(j + njp1*k) ]) -
              ( fpBj[ i + nip1*(j+1 + njp1*k) ]) +
              ( fpBk[ i + nip1*(j + njp1*k) ]) -
              ( fpBk[ i + nip1*(j + njp1*(k+1)) ]))/
                   Bfmax;
                   }}}


  /* Now we construct the OpenDX field */
  iSize = ni*njp2*nkp1;
  nv = 14;
  fpVar = malloc(sizeof(float)*iSize*nv);
  /* Store a copy of Bx, By, and Bz for current calcs */
  fpRho = malloc(sizeof(float)*ni*nj*nk);
  fpP = malloc(sizeof(float)*ni*nj*nk);
  fpBx = malloc(sizeof(float)*ni*nj*nk);
  fpBy = malloc(sizeof(float)*ni*nj*nk);
  fpBz = malloc(sizeof(float)*ni*nj*nk);

        /* Density in #_H/cm^3 
	 assume helium = 4% */
  massconv = 1.0/(1.67e-24*(1.0 + 0.04*4.0));
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units */
        fpVar[vijk(0,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*massconv;
        fpRho[ijk(i,j,k)]=fpReadVar[vijk1(0,i,j,k)];
        /* Vx in km/s */
        fpVar[vijk(1,i,j+1,k)]=fpReadVar[vijk1(1,i,j,k)]*1.0e-5;
        /* Vy in km/s */
        fpVar[vijk(2,i,j+1,k)]=fpReadVar[vijk1(2,i,j,k)]*1.0e-5;
        /* Vz in km/s */
        fpVar[vijk(3,i,j+1,k)]=fpReadVar[vijk1(3,i,j,k)]*1.0e-5;
        /* Pressure kev/cm^3 */
        fpVar[vijk(4,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*
		               fpReadVar[vijk1(4,i,j,k)]*
                               fpReadVar[vijk1(4,i,j,k)]*3.75e8; 
        fpP[ijk(i,j,k)]=fpReadVar[vijk1(4,i,j,k)];
        /* Bx in nT */
        fpVar[vijk(5,i,j+1,k)]=(fpReadVar[vijk1(5,i,j,k)]+
         (fpBXqnK[ijk(i,j,k)]+fpBXqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
        fpBx[ijk(i,j,k)]=fpReadVar[vijk1(5,i,j,k)];
        /* By in nT */
        fpVar[vijk(6,i,j+1,k)]=(fpReadVar[vijk1(6,i,j,k)]+
         (fpBYqnK[ijk(i,j,k)]+fpBYqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
        fpBy[ijk(i,j,k)]=fpReadVar[vijk1(6,i,j,k)];
        /* Bz in nT */
        fpVar[vijk(7,i,j+1,k)]=(fpReadVar[vijk1(7,i,j,k)]+
         (fpBZqnK[ijk(i,j,k)]+fpBZqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
        fpBz[ijk(i,j,k)]=fpReadVar[vijk1(7,i,j,k)];
      }
    }
  }
  

  /* Get Cartesian E's at regular grid points */
  /* First read the Ei,Ej, and Ek */
  sprintf(caSec,"ei_time_step_%d",iStepfix);
  fpEi = mffpGetData(liFileId,caSec);
  sprintf(caSec,"ej_time_step_%d",iStepfix);
  fpEj = mffpGetData(liFileId,caSec);
  sprintf(caSec,"ek_time_step_%d",iStepfix);
  fpEk = mffpGetData(liFileId,caSec);
  /* Need space for cell volume which will be calc here and used in
   * determination of the current 
   */
  fpVol = malloc(sizeof(float)*nip1*njp1*nkp1);
  
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) { 
      for(i=0;i<ni;i++) {
           e_i = 0.25*(fpEi[ijkei(i,j,k)]+fpEi[ijkei(i,j,k+1)] +
                 fpEi[ijkei(i,j+1,k)]+fpEi[ijkei(i,j+1,k+1)]);
           e_j = 0.25*(fpEj[ijkej(i,j,k)]+fpEj[ijkej(i,j,k+1)] +
                 fpEj[ijkej(i+1,j,k)]+fpEj[ijkej(i+1,j,k+1)]);
           e_k = 0.25*(fpEk[ijkek(i,j,k)]+fpEk[ijkek(i+1,j,k)] +
                fpEk[ijkek(i,j+1,k)]+ fpEk[ijkek(i+1,j+1,k)]);
     x_i = 0.25*((fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]+
                 fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -
                (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+
                 fpX[ijk1(i,j,k+1)]+fpX[ijk1(i,j+1,k+1)]));
     y_i = 0.25*((fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]+
                  fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) - 
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+
                  fpY[ijk1(i,j,k+1)]+fpY[ijk1(i,j+1,k+1)]));
     z_i = 0.25*((fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]+
                  fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) - 
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+
                  fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]));
     x_j = 0.25*((fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+
                  fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -  
                 (fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)]+
                  fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]));
     y_j = 0.25*((fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+
                  fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) -  
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)]+
                  fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]));
     z_j = 0.25*((fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+
                  fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) -  
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)]+
                  fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]));
     x_k = 0.25*((fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]+
                  fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -  
                 (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+
                  fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]));
     y_k = 0.25*((fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]+
                  fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) -  
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+
                  fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]));
     z_k = 0.25*((fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]+
                  fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) -  
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+
                 fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]));
                 cx[0] = x_i;
                 cx[1] = x_j;
                 cx[2] = x_k;
                 cy[0] = y_i;
                 cy[1] = y_j;
                 cy[2] = y_k;
                 cz[0] = z_i;
                 cz[1] = z_j;
                 cz[2] = z_k;
                 et[0] = e_i;
                 et[1] = e_j;
                 et[2] = e_k;
                 det = 1.e-6/triple(cx,cy,cz);
		 fpVol[ijk1(i,j,k)] = fabs(triple(cx,cy,cz));
                 fpVar[vijk(8,i,j+1,k)] = triple(et,cy,cz)*det;
                 fpVar[vijk(9,i,j+1,k)] = triple(cx,et,cz)*det;
                 fpVar[vijk(10,i,j+1,k)] = triple(cx,cy,et)*det;
     } /* End i Loop */
    } /* End j Loop */
  } /* End k Loop */
 
  /* Next we move onto calculating the cartesian currents */

  

  /* First we need to get the magnetic field information for the curl */
  fpCur = malloc(sizeof(float)*3*iSize);
  mfjcalc2ptr_(fpCur,fpBx,fpBy,fpBz,fpX,fpY,fpZ,fpVol,
          &ni,&nj,&nk,&nip1,&njp1,&nkp1,&njp2);

  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
	for(l=0; l<3; l++) {
	  fpVar[vijk(11+l,i,j+1,k)] = fpCur[l+3*(i+ni*((j+1)+njp2*k))];
	}}}}
DXMessage("cruising for getting data");

  /* Go off and do the field line tracing for the flux tube quanties */
  /*  comment out for now
      GetTubes(fpTubeData,ni,nj,nk);*/ 
  /* Calculate Axis values */
  DXMessage("cruising for getting data %d %G\n",fpCur, fpCur[15000]);
 
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpVar[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpVar[vijk(l,i,j,k)]=fAxisVal;;
        }
      }
    }
  }
//
  /* Close the k cells for the field */
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpVar[vijk(l,i,j,nk)]=fpVar[vijk(l,i,j,0)];
      }
    }
  }
DXMessage("cruising for getting data");

  /* Pass the field data back */
  for(i=0;i<nv*iSize;i++) {
    fpMHDData[i] = fpVar[i];
  } 

  /* Clean up data pointers */
  free(fpReadVar);
  free(fpBzeroVar);
  free(fpFace_i);
  free(fpFace_j);
  free(fpFace_k);
  free(fpVar);
  free(fpRho);
  free(fpP);
  free(fpBx);
  free(fpBy);
  free(fpBz);
  free(fpEi);
  free(fpEj);
  free(fpEk);
  free(fpVol);
  free(fpCur);

/*    >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
          multifluid Field
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  */

  /* Now we construct the OpenDX field */
  iSize = ni*njp2*nkp1;
  nv = 11;
  fpVar2 = malloc(sizeof(float)*iSize*nv);
  DXMessage("cruising for getting data");

  DXMessage("cruising past malloc");

  /*Get the hydrogen data */ 
sprintf(H_caSec,"H_time_step_%d",iStepfix);
fpReadVar = mffpGetData(liFileId,H_caSec);
  DXMessage("cruising past getting data");

/*              

*/
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units */
        /* Density in #/cm^3 */
        fpVar2[vijk(0,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*massconv;
        /* Vx in km/s */
        fpVar2[vijk(1,i,j+1,k)]=fpReadVar[vijk1(1,i,j,k)]*1.0e-5;
        /* Vy in km/s */
        fpVar2[vijk(2,i,j+1,k)]=fpReadVar[vijk1(2,i,j,k)]*1.0e-5;
        /* Vz in km/s */
        fpVar2[vijk(3,i,j+1,k)]=fpReadVar[vijk1(3,i,j,k)]*1.0e-5;
        /* Pressure kev/cm^3 */
        fpVar2[vijk(4,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*
		               fpReadVar[vijk1(4,i,j,k)]*
                               fpReadVar[vijk1(4,i,j,k)]*3.75e8; 
      }}}

/*Get the oxygen data */
sprintf(O_caSec,"O_time_step_%d",iStepfix);
fpReadVar = mffpGetData(liFileId,O_caSec);

/*              

*/
 mass = 1./(16.0*1.67e-24);
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units */
        /* Density in #/cm^3 */
        fpVar2[vijk(5,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*massconv;
        /* Vx in km/s */
        fpVar2[vijk(6,i,j+1,k)]=fpReadVar[vijk1(1,i,j,k)]*1.0e-5;
        /* Vy in km/s */
        fpVar2[vijk(7,i,j+1,k)]=fpReadVar[vijk1(2,i,j,k)]*1.0e-5;
        /* Vz in km/s */
        fpVar2[vijk(8,i,j+1,k)]=fpReadVar[vijk1(3,i,j,k)]*1.0e-5;
        /* Pressure kev/cm^3 */
        fpVar2[vijk(9,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*
		               fpReadVar[vijk1(4,i,j,k)]*
                               fpReadVar[vijk1(4,i,j,k)]*3.75e8; 
      }}}

free(fpReadVar);

/*Get the electron pressure data */
sprintf(PE_caSec,"PE_time_step_%d",iStepfix);
fpReadVar = mffpGetData(liFileId,PE_caSec);

/*              

*/
 mass = 1./(16.0*1.67e-24);
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units */
        /* Electron Pressure in keV/cm^3 */
        fpVar2[vijk(10,i,j+1,k)]=fpReadVar[ijk(i,j,k)]*1.66666*3.75e8;

      }}}

 
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpVar2[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpVar2[vijk(l,i,j,k)]=fAxisVal;;
        }
      }
    }
  }
//
  /* Close the k cells for the field */
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpVar2[vijk(l,i,j,nk)]=fpVar2[vijk(l,i,j,0)];
      }
    }
  }

  /* Pass the field data back */
  for(i=0;i<nv*iSize;i++) {
    fpMF_MHDData[i] = fpVar2[i];
  }


free(fpReadVar);

/*   end of multifluid section  */
  
  /* We leave the grid pointers existence since they are used in the CalcIonGrid
   * function.
   */

  return 1;
}


