#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"

int mfGetGrid(int32 liFileId, float fNorm, int ni, int nj, int nk,
            float *fpMHDPts, float *fpFluxPts, float *fpDivBPts) 
{
  int i,j,k, nip1, njp1, nkp1, njp2, iNumPts, iAxis, iMin;
  int iNumFluxPts, iNumDivBPts;
  float fAxisVal, fNormInv;
  FILE *fpOut, *FpDebug;
  float *darc, *cosarc, *deltax, *deltay, *dx, *dy, *facek, *ybari, *ybarj;
  float deldx, deldy, jeta, jpsi, j0;


  nip1 = ni+1;
  njp1 = nj+1;
  njp2 = nj+2;
  nkp1 = nk+1;
  DXMessage("one\n");
  /* Now we get the XZ,Y2 and Phi reconstruction arrays */
  fpX2 = mffpGetData(liFileId,"grid_x2");
  DXMessage("two\n");
  fpY2 = mffpGetData(liFileId,"grid_y2");
  DXMessage("three\n");
  fpPhi = mffpGetData(liFileId,"grid_phi");
  DXMessage("four\n");
#ifdef DDEBUG
  printf("X2[0,0] %g X2[10,15] %g \n",fpX2[ij(0,0)],fpX2[ij1(10,15)]);
  for(i=0;i<nip1;i++) {
    printf("X2[%d,0] %g \n",i,fpX2[ij1(i,0)]);
  }
  for(i=0;i<njp1;i++) {
    printf("X2[0,%d] %g \n",i,fpX2[ij1(0,i)]);
  }
  fpOut = fopen("x2.dat","w");
  for(j=0;j<njp1;j++) {
    for(i=0;i<nip1;i++) {
      fprintf(fpOut,"%g \n",fpX2[ij1(i,j)]);
    }
  }
  fclose(fpOut);
  printf("Y2[0,0] %g Y2[10,15] %g \n",fpY2[ij(0,0)],fpY2[ij1(10,15)]);
  for(i=0;i<nip1;i++) {
    printf("Y2[%d,0] %g \n",i,fpY2[ij1(i,0)]);
  }
  for(i=0;i<njp1;i++) {
    printf("Y2[0,%d] %g \n",i,fpY2[ij1(0,i)]);
  }
  fpOut = fopen("phi.dat","w");
  for(i=0;i<nkp1;i++) fprintf(fpOut,"%g \n",fpPhi[i]);
  fclose(fpOut);
#endif
  /* Calc locations of cell corners */
  fpCosPhi = malloc(sizeof(float)*nkp1);
  fpSinPhi = malloc(sizeof(float)*nkp1);
  for(i=0;i<nk;i++) {
    fpCosPhi[i]=cos(fpPhi[i]);
    fpSinPhi[i]=sin(fpPhi[i]);
  }
  fpCosPhi[nk] = fpCosPhi[0];
  fpSinPhi[nk] = fpSinPhi[0];
#ifdef DDEBUG
  for(i=0;i<nkp1;i++) {
    printf("Phi[%d] %g\n",i,fpPhi[i]);
  }
  for(i=0;i<nkp1;i++) {
    printf("SinPhi[%d] %g\n",i,fpSinPhi[i]);
  }
#endif
  /* get the Face areas: needed for bflux calculations */

  fpFace_i = (float *)malloc(sizeof(float)*nip1*nj*nk);
  fpFace_j = (float *)malloc(sizeof(float)*ni*njp1*nk);
  fpFace_k = (float *)malloc(sizeof(float)*ni*nj*nkp1);

/* temporaries */
darc = (float *)malloc(sizeof(float)*nkp1);
cosarc = (float *)malloc(sizeof(float)*nkp1);
deltay = (float *)malloc(sizeof(float)*nip1*njp1);
deltax = (float *)malloc(sizeof(float)*nip1*njp1);
dx = (float *)malloc(sizeof(float)*ni*njp1);
dy = (float *)malloc(sizeof(float)*ni*njp1);
facek = (float *)malloc(sizeof(float)*ni*nj);
ybarj = (float *)malloc(sizeof(float)*ni*njp1);
ybari = (float *)malloc(sizeof(float)*nip1*nj);

for( j=0; j<njp1; j++) {
 for (i=0; i<ni; i++) {
                        dx[ij(i,j)] = fpX2[i1j(i+1,j)]-fpX2[i1j(i,j)];
                        dy[ij(i,j)] = fpY2[i1j(i+1,j)]-fpY2[i1j(i,j)];
          }}

for ( j=0; j<nj; j++ ) {
 for (i=0; i<nip1; i++ ) {
   deltax[i1j(i,j)] = fpX2[i1j(i,j+1)]-fpX2[i1j(i,j)];
   deltay[i1j(i,j)] = fpY2[i1j(i,j+1)]-fpY2[i1j(i,j)];
            }}
for (j=0; j<nj; j++) {
 for (i=0; i<ni; i++) {

   deldx = dx[ij(i,j+1)]-dx[ij(i,j)];
   deldy= dy[ij(i,j+1)]-dy[ij(i,j)];
   j0 = dx[ij(i,j)]*deltay[i1j(i,j)] - dy[ij(i,j)]*deltax[i1j(i,j)];
   jeta= dx[ij(i,j)]*deldy - dy[ij(i,j)]*deldx;
   jpsi = deltay[i1j(i,j)]*deldx - deltax[i1j(i,j)]*deldy;
  facek[ij(i,j)] = j0 + 0.5*(jeta+jpsi);
 }}
for ( k=0; k<nkp1; k++) {
 for (j=0; j<nj; j++ ) {
   for (i=0; i<ni; i++) {
     fpFace_k[ijk(i,j,k)] = facek[ij(i,j)];
   }}}
for ( k=0; k<nkp1; k++) {
 darc[k] = 2.0*sin( 0.5*(fpPhi[k+1]-fpPhi[k] ));
                     cosarc[k] = cos( 0.5*(fpPhi[k+1]-fpPhi[k]));
                     }
 for (j=0; j<njp1; j++ ) {
   for (i=0; i<ni; i++ ) {
     ybarj[ij(i,j)] = 0.5*(fpY2[i1j(i,j)]+fpY2[i1j(i+1,j)]);
   }}
for (k=0; k<nk; k++) {
   for (j=0; j<njp1; j++) {
     for (i=0; i<ni; i++ ) {
        fpFace_j[ij1k(i,j,k)] = darc[k]*ybarj[ij(i,j)]*
          sqrt( pow(cosarc[k]*dy[ij(i,j)],2) + pow(dx[ij(i,j)],2));
     }}}
 for (j=0; j<nj; j++) {
   for (i=0; i<nip1; i++) {
     ybari[i1j(i,j)] = 0.5*(fpY2[i1j(i,j+1)]+fpY2[i1j(i,j)]);
   }}

 for (k=0; k<nk; k++) {
   for (j=0; j<nj; j++) {
     for (i=0; i<nip1; i++) {
        fpFace_i[i1jk(i,j,k)] = darc[k]*ybari[i1j(i,j)]*
          sqrt( pow(cosarc[k]*deltay[i1j(i,j)],2) + pow(deltax[i1j(i,j)],2));
     }}}

        free(darc);  free(cosarc);
        free(deltay); free(deltax); free(dx); free(dy);
        free(facek);
        free(ybarj); free(ybari);

        /* calculate the corner values */
  fpX = malloc(sizeof(float)*(nip1*njp1*nkp1));
  fpY = malloc(sizeof(float)*(nip1*njp1*nkp1));
  fpZ = malloc(sizeof(float)*(nip1*njp1*nkp1));
  
  for(k=0;k<nkp1;k++) {
    for(j=0;j<njp1;j++) {
      for(i=0;i<nip1;i++) {
	fpX[ijk1(i,j,k)]=fpX2[ij1(i,j)];
	fpY[ijk1(i,j,k)]=fpY2[ij1(i,j)]*fpCosPhi[k];
	fpZ[ijk1(i,j,k)]=fpY2[ij1(i,j)]*fpSinPhi[k];
      }
    }
  }

  /* Calc locations of Cell centers */
  fpXc = malloc(sizeof(float)*(ni*njp2*nkp1));
  fpYc = malloc(sizeof(float)*(ni*njp2*nkp1));
  fpZc = malloc(sizeof(float)*(ni*njp2*nkp1));

  fNormInv = 1.0/fNorm;
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
	fpXc[ijkc(i,j+1,k)]=(fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)] +
			     fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+fpX[ijk1(i,j,k+1)] +
			     fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
	fpYc[ijkc(i,j+1,k)]=(fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)] +
			     fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+fpY[ijk1(i,j,k+1)] +
			     fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
	fpZc[ijkc(i,j+1,k)]=(fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)] +
			     fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+fpZ[ijk1(i,j,k+1)] +
			     fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
      }
    }
  }
  
  /* Calc Axes values */
  for(j=0;j<njp2;j++) {
    iAxis=max(1,min(nj,j));
#ifdef DDEBUG
    printf("iAxis %d \n",iAxis);
#endif
    for(i=0;i<ni;i++) {
      fAxisVal=0.0;
      /*
	iMin = (nj < j) ? nj: j;
	iAxis= (1 > iMin) ? 1 : iMin; 
      */
      for(k=0;k<nk;k++) {
        fAxisVal += fpXc[ijkc(i,iAxis,k)];
      }
      fAxisVal /= nk;
      for(k=0;k<nk;k++) {
        fpXc[ijkc(i,j,k)]=fAxisVal;
      }
    }
  }

  /* Close the k cells for the field */
  for(j=0;j<njp2;j++) {
    for(i=0;i<ni;i++)  {
      fpXc[ijkc(i,j,nk)]=fpXc[ijkc(i,j,0)];
      fpYc[ijkc(i,j,nk)]=fpYc[ijkc(i,j,0)];
      fpZc[ijkc(i,j,nk)]=fpZc[ijkc(i,j,0)];
    }
  }

  /* Assign return quanties */
  iNumPts = ni*njp2*nkp1;
  for(i=0;i<iNumPts;i++) {
    fpMHDPts[i*3]=fpXc[i];
    fpMHDPts[i*3+1]=fpYc[i];
    fpMHDPts[i*3+2]=fpZc[i];
  }

 /* Assign return quanties */
 iNumFluxPts = nip1*njp1*nkp1;
 for(i=0;i<iNumFluxPts;i++) {
     fpFluxPts[i*3]=fpX[i]*fNormInv;
     fpFluxPts[i*3+1]=fpY[i]*fNormInv;
     fpFluxPts[i*3+2]=fpZ[i]*fNormInv;
 }

/* Assign return quanties */
 iNumDivBPts = ni*nj*nk;
 for(k=0; k<nk+1; k++ ) {
   for (j=0;j<nj; j++) {
     for(i=0;i<ni;i++) {
          fpDivBPts[ijk(i,j,k)*3]=fpXc[ijkc(i,j+1,k)];
          fpDivBPts[ijk(i,j,k)*3+1]=fpYc[ijkc(i,j+1,k)];
          fpDivBPts[ijk(i,j,k)*3+2]=fpZc[ijkc(i,j+1,k)];
     }}}


  
  /* NB: We don't cleanup the grid pointers since they are needed in the
   * GetMHDData function to compute the carteisan electric field and current
   */
  return 1;

} 


