#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"
#include "bzero_pointers.h"

#define MAXLINE  10000

int mfGetTubes(float *fpTubeData, int ni, int nj, int nk)
{
  int nip1,njp1,njp2,nkp1,nv;
  int i,j,k,l;
  /* Lines in logical space */
  float faILine[MAXLINE], faJLine[MAXLINE], faKLine[MAXLINE];
  /* Is Field line Connected to northern hemisphere (n), southern hemisphere
  * (s), closed (c), or solar wind (w)
  */
  char cLineType;
  /* Entry point of field in logical space for northern and southern hemis */
  float fIN,fJN,fKN,fIS,fJS,fKS;
  /* Line Length */
  int iLineLen;
  /* Why iInterp */
  int iInterp;
  /* Real space values */
  float *fpXLine, *fpYLine, *fpZLine;
  float *fpBxLine, *fpByLine, *fpBzLine;
  float *fpRhoLine, *fpPLine;
  
  float fi,fj,fk;
  float fTubeVol,fTubeMass,fTubeP;
  float fDs, fBmag, fDVol;
  float fReInv;


  nip1 = ni + 1;
  njp1 = nj + 1;
  nkp1 = nk + 1;
  fReInv = 1.0/6.38e8; /* One over Earth Radius in cm */

  
  /* Loop over all cells in magnetospheric grid */
  for(k=0;k<nk;k++) {
     fk = k +0.5;
    for(j=0;j<nj;j++) {
      fj = j+0.5;
      for(i=0;i<ni;i++) {
        fi = i+0.5;
	/*        btrace(fi,fj,fk,iInterp,
               iLineLen,cLineType,faILine,faJLine,faKLine,
               fIN,fJN,fKN,fIS,fJS,fKS); */
        fpXLine = malloc(sizeof(float)*iLineLen);
        fpYLine = malloc(sizeof(float)*iLineLen);
        fpZLine = malloc(sizeof(float)*iLineLen);
        fpBxLine = malloc(sizeof(float)*iLineLen);
        fpByLine = malloc(sizeof(float)*iLineLen);
        fpBzLine = malloc(sizeof(float)*iLineLen);
        fpRhoLine = malloc(sizeof(float)*iLineLen);
        fpPLine = malloc(sizeof(float)*iLineLen);
        mfinterpline(fpXLine,fpX,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline(fpYLine,fpY,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline(fpZLine,fpZ,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline2(fpBxLine,fpBx,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline2(fpByLine,fpBy,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline2(fpBzLine,fpBz,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline2(fpRhoLine,fpRho,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        mfinterpline2(fpPLine,fpP,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        for(l=0;l<iLineLen;l++) 
          fpPLine[l] = 0.6*fpRhoLine[l]*fpPLine[l]*fpPLine[l]/1.6e-9; 
        fTubeVol =0.0;
        fTubeMass = 0.0;
        fTubeP = 0.0;
        for(l=0;l<iLineLen-1;l++) {
          fDs = sqrt((fpXLine[l+1]-fpXLine[i])*(fpXLine[l+1]-fpXLine[i])+ 
                     (fpYLine[l+1]-fpYLine[i])*(fpYLine[l+1]-fpYLine[i])+ 
                     (fpZLine[l+1]-fpZLine[i])*(fpZLine[l+1]-fpZLine[i])); 
          fBmag = sqrt((fpBxLine[l+1]-fpBxLine[i])*(fpBxLine[l+1]-fpBxLine[i])+ 
                     (fpByLine[l+1]-fpByLine[i])*(fpByLine[l+1]-fpByLine[i])+ 
                     (fpBzLine[l+1]-fpBzLine[i])*(fpBzLine[l+1]-fpBzLine[i])); 
          fDVol = fReInv*fDs/fBmag;
          fTubeVol += fDVol;
          fTubeMass += 0.5*(fpRhoLine[l+1]+fpRhoLine[l])*fDVol;
          fTubeP += 0.5*(fpPLine[l+1]+fpPLine[l])*fDVol;
        }
        nv = 3;
        switch (cLineType) {
         case 'c':
          fpTubeData[vijk(0,i,j,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j,k)] = fTubeP;
          break;
         case 'n':
          fpTubeData[vijk(0,i,j,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j,k)] = fTubeP;
          break;
         case 's':
          fpTubeData[vijk(0,i,j,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j,k)] = fTubeP;
          break;
         default:
          fpTubeData[vijk(0,i,j,k)] = 0.0;
          fpTubeData[vijk(1,i,j,k)] = 0.0;
          fpTubeData[vijk(2,i,j,k)] = 0.0;
          break;
        }
        free(fpXLine);
        free(fpYLine);
        free(fpZLine);
        free(fpBxLine);
        free(fpByLine);
        free(fpBzLine);
        free(fpRhoLine);
        free(fpPLine);
      }
    }
  }
  return 1;
}

void mfinterpline(float *fpOut, float *fpIn, 
                float *fpILine, float *fpJLine, float *fpKLine,
                int iLineLen,
                int ni, int nj, int nk)
{
  float xxq, yyq, zzq, dx0, dy0, dz0 ;
  int i,i0,j0,k0,ip,jp,kp;
  int nip1,njp1,nkp1;
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  for(i=0;i<iLineLen;i++) {
    xxq = fpILine[i];
    yyq = fpJLine[i];
    zzq = fpKLine[i];
    i0=min(ni-1,floor(xxq+1.00001));
    j0=min(nj-1,floor(yyq+1.00001));
    k0=min(nk-1,floor(zzq+1.00001));
    ip = i0 + 1;
    jp = j0 + 1;
    kp = k0 + 1;
    dx0 = xxq - (i0-1);
    dy0 = yyq - (j0-1);
    dz0 = zzq - (k0-1);
    fpOut[i] = (1.0-dx0)*(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(i0,j0,k0)] +
                    dx0 *(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(ip,j0,k0)] +
               (1.0-dx0)*     dy0 *(1.0-dz0)*fpIn[ijk1(i0,jp,k0)] +
                    dx0 *     dy0 *(1.0-dz0)*fpIn[ijk1(ip,jp,k0)] +
               (1.0-dx0)*(1.0-dy0)*     dz0 *fpIn[ijk1(i0,j0,kp)] +
                    dx0 *(1.0-dy0)*     dz0 *fpIn[ijk1(ip,j0,kp)] +
               (1.0-dx0)*     dy0 *     dz0 *fpIn[ijk1(i0,jp,kp)] +
                    dx0 *     dy0 *     dz0 *fpIn[ijk1(ip,jp,kp)];
  }
  return;
}
    
void mfinterpline2(float *fpOut, float *fpIn, 
                float *fpILine, float *fpJLine, float *fpKLine,
                int iLineLen,
                int ni, int nj, int nk)
{
  float xxq, yyq, zzq, dx0, dy0, dz0 ;
  int i,i0,j0,k0,ip,jp,kp;
  int nip1,njp1,nkp1;
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  for(i=0;i<iLineLen;i++) {
    xxq = fpILine[i]+0.5;
    yyq = fpJLine[i]+0.5;
    zzq = fpKLine[i]+0.5;
    i0=min(ni-2,max(1,xxq));
    j0=min(nj-2,max(1,yyq));
    k0=min(nk-1,zzq);
    dz0 = zzq - k0;
    k0 = (k0+31)%nk+1;
    ip = i0 + 1;
    jp = j0 + 1;
    kp = (k0 % nk)+ 1;
    dx0 = xxq - (i0);
    dy0 = yyq - (j0);
    fpOut[i] = (1.0-dx0)*(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(i0,j0,k0)] +
                    dx0 *(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(ip,j0,k0)] +
               (1.0-dx0)*     dy0 *(1.0-dz0)*fpIn[ijk1(i0,jp,k0)] +
                    dx0 *     dy0 *(1.0-dz0)*fpIn[ijk1(ip,jp,k0)] +
               (1.0-dx0)*(1.0-dy0)*     dz0 *fpIn[ijk1(i0,j0,kp)] +
                    dx0 *(1.0-dy0)*     dz0 *fpIn[ijk1(ip,j0,kp)] +
               (1.0-dx0)*     dy0 *     dz0 *fpIn[ijk1(i0,jp,kp)] +
                    dx0 *     dy0 *     dz0 *fpIn[ijk1(ip,jp,kp)];
  }
  return;
}
    
