#define SMALL   1.e-15

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include <dx/dx.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"

FILE *FpDebug;

float *fpPGetData(int32, char*);

int PGetGrid(int32 liFileId, float fNorm, float *fpMHDPts, float *fpFluxPts, float *fpDivBPts) 
{
   int i,j,k, ni,nj,nk, nip1, njp1, nkp1, njp2, iAxis, iMin;
   int iNi,iNj,iNk,iNumPts, iNumFluxPts, iNumDivBPts;
   float fAxisVal, fNormInv;
   int32 iErr, liSdsId, liIndex, liNumType, liAttr, liRank, liSize;
   int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride; 
   char caDataName[MAX_NC_NAME];
   FILE *fpOut;

   /* First we get the Array size information */
   /* Since X2 is nip1xnjp1 we have*/

#ifdef DEBUG
   FpDebug = fopen("Debug.txt","w");
#endif

   DXMessage("Loc: GetGrid\n");
#ifdef DEBUG  
   DXMessage("before grid read\n");
#endif
   liIndex = SDnametoindex(liFileId,"X_grid");
   liSdsId = SDselect(liFileId,liIndex);
   liAttr = SDfindattr(liSdsId,"ni");
   SDreadattr(liSdsId, liAttr, &iNi); 
   liAttr = SDfindattr(liSdsId,"nj");
   SDreadattr(liSdsId, liAttr, &iNj); 
   liAttr = SDfindattr(liSdsId,"nk");
   SDreadattr(liSdsId, liAttr, &iNk); 
   DXMessage("Loc 1\n");
   nip1 = iNi;
   njp1 = iNj;
   nkp1 = iNk;
   ni = nip1-1;
   nj = njp1-1;
   nk = nkp1-1;
   njp2 = njp1 + 1;
   fpX = fpPGetData(liFileId,"X_grid");
   fpY = fpPGetData(liFileId,"Y_grid");
   fpZ = fpPGetData(liFileId,"Z_grid");
#ifdef DEBUG
   fprintf(FpDebug,"initial load of grid \n");
	   for (k=0; k<3; k++) {
	   for (j=0; j<3; j++) {
	   for (i=0; i<3; i++) {
	     fprintf(FpDebug,"[%d,%d,%d] => %g, %g, %g\n",i,j,k,fpX[ijk1(i,j,k)],fpY[ijk1(i,j,k)],
		     fpZ[ijk1(i,j,k)]); }}}
#endif

#ifdef DEBUG  
   DXMessage("after grid read\n");
#endif

  /* Calc locations of Cell centers */
   fpXc = malloc(sizeof(float)*(ni*njp2*nkp1));
   fpYc = malloc(sizeof(float)*(ni*njp2*nkp1));
   fpZc = malloc(sizeof(float)*(ni*njp2*nkp1));



   fNormInv = 1.0/fNorm;
   DXMessage("fnorm %g",fNorm);
   for(k=0;k<nk;k++) {
     for(j=0;j<nj;j++) {
       for(i=0;i<ni;i++) {
         fpXc[ijkc(i,j+1,k)]=(fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)] +
          fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+fpX[ijk1(i,j,k+1)] +
          fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
         fpYc[ijkc(i,j+1,k)]=(fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)] +
          fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+fpY[ijk1(i,j,k+1)] +
          fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
         fpZc[ijkc(i,j+1,k)]=(fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)] +
          fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+fpZ[ijk1(i,j,k+1)] +
          fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
       }
     }
   }

#ifdef DEBUG  
   DXMessage("after grid centers\n");
#endif
  
  /* Calc Axes values */
   for(j=0;j<njp2;j+=nj+1) {
    iAxis=max(1,min(nj,j));
#ifdef DEBUG
    fprintf(FpDebug,"iAxis %d \n",iAxis);
#endif
    for(i=0;i<ni;i++) {
      fAxisVal=0.0;
/*
      iMin = (nj < j) ? nj: j;
      iAxis= (1 > iMin) ? 1 : iMin; 
*/
      for(k=0;k<nk;k++) {
        fAxisVal += fpXc[ijkc(i,iAxis,k)];
      }
      fAxisVal /= nk;
      for(k=0;k<nk;k++) {
        fpXc[ijkc(i,j,k)]=fAxisVal;
      }
    }
  }

#ifdef DEBUG  
   DXMessage("after axis fill\n");
#endif

  /* Close the k cells for the field */
  for(j=0;j<njp2;j++) {
     for(i=0;i<ni;i++)  {
       fpXc[ijkc(i,j,nk)]=fpXc[ijkc(i,j,0)];
       fpYc[ijkc(i,j,nk)]=fpYc[ijkc(i,j,0)];
       fpZc[ijkc(i,j,nk)]=fpZc[ijkc(i,j,0)];
     }
  }

#ifdef DEBUG  
   DXMessage("before assign return\n");
   DXMessage("fpMHDPts pointer %x\n",fpMHDPts);
#endif

  /* Assign return quanties */
  iNumPts = ni*njp2*nkp1; 
#ifdef DEBUG  
   DXMessage("iNumPts: %d, ni: %d, njp2: %d, nkp1: %d\n", iNumPts, ni,njp2,nkp1);
#endif

  for(i=0;i<iNumPts;i++) {
      fpMHDPts[i*3]=fpXc[i];
      fpMHDPts[i*3+1]=fpYc[i];
      fpMHDPts[i*3+2]=fpZc[i];
  }


  /* Assign return quanties */
  iNumFluxPts = nip1*njp1*nkp1;
  DXMessage("num of flux pts %d\n",iNumFluxPts);
  for(i=0;i<iNumFluxPts;i++) {
      fpFluxPts[i*3]=fpX[i]*fNormInv;
      fpFluxPts[i*3+1]=fpY[i]*fNormInv;
      fpFluxPts[i*3+2]=fpZ[i]*fNormInv;
  }
  DXMessage("%f %f %f \n",fpFluxPts[30 + njp1*10],fpFluxPts[31 + njp1*10],fpFluxPts[32 + njp1*10]);

  /* Assign return quanties */
  iNumDivBPts = ni*nj*nk;
  DXMessage("num of div pts %d\n",iNumDivBPts);
  for(k=0; k<nk+1; k++ ) {
    for (j=0;j<nj; j++) {
      for(i=0;i<ni;i++) {
	  fpDivBPts[ijk(i,j,k)*3]=fpXc[ijkc(i,j+1,k)];
	  fpDivBPts[ijk(i,j,k)*3+1]=fpYc[ijkc(i,j+1,k)];
	  fpDivBPts[ijk(i,j,k)*3+2]=fpZc[ijkc(i,j+1,k)];
      }}}
  DXMessage("%f %f %f \n",fpDivBPts[30 + njp1*10],fpDivBPts[31 + njp1*10],fpDivBPts[32 + njp1*10]);
  
#ifdef DEBUG
  fprintf(stderr,"Before return from GetGrid \n");
#endif
  return 1;

} 


  
int PGetIntAtr(int32 liId, char *cpName) 
{
  int32 iErr, liAtrId, iVal;
  liAtrId = SDfindattr(liId,cpName);
  iErr = SDreadattr(liId,liAtrId,&iVal); 
  return iVal;
}
  
int min(int a, int b) {
  if(a < b) return a;
  else return b;
}

int max(int a, int b) {
  if(a > b) return a;
  else return b;
}
 
int32 PgetFileId(char *cpName) 
{
  int32 iFileId;
  FILE *FpOut;
  FpOut=fopen("info.txt","w");
  iFileId = SDstart(cpName,DFACC_RDONLY);
#ifdef DEBUG
  fprintf(FpOut,"FileName %s Status %d \n",cpName,iFileId);
#endif
  fclose(FpOut);
  return iFileId;
}


int PGetMHDData(int32 liFileId, int ni, int nj, int nk,float *fpMHDData, float *fpFluxData, float *fpDivBData)
{
  int32 iErr;
  float p_triple(), p_dot();
  void p_cross3();
  char *cpDum, caSec[MAX_NC_NAME],caStep[10];
  int i,j,k,l;
  int nip1, njp1, nkp1, njp2, nv, iSize;
  int iAxis;
  float *fpVar, fAxisVal;
  float *fpVol, *fpCur;
  float x_i, y_i, z_i, e_i, e_j, e_k;
  float x_j, y_j, z_j, x_k, y_k, z_k;
  float cx[3], cy[3], cz[3], et[3];
  float det;
  float Bfmax;

  /*set some useful parameters */
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  njp2 = nj+2;
  iSize = nip1*njp1*nkp1;
  
  /*Get the data */
  cpDum = strcpy(caSec,"rho_");
  fpRho = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vx_");
  fpVx = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vy_");
  fpVy = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vz_");
  fpVz = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"c_");
  fpC = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bx_");
  fpBx = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"by_");
  fpBy = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bz_");
  fpBz = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bi_");
  fpBi = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bj_");
  fpBj = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bk_");
  fpBk = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"ei_");
  fpEi = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"ej_");
  fpEj = fpPGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"ek_");
  fpEk = fpPGetData(liFileId,caSec);

  /*       Create the Flux Field  */
  iSize = nip1*njp1*nkp1;
    for ( i=0; i<iSize; i++ ) {
      fpFluxData[3*i] = fpBi[i];
      fpFluxData[3*i+1] = fpBj[i];
      fpFluxData[3*i+2] = fpBk[i];
    }

  /*      Create the Div B Field  */
    for (k=0; k<nk; k++) {
    for (j=0; j<nj; j++) {
      for (i=0; i<ni; i++) {
	Bfmax = fabs( fpBi[ i + nip1*(j + njp1*k) ]) +
	  fabs( fpBi[ i+1 + nip1*(j + njp1*k) ]) +
	  fabs( fpBj[ i + nip1*(j + njp1*k) ]) +
	  fabs( fpBj[ i + nip1*(j+1 + njp1*k) ]) +
	  fabs( fpBk[ i + nip1*(j + njp1*k) ]) +
	  fabs( fpBk[ i + nip1*(j + njp1*(k+1)) ]) +
	  SMALL;
	fpDivBData[ijk(i,j,k)] =
	  fabs( ( fpBi[ i + nip1*(j + njp1*k) ]) -
	      ( fpBi[ i+1 + nip1*(j + njp1*k) ]) +
	      ( fpBj[ i + nip1*(j + njp1*k) ]) -
	      ( fpBj[ i + nip1*(j+1 + njp1*k) ]) +
	      ( fpBk[ i + nip1*(j + njp1*k) ]) -
	      ( fpBk[ i + nip1*(j + njp1*(k+1)) ]))/
		   Bfmax;
		   }}}


  // Create the AVS Field data 
  iSize = ni*njp2*nkp1;
  nv = 14;
  fpVar = malloc(sizeof(float)*iSize*nv);
  fpVol = malloc(sizeof(float)*nip1*njp1*nkp1);
  fpCur = malloc(sizeof(float)*3*iSize);
#ifdef DEBUG
  fprintf(FpDebug,"output of Ei -- first 153 elements at k=15(C)\n"); 
  for (i=0; i<153; i++) {
    k = i + nip1*njp1*15;
    fprintf(FpDebug,"Ei %d %g\n",i,fpEi[k]); }
  fprintf(FpDebug,"output of Ej -- first 153 elements at k=15(C)\n"); 
  for (i=0; i<153; i++) {
    k = i + nip1*njp1*15;
    fprintf(FpDebug,"Ej %d %g\n",i,fpEj[k]); }
  fprintf(FpDebug,"output of Bz -- first 153 elements at k=15(C)\n"); 
  for (i=0; i<153; i++) {
    k = i + nip1*njp1*15;
    fprintf(FpDebug,"Bz %d %g\n",i,fpBz[k]); }
#endif
  
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        // Convert to correct units
        // Density in #/cm^3
        fpVar[vijk(0,i,j+1,k)]=fpRho[ijk1(i,j,k)]*4.7619e23;
        // Vx in km/s
        fpVar[vijk(1,i,j+1,k)]=fpVx[ijk1(i,j,k)]*1.0e-5;
        // Vy in km/s
        fpVar[vijk(2,i,j+1,k)]=fpVy[ijk1(i,j,k)]*1.0e-5;
        // Vz in km/s
        fpVar[vijk(3,i,j+1,k)]=fpVz[ijk1(i,j,k)]*1.0e-5;
        // Pressure kev/cm^3
        fpVar[vijk(4,i,j+1,k)]=fpRho[ijk1(i,j,k)]*fpC[ijk1(i,j,k)]*
                               fpC[ijk1(i,j,k)]*3.75e8; 
        // Bx in nT
        fpVar[vijk(5,i,j+1,k)]=fpBx[ijk1(i,j,k)]*1.0e+5;
        // By in nT
        fpVar[vijk(6,i,j+1,k)]=fpBy[ijk1(i,j,k)]*1.0e+5;
        // Bz in nT
        fpVar[vijk(7,i,j+1,k)]=fpBz[ijk1(i,j,k)]*1.0e+5;
      }
    }
  }

#ifdef DEBUG
  fprintf(FpDebug,"output of Bz in fpVar -- i=0,52,j=0,2\n"); 
	  for (j=0; j<3; j++) {
  for (i=0; i<51; i++) {
    k = vijk(7,i,j,15);
    fprintf(FpDebug,"Bz %d %d %d %g\n",i,j,k,fpVar[k]); }}
#endif

  /*  get cartesian E's at regular grid points  */
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
           e_i = 0.25*(fpEi[ijk1(i,j,k)]+fpEi[ijk1(i,j,k+1)] +
                 fpEi[ijk1(i,j+1,k)]+fpEi[ijk1(i,j+1,k+1)]);
           e_j = 0.25*(fpEj[ijk1(i,j,k)]+fpEj[ijk1(i,j,k+1)] +
                 fpEj[ijk1(i+1,j,k)]+fpEj[ijk1(i+1,j,k+1)]);
           e_k = 0.25*(fpEk[ijk1(i,j,k)]+
                fpEk[ijk1(i+1,j,k)] +
                fpEk[ijk1(i,j+1,k)]+
                fpEk[ijk1(i+1,j+1,k)]);
     x_i = 0.25*(
           (fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]+fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i+1,j+1,k+1)])
        -  (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+fpX[ijk1(i,j,k+1)]+fpX[ijk1(i,j+1,k+1)]));
     y_i = 0.25*(
           (fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]+fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i+1,j+1,k+1)])
          - (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+fpY[ijk1(i,j,k+1)]+fpY[ijk1(i,j+1,k+1)]));
     z_i = 0.25*(
           (fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]+fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i+1,j+1,k+1)])
        -  (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]));
     x_j = 0.25*(
           (fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)])
        -  (fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)]+fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]));
     y_j = 0.25*(
           (fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)])
        -  (fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)]+fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]));
     z_j = 0.25*(
           (fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)])
        -  (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]));
     x_k = 0.25*(
           (fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)])
        -  (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]));
     y_k = 0.25*(
           (fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)])
        -  (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]));
     z_k = 0.25*(
           (fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)])
        -  (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]));
                 cx[0] = x_i;
                 cx[1] = x_j;
                 cx[2] = x_k;
                 cy[0] = y_i;
                 cy[1] = y_j;
                 cy[2] = y_k;
                 cz[0] = z_i;
                 cz[1] = z_j;
                 cz[2] = z_k;
                 et[0] = e_i;
                 et[1] = e_j;
                 et[2] = e_k;
                 det = 1.e-6/p_triple(cx,cy,cz);
		 fpVol[ijk1(i,j,k)] = fabs(p_triple(cx,cy,cz));
                 fpVar[vijk(8,i,j+1,k)] = p_triple(et,cy,cz)*det;
                 fpVar[vijk(9,i,j+1,k)] = p_triple(cx,et,cz)*det;
                 fpVar[vijk(10,i,j+1,k)] = p_triple(cx,cy,et)*det;
#ifdef DEBUG
		 if ( i >= 28 && i <=31 ) {
		   if ( j >=21 ) {
		     fprintf(FpDebug,">>>>>>>>>>>>\n %d %d %d \nelectric field\n",i,j,k);
		       fprintf(FpDebug,"%g %g %g\n",e_i,e_j,e_k);
		       fprintf(FpDebug,"%g %g %g\n",x_i,x_j,x_k);
		       fprintf(FpDebug,"%g %g %g\n",y_i,y_j,y_k);
		       fprintf(FpDebug,"%g %g %g\n",z_i,z_j,z_k);
		       fprintf(FpDebug,"%g %g\n",det,fpVol[ijk1(i,j,k)]);
		       fprintf(FpDebug,"%g %g %g\n",fpVar[vijk(8,i,j+1,k)],
			       fpVar[vijk(9,i,j+1,k)],fpVar[vijk(10,i,j+1,k)]); }}
#endif
      }}}
 
  pjcalc2_(fpCur,fpBx,fpBy,fpBz,fpX,fpY,fpZ,fpVol,&ni,&nj,&nk,&nip1,&njp1,&nkp1,&njp2);

  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
	for(l=0; l<3; l++) {
	  fpVar[vijk(11+l,i,j+1,k)] = fpCur[l+3*(i+ni*((j+1)+njp2*k))];
	}}}}

  // Calculate Axis values
 
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpVar[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpVar[vijk(l,i,j,k)]=fAxisVal;
        }
      }
    }
  }

  // Close the k cells for the field
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpVar[vijk(l,i,j,nk)]=fpVar[vijk(l,i,j,0)];
      }
    }
  }

#ifdef DEBUG
  fprintf(FpDebug,"checking variables \n");
    fprintf(FpDebug,"X[1,1,1] = %g \n",fpX[ijk1(1,1,1)]);
    fprintf(FpDebug,"Y[1,1,1] = %g \n",fpY[ijk1(1,1,1)]);
    fprintf(FpDebug,"Z[1,1,1] = %g \n",fpZ[ijk1(1,1,1)]);

    fprintf(FpDebug,"Rho[1,1,1] = %g \n",fpRho[ijk1(1,1,1)]);
    fprintf(FpDebug,"C[1,1,1] = %g \n",fpC[ijk1(1,1,1)]);
    fprintf(FpDebug,"vX[1,1,1] = %g \n",fpVx[ijk1(1,1,1)]);
    fprintf(FpDebug,"vY[1,1,1] = %g \n",fpVy[ijk1(1,1,1)]);
    fprintf(FpDebug,"vZ[1,1,1] = %g \n",fpVz[ijk1(1,1,1)]);
    fprintf(FpDebug,"bX[1,1,1] = %g \n",fpBx[ijk1(1,1,1)]);
    fprintf(FpDebug,"bY[1,1,1] = %g \n",fpBy[ijk1(1,1,1)]);
    fprintf(FpDebug,"bZ[1,1,1] = %g \n",fpBz[ijk1(1,1,1)]);
    for (i=0; i<14; i++) {
      fprintf(FpDebug,"variable at [%d, 1,2,1] = %g \n",i,fpVar[vijk(i,1,2,1)]);}
    fflush(NULL);
#endif 

  // Pass the field data back
  for(i=0;i<nv*iSize;i++) {
    fpMHDData[i] = fpVar[i];
  } 

  DXMessage("O.K. to here, Bill!\n");
  //free(fpCosPhi);
  DXMessage("O.K. to here, Bill!\n");
  //free(fpSinPhi);
  DXMessage("O.K. to here, Bill!\n");
  free(fpX);
  free(fpY);
  free(fpZ);
  free(fpXc);
  free(fpYc);
  free(fpZc);
  //free(fpX2);  free(fpY2);

  DXMessage("O.K. to here, Bill!\n");

  free(fpRho);
  free(fpVx);
  free(fpVy);
  free(fpVz);
  free(fpC);
  free(fpBx);
  free(fpBy);
  free(fpBz);
  free(fpBi);
  free(fpBj);
  free(fpBk);
  free(fpEi);
  free(fpEj);
  free(fpEk);
  free(fpVol);
  free(fpVar); 
  free(fpCur);
  
  return 1;   
  
}

 int PChkStep(int iTimeStep, int iInital, int iFinal, int iInterval) {

  if( iTimeStep >= iInital && iTimeStep <= iFinal) {
    if( iTimeStep % iInterval == 0 ) {
      return iTimeStep;
    } else {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep, 
              iTimeStep - iTimeStep % iInterval);
      return iTimeStep - iTimeStep % iInterval;
    }
  } else {
    if (iTimeStep < iInital) {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep,iInital);
      return iInital;
    } else {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep, iFinal );
      return iFinal;
    }
  }
}

float p_dot(float *x,float *y) {
  return x[0]*y[0] + x[1]*y[1] + x[2]*y[2]; }

void p_cross3(float *x, float *y, float *z) {
  z[0] = x[1]*y[2] - x[2]*y[1];
  z[1] = x[2]*y[0] - x[0]*y[2];
  z[2] = x[0]*y[1] - x[1]*y[0];
}

float p_triple(float *x, float *y, float *z) {
  float dum[3];
  p_cross3(y,z,dum);
  return p_dot(x,dum);
}
 

