#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"


int CalcIonGrid(float fNorm, int nip1, int njp2, int nk2p3,float *fpIonPts)
{
  int i,j,k,l,m,lm,iNumPts;
  int nj,njp1,nk2,nk2p1, nk2p2;
  float fAxisVal, fNormInv;
  float fRo, fTheta, fPhi;
  float *fpX2Ion, *fpY2Ion;
  float *fpXa2,*fpYa2,*fpZa2;
  float fRouter, fS2Theta, fTheta2, fPhi2;
  float fAzero, fR0, fRionm, fXbar, fYbar, fDxj, fDxk, fDyj, fDyk;
  float fAaa, fRbar, fAng, fThfac,fXu,fYu;
  float faDdj[2][2], faDdk[2][2];
  char caDataName[MAX_NC_NAME];
  FILE *fpOut;


  njp1 = njp2 -1;
  nj = njp1 - 1;
  nk2  = nk2p3 -3;
  nk2p1 = nk2 + 1; 
  nk2p2 = nk2 + 2; 

  /* First step is to determine the XA2, YA2, and ZA2 ionospheric grid info from
   * the magnetospheric grid
   */

  iNumPts = njp1*nk2p3;
  fpXa2 = malloc(sizeof(float)*iNumPts);
  fpYa2 = malloc(sizeof(float)*iNumPts);
  fpZa2 = malloc(sizeof(float)*iNumPts);
  for(j=0;j<njp1;j++) {
    for(k=0;k<nk2p1;k++) {
      fpXa2[jk(j,k+1)]=fpX[ijk1(1,j,0)];
      fpYa2[jk(j,k+1)]=fpY[ijk1(1,j,k)];
      fpZa2[jk(j,k+1)]=fpZ[ijk1(1,j,k)];
    }
  }
  for(k=0;k<nk2p1;k++) {
    fpXa2[jk(0,k+1)]=fpX[ijk1(1,0,nk2/2)];
    fpYa2[jk(0,k+1)]=fpY[ijk1(1,0,nk2/2)];
    fpZa2[jk(0,k+1)]=fpZ[ijk1(1,0,nk2/2)];
    fpXa2[jk(nj,k+1)]=fpX[ijk1(1,nj,nk2/2)];
    fpYa2[jk(nj,k+1)]=fpY[ijk1(1,nj,nk2/2)];
    fpZa2[jk(nj,k+1)]=fpZ[ijk1(1,nj,nk2/2)];
  }

  for(j=0;j<njp1;j++) {
    fpXa2[jk(j,0)]=fpX[ijk1(0,j,0)];
    fpYa2[jk(j,0)]=fpY[ijk1(0,j,0)];
    fpZa2[jk(j,0)]=fpZ[ijk1(0,j,0)];
    fpXa2[jk(j,nk2p2)]=fpX[ijk1(0,j,nk2)];
    fpYa2[jk(j,nk2p2)]=fpY[ijk1(0,j,nk2)];
    fpZa2[jk(j,nk2p2)]=fpZ[ijk1(0,j,nk2)];
  }

  fpX2Ion = malloc(sizeof(float)*iNumPts);
  fpY2Ion = malloc(sizeof(float)*iNumPts);
  for(j=0;j<njp1;j++) {
    for(k=0;k<nk2p3;k++) {
       fRouter=sqrt(pow(fpXa2[jk(j,k)],2.0) + pow(fpYa2[jk(j,k)],2.0) 
                    + pow(fpZa2[jk(j,k)],2.0));
       fS2Theta = fNorm/fRouter*(1.0-pow(fpZa2[jk(j,k)]/fRouter,2.0));
       fTheta2 = asin(sqrt(fS2Theta));
       fPhi2 = atan2(fpYa2[jk(j,k)],fpXa2[jk(j,k)]);
       fpX2Ion[jk(j,k)] = fTheta2*cos(fPhi2);
       fpY2Ion[jk(j,k)] = fTheta2*sin(fPhi2);
    }
  }

  /* These arrays
  X2Ion = theta*cos(phi)
  Y2Ion = theta*sin(phi)
  so we can go 3d by using by extracting the angles 
  theta = sqrt(x2ion^2+y2ion^2)
  phi = atan2(y2ion,x2ion)
  and then applying the usual transformations
  x = r0*sin(theta)*cos(phi)
  y = r0*sin(theta)*cos(phi)
  z = +/-r0*cos(theta)  sign defines hemisphere
  */

  fRo = 6.5e8/fNorm;
  for(i=0;i<iNumPts;i++) {
    fTheta = sqrt(fpX2Ion[i]*fpX2Ion[i]+fpY2Ion[i]*fpY2Ion[i]);
    fPhi = atan2(fpY2Ion[i],fpX2Ion[i]);
    fpIonPts[i*3] = fRo*sin(fTheta)*cos(fPhi);
    fpIonPts[i*3+1] = fRo*sin(fTheta)*sin(fPhi);
    fpIonPts[i*3+2] = fRo*cos(fTheta);
  }

  /* In order to compute the NS/EW componets of electric field and current we
     need the grid dependent Coefficents
   */

  fpEthCoef = malloc(sizeof(float)*nj*nk2p2*4);
  fpEphiCoef = malloc(sizeof(float)*nj*nk2p2*4);
  for(i=0;i<nj*nk2p2*4;i++) {
    fpEthCoef[i]=0.0;
    fpEphiCoef[i]=0.0;
  }
  fRionm = 6.5e6;
  fAzero = fRionm*fRionm;
  for(k=0;k<nk2p2;k++) {
    for(j=0;j<nj;j++)  {
      fXbar = 0.25*(fpX2Ion[jk(j,k)]+fpX2Ion[jk(j+1,k)]+
                    fpX2Ion[jk(j,k+1)]+fpX2Ion[jk(j+1,k+1)]);
      fYbar = 0.25*(fpY2Ion[jk(j,k)]+fpY2Ion[jk(j+1,k)]+
                    fpY2Ion[jk(j,k+1)]+fpY2Ion[jk(j+1,k+1)]);
      fDxj = 0.5*(fpX2Ion[jk(j+1,k)]+fpX2Ion[jk(j+1,k+1)]-
                  fpX2Ion[jk(j,k)]-fpX2Ion[jk(j,k+1)]);
      fDxk = 0.5*(fpX2Ion[jk(j,k+1)]+fpX2Ion[jk(j+1,k+1)]-
                  fpX2Ion[jk(j,k)]-fpX2Ion[jk(j+1,k)]);
      fDyj = 0.5*(fpY2Ion[jk(j+1,k)]+fpY2Ion[jk(j+1,k+1)]-
                  fpY2Ion[jk(j,k)]-fpY2Ion[jk(j,k+1)]);
      fDyk = 0.5*(fpY2Ion[jk(j,k+1)]+fpY2Ion[jk(j+1,k+1)]-
                  fpY2Ion[jk(j,k)]-fpY2Ion[jk(j+1,k)]);
      fAaa = fDxj*fDyk-fDxk*fDyj;
      fRbar = sqrt(fXbar*fXbar+fYbar*fYbar);
      fAng = atan2(fYbar,fXbar);
      if ( fRbar < 1.0e-3) {
         fThfac = 1.0;
         fXu = cos(fAng);
         fYu = sin(fAng);
      } else {
         fThfac = sin(fRbar)/fRbar;
         fXu = fXbar/fRbar;
         fYu = fYbar/fRbar;
      }
      faDdj[0][0] = -1.0;
      faDdj[0][1] = -1.0;
      faDdj[1][0] = 1.0;
      faDdj[1][1] = 1.0;
      faDdk[0][0] = -1.0;
      faDdk[0][1] = 1.0;
      faDdk[1][0] = -1.0;
      faDdk[1][1] = 1.0;
      for(l=0;l<2;l++) {
        for(m=0;m<2;m++) {
          lm = m+2*l;
          fpEthCoef[jkl(j,k,lm)] = 0.5*(faDdj[m][l]*(fXu*fDyk-fYu*fDxk) +
                                           faDdk[m][l]*(fYu*fDxj-fXu*fDyj))/
                                           fAaa/fRionm;
          fpEphiCoef[jkl(j,k,lm)] = 0.5*(-1.0*faDdj[m][l]*(fYu*fDyk-fXu*fDxk)+
                                      faDdk[m][l]*(fYu*fDyj+fXu*fDxj))/
                                           fAaa/fRionm;
        }
      }
	  
    }
   }

  free(fpX2Ion);
  free(fpY2Ion);
  free(fpXa2);
  free(fpYa2);
  free(fpZa2);

 /* Since all the pointers are 'shared' we must clean up the grid pointers */
  free(fpCosPhi);
  free(fpSinPhi);
  free(fpX2);
  free(fpY2);
  free(fpPhi);
  free(fpX);
  free(fpY);
  free(fpZ);
  free(fpXc);
  free(fpYc);
  free(fpZc);

  return 1;
}


