#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"
#include "bzero_pointers.h"

#define SMALL 1.e-15
FILE *FpDebug;

int GetMHDData(int32 liFileId, int iStepfix, int ni, int nj, int nk,
               int iComputeTubes,
	       float *fpMHDData, float *fpFluxData, float *fpDivBData,
               float *fpTubeData)
{
  int32 iErr, liIndex,liSecId,liStart,liStride,liEdge,liStat,liBzeroSize;
  float triple(), dot();
  void cross3();
  char *cpDum, caSec[MAX_NC_NAME],caStep[10];
  int i,j,k,l;
  int nip1, njp1, nkp1, njp2, nv, iSize;
  int iAxis;
  int Nbtot;
  float *fpVar, *fpReadVar, fAxisVal, *fpVol, *fpCur;
  float *fpBzeroVar;
  float *fpBXqnK, *fpBYqnK, *fpBZqnK;
  float x_i, y_i, z_i, e_i, e_j, e_k;
  float x_j, y_j, z_j, x_k, y_k, z_k;
  float cx[3], cy[3], cz[3], et[3];
  float det;
  float Bfmax;

  /*set some useful parameters */
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  njp2 = nj+2;
  iSize = nip1*njp1*nkp1;
  Nbtot = 8*nip1*nj*nk + 3*nip1*njp1*nk +
          8*ni*njp1*nk + 3*ni*njp1*nkp1 +
          8*ni*nj*nkp1 + 3*nip1*nj*nkp1;

  /*Get the Bzero data*/
  fpBzeroVar = fpGetData(liFileId,"bzeros");
  if (NULL != fpBzeroVar) {
    fp_bnqfi = fpBzeroVar;
#include "bzero_assign.h"
    fpBXqnK = fp_bxqnk;
    fpBYqnK = fp_byqnk;
    fpBZqnK = fp_bzqnk;
  }
  
  
  /*Get the data */
  sprintf(caSec,"time_step_%d",iStepfix);
  fpReadVar = fpGetData(liFileId,caSec);
  
 /* Get Bfluxes */
  /* First read the Bi,Bj, and Bk */
  sprintf(caSec,"bi_time_step_%d",iStepfix);
  fpBi = fpGetData(liFileId,caSec);
  sprintf(caSec,"bj_time_step_%d",iStepfix);
  fpBj = fpGetData(liFileId,caSec);
  sprintf(caSec,"bk_time_step_%d",iStepfix);
  fpBk = fpGetData(liFileId,caSec);

  /*       Create the Flux Field  */
  iSize = nip1*njp1*nkp1;
  nv = 3;
  /* First initialize the entire field to zero */
  for ( i=0; i<iSize; i++ ) {
    fpFluxData[i] = 0.0;
  }
  if (NULL != fpBzeroVar) { /* Bzero information in HDF file */
  /* The i faces have size nip1,nj,nk */
    for (i=0;i<nip1;i++) {
      for (j=0;j<nj;j++) {
        for (k=0;k<nk;k++) {
          fpBi[i1jk(i,j,k)] += fpFace_i[i1jk(i,j,k)]*fp_bnqfi[i1jk(i,j,k)];
          fpBi[i1jk(i,j,k)] = fpBi[i1jk(i,j,k)]/(6.38e8*6.38e8);
          //fpBi[i1jk(i,j,k)] = fpFace_i[i1jk(i,j,k)];
          fpFluxData[vi1j1k(0,i,j,k)] = fpBi[i1jk(i,j,k)];
        }
      }
    }
    /* The j faces have size ni,njp1,nk */
    for (i=0;i<ni;i++) {
      for (j=0;j<njp1;j++) {
        for (k=0;k<nk;k++) {
          fpBj[ij1k(i,j,k)] += fpFace_j[ij1k(i,j,k)]*fp_bnqfj[ij1k(i,j,k)];
          fpBj[ij1k(i,j,k)] = fpBj[ij1k(i,j,k)]/(6.38e8*6.38e8);
          //fpBj[ij1k(i,j,k)] = fpFace_j[ij1k(i,j,k)];
          fpFluxData[vi1j1k(1,i,j,k)] = fpBj[ij1k(i,j,k)];
        }
      }
    }
    /* The k faces have size ni,nj,nkp1 */
    for (i=0;i<ni;i++) {
      for (j=0;j<nj;j++) {
        for (k=0;k<nkp1;k++) {
          fpBk[ijk(i,j,k)] += fpFace_k[ijk(i,j,k)]*fp_bnqfk[ijk(i,j,k)];
          fpBk[ijk(i,j,k)] = fpBk[ijk(i,j,k)]/(6.38e8*6.38e8);
          //fpBk[ijk(i,j,k)] = fpFace_k[ijk(i,j,k)];
          fpFluxData[vi1j1k(2,i,j,k)] = fpBk[ijk(i,j,k)];
        }
      }
    }
  } else { /* Total magnetic field information in file */
    for (i=0;i<nip1;i++) {
      for (j=0;j<nj;j++) {
        for (k=0;k<nk;k++) {
          fpFluxData[vi1j1k(0,i,j,k)] = fpBi[i1jk(i,j,k)];
        }
      }
    }
    /* The j faces have size ni,njp1,nk */
    for (i=0;i<ni;i++) {
      for (j=0;j<njp1;j++) {
        for (k=0;k<nk;k++) {
          fpFluxData[vi1j1k(1,i,j,k)] = fpBj[ij1k(i,j,k)];
        }
      }
    }
    /* The k faces have size ni,nj,nkp1 */
    for (i=0;i<ni;i++) {
      for (j=0;j<nj;j++) {
        for (k=0;k<nkp1;k++) {
          fpFluxData[vi1j1k(2,i,j,k)] = fpBk[ijk(i,j,k)];
        }
      }
    }
  }
  /*      Create the Div B Field  */
    for (k=0; k<nk; k++) {
    for (j=0; j<nj; j++) {
      for (i=0; i<ni; i++) {
        Bfmax = fabs( fpBi[i1jk(i,j,k)]) +
          fabs( fpBi[i1jk(i+1,j,k)]) +
          fabs( fpBj[ij1k(i,j,k) ]) +
          fabs( fpBj[ij1k(i,j+1,k)]) +
          fabs( fpBk[ijk(i,j,k)]) +
          fabs( fpBk[ijk(i,j,k+1) ]) +
          SMALL;
        fpDivBData[ijk(i,j,k)] =
          fabs( ( fpBi[i1jk(i,j,k) ]) -
              ( fpBi[i1jk(i+1,j,k) ]) +
              ( fpBj[ij1k(i,j,k) ]) -
              ( fpBj[ij1k(i,j+1,k) ]) +
              ( fpBk[ijk(i,j,k) ]) -
              ( fpBk[ijk(i,j,k+1)]))/
                   Bfmax;

                   }}}

  /* Now we construct the OpenDX field */
  iSize = ni*njp2*nkp1;
  nv = 14;
  fpVar = malloc(sizeof(float)*iSize*nv);
  /* Store a copy of Bx, By, and Bz for current calcs */
  fpBx = malloc(sizeof(float)*ni*nj*nk);
  fpBy = malloc(sizeof(float)*ni*nj*nk);
  fpBz = malloc(sizeof(float)*ni*nj*nk);
  fpRho = malloc(sizeof(float)*ni*nj*nk);
  fpP = malloc(sizeof(float)*ni*nj*nk);
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units */
        /* Density in #/cm^3 */
        fpVar[vijk(0,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*4.7619e23;
        fpRho[ijk(i,j,k)]=fpReadVar[vijk1(0,i,j,k)]; 
        /* Vx in km/s */
        fpVar[vijk(1,i,j+1,k)]=fpReadVar[vijk1(1,i,j,k)]*1.0e-5;
        /* Vy in km/s */
        fpVar[vijk(2,i,j+1,k)]=fpReadVar[vijk1(2,i,j,k)]*1.0e-5;
        /* Vz in km/s */
        fpVar[vijk(3,i,j+1,k)]=fpReadVar[vijk1(3,i,j,k)]*1.0e-5;
        /* Pressure kev/cm^3 */
        fpVar[vijk(4,i,j+1,k)]=fpReadVar[vijk1(0,i,j,k)]*
		               fpReadVar[vijk1(4,i,j,k)]*
                               fpReadVar[vijk1(4,i,j,k)]*3.75e8; 
        fpP[ijk(i,j,k)]=fpReadVar[vijk1(4,i,j,k)]; 
      }
    }
  }
  if (NULL != fpBzeroVar) { /*Bzero information in HDF file */
    for(k=0;k<nk;k++) {
      for(j=0;j<nj;j++) {
        for(i=0;i<ni;i++) {
          /* Bx in nT */
          fpVar[vijk(5,i,j+1,k)]=(fpReadVar[vijk1(5,i,j,k)]+
           (fpBXqnK[ijk(i,j,k)]+fpBXqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
          //fpBx[ijk(i,j,k)]=fpReadVar[vijk1(5,i,j,k)];
          fpBx[ijk(i,j,k)]=(fpReadVar[vijk1(5,i,j,k)]+
	   (fpBXqnK[ijk(i,j,k)]+fpBXqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
          /* By in nT */
          fpVar[vijk(6,i,j+1,k)]=(fpReadVar[vijk1(6,i,j,k)]+
           (fpBYqnK[ijk(i,j,k)]+fpBYqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
          //fpBy[ijk(i,j,k)]=fpReadVar[vijk1(6,i,j,k)];
          fpBy[ijk(i,j,k)]=(fpReadVar[vijk1(6,i,j,k)]+
	   (fpBYqnK[ijk(i,j,k)]+fpBYqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
          /* Bz in nT */
          fpVar[vijk(7,i,j+1,k)]=(fpReadVar[vijk1(7,i,j,k)]+
           (fpBZqnK[ijk(i,j,k)]+fpBZqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
          //fpBz[ijk(i,j,k)]=fpReadVar[vijk1(7,i,j,k)];
          fpBz[ijk(i,j,k)]=(fpReadVar[vijk1(7,i,j,k)]+
	   (fpBZqnK[ijk(i,j,k)]+fpBZqnK[ijk(i,j,k+1)])*0.5)*1.0e+5;
        }
      }
    }
  } else { /* Total magnetic field in HDF file */
    for(k=0;k<nk;k++) {
      for(j=0;j<nj;j++) {
        for(i=0;i<ni;i++) {
          /* Bx in nT */
          fpVar[vijk(5,i,j+1,k)]=fpReadVar[vijk1(5,i,j,k)]*1.0e+5;
          fpBx[ijk(i,j,k)]=fpReadVar[vijk1(5,i,j,k)];
          /* By in nT */
          fpVar[vijk(6,i,j+1,k)]=fpReadVar[vijk1(6,i,j,k)]*1.0e+5;
          fpBy[ijk(i,j,k)]=fpReadVar[vijk1(6,i,j,k)];
          /* Bz in nT */
          fpVar[vijk(7,i,j+1,k)]=fpReadVar[vijk1(7,i,j,k)]*1.0e+5;
          fpBz[ijk(i,j,k)]=fpReadVar[vijk1(7,i,j,k)];
        }
      }
    }
  }

  /* Get Cartesian E's at regular grid points */
  /* First read the Ei,Ej, and Ek */
  sprintf(caSec,"ei_time_step_%d",iStepfix);
  fpEi = fpGetData(liFileId,caSec);
  sprintf(caSec,"ej_time_step_%d",iStepfix);
  fpEj = fpGetData(liFileId,caSec);
  sprintf(caSec,"ek_time_step_%d",iStepfix);
  fpEk = fpGetData(liFileId,caSec);
  /* Need space for cell volume which will be calc here and used in
   * determination of the current 
   */
  fpVol = malloc(sizeof(float)*nip1*njp1*nkp1);
  
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) { 
      for(i=0;i<ni;i++) {
           e_i = 0.25*(fpEi[ijkei(i,j,k)]+fpEi[ijkei(i,j,k+1)] +
                 fpEi[ijkei(i,j+1,k)]+fpEi[ijkei(i,j+1,k+1)]);
           e_j = 0.25*(fpEj[ijkej(i,j,k)]+fpEj[ijkej(i,j,k+1)] +
                 fpEj[ijkej(i+1,j,k)]+fpEj[ijkej(i+1,j,k+1)]);
           e_k = 0.25*(fpEk[ijkek(i,j,k)]+fpEk[ijkek(i+1,j,k)] +
                fpEk[ijkek(i,j+1,k)]+ fpEk[ijkek(i+1,j+1,k)]);
     x_i = 0.25*((fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]+
                 fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -
                (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+
                 fpX[ijk1(i,j,k+1)]+fpX[ijk1(i,j+1,k+1)]));
     y_i = 0.25*((fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]+
                  fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) - 
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+
                  fpY[ijk1(i,j,k+1)]+fpY[ijk1(i,j+1,k+1)]));
     z_i = 0.25*((fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]+
                  fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) - 
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+
                  fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]));
     x_j = 0.25*((fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+
                  fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -  
                 (fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)]+
                  fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]));
     y_j = 0.25*((fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+
                  fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) -  
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)]+
                  fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]));
     z_j = 0.25*((fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+
                  fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) -  
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)]+
                  fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]));
     x_k = 0.25*((fpX[ijk1(i,j,k+1)]+fpX[ijk1(i+1,j,k+1)]+
                  fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) -  
                 (fpX[ijk1(i,j,k)]+fpX[ijk1(i,j+1,k)]+
                  fpX[ijk1(i+1,j,k)]+fpX[ijk1(i+1,j+1,k)]));
     y_k = 0.25*((fpY[ijk1(i,j,k+1)]+fpY[ijk1(i+1,j,k+1)]+
                  fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) -  
                 (fpY[ijk1(i,j,k)]+fpY[ijk1(i,j+1,k)]+
                  fpY[ijk1(i+1,j,k)]+fpY[ijk1(i+1,j+1,k)]));
     z_k = 0.25*((fpZ[ijk1(i,j,k+1)]+fpZ[ijk1(i+1,j,k+1)]+
                  fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) -  
                 (fpZ[ijk1(i,j,k)]+fpZ[ijk1(i,j+1,k)]+
                 fpZ[ijk1(i+1,j,k)]+fpZ[ijk1(i+1,j+1,k)]));
                 cx[0] = x_i;
                 cx[1] = x_j;
                 cx[2] = x_k;
                 cy[0] = y_i;
                 cy[1] = y_j;
                 cy[2] = y_k;
                 cz[0] = z_i;
                 cz[1] = z_j;
                 cz[2] = z_k;
                 et[0] = e_i;
                 et[1] = e_j;
                 et[2] = e_k;
                 det = 1.e-6/triple(cx,cy,cz);
		 fpVol[ijk1(i,j,k)] = fabs(triple(cx,cy,cz));
                 fpVar[vijk(8,i,j+1,k)] = triple(et,cy,cz)*det;
                 fpVar[vijk(9,i,j+1,k)] = triple(cx,et,cz)*det;
                 fpVar[vijk(10,i,j+1,k)] = triple(cx,cy,et)*det;
     } /* End i Loop */
    } /* End j Loop */
  } /* End k Loop */
 
  /* Next we move onto calculating the cartesian currents */

  

  /* First we need to get the magnetic field information for the curl */
  fpCur = malloc(sizeof(float)*3*iSize);
  jcalc2ptr_(fpCur,fpBx,fpBy,fpBz,fpX,fpY,fpZ,fpVol,
          &ni,&nj,&nk,&nip1,&njp1,&nkp1,&njp2);

  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
	for(l=0; l<3; l++) {
	  fpVar[vijk(11+l,i,j+1,k)] = fpCur[l+3*(i+ni*((j+1)+njp2*k))];
	}}}}

  /* Go off and do the field line tracing for the flux tube quanties */
  if (iComputeTubes != 0) {
    GetTubes(fpTubeData,ni,nj,nk); 
  } 
    

  /* Calculate Axis values */
 
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpVar[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpVar[vijk(l,i,j,k)]=fAxisVal;;
        }
      }
    }
  }
//
  /* Close the k cells for the field */
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpVar[vijk(l,i,j,nk)]=fpVar[vijk(l,i,j,0)];
      }
    }
  }

  /* Pass the field data back */
  for(i=0;i<nv*iSize;i++) {
    fpMHDData[i] = fpVar[i];
  } 

//Repeat axis and k cells for tube data
  nv = 4;
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpTubeData[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpTubeData[vijk(l,i,j,k)]=fAxisVal;;
        }
      }
    }
  }
//
  /* Close the k cells for the field */
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpTubeData[vijk(l,i,j,nk)]=fpTubeData[vijk(l,i,j,0)];
      }
    }
  }



  /* Clean up data pointers */
  free(fpReadVar);
  free(fpBzeroVar);
  free(fpFace_i);
  free(fpFace_j);
  free(fpFace_k);
  free(fpVar);
  free(fpRho);
  free(fpP);
  free(fpBx);
  free(fpBy);
  free(fpBz);
  free(fpBi);
  free(fpBj);
  free(fpBk);
  free(fpEi);
  free(fpEj);
  free(fpEk);
  free(fpVol);
  free(fpCur);
  
  /* We leave the grid pointers existence since they are used in the CalcIonGrid
   * function.
   */

  return 1;
}
