#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"
#include "fpointers.h"
#include "bzero_pointers.h"

#define MAXLINE  10000

int GetTubes(float *fpTubeData, int ni, int nj, int nk)
{
  int nip1,njp1,njp2,nkp1,nv;
  int i,j,k,l,m;
  /* Lines in logical space */
  float faILine[MAXLINE], faJLine[MAXLINE], faKLine[MAXLINE];
  /* Is Field line Connected to northern hemisphere (n), southern hemisphere
  * (s), closed (c), or solar wind (w)
  */
  char cLineType;
  /* Entry point of field in logical space for northern and southern hemis */
  float fIN,fJN,fKN,fIS,fJS,fKS;
  /* Line Length */
  int iLineLen;
  /* Real space values */
  float *fpXLine, *fpYLine, *fpZLine;
  float *fpBxLine, *fpByLine, *fpBzLine;
  float *fpRhoLine, *fpPLine;
  
  float fi,fj,fk;
  float fTubeVol,fTubeMass,fTubeP,fTubeEntropy;
  float fDs, fBmag, fDVol;
  float fReInv,fGamma,fGammaInv;

  float f1,f2,f3,f4,f5,f6,f7,f8;

  int flag=0;

  nip1 = ni + 1;
  njp1 = nj + 1;
  njp2 = nj + 2;
  nkp1 = nk + 1;
  fReInv = 1.0/6.38e8; /* One over Earth Radius in cm */
  fGamma = 5.0/3.0;
  fGammaInv = 1.0/fGamma;

  /* Loop over all cells in magnetospheric grid */
  for(k=0;k<nk;k++) {
     fk = k +0.5;
    for(j=0;j<nj;j++) {
      fj = j+0.5;
      for(i=1;i<ni;i++) {
        fi = i+0.5;
        btrace(fi,fj,fk,ni,nj,nk,
               &iLineLen,&cLineType,faILine,faJLine,faKLine,
               &fIN,&fJN,&fKN,&fIS,&fJS,&fKS);
      
        fpXLine = malloc(sizeof(float)*iLineLen);
        fpYLine = malloc(sizeof(float)*iLineLen);
        fpZLine = malloc(sizeof(float)*iLineLen);
        fpBxLine = malloc(sizeof(float)*iLineLen);
        fpByLine = malloc(sizeof(float)*iLineLen);
        fpBzLine = malloc(sizeof(float)*iLineLen);
        fpRhoLine = malloc(sizeof(float)*iLineLen);
        fpPLine = malloc(sizeof(float)*iLineLen);

        interpline(fpXLine,fpX,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        interpline(fpYLine,fpY,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        interpline(fpZLine,fpZ,faILine,faJLine,faKLine,iLineLen,ni,nj,nk);
        interpline2(fpBxLine,fpBx,faILine,faJLine,faKLine,iLineLen,ni,nj,nk,0);
        interpline2(fpByLine,fpBy,faILine,faJLine,faKLine,iLineLen,ni,nj,nk,0);
        interpline2(fpBzLine,fpBz,faILine,faJLine,faKLine,iLineLen,ni,nj,nk,0);
        interpline2(fpRhoLine,fpRho,faILine,faJLine,faKLine,iLineLen,ni,nj,nk,0);
        interpline2(fpPLine,fpP,faILine,faJLine,faKLine,iLineLen,ni,nj,nk,0);

        for(l=0;l<iLineLen;l++) {
          fpPLine[l] = 6.0e7*fpRhoLine[l]*fpPLine[l]*fpPLine[l]; 
          //fpPLine[l] =fpPLine[l]*1.6e-1; 
	}

        fTubeVol =0.0;
        fTubeMass = 0.0;
        fTubeP = 0.0;
        fTubeEntropy = 0.0;
        for(l=0;l<iLineLen-1;l++) {
          fDs = sqrt(pow(fpXLine[l+1]-fpXLine[l],2)+ 
                     pow(fpYLine[l+1]-fpYLine[l],2)+ 
                     pow(fpZLine[l+1]-fpZLine[l],2)); 
          fBmag = 0.5*sqrt(pow(fpBxLine[l+1]+fpBxLine[l],2)+ 
                     pow(fpByLine[l+1]+fpByLine[l],2)+ 
                     pow(fpBzLine[l+1]+fpBzLine[l],2)); 
          fDVol = fReInv*fDs/fBmag;
          //fDVol = fReInv*fDs;
          fTubeVol += fDVol;
          fTubeMass += 0.5*(fpRhoLine[l+1]+fpRhoLine[l])*fDVol;
          fTubeP += 0.5*(fpPLine[l+1]+fpPLine[l])*fDVol;
          fTubeEntropy += pow(0.5*(fpPLine[l+1]+fpPLine[l]),fGammaInv)*fDVol;
        }

	fTubeP /= fTubeVol;
        fTubeEntropy = pow(fTubeEntropy,fGamma);

        nv = 4;
        switch (cLineType) {
         case 'C':
          fpTubeData[vijk(0,i,j+1,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j+1,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j+1,k)] = fTubeP;
          fpTubeData[vijk(3,i,j+1,k)] = 0.0;
          break;
         case 'N':
          fpTubeData[vijk(0,i,j+1,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j+1,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j+1,k)] = fTubeP;
          fpTubeData[vijk(3,i,j+1,k)] = 1.0;
          break;
         case 'S':
          fpTubeData[vijk(0,i,j+1,k)] = fTubeVol;
          fpTubeData[vijk(1,i,j+1,k)] = fTubeMass;
          fpTubeData[vijk(2,i,j+1,k)] = fTubeP;
          fpTubeData[vijk(3,i,j+1,k)] = 2.0;
          break;
         default:
          fpTubeData[vijk(0,i,j+1,k)] = 0.0;
          fpTubeData[vijk(1,i,j+1,k)] = 0.0;
          fpTubeData[vijk(2,i,j+1,k)] = 0.0;
          fpTubeData[vijk(3,i,j+1,k)] = 3.0;
          break;
        }

        free(fpXLine);
        free(fpYLine);
        free(fpZLine);
        free(fpBxLine);
        free(fpByLine);
        free(fpBzLine);
        free(fpRhoLine);
        free(fpPLine);
      }
// Assigning values to inner boundary
      fpTubeData[vijk(0,0,j+1,k)] = 0.0;
      fpTubeData[vijk(1,0,j+1,k)] = 0.0;
      fpTubeData[vijk(2,0,j+1,k)] = 0.0;
      fpTubeData[vijk(3,0,j+1,k)] = 0.0;
      //fpTubeData[vijk(4,0,j+1,k)] = 0.0;
    }
  }
  return 1;
}

void btrace(float fX,float fY,float fZ, int ni, int nj, int nk,
            int *ipLineLen, char *cpLineType, 
            float *fpXLine,float *fpYLine,float *fpZLine,
            float *fpXN,float *fpYN,float *fpZN,
            float *fpXS,float *fpYS,float *fpZS) {

/* A few notes on the calling arguments 
   INPUTS
   fX,fY,fZ - initial logical space coordinates 
   ni,nj,nk - size of grid
   OUTPUTS
   iLineLen - Number of points on line (valid space of output arrays is less
                                        than this amount)
   
   cpLineType - Type of magnetic field line
   fp(XYZ)Line - Array of logical coordinates of line
   fp(XYZ)(NS) - Entry in logical space into northern/southern hemisphere
*/

  int i,j,k,l;
  int i0,j0,k0,i00,j00,k00;
  int nip1,nim1,njp1,njm1,nkp1,nkm1,nk2,nk2m1;
  int nimin,nimax,njmin,njmax;
  int n,nd,n0;
  float *fpXq, *fpYq, *fpZq;
  float *fpXlow, *fpXhigh, *fpYlow, *fpYhigh, *fpZlow, *fpZhigh; 
  float fX0, fY0, fZ0;
  float fDirect;
  float fDBmag, fAvB;
  float fTmp;
  float fDBx, fBx0, fSgni, fDx, fCx, fCxDx, fDx0Bx, fXLgln, fSx;
  float fDBy, fBy0, fSgnj, fDy, fCy, fCyDy, fDy0By, fYLgln, fSy;
  float fDBz, fBz0, fSgnk, fDz, fCz, fCzDz, fDz0Bz, fZLgln, fSz;
  float fS0a, fS0;
  int iPl, jPl, kPl;
  float fX0q, fY0q, fZ0q;
  float fSqq, fDBxs, fBxs, fDBys, fBys, fDBzs, fBzs;
  float faXXLine[MAXLINE], faYYLine[MAXLINE], faZZLine[MAXLINE];
  float fXNearE;
  int istop,jstop,stop;
  float f1,f2,f3;

 
  /* Define some sizes */
  nip1 = ni + 1;
  njp1 = nj + 1;
  nkp1 = nk + 1;
  nim1 = ni - 1;
  njm1 = nj - 1;
  nkm1 = nk - 1;
  nk2 = nk/2;
  nk2m1 = nk2 - 1;
  nimin = 1;
  njmin = 0;
  

  /* Create work space */
  fpXq = malloc(sizeof(float)*nip1);
  fpYq = malloc(sizeof(float)*njp1);
  fpZq = malloc(sizeof(float)*nkp1);
  fpXlow = malloc(sizeof(float)*nip1);
  fpYlow = malloc(sizeof(float)*njp1);
  fpZlow = malloc(sizeof(float)*nkp1);
  fpXhigh = malloc(sizeof(float)*nip1);
  fpYhigh = malloc(sizeof(float)*njp1);
  fpZhigh = malloc(sizeof(float)*nkp1);

  /* Make arrays of logical coordinates as floats */
  for(i=0;i<nip1;i++) fpXq[i] = 1.0*i;
  for(j=0;j<njp1;j++) fpYq[j] = 1.0*j;
  for(k=0;k<nkp1;k++) fpZq[k] = 1.0*k;

  /* Create arrays with sides */
  for(i=0;i<ni;i++) {
   fpXlow[i] = fpXq[i];
   fpXhigh[i] = fpXq[i+1];
  }
  for(j=0;j<nj;j++) {
   fpYlow[j] = fpYq[j];
   fpYhigh[j] = fpYq[j+1];
  }
  for(k=0;k<nk;k++) {
   fpZlow[k] = fpZq[k];
   fpZhigh[k] = fpZq[k+1];
  }

  /* Initialize number of points in direction along field line */
  nd = 0;
  
  /* Set initial point */
  fX0 = fX;
  fY0 = fY;
  fZ0 = fZ;
  i00 = floor(fX);
  j00 = floor(fY);
  k00 = floor(fZ);
  i0 = i00;
  j0 = j00;
  k0 = k00;
  fpXLine[0]=fX0;
  fpYLine[0]=fY0;
  fpZLine[0]=fZ0;

  /* Initialize volume */
  
  /* Only allow line to extend half max line length in one direction */
  fDirect = 1.0;

  n = 0;
  stop = 0;
  istop = 0;
  jstop = 0;
  
  while ( (n < MAXLINE-1) && (stop != 1) ) {

    /* X Block */
    fDBx = (fpBi[i1jk(i0+1,j0,k0)]-fpBi[i1jk(i0,j0,k0)])/(fpXq[i0+1]-fpXq[i0]);
    fDBx = sign(fabs(fDBx+1.0e-10),fDBx);
    fBx0 = fpBi[i1jk(i0,j0,k0)]+(fX0-fpXq[i0])*fDBx;
    fDBx = fDirect*fDBx;
    fBx0 = fDirect*fBx0;
    fSgni = sign(1.0,fBx0);
    fBx0 = fSgni*dmax(1.0e-10,fabs(fBx0));
    fDx = 0.5*(-1.0*(fX0-fpXq[i0])*(1.0-fSgni)+(fpXq[i0+1]-fX0)*(1.0+fSgni));
    fAvB = (fabs(fpBi[i1jk(i0+1,j0,k0)])+fabs(fpBi[i1jk(i0,j0,k0)])+1.0e-8);
    fDBmag = fabs(fBx0)/fAvB;
    if (fDBmag < 1.0e-4) fBx0 = sign(1.0e-4*fAvB,fBx0);
    fCx = fDBx/fBx0;
    fCxDx = fCx*fDx;
    fDx0Bx = fDx/fBx0;
    fXLgln = sign(0.5,fabs(fCxDx)-0.1);
    if (fCxDx <= -1.0) {
      fSx = 1.0e6;
    } else {
      fSx = (0.5+fXLgln)*log(1.0+fCxDx)/fDBx+(0.5-fXLgln)*alog1(fCxDx,fDx0Bx);
      fSx = fabs(fSx);
    } 


    /* Y Block */
    fDBy = (fpBj[ij1k(i0,j0+1,k0)]-fpBj[ij1k(i0,j0,k0)])/
           (fpYq[j0+1]-fpYq[j0]);
    fDBy = sign(fabs(fDBy+1.0e-10),fDBy);
    fBy0 = fpBj[ij1k(i0,j0,k0)]+(fY0-fpYq[j0])*fDBy;
    fDBy = fDirect*fDBy;
    fBy0 = fDirect*fBy0;
    fSgnj = sign(1.0,fBy0);
    fBy0 = fSgnj*dmax(1.0e-10,fabs(fBy0));
    fDy = 0.5*(-1.0*(fY0-fpYq[j0])*(1.0-fSgnj)+(fpYq[j0+1]-fY0)*(1.0+fSgnj));
    fAvB = (fabs(fpBj[ij1k(i0,j0+1,k0)])+fabs(fpBj[ij1k(i0,j0,k0)])+1.0e-8);
    fDBmag = fabs(fBy0)/fAvB;
    if (fDBmag < 1.0e-4) fBy0 = sign(1.0e-4*fAvB,fBy0);
    fCy = fDBy/fBy0;
    fCyDy = fCy*fDy;
    fYLgln = sign(0.5,fabs(fCyDy)-1.0e-3);
    fDy0By = fDy/fBy0;
    fYLgln = sign(0.5,fabs(fCyDy)-0.1);
    if (fCyDy <= -1.0) {
      fSy = 1.0e6;
    } else {
      fSy = (0.5+fYLgln)*log(1.0+fCyDy)/fDBy+(0.5-fYLgln)*alog1(fCyDy,fDy0By);
      fSy = fabs(fSy);
    } 

    /* Z Block */
    fDBz = (fpBk[ijk(i0,j0,k0+1)]-fpBk[ijk(i0,j0,k0)])/
           (fpZq[k0+1]-fpZq[k0]);
    fDBz = sign(fabs(fDBz+1.0e-10),fDBz);
    fBz0 = fpBk[ijk(i0,j0,k0)]+(fZ0-fpZq[k0])*fDBz;
    fDBz = fDirect*fDBz;
    fBz0 = fDirect*fBz0;
    fSgnk = sign(1.0,fBz0);
    fBz0 = fSgnk*dmax(1.0e-10,fabs(fBz0));
    fDz = 0.5*(-1.0*(fZ0-fpZq[k0])*(1.0-fSgnk)+
               (fpZq[k0+1]-fZ0)*(1.0+fSgnk));
    fAvB = (fabs(fpBk[ijk(i0,j0,k0+1)])+fabs(fpBk[ijk(i0,j0,k0)])+1.0e-8);
    fDBmag = fabs(fBz0)/fAvB;
    if (fDBmag < 1.0e-4) fBz0 = sign(1.0e-4*fAvB,fBz0);
    fCz = fDBz/fBz0;
    fCzDz = fCz*fDz;
    fDz0Bz = fDz/fBz0;
    fZLgln = sign(0.5,fabs(fCzDz)-0.1);
    if (fCzDz <= -1.0) {
     fSz = 1.0e6;
    } else {
     fSz = (0.5+fZLgln)*log(1.0+fCzDz)/fDBz +
           (0.5-fZLgln)*alog1(fCzDz,fDz0Bz);
     fSz = fabs(fSz);
    }
/*
if(fX == 30.5 && fY==0.5 && fZ==57.5 && fSz < 0){
    DXMessage("0 %f %f %f",fX0,fY0,fZ0);
    DXMessage("1 %d %d %d %g %g %g %g",i0,j0,k0,fpBk[ijk(i0,j0,k0)],fpBk[ijk(i0,j0,k0+1)],fpZq[k0],fpZq[k0+1]);
    fDBz = (fpBk[ijk(i0,j0,k0+1)]-fpBk[ijk(i0,j0,k0)])/(fpZq[k0+1]-fpZq[k0]);
    DXMessage("1 %g",fDBz);
    DXMessage("2 %g",fDBz);
    fDBz = sign(fabs(fDBz+1.0e-10),fDBz);
    DXMessage("2 %g",fDBz);
    DXMessage("3 %g %f %g %g",fpBk[ijk(i0,j0,k0)],fZ0,fpZq[k0],fDBz);
    fBz0 = fpBk[ijk(i0,j0,k0)]+(fZ0-fpZq[k0])*fDBz;
    DXMessage("3 %g",fBz0);
    DXMessage("4 %g %g",fDirect,fDBz);
    fDBz = fDirect*fDBz;
    DXMessage("4 %g",fDBz);
    DXMessage("5 %g %g",fDirect,fBz0);
    fBz0 = fDirect*fBz0;
    DXMessage("5 %g",fBz0);
    DXMessage("6 %g",fBz0);
    fSgnk = sign(1.0,fBz0);
    DXMessage("6 %g",fSgnk);
    DXMessage("7 %g %g",fBz0,fSgnk);
    fBz0 = fSgnk*dmax(1.0e-10,fabs(fBz0));
    DXMessage("7 %g",fBz0);
    DXMessage("8 %g %g %g %g",fZ0,fpZq[k0],fSgnk,fpZq[k0+1]);
    fDz = 0.5*(-1.0*(fZ0-fpZq[k0])*(1.0-fSgnk)+(fpZq[k0+1]-fZ0)*(1.0+fSgnk));
    DXMessage("8 %g",fDz);
    DXMessage("9 %g %g",fpBk[ijk(i0,j0,k0+1)],fpBk[ijk(i0,j0,k0)]);
    fAvB = (fabs(fpBk[ijk(i0,j0,k0+1)])+fabs(fpBk[ijk(i0,j0,k0)])+1.0e-8);
    DXMessage("9 %g",fAvB);
    DXMessage("10 %g %g",fBz0,fAvB);
    fDBmag = fabs(fBz0)/fAvB;
    DXMessage("10 %g",fDBmag);
    DXMessage("11 %g %g %g",fDBmag,fAvB,fBz0);
    if (fDBmag < 1.0e-4) fBz0 = sign(1.0e-4*fAvB,fBz0);
    DXMessage("11 %g",fBz0);
    DXMessage("12 %g %g",fDBz,fBz0);
    fCz = fDBz/fBz0;
    DXMessage("12 %g",fCz);
    DXMessage("13 %g %g",fCz,fDz);
    fCzDz = fCz*fDz;
    DXMessage("13 %g",fCzDz);
    DXMessage("14 %g %g",fDz,fBz0);
    fDz0Bz = fDz/fBz0;
    DXMessage("14 %g",fDz0Bz);
    DXMessage("15 %g",fCzDz);
    fZLgln = sign(0.5,fabs(fCzDz)-0.1);
    DXMessage("15 %g",fZLgln);
    DXMessage("16 %g %g %g %g",fCzDz,fZLgln,fDBz,fDz0Bz);
    if (fCzDz <= -1.0) {
     fSz = 1.0e6;
    } else {
     fSz = (0.5+fZLgln)*log(1.0+fCzDz)/fDBz +
           (0.5-fZLgln)*alog1(fCzDz,fDz0Bz);
    } 
    DXMessage("16 %g-------------",fSz);
}
*/
    fS0a = dmin(fSx,fSy);
    fS0 = dmin(fS0a,fSz);
    iPl = inchng(fSgni,fSx,fS0);
    jPl = inchng(fSgnj,fSy,fS0);
    kPl = inchng(fSgnk,fSz,fS0);
    fX0q = fX0;
    fY0q = fY0;
    fZ0q = fZ0;

    for(l=1;l<4;l++) {
      fSqq = fS0*l/3.0;
      fDBxs = fDBx*fSqq;   
      fBxs = fBx0*fSqq;
      fXLgln = sign(0.5,fabs(fDBxs)-0.1);
      fDBys = fDBy*fSqq;
      fBys = fBy0*fSqq;
      fYLgln = sign(0.5,fabs(fDBys)-0.1);
      fDBzs = fDBz*fSqq;
      fBzs = fBz0*fSqq;
      fZLgln = sign(0.5,fabs(fDBzs)-0.1);
      fX0 = fX0q + (0.5-fXLgln)*exp1(fDBxs,fBxs) + 
                  (0.5+fXLgln)*(exp(fDBx*fSqq)-1.0)/(fCx+1.0e-20);
      fY0 = fY0q + (0.5-fYLgln)*exp1(fDBys,fBys) + 
                  (0.5+fYLgln)*(exp(fDBy*fSqq)-1.0)/(fCy+1.0e-20);
      fZ0 = fZ0q + (0.5-fZLgln)*exp1(fDBzs,fBzs) + 
                  (0.5+fZLgln)*(exp(fDBz*fSqq)-1.0)/(fCz+1.0e-20);

      fpXLine[n+l]=fX0;
      fpYLine[n+l]=fY0;
      fpZLine[n+l]=fZ0;
    }

    n += 3;
    i0 += iPl;
    j0 += jPl;
    k0 = ((k0+1 + nk + kPl-1) % nk);
    if (k0 == 0 && fZ0 > nk2) {
      fZ0 = dmax(0,fZ0-nk);
    }
    if (k0 == nkm1 && fZ0 < nk2) {
      fZ0 = dmin(nk,fZ0+nk);
    }
    if ((i0 > nim1) || (i0 < nimin)) istop = 1;
    if ((j0 > njm1) || (j0 < njmin)) jstop = 1;

     
    /* Flip direction if first time through loop otherwise stop trace */
    if ( ((istop == 1) || (jstop == 1))) {
       if ((fDirect < 0)) {
         //DXMessage("Completed tracing");
         stop = 1; 
       } else {
         //DXMessage("Changing direction");
         fDirect = -1.0;
         nd = n;
         fX0 = fpXLine[0];
         fY0 = fpYLine[0];
         fZ0 = fpZLine[0];
         i0=i00;
         j0=j00;
         k0=k00;
         istop = 0;
         jstop = 0;
       }
    }
 
}
  /* determine number of points in the second half of field line */
  n0 = n - nd;
  *ipLineLen = n + 1;
  /* then sort into continious line */

  for(i=0;i<n0;i++) {
    faXXLine[i] = fpXLine[n-i];
    faYYLine[i] = fpYLine[n-i];
    faZZLine[i] = fpZLine[n-i];
  }
  for(i=0;i<nd+1;i++) {
    faXXLine[i+n0]=fpXLine[i];
    faYYLine[i+n0]=fpYLine[i];
    faZZLine[i+n0]=fpZLine[i];
  }
  for(i=0;i<n+1;i++) {
    fpXLine[i]=faXXLine[i];
    fpYLine[i]=faYYLine[i];
    fpZLine[i]=faZZLine[i];
  }

  /* Finally we classify the field lines as closed (C), open from the northern
 * hemisphere (N), open from the southern hemisphere (S), or not a
 * magnetospheric field line W
 */
 fXNearE = nimin + 0.5;
 if ((fpXLine[0] < fXNearE) && (fpXLine[n] < fXNearE)) {
   cpLineType[0] = 'C';
 }
 if ((fpXLine[0] < fXNearE) && (fpXLine[n] > fXNearE)) {
   if (fpZLine[0] <= nk2) {
      cpLineType[0] = 'N';
      DXMessage("Problem with sort on N line");
   } else {
      cpLineType[0] = 'S';
   }
 }
 if ((fpXLine[0] > fXNearE) && (fpXLine[n] <= fXNearE)) {
   if (fpZLine[n] <= nk2) {
      cpLineType[0] = 'N';
   } else {
      cpLineType[0] = 'S';
      DXMessage("Problem with sort on S line");
   }
 }
 if ((fpXLine[0] > fXNearE) && (fpXLine[n] > fXNearE)) {
   cpLineType[0] ='W'; 
 } 

 /* The last major task is determine the northern and southern hemisphere entry
 * points from the arrays
 */
 if (cpLineType[0] == 'C') {
   if((fpZLine[0] > 0.0) && (fpZLine[0] <= nk2)) {
     fpXN[0] = fpXLine[0];
     fpYN[0] = fpYLine[0];
     fpZN[0] = fpZLine[0];
     fpXS[0] = fpXLine[n];
     fpYS[0] = fpYLine[n];
     fpZS[0] = fpZLine[n];
   } else {
     fpXS[0] = fpXLine[0];
     fpYS[0] = fpYLine[0];
     fpZS[0] = fpZLine[0];
     fpXN[0] = fpXLine[n];
     fpYN[0] = fpYLine[n];
     fpZN[0] = fpZLine[n];
   }
 }
 if (cpLineType[0] == 'N') {
   if (fpXLine[0] < (nimin+1.0)) {
     fpXN[0] = fpXLine[0];
     fpYN[0] = fpYLine[0];
     fpZN[0] = fpZLine[0];
   } else {
     fpXN[0] = fpXLine[n];
     fpYN[0] = fpYLine[n];
     fpZN[0] = fpZLine[n];
   }
 }
 
 if (cpLineType[0] == 'S') {
   if (fpXLine[0] < (nimin+1.0)) {
     fpXS[0] = fpXLine[0];
     fpYS[0] = fpYLine[0];
     fpZS[0] = fpZLine[0];
   } else {
     fpXS[0] = fpXLine[n];
     fpYS[0] = fpYLine[n];
     fpZS[0] = fpZLine[n];
   }
 }
  /* Free work space */
  free(fpXq);
  free(fpYq);
  free(fpZq);
  free(fpXlow);
  free(fpYlow);
  free(fpZlow);
  free(fpXhigh);
  free(fpYhigh);
  free(fpZhigh);
  return;
}
  
  


void interpline(float *fpOut, float *fpIn, 
                float *fpILine, float *fpJLine, float *fpKLine,
                int iLineLen,
                int ni, int nj, int nk)
{
  float xxq, yyq, zzq, dx0, dy0, dz0 ;
  int i,i0,j0,k0,ip,jp,kp;
  int nip1,njp1,nkp1;

  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;


  for(i=0;i<iLineLen;i++) {
    xxq = fpILine[i]-1;
    yyq = fpJLine[i]-1;
    zzq = fpKLine[i]-1;
    i0=dmin(ni-1,floor(xxq+1.00001));
    j0=dmin(nj-1,floor(yyq+1.00001));
    k0=dmin(nk-1,floor(zzq+1.00001));
    ip = i0 + 1;
    jp = j0 + 1;
    kp = k0 + 1;
    dx0 = xxq - (i0-1);
    dy0 = yyq - (j0-1);
    dz0 = zzq - (k0-1);

    fpOut[i] = (1.0-dx0)*(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(i0,j0,k0)] +
                    dx0 *(1.0-dy0)*(1.0-dz0)*fpIn[ijk1(ip,j0,k0)] +
               (1.0-dx0)*     dy0 *(1.0-dz0)*fpIn[ijk1(i0,jp,k0)] +
                    dx0 *     dy0 *(1.0-dz0)*fpIn[ijk1(ip,jp,k0)] +
               (1.0-dx0)*(1.0-dy0)*     dz0 *fpIn[ijk1(i0,j0,kp)] +
                    dx0 *(1.0-dy0)*     dz0 *fpIn[ijk1(ip,j0,kp)] +
               (1.0-dx0)*     dy0 *     dz0 *fpIn[ijk1(i0,jp,kp)] +
                    dx0 *     dy0 *     dz0 *fpIn[ijk1(ip,jp,kp)];
  }
  return;
}
    
void interpline2(float *fpOut, float *fpIn, 
                float *fpILine, float *fpJLine, float *fpKLine,
                int iLineLen,
                int ni, int nj, int nk,int flag)
{
  float xxq, yyq, zzq, dx0, dy0, dz0 ;
  float *fpXq, *fpYq, *fpZq;
  float *fpXlow, *fpXhigh, *fpYlow, *fpYhigh, *fpZlow, *fpZhigh; 
  int j,k;
  int i,i0,j0,k0,ip,jp,kp;
  int nip1,njp1,nkp1,nk2,nk2m1;
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  nk2 = nk/2;
  nk2m1= nk2-1;

  /* Create work space */
  fpXq = malloc(sizeof(float)*nip1);
  fpYq = malloc(sizeof(float)*njp1);
  fpZq = malloc(sizeof(float)*nkp1);

  for(i=0;i<iLineLen;i++) {
    xxq = fpILine[i]+0.5-1;
    yyq = fpJLine[i]+0.5-1;
    zzq = fpKLine[i]+0.5-1;
    i0=dmin(ni-2,dmax(0,xxq));
    j0=dmin(nj-2,dmax(0,yyq));
    k0=dmin(nk-1,zzq);
    dz0 = zzq - k0;
    k0 = (k0+nk2)%nk;
    ip = i0 + 1;
    jp = j0 + 1;
    kp = ((k0+1) % nk);
    if(flag)
	DXMessage("%d %d %d %d %d",k0,k0+1,nk,(k0+1)%nk,kp);
    dx0 = xxq - (i0);
    dy0 = yyq - (j0);
    fpOut[i] = (1.0-dx0)*(1.0-dy0)*(1.0-dz0)*fpIn[ijk(i0,j0,k0)] +
                    dx0 *(1.0-dy0)*(1.0-dz0)*fpIn[ijk(ip,j0,k0)] +
               (1.0-dx0)*     dy0 *(1.0-dz0)*fpIn[ijk(i0,jp,k0)] +
                    dx0 *     dy0 *(1.0-dz0)*fpIn[ijk(ip,jp,k0)] +
               (1.0-dx0)*(1.0-dy0)*     dz0 *fpIn[ijk(i0,j0,kp)] +
                    dx0 *(1.0-dy0)*     dz0 *fpIn[ijk(ip,j0,kp)] +
               (1.0-dx0)*     dy0 *     dz0 *fpIn[ijk(i0,jp,kp)] +
                    dx0 *     dy0 *     dz0 *fpIn[ijk(ip,jp,kp)];
  }
  /* Make arrays of logical coordinates as floats */
  for(i=0;i<nip1;i++) fpXq[i] = 1.0*i;
  for(j=0;j<njp1;j++) fpYq[j] = 1.0*j;
  for(k=0;k<nkp1;k++) fpZq[k] = 1.0*k;

  free(fpXq);
  free(fpYq);
  free(fpZq);
  return;
}
    
float exp1(float fXt, float fYt) {
  return fYt*(((((1.3888888e-3*fXt+8.3333333e-3)*fXt +
               4.1666666e-2)*fXt+0.1666666)*fXt + 0.5)*fXt+1.0);
}

float alog1(float fXt, float fYt) {
   return fYt*(((((-1.0*fXt*0.1666666+0.2)*fXt-0.25)*fXt+
                0.3333333)*fXt-0.5)*fXt+1.0);
}

int inchng(float fXt, float fYt, float fSo) {
  return floor(fXt)*
           (1 - dmin(1.0,floor(dmin(2.0,0.999999*fYt/(fSo+1.0e-25)))));
}

float sign(float fX, float fY) {
  float fVal;
  if (fY >= 0.0) {
   fVal = 1.0*fX;
  } else {
   fVal = -1.0*fX;
  }
  return fVal ;
}

