#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"


int GetGrid(int32 liFileId, float fNorm, int *ipNi, int *ipNj, int *ipNk, 
            float **fppGridX, float **fppGridY, float **fppGridZ) 
{
   int i,j,k, ni,nj,nk, nip1, njp1, nkp1, njp2, iAxis, iMin;
   int iNi,iNj,iNk;
   float *fpX2, *fpY2, *fpPhi, *fpSinPhi, *fpCosPhi, *fpX, *fpY, *fpZ;
   float *fpXc, *fpYc, *fpZc;
   float fAxisVal, fNormInv;
   int32 iErr, liSdsId, liIndex, liNumType, liAttr, liRank, liSize;
   int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride; 
   char caDataName[MAX_NC_NAME];
   FILE *fpOut;

   WHERE(Inside GetGrid);
   /* First we get the Array size information */
   /* Since X2 is nip1xnjp1 we have*/
   liIndex = SDnametoindex(liFileId,"grid_x2");
   liSdsId = SDselect(liFileId,liIndex);
   liAttr = SDfindattr(liSdsId,"ni");
   SDreadattr(liSdsId, liAttr, &iNi); 
   liAttr = SDfindattr(liSdsId,"nj");
   SDreadattr(liSdsId, liAttr, &iNj); 
   nip1 = iNi;
   njp1 = iNj;
   ni = nip1-1;
   nj = njp1-1;
   njp2 = njp1 + 1;
   /* Since Phi is nkp1 we have */
   liIndex = SDnametoindex(liFileId,"grid_phi");
   liSdsId = SDselect(liFileId,liIndex);
   liAttr = SDfindattr(liSdsId,"ni");
   SDreadattr(liSdsId, liAttr, &iNi); 
   nkp1 = iNi;
   nk = nkp1 -1;

   /* Now we get the XZ,Y2 and Phi reconstruction arrays */
   fpX2 = fpGetData(liFileId,"grid_x2");
   fpY2 = fpGetData(liFileId,"grid_y2");
   fpPhi = fpGetData(liFileId,"grid_phi");
#ifdef DEBUG
   printf("X2[0,0] %g X2[10,15] %g \n",fpX2[ij(0,0)],fpX2[ij1(10,15)]);
   for(i=0;i<nip1;i++) {
     printf("X2[%d,0] %g \n",i,fpX2[ij1(i,0)]);
   }
   for(i=0;i<njp1;i++) {
     printf("X2[0,%d] %g \n",i,fpX2[ij1(0,i)]);
   }
   fpOut = fopen("x2.dat","w");
   for(j=0;j<njp1;j++) {
     for(i=0;i<nip1;i++) {
        fprintf(fpOut,"%g \n",fpX2[ij1(i,j)]);
     }
   }
   fclose(fpOut);
   printf("Y2[0,0] %g Y2[10,15] %g \n",fpY2[ij(0,0)],fpY2[ij1(10,15)]);
   for(i=0;i<nip1;i++) {
     printf("Y2[%d,0] %g \n",i,fpY2[ij1(i,0)]);
   }
   for(i=0;i<njp1;i++) {
     printf("Y2[0,%d] %g \n",i,fpY2[ij1(0,i)]);
   }
#endif
   /* Calc locations of cell corners */
   fpCosPhi = malloc(sizeof(float)*nkp1);
   fpSinPhi = malloc(sizeof(float)*nkp1);
   for(i=0;i<nk;i++) {
     fpCosPhi[i]=cos(fpPhi[i]);
     fpSinPhi[i]=sin(fpPhi[i]);
   }
   fpCosPhi[nk] = fpCosPhi[0];
   fpSinPhi[nk] = fpSinPhi[0];
#ifdef DEBUG
   for(i=0;i<nkp1;i++) {
    printf("Phi[%d] %g\n",i,fpPhi[i]);
   }
   for(i=0;i<nkp1;i++) {
    printf("SinPhi[%d] %g\n",i,fpSinPhi[i]);
   }
#endif

   fpX = malloc(sizeof(float)*(nip1*njp1*nkp1));
   fpY = malloc(sizeof(float)*(nip1*njp1*nkp1));
   fpZ = malloc(sizeof(float)*(nip1*njp1*nkp1));
  
   for(k=0;k<nkp1;k++) {
     for(j=0;j<njp1;j++) {
       for(i=0;i<nip1;i++) {
         fpX[ijk1(i,j,k)]=fpX2[ij1(i,j)];
         fpY[ijk1(i,j,k)]=fpY2[ij1(i,j)]*fpCosPhi[k];
         fpZ[ijk1(i,j,k)]=fpY2[ij1(i,j)]*fpSinPhi[k];
       }
     }
  }
  WHERE(Done with cell corners); 

  /* Calc locations of Cell centers */
   fpXc = malloc(sizeof(float)*(ni*njp2*nkp1));
   fpYc = malloc(sizeof(float)*(ni*njp2*nkp1));
   fpZc = malloc(sizeof(float)*(ni*njp2*nkp1));

   fNormInv = 1.0/fNorm;
   for(k=0;k<nk;k++) {
     for(j=0;j<nj;j++) {
       for(i=0;i<ni;i++) {
         fpXc[ijkc(i,j+1,k)]=(fpX[ijk1(i,j,k)]+fpX[ijk1(i+1,j,k)] +
          fpX[ijk1(i,j+1,k)]+fpX[ijk1(i+1,j+1,k)]+fpX[ijk1(i,j,k+1)] +
          fpX[ijk1(i+1,j,k+1)]+fpX[ijk1(i,j+1,k+1)]+fpX[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
         fpYc[ijkc(i,j+1,k)]=(fpY[ijk1(i,j,k)]+fpY[ijk1(i+1,j,k)] +
          fpY[ijk1(i,j+1,k)]+fpY[ijk1(i+1,j+1,k)]+fpY[ijk1(i,j,k+1)] +
          fpY[ijk1(i+1,j,k+1)]+fpY[ijk1(i,j+1,k+1)]+fpY[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
         fpZc[ijkc(i,j+1,k)]=(fpZ[ijk1(i,j,k)]+fpZ[ijk1(i+1,j,k)] +
          fpZ[ijk1(i,j+1,k)]+fpZ[ijk1(i+1,j+1,k)]+fpZ[ijk1(i,j,k+1)] +
          fpZ[ijk1(i+1,j,k+1)]+fpZ[ijk1(i,j+1,k+1)]+fpZ[ijk1(i+1,j+1,k+1)]) *
          0.125*fNormInv;
       }
     }
   }
  WHERE(Done with cell centers);
  
  /* Calc Axes values */
  printf("njp2 %d\n",njp2);
  for(j=0;j<njp2;j++) {
    iAxis=max(1,min(nj,j));
#ifdef DEBUG
    printf("iAxis %d \n",iAxis);
#endif
    for(i=0;i<ni;i++) {
      fAxisVal=0.0;
/*
      iMin = (nj < j) ? nj: j;
      iAxis= (1 > iMin) ? 1 : iMin; 
*/
      for(k=0;k<nk;k++) {
        fAxisVal += fpXc[ijkc(i,iAxis,k)];
      }
      fAxisVal /= nk;
      for(k=0;k<nk;k++) {
        fpXc[ijkc(i,j,k)]=fAxisVal;
      }
    }
  }
  WHERE(Done with axis values);

  /* Close the k cells for the field */
  for(j=0;j<njp2;j++) {
     for(i=0;i<ni;i++)  {
       fpXc[ijkc(i,j,nk)]=fpXc[ijkc(i,j,0)];
       fpYc[ijkc(i,j,nk)]=fpYc[ijkc(i,j,0)];
       fpZc[ijkc(i,j,nk)]=fpZc[ijkc(i,j,0)];
     }
  }
  WHERE(Done with closing cells);

  /* Assign return quantties */
  fprintf(stderr,"Before setting size outs %x %x\n",ipNi,fppGridX);
  *ipNi = ni;
  printf("ipNi %x *ipNi %d ni %d\n",ipNi,*ipNi,ni);
  *ipNj = nj;
  printf("ipNj %x *ipNj %d nj %d\n",ipNj,*ipNj,nj);
  *ipNk = nk;
  printf("ipNk %x *ipNk %d nk %d\n",ipNk,*ipNk,nk);
  fprintf(stderr,"Before setting grid outs \n");
  *fppGridX = fpXc;
  *fppGridY = fpYc;
  *fppGridZ = fpZc;

  fprintf(stderr,"Before releasing temps \n");
  /* Free up temoraries */
  free(fpCosPhi);
  free(fpSinPhi);
  free(fpX);
  free(fpY);
  free(fpZ);

  WHERE(Done with Grid Calc);
  
  fprintf(stderr,"Before return from GetGrid \n");
  return 1;

} 


float *fpGetData(int32 liFileId, char *cpName)
{
   int32 iErr, liSdsId, liIndex, liNumType, liAttr, liRank, liSize;
   int32 liaDimSize[MAX_VAR_DIMS], liStart, liEdge, liStride; 
   int i, iNi, iNj,iNk;
   char caDataName[MAX_NC_NAME];
   float *fpData;

   /*Get the info about thie dataset */
   WHERE(Getting Data);
   liIndex = SDnametoindex(liFileId,cpName);
   liSdsId = SDselect(liFileId,liIndex);
   iErr = SDgetinfo(liSdsId,caDataName,&liRank,liaDimSize,&liNumType,&liAttr);
   /* Create reterival information arrays and allocate space */
   liAttr = SDfindattr(liSdsId,"ni");
   SDreadattr(liSdsId, liAttr, &iNi); 
   liSize=iNi;
   liAttr = SDfindattr(liSdsId,"nj");
   if (liAttr > 0) {
     SDreadattr(liSdsId, liAttr, &iNj); 
     liSize*=iNj;
   }
   liAttr = SDfindattr(liSdsId,"nk");
   if (liAttr > 0) {
     SDreadattr(liSdsId, liAttr, &iNk); 
     liSize*=iNk;
   }
   liStart = 0;
   liEdge = liSize;
   liStride = 1;
   fpData = malloc(liSize*sizeof(float));
   /* Get dataset */
   iErr = SDreaddata(liSdsId,&liStart,&liStride,&liEdge,fpData);
   
   /* Cleanup */
   return fpData;
}

  
int GetIntAtr(int32 liId, char *cpName) 
{
  int32 iErr, liAtrId, iVal;
  liAtrId = SDfindattr(liId,cpName);
  iErr = SDreadattr(liId,liAtrId,&iVal); 
  return iVal;
}
  
int min(int a, int b) {
  if(a < b) return a;
  else return b;
}

int max(int a, int b) {
  if(a > b) return a;
  else return b;
}
 
