#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"

int GetMHDData(int32 liFileId, int ni, int nj, int nk,float **fppMHDData)
{
  int32 iErr;
  char *cpDum, caSec[MAX_NC_NAME],caStep[10];
  float *fpVar, *fpRho, *fpVx, *fpVy, *fpVz, *fpC, *fpBx, *fpBy, *fpBz;
  int i,j,k,l;
  int nip1, njp1, nkp1, njp2, nv, iSize;
  int iAxis;
  float fAxisVal;

  /*set some useful parameters */
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  njp2 = nj+2;
  
  /* Firtt we allocate space for the MHD data */
  iSize = nip1*njp1*nkp1;
  fpRho = malloc(sizeof(float)*iSize);
  fpVx = malloc(sizeof(float)*iSize);
  fpVy = malloc(sizeof(float)*iSize);
  fpVz = malloc(sizeof(float)*iSize);
  fpC = malloc(sizeof(float)*iSize);
  fpBx = malloc(sizeof(float)*iSize);
  fpBy = malloc(sizeof(float)*iSize);
  fpBz = malloc(sizeof(float)*iSize);
  
  /*Get the data */
  cpDum = strcpy(caSec,"rho_0");
  fpRho = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vx_0");
  fpVx = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vy_0");
  fpVy = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"vz_0");
  fpVz = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"c_0");
  fpC = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bx_0");
  fpBx = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"by_0");
  fpBy = fpGetData(liFileId,caSec);
  cpDum = strcpy(caSec,"bz_0");
  fpBz = fpGetData(liFileId,caSec);

  /* Create the AVS Field data */
  iSize = ni*njp2*nkp1;
  nv = 8;
  fpVar = malloc(sizeof(float)*iSize*nv);
  
  for(k=0;k<nk;k++) {
    for(j=0;j<nj;j++) {
      for(i=0;i<ni;i++) {
        /* Convert to correct units*/
        /* Density in #/cm^3*/
        fpVar[vijk(0,i,j+1,k)]=fpRho[ijk1(i,j,k)]*4.7619e23;
        /* Vx in km/s*/
        fpVar[vijk(1,i,j+1,k)]=fpVx[ijk1(i,j,k)]*1.0e-5;
        /* Vy in km/s*/
        fpVar[vijk(2,i,j+1,k)]=fpVy[ijk1(i,j,k)]*1.0e-5;
        /* Vz in km/s*/
        fpVar[vijk(3,i,j+1,k)]=fpVz[ijk1(i,j,k)]*1.0e-5;
        /* Pressure kev/cm^3*/
        fpVar[vijk(4,i,j+1,k)]=fpRho[ijk1(i,j,k)]*fpC[ijk1(i,j,k)]*
                               fpC[ijk1(i,j,k)]*3.75e8; 
        /* Bx in nT*/
        fpVar[vijk(5,i,j+1,k)]=fpBx[ijk1(i,j,k)]*1.0e+5;
        /* By in nT*/
        fpVar[vijk(6,i,j+1,k)]=fpBy[ijk1(i,j,k)]*1.0e+5;
        /* Bz in nT*/
        fpVar[vijk(7,i,j+1,k)]=fpBz[ijk1(i,j,k)]*1.0e+5;
      }
    }
  }

  /* Calculate Axis values*/
 
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j+=njp1) {
      iAxis=max(1,min(nj,j));
      for(i=0;i<ni;i++) {
        fAxisVal=0.0;
        for(k=0;k<nk;k++) {
           fAxisVal += fpVar[vijk(l,i,iAxis,k)];
        }
        fAxisVal/=nk;
        for(k=0;k<nk;k++) {
            fpVar[vijk(l,i,j,k)]=fAxisVal;
        }
      }
    }
  }

  /* Close the k cells for the field*/
  for(l=0;l<nv;l++) {
    for(j=0;j<njp2;j++) {
      for(i=0;i<ni;i++)  {
         fpVar[vijk(l,i,j,nk)]=fpVar[vijk(l,i,j,0)];
      }
    }
  }

  /* Pass the field data back*/
  *fppMHDData = fpVar;
  
  /* Clean up*/
  free(fpRho);
  free(fpVx);
  free(fpVy);
  free(fpVz);
  free(fpC);
  free(fpBx);
  free(fpBy);
  free(fpBz);
  
  return 1;   
  
}
 int ChkStep(int iTimeStep, int iInital, int iFinal, int iInterval) {

  if( iTimeStep >= iInital && iTimeStep <= iFinal) {
    if( iTimeStep % iInterval == 0 ) {
      return iTimeStep;
    } else {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep, 
              iTimeStep - iTimeStep % iInterval);
      return iTimeStep - iTimeStep % iInterval;
    }
  } else {
    if (iTimeStep < iInital) {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep,iInital);
      return iInital;
    } else {
      printf("Requested timestep is not VALID!!\n");
      printf("Changed from %d to %d\n", iTimeStep, iFinal );
      return iFinal;
    }
  }
}
 
