#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mfhdf.h>
#include "debug.h"
#include "funcs.h"

#define jk(j,k) ((j)+njp1*(k))

int GetIonGrid(int32 liFileId, float fNorm, float *fpIonPts)
{
  int i,j,k,njp1,nk2p3,iNumPts;
  int32 iErr, liSdsId, liIndex, liNumType, liAttr, liRank, liSize;
  int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride; 
  float fAxisVal, fNormInv;
  float fRo, fTheta, fPhi;
  float *fpX2Ion, *fpY2Ion;
  char caDataName[MAX_NC_NAME];

  liIndex = SDnametoindex(liFileId,"x2ion");
  liSdsId = SDselect(liFileId,liIndex);
  liAttr = SDfindattr(liSdsId,"ni");
  SDreadattr(liSdsId, liAttr, &njp1);
  liAttr = SDfindattr(liSdsId,"nj");
  SDreadattr(liSdsId, liAttr, &nk2p3);
  fpX2Ion = fpGetData(liFileId,"x2ion");
  fpY2Ion = fpGetData(liFileId,"y2ion");


  iNumPts = njp1*nk2p3;
  /* These arrays
  X2Ion = theta*cos(phi)
  Y2Ion = theta*sin(phi)
  so we can go 3d by using by extracting the angles 
  theta = sqrt(x2ion^2+y2ion^2)
  phi = atan2(y2ion,x2ion)
  and then applying the usual transformations
  x = r0*sin(theta)*cos(phi)
  y = r0*sin(theta)*cos(phi)
  z = +/-r0*cos(theta)  sign defines hemisphere
  */

  fRo = 6.5e8/fNorm;
  for(i=0;i<iNumPts-1;i++) {
    fTheta = sqrt(fpX2Ion[i]*fpX2Ion[i]+fpY2Ion[i]*fpY2Ion[i]);
    fPhi = atan2(fpY2Ion[i],fpX2Ion[i]);
    fpIonPts[i*3] = fRo*sin(fTheta)*cos(fPhi);
    fpIonPts[i*3+1] = fRo*sin(fTheta)*sin(fPhi);
    fpIonPts[i*3+2] = fRo*cos(fTheta);
  }
     
  free(fpX2Ion);
  free(fpY2Ion);

  return 1;
}


