#include <dx/dx.h>  
#include <stdio.h>  
#include <stdlib.h>
#include <stddef.h>
#include "read_hdf_sd.h"

Error m_LoadHDFSDS(Object *in, Object *out)
{

  static SDS_structure SDS;
  int stat;        

  char *filename;      /* Input 0 */
  int sds_index = 0;   /* Input 1 */
  int verbosity  = 0; /* Input 2 */


  int i = 0;
  int j = 0;
  int k = 0;
  int M = 3; /* Returned field has a data component that is a M-vector.    */
  int N = 1; /* Returned field has a data component with N M-vectors.      */
  int Nt = 1;/* Number of elements in array returned by read_hdf_sd        */
  int return_data = 0; /* If 0, call to read_hdf_sd returns only file info */
  int verbosity2  = 0; /* If 1, put info about selected sds to stdout      */

  Array aPositionVector = NULL;  /* array for position vector    */
  Array aConnect = NULL;         /* array for connections vector */
  Array aDataVector = NULL;      /* array for data vector        */
  float *fpDataVec;

  Field fOutputField = NULL;     /* field for output             */

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  DXExtractString(in[0],  &filename);
  DXExtractInteger(in[1], &sds_index);
  DXExtractInteger(in[1], &verbosity);
  /******************************************************************/
  
  /*******************************************************************/
  /* Get data from first sds in filename with index 0  ***************/
  /*******************************************************************/
  SDS.sd_filename = (char*)calloc(MAX_LEN,sizeof(char));
  strcpy(SDS.sd_filename,filename);

  SDS.sds_index = sds_index;
  return_data   = 1;
  if (verbosity2 == 1)
    DXMessage("Attempting to call function read_hdf_sd ...\n");
  stat = read_hdf_sd(&SDS, return_data, verbosity2);
  if (stat == 0){
    DXMessage("Problem with call to read_hdf_sd.  Aborting.\n");
    return ERROR;
  }
  else {
    if (verbosity2 == 1) {
      DXMessage("... Completed call to function read_hdf_sd.\n");
      DXMessage("Name of SDS %d/%d = %s\n",
		SDS.sds_index,SDS.n_datasets,SDS.name);
    }
  }

  if (SDS.rank != 1 & SDS.rank != 2) {
    DXSetError(ERROR_INTERNAL, 
	       "LoadHDFSDS can only handle SDSs with rank of 1 or 2.\n Returned SDS has rank of %d.\n",SDS.rank);
    return ERROR;  
  }

  N = SDS.dim_sizes[0];
  M = SDS.dim_sizes[1];

  if (M > 9){
    DXSetError(ERROR_INTERNAL, 
	       "function read_hdf_sd returned a %d-vector.\nOpenDX can only handle 1 to 9-vectors.\n",M);
    return ERROR;
  }

  /* Compute the size of the returned array */
  for (i = 0; i < SDS.rank; i++)  
    Nt = Nt*SDS.dim_sizes[i];


  /***************************************************************/
  if (verbosity == 1) {
    printf("_____________________________________________________________________\n");
    printf("sds_id        = %d\n",SDS.sds_id);
    printf("sds_index     = %d of %d\n", 
	   SDS.sds_index, (SDS.n_datasets)-1);
    printf("name          = %s\n", SDS.name);
    printf("label         = %s\n", SDS.label);
    printf("unit          = %s\n", SDS.unit);
    printf("format        = %s\n", SDS.format);
    printf("coord_system  = %s\n", SDS.coord_system);
    printf("num_type      = %d\n", SDS.num_type);
    printf("n_attributes  = %d\n", SDS.n_attributes);
    printf("rank          = %d\n", SDS.rank);
    for (i = 0; i < SDS.rank; i++)
      printf ("dim_sizes[%d] = %i\n", i, SDS.dim_sizes[i]);

    printf ("\nFirst 5 points in array out of %d: \n",N);
    for (i = 0; i < 5; i++)
      printf ("%10.2f", SDS.float_array_1D[i]);
    printf("\n");
    
    printf ("\nAttributes associated with above array: \n");
    for (i = 0; i < SDS.n_attributes; i++) {
      printf("attr_name  = %s\n",SDS.attributes.attr_names[i]);
      printf("attr_value = %s\n",SDS.attributes.attr_values[i]);
    }
    printf("_____________________________________________________________________\n");
    fflush(stdout);
  }
  /*********************************************************************/


  /******************************************************************/
  /* Allocate and declare OpenDX arrays as M-vectors ****************/
  /* with N elements ************************************************/
  /******************************************************************/
  aPositionVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
  DXAddArrayData(aPositionVector, 0, N, NULL);

  aDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
  DXAddArrayData(aDataVector, 0, N, NULL);

  fpDataVec = (float*)DXGetArrayData(aDataVector);
  /******************************************************************/

  /******************************************************************/  
  /* Fill in the arrays with data ***********************************/
  /******************************************************************/  
  for (i=0; i<N; i++){
    for (j=0; j<M ; j++){
      fpDataVec[j+M*i] = SDS.float_array_1D[k];
      k = k + 1;
    }
  }
  /******************************************************************/  

  /******************************************************************/   
  /* Form output field **********************************************/
  /******************************************************************/   
  fOutputField = DXNewField();

  /* Save memory by making positions a Grid object */
  aPositionVector = DXMakeGridPositions(1,14610,1.0,1.0);
  DXSetComponentValue(fOutputField, "positions", (Object)aPositionVector);
  
  DXSetStringAttribute((Object)aDataVector,"dep", "positions");
  DXSetStringAttribute((Object)aDataVector,"Name", SDS.name);
  DXSetComponentValue(fOutputField, "data", (Object)aDataVector);
  
  aConnect = DXMakeGridConnectionsV(1,&N);
  DXSetComponentValue(fOutputField, "connections", (Object)aConnect);
  
  DXEndField(fOutputField);  /* this closes the field */
  /******************************************************************/   

  out[0] = (Object)fOutputField;

  free_SDS(&SDS,return_data);
  
  return OK;
}

