#include <stdio.h>
#include <math.h>
#include <dx/dx.h>

Error m_Ion2FtPt(Object *in, Object *out)
{
  float fX, fY, fZ, fR, fPhi, fTheta, fXln, fSign, fRmag;
  int i, iNPts;
  Array aFtPtsCoord, aFtPtsData, aIonFtPts;
  float *fpFtPtsCoord, *fpFtPtsData, *fpIonFtPtsCoord;
  Field FFtPts;

  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Needs ionospheric Foot Points");
    goto error;
  }
  if(!in[1]) {
   fRmag = 3.5;
  } else if (!DXExtractFloat(in[1],&fRmag)) {
    DXSetError(ERROR_BAD_PARAMETER, "fRmag must be a float");
    goto error;
  }

   

  /* First we get the positions to use as starting points for magnetospheric
   * field lines 
   */
  
  aIonFtPts = (Array) DXGetComponentValue((Field)in[0],"positions"); 
  fpIonFtPtsCoord = (float*) DXGetArrayData(aIonFtPts);
  DXGetArrayInfo(aIonFtPts,&iNPts,NULL,NULL,NULL,NULL);

  /* Now that we know how many foot points we can begin to create the output
   * object for the module
   */
 
   aFtPtsData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
   if(!aFtPtsData) {
     DXSetError(ERROR_NO_MEMORY,"Creation of FtPtsData failed");
     goto error;
   }
   if(!DXAddArrayData(aFtPtsData,0,iNPts,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of FtPtsData failed");
     goto error;
   }
   fpFtPtsData = (float*)DXGetArrayData(aFtPtsData);

   aFtPtsCoord = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
   if(!aFtPtsCoord) {
     DXSetError(ERROR_NO_MEMORY,"Creation of FtPtsCoord failed");
     goto error;
   }
   if(!DXAddArrayData(aFtPtsCoord,0,iNPts,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of FtPtsCoord failed");
     goto error;
   }
   fpFtPtsCoord = (float*)DXGetArrayData(aFtPtsCoord);

   for(i=0;i<iNPts;i++) {
     /* Get location in ionosphere */
     fX = fpIonFtPtsCoord[i*3];
     fY = fpIonFtPtsCoord[i*3+1];
     fZ = fpIonFtPtsCoord[i*3+2];
     fR = sqrt(fX*fX+fY*fY+fZ*fZ);
     fPhi = atan2(fY,fX);
     /* Map out to Mag Re using dipole field line equations 
      * Phi = Phio
      * R = Ro sin^2(Theta) 
      * so that for a given R1,Theata1 we can find Theta2 for a desired R2 by
      * Theata2 = asin(sqrt(R2*(X1^2+Y1^2)/R1^3))
      */
     fXln = (fX*fX+fY*fY)/(fR*fR*fR);
     fTheta = asin(sqrt(fRmag*fXln));
     /* Use sign of z to determine hemisphere */
     fSign = (fZ > 0) ? 1.0 : -1.0;
     fpFtPtsCoord[i*3] = fRmag*sin(fTheta)*cos(fPhi);
     fpFtPtsCoord[i*3+1] = fRmag*sin(fTheta)*sin(fPhi);
     fpFtPtsCoord[i*3+2] = fSign*fRmag*cos(fTheta);
     fpFtPtsData[i] = 1.0;
   }

   /* Now we complete the assignment of array objects to the output field */

   FFtPts = DXNewField();
   if(!FFtPts) {
     DXSetError(ERROR_NO_MEMORY,"Creation of FtPts field failed");
     goto error;
   }
   /* Set the Data array as the data component of the field */
   if(!DXSetStringAttribute((Object)aFtPtsData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set FtPtsData Dep");
    goto error;
   }
   if(!DXSetComponentValue(FFtPts,"data",(Object)aFtPtsData)) {
      DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of FtPts Field");
     goto error;
   }
   /* Set the Coord array as the position component of the field */
   if(!DXSetComponentValue(FFtPts,"positions",(Object)aFtPtsCoord)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of FtPts Field");
     goto error;
   }

  out[0] = (Object) FFtPts;

  return OK;
  error:
  DXDelete((Object) FFtPts);
  return ERROR;
}
