#include <stdio.h>
#include <math.h>
#include <dx/dx.h>

Error m_MapFtPts(Object *in, Object *out)
{
  float fX,fY,fZ;
  int i,iNLines, iStreamLineNumPts;
  Array aIonFtPtsCoord, aIonFtPtsData, aStreamLinePts;
  float *fpIonFtPtsCoord, *fpIonFtPtsData, *fpStreamLinePts;
  float fR, fPhi, fTheta, fXln, fSign;
  Field FIonFtPts, FStreamLine;

  if (!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Needs magnetic field lines");
    goto error;
  }

  DXMessage("Inside MapFtPts");
  /* A set of field lines is a group in DX*/
  DXGetMemberCount((Group)in[0],&iNLines);
  /* Now that we know the number of lines we set up the Field to contain these*/
  /* Ionospheric footpoints*/
  aIonFtPtsData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  if(!aIonFtPtsData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of IonFtPtsData failed");
    goto error;
  }
  if(!DXAddArrayData(aIonFtPtsData,0,iNLines,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of IonFtPtsData failed");
     goto error;
  } 
  fpIonFtPtsData = (float*)DXGetArrayData(aIonFtPtsData);
  aIonFtPtsCoord = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aIonFtPtsCoord) {
    DXSetError(ERROR_NO_MEMORY,"Creation of IonFtPtsCoord failed");
    goto error;
  }
  if(!DXAddArrayData(aIonFtPtsCoord,0,iNLines,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of IonFtPtsCoord failed");
     goto error;
  } 
  fpIonFtPtsCoord = (float*)DXGetArrayData(aIonFtPtsCoord);
  /* Now we loop over the input streamlines and extract the endpoints to the*/
  /* output field*/
  for(i=0;i<iNLines;i++) {
     FStreamLine = (Field) DXGetEnumeratedMember((Group)in[0],i,NULL);
     aStreamLinePts = (Array) DXGetComponentValue((Field)FStreamLine,
		      "positions");
     fpStreamLinePts = (float*)DXGetArrayData(aStreamLinePts);
     DXGetArrayInfo(aStreamLinePts,&iStreamLineNumPts,NULL,NULL,NULL,NULL);
     /* Last point is at inner edge of simulation if closed*/
     fX = fpStreamLinePts[(iStreamLineNumPts-1)*3];
     fY = fpStreamLinePts[(iStreamLineNumPts-1)*3+1];
     fZ = fpStreamLinePts[(iStreamLineNumPts-1)*3+2];
     /* Map along dipole to ionosphere*/
     fR = sqrt(fX*fX+fY*fY+fZ*fZ);
     fPhi = atan2(fY,fX);
     fXln = fR*fR*fR/(fY*fY+fX*fX+1.0e-20);
     fTheta = asin(sqrt(1.0/fXln));
     /* Use sign of z to determine hemisphere*/
     fSign = (fZ > 0) ? 1.0 : -1.0;
     fpIonFtPtsCoord[i*3] = sin(fTheta)*cos(fPhi);
     fpIonFtPtsCoord[i*3+1] = sin(fTheta)*sin(fPhi);
     fpIonFtPtsCoord[i*3+2] = fSign*cos(fTheta);
     fpIonFtPtsData[i] = 1.0;
  }

  /* Now we complete the assigment of Array objects to the output Field*/
  FIonFtPts = DXNewField();
  if(!FIonFtPts) {
   DXSetError(ERROR_NO_MEMORY,"Creation of FIonFtPts field failed");
   goto error;
  }

  if(!DXSetStringAttribute((Object)aIonFtPtsData,"dep","positions")) {
    DXSetError(ERROR_INTERNAL,"Couldn't set IonFtPtsData Dep");
    goto error;
   }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(FIonFtPts,"data",(Object)aIonFtPtsData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of IonFtPts Field");
    goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(FIonFtPts,"positions",(Object)aIonFtPtsCoord)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of IonFtPts Field");
     goto error;
  }

  out[0] = (Object) FIonFtPts;

  return OK;
  error:
  DXDelete((Object) FIonFtPts);
  return ERROR;
}
