#ifdef FDRIVER
      Program driver
      dimension solution(200,501,2)
      character*4 profile,algorithm
      logical switch,diffuse
      namelist /go/ nbox,nstep,width,eps,profile,switch,
     $     norder,morder,diffuse,algorithm,switchParam,
     $     diffParam
c
      open(unit=12,file='go.dat',form='formatted')
      read (12,nml=go)
      call advect(solution,nbox,nstep,width,eps,profile,switch,
     $     norder, morder, diffuse,algorithm,switchParam,diffParam)
c
      end
#endif

      Subroutine advect(solution,position,
     $     nbox, nstep, width, eps, profile,switch,
     $     norder,morder, diffuse, algorithm,switchParam,
     $     diffParam)
c
      dimension solution(2,nbox,0:nstep),position(2,nbox,0:nstep)
      logical switch,diffuse
      character*4 profile,algorithm,type
      dimension v(200,0:500,2)
      dimension v0(-20:220),v1(-20:220),v2(-20:220)
      dimension vprofile(-20:220)
      dimension vnm1(200)
      dimension vnp1(200)
      dimension f(-20:220),df(-20:220)
      dimension flft(-20:220)
      dimension frt(-20:220)
      dimension fcoef(20),dcoef(20)
c
c   set up some constants
c
      write (15,*) 'diffuse',diffuse
      write (15,*) 'nbox,etc', nbox,nstep,width,eps,profile,
     $     switch,norder
      write (15,*)'morder,etc', morder,diffuse,algorithm,
     $     switchParam,diffParam
      write(15,*) 'end of block'

      if (diffuse) then
         adiff = diffParam
      else
         adiff = 0.
      endif
      SWparam = 2.*(1-switchParam)/switchParam
      pi = 4.*atan(1.)
      call derco(fcoef,norder)
      call difco(dcoef,morder)
      n2 = norder/2
      m2 = morder/2
c
c  create initial profile
c
      if ( profile .eq. 'sqwv' ) then
         do i=1,nbox
            v(i,0,1) = 0.0
         enddo
         istart = 21 
         istop = 20.0+width
         do i=istart,istop
            v(i,0,1) = 1.0
         enddo
      elseif (profile .eq. 'sine' ) then
         wave0 = 2.*pi/float(nbox)
         iwave = width
         wave1 = iwave*wave0
         do i=1,nbox
            v(i,0,1) = cos(float(i-1)*wave1)
         enddo
      elseif (profile .eq. 'gaus' ) then
         awidth = 0.5*width
         sigma = awidth*sqrt(alog(2.0))
         xmid = nbox/2
         do i=1,nbox
            v(i,0,1) = exp(-((float(i)-xmid)/sigma)**2)
         enddo
      elseif (profile .eq. 'half' ) then
         awidth = 0.5*width
         xmid = nbox/2
         do i=1,nbox
            v(i,0,1) = sqrt( max(0.,1.-((float(i)-xmid)/awidth)**2))
         enddo
      else 
         do i=1,nbox
            v(i,0,1) = 0.0
         enddo
         istart = 21 
         istop = 40
         do i=istart,istop
            v(i,0,1) = 1.0
         enddo
      endif
      do i=1,nbox
         v(i,0,2) = v(i,0,1)
         vprofile(i) = v(i,0,1)
      enddo
      call bc(vprofile,nbox)
c
c
      do i=1,nbox
         position(1,i,0) = float(i)
         position(2,i,0) = 0.0
      enddo
c
c load solution temporaries
c
c   going to use a three time-level system:
c        v0 is base level for time step
c        v1 is the mid-point for step
c        v2 is the time advanced value
c
      do i =-20,220
         v0(i) = vprofile(i)
         v1(i) = vprofile(i)
         v2(i) = vprofile(i)
      enddo

c
c     main loop
c
      do ll=1,nstep
         time = float(ll)*eps
         IF ( algorithm .eq. 'Leap') time = 0.5*float(ll)*eps
         dt = eps
c
c
c  intermediate time stepping
c
c
c   Donor Cell
         if ( algorithm .eq. 'DonC' ) then
           do i=1,nbox+1
            f(i) = v0(i-1)
           enddo
c   Lax-Wendroff
         elseif ( algorithm .eq. 'LWen'  ) then
         do i=1,nbox+1
           f(i) = 0.5*((v0(i-1)+v0(i))+dt*(v0(i-1)-v0(i)))
         enddo
c   Fromm (or MUSCL)
         elseif (algorithm .eq. 'From') then
           do i=0,nbox+1
              df(i) = v0(i+1)-v0(i-1)
           enddo
           if (switch) then  ! this gives the MUSCL algorithm
              do i=1,nbox+1
                 peak = (v0(i)-v0(i-1))*(v0(i+1)-v0(i))
                 if ( peak .le. 0. ) then
                    df(i) = 0.
                 else
                    s = sign(1.,df(i))
                    df(i) = s*min(abs(df(i)),2*abs(v0(i)-v0(i-1)))
                 endif
              enddo
           endif
           do i=1,nbox+1
           f(i) = v0(i-1) + 0.5*(1.-eps)*df(i-1)
           enddo
c
c  Centered Differences Using Leapfrog or AdamsBashforth
c
         elseif ( algorithm .eq. 'Leap' .or.
     $        algorithm .eq. 'AdBa') then
         call high(v1,f,fcoef,nbox,n2)
         call dhigh(v0,df,dcoef,nbox,m2)
         do i=1,nbox+1
            f(i) = f(i) + adiff*df(i)
         enddo
c
         if (switch) then
            call clip(f,v0,nbox)
            call PDMswitch(f,v0,flft,frt,nbox,SWparam)
            do i=1,nbox+1
               f(i) = flft(i)
            enddo
         endif
        endif
c
c   time advancement
c
         do i=1,nbox
            v(i,ll,1) = v0(i) + dt*(f(i)-f(i+1))
         enddo
         do i=1,nbox
            v2(i) = v(i,ll,1)
         enddo
         call bc(v2,nbox)
c
c   shuffle arrays to get proper time sequence
c
         if ( algorithm .eq. 'LWen' .or.
     $        algorithm .eq. 'DonC' .or.
     $        algorithm .eq. 'From') then
         do i=1,nbox
            v0(i) = v2(i)
         enddo
         elseif ( algorithm .eq. 'Leap' ) then
         do i=1,nbox
           v0(i) = v1(i)
           v1(i) = v2(i)
         enddo
         elseif ( algorithm .eq. 'AdBa' ) then
           do i = 1,nbox
              v1(i) = 1.5*v2(i) - 0.5*v0(i)
              v0(i) = v2(i)
           enddo
         endif
c
c
         call bc(v0,nbox)
         call bc(v1,nbox)
         call bc(v2,nbox)
c
c
c  analytic solution
c
         i0 = time
         delta = time -i0
         ishift = mod(i0-1,nbox)+1
         do i =1,nbox
            ilow = mod(i-ishift-2,nbox)+1
            ihigh = mod(i-ishift-1+nbox,nbox)+1
            v(i,ll,2) = vprofile(ihigh)*(1.-delta) +
     $           vprofile(ilow)*delta
         enddo
c
      do i=1,nbox
         position(1,i,ll) = float(i)
         position(2,i,ll) = time
      enddo
c
c
      enddo
c
c
      do kk=1,2
      do j=0,nstep
         do i=1,nbox
            solution(kk,i,j) = v(i,j,kk)
         enddo
      enddo
      enddo
      write (15,*) 'solution', (solution(1,i,100),i=1,100)
      write (15,*) 'analytic', (solution(1,i,100),i=1,100)
c
c
      return
      end


      subroutine bc(v,nbox)
      dimension v(-20:nbox+20)
c
      do i=1,20
         v(-20+i) = v(nbox-20+i)
         v(nbox+i) = v(i)
      enddo
c
      return
      end
      subroutine clip(fhf,f0,nbox)
C
      dimension fhf(-20:nbox+20),f0(-20:nbox+20)
C
        do  i=1,nbox
           fhf(i) = amax1( amin1(f0(i-1),f0(i)),
     $          amin1( fhf(i), amax1(f0(i-1),f0(i))))
        enddo
C
        return
        end
      SUBROUTINE PDMswitch(F,F0,FLFT,FRT,nbox,B)
C
      character*4 type
      dimension f(-20:nbox+20)
      dimension f0(-20:nbox+20)
      dimension frt(-20:nbox+20)
      dimension flft(-20:nbox+20)
      dimension df(-20:220)
      dimension s(-20:220)
      dimension q(-20:220)
      dimension dflft(-20:220)
      dimension dfrt(-20:220)
C
C
C
      pdmb = 0.25*b
      DO I = 0,nbox+2
      DF(I) = PDMB*(F0(I)-F0(I-1))
      S(I) = SIGN(1.,DF(I))
      DF(I) = ABS(DF(I))
      enddo
C
      DO I = 0,nbox+1
         Q(I) = ABS(S(I) +S(I+1))
      enddo
C
      DO I = 1,nbox+1
      DFLFT(I) = F(I) - F0(I-1)
      DFRT(I) = F0(I) - F(I)
      FLFT(I) = F(I) - S(I)*AMAX1(0.,
     $     ABS(DFLFT(I))- Q(I-1)*DF(I-1))
      FRT(I) = F(I) + S(I)*AMAX1(0.,
     $     ABS(DFRT(I))-Q(I)*DF(I+1))
      enddo
C
      RETURN
      END
      SUBROUTINE DERCO(FCOEF,NORDER)
      DIMENSION BCOEF(100),DCOEF(100)
      DIMENSION FCOEF(NORDER)
      COMMON /ASCRACH/ A(200),B(200),C(200),F(200)
C
      N = NORDER
      N2 = N/2
      NB2=N2
C
      A(1) = 1.
      DO 200 I = 2,N2
  200 A(I) = A(I-1)*FLOAT(N+2-I)/FLOAT(I-1)
C
      DO 201 I = 1,N2
  201 B(I) = A(N2+1-I)
C
      DO 202 I = 1,N2,2
  202 B(I) = B(I)/FLOAT(I)
C
      DO 203 I = 2,N2,2
  203 B(I) = - B(I)/FLOAT(I)
C
       WRITE (6,101)(A(I),I=1,N2)
  101 FORMAT (20(1X,5(1PE14.4)/))
      WRITE (6,101) (B(I),I=1,N2)
      C(1) = 0.
      DO 300 I = 1,N2
  300 C(I+1) = B(I)
C
      F(1) = B(N2)
      DO 400 I = 1,N2
  400 F(I+1) = C(N2+1-I) + F(I)
C
      N2P1 = N2+1
      WRITE(6,101) (C(I),I=1,N2P1)
      WRITE (6,101) (F(I),I=1,N2P1)
      SUM = 0.
      DO 401 I = 1,N2
  401 SUM = SUM + F(I)
      SUM = SUM*2.
C
      DO 410 I = 1,N2
  410 F(I) = F(I)/SUM
C
      DO 500 I = 1,N2
      FCOEF(I) = F(I)
      BCOEF(I) = F(I)
      DCOEF(I) = B(N2+1-I)/SUM
  500 FCOEF(N+1-I) = F(I)
      write (6,*) 'flux coefficients'
      write (6,911) (fcoef(i),i=1,n2)
      write (6,*) 'dflux coefficients'
      write (6,911) (dcoef(i),i=1,n2)
 911  format(5(1pe12.5))
C
      RETURN
      END
      SUBROUTINE DIFCO(DCOEF,NORDER)
      DIMENSION BCOEF(100),DCOEF(100)
      DIMENSION FCOEF(NORDER)
      COMMON /ASCRACH/ A(200),B(200),C(200),F(200)
C
      N = NORDER
      N2 = N/2
      NB2=N2
C
      A(1) = 1.
      DO 200 I = 2,N2
  200 A(I) = A(I-1)*FLOAT(N+2-I)/FLOAT(I-1)
C
      DO 201 I = 1,N2
  201 B(I) = A(N2+1-I)
C
      DO 202 I = 1,N2,2
  202 B(I) = B(I)/FLOAT(I)
C
      DO 203 I = 2,N2,2
  203 B(I) = - B(I)/FLOAT(I)
C
       WRITE (6,101)(A(I),I=1,N2)
  101 FORMAT (20(1X,5(1PE14.4)/))
      WRITE (6,101) (B(I),I=1,N2)
      C(1) = 0.
      DO 300 I = 1,N2
  300 C(I+1) = B(I)
C
      F(1) = B(N2)
      DO 400 I = 1,N2
  400 F(I+1) = C(N2+1-I) + F(I)
C
      N2P1 = N2+1
      WRITE(6,101) (C(I),I=1,N2P1)
      WRITE (6,101) (F(I),I=1,N2P1)
      SUM = 0.
      DO 401 I = 1,N2
  401 SUM = SUM + F(I)
      SUM = SUM*2.
C
      DO 410 I = 1,N2
  410 F(I) = F(I)/SUM
C
      DO 500 I = 1,N2
      FCOEF(I) = F(I)
      BCOEF(I) = F(I)
      DCOEF(I) = B(N2+1-I)/SUM
  500 FCOEF(N+1-I) = F(I)
      write (6,*) 'flux coefficients'
      write (6,911) (fcoef(i),i=1,n2)
      write (6,*) 'dflux coefficients'
      write (6,911) (dcoef(i),i=1,n2)
 911  format(5(1pe12.5))
C
      RETURN
      END
      SUBROUTINE HIGH(F,FHF,fcoef,nbox,ord2)
C
      integer ord2,order
      DIMENSION F(-20:220),FHF(-20:220),fcoef(20)
      order = 2*ord2
c
        do i=1,nbox+1
           fhf(i) = 0.
        enddo
        DO  K=1,ORD2
        DO  I=1,nbox+1
        FHF(I) = FHF(I) + FCOEF(K)*
     $          ( F(I+K-ORD2-1) + F(I+ORD2-K) )
        enddo
        enddo
C
        RETURN
        END
      SUBROUTINE dHIGH(F,FHF,dfcoef,nbox,ord2)
C
      integer ord2,order
      DIMENSION F(-20:220),FHF(-20:220),dfcoef(20)
      order =2*ord2
c
        do i=1,nbox+1
           fhf(i) = 0.
        enddo
        DO  K=1,ORD2
        DO  I=1,nbox+1
        FHF(I) = FHF(I) + dFCOEF(K)*
     $          ( F(I+K-ORD2-1) - F(I+ORD2-K) )
        enddo
        enddo
C
        RETURN
        END
         


