#include <dx/dx.h>

/* Function Prototypes for Numerics Module */


Error
m_Numerics(Object *in, Object *out)
{
   /***********************/
   /*  Declare variables  */
   /***********************/
   int  Nbox;
   int  Nstep;
   float  Eps;
   float  Scale;
   int Ptype;
   int Switch;
   float SwitchParam;
   int Algorithm;
   int Order;
   int Dorder, Diffuse;
   float DiffParam;

   Array a1=NULL;
   float *fpa1;
   Array a1p=NULL;
   float *fpa1p;
   Array a1c=NULL;
   Field f1=NULL;

   char *AProfiles[4];
   char *AAlgorithm[6];


   //int advect_(*int,*int,*float,*float,*float,*float);
   int iErr;
   int NUMVARS;
   int SIZEOFARRAY;
   int NUMDIMS;

   AProfiles[0] = "sqwv";
   AProfiles[1] = "sine";
   AProfiles[2] = "gaus";
   AProfiles[3] = "half";

   AAlgorithm[0] = "DonC";
   AAlgorithm[1] = "LWen";
   AAlgorithm[2] = "From";
   AAlgorithm[3] = "Leap";
   AAlgorithm[4] = "AdBa";
   AAlgorithm[5] = "RunK";

   Diffuse = 0;
   Dorder = 0;
   DiffParam=0.0;



   /***********************/
   /*  Get input values   */
   /***********************/
   /* Get Int1's value */
   if (!in[0]) {
      DXSetError(ERROR_BAD_PARAMETER,"Missing Nbox");
      goto error;
   }
   else if (!DXExtractInteger(in[0], &Nbox)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Nbox parameter must be a Integer");
     goto error;
   }

   /* Get Nstep's value */
   if (!in[1]) {
      DXSetError(ERROR_BAD_PARAMETER,"Missing Nstep");
      goto error;
   }
   else if (!DXExtractInteger(in[1], &Nstep)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Nstep parameter must be a Integer");
     goto error;
   }

   /* Get Scal1's value */
   if (!in[2]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing scal1 parameter");
	goto error; 
   }
   else if (!DXExtractFloat(in[2], &Eps)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Eps must be a float");
     goto error;
   }

   /* Get Scales's value */
   if (!in[3]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing scal2 parameter");
	goto error;
   }
   else if (!DXExtractFloat(in[3], &Scale)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Scale must be a float");
     goto error;
   }

   /* Get Scales's value */
   if (!in[4]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing Ptype parameter");
	goto error;
   }
   else if (!DXExtractInteger(in[4], &Ptype)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Ptype must be an integer");
     goto error;
   }

   /* Get Switch's value */
   if (!in[5]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing Switch parameter");
	goto error;
   }
   else if (!DXExtractInteger(in[5], &Switch)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Switch must be an integer");
     goto error;
   }
   /* Get SwitchParam's value */
   if (!in[6]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing SwitchParam parameter");
	goto error;
   }
   else if (!DXExtractFloat(in[6], &SwitchParam)) {   
     DXSetError(ERROR_BAD_PARAMETER, "SwitchParam must be a float");
     goto error;
   }
   /* Get Algorithm's value */
   if (!in[7]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing Algorithm parameter");
	goto error;
   }
   else if (!DXExtractInteger(in[7], &Algorithm)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Algorithm must be an integer");
     goto error;
   }
   /* Get Order's value */
   if (!in[8]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing order parameter");
	goto error;
   }
   else if (!DXExtractInteger(in[8], &Order)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Order must be an integer");
     goto error;
   }


   /***********************/
   /* Function's Body     */
   /***********************/
   
   NUMVARS = 2;
   SIZEOFARRAY = NUMVARS*Nbox*(Nstep+1);
   NUMDIMS=2;
   /* Create data array */
   a1 = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,NUMVARS);
   if (!a1) goto error;
   /* allocate space in the data array */
   if (!DXAddArrayData(a1,0,SIZEOFARRAY,NULL)) goto error;
   /* get a pointer to the data array */
   fpa1 = (float *)DXGetArrayData(a1);
   printf("after DXGetArrayData\n"); 


   /* Create points array */
   a1p = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,NUMDIMS);
   if (!a1p) goto error;
   /* allocate space in the data array */
   if (!DXAddArrayData(a1p,0,SIZEOFARRAY,NULL)) goto error;
   /* get a pointer to the data array */
   fpa1p = (float *)DXGetArrayData(a1p);

   /* Create Connections */
   a1c = DXMakeGridConnections(2,Nstep,Nbox);

   printf("after DXGetArrayData\n");
   /* Call Numerics Subroutine */
   iErr = advect_(fpa1,fpa1p,&Nbox,&Nstep,&Scale,&Eps,AProfiles[Ptype],
		  &Switch,&Order,&Dorder,&Diffuse,AAlgorithm[Algorithm],&SwitchParam,&DiffParam);
     printf("after advect\n"); 
     /* Create Output Field */
     f1 = DXNewField();
     if (!f1) goto error;
     /* set the dependency of the data to be on positions */
     if (!DXSetStringAttribute((Object)a1, "dep", "positions"))
        goto error;
     printf("after positions\n");
     /* set the data array as the data component of f */
     if (!DXSetComponentValue(f1, "data", (Object)a1))
        goto error;
     printf("after positions\n");
     /* Load position data */      
     if (!DXSetComponentValue(f1, "positions", (Object)a1p))
        goto error;
     if (!DXSetComponentValue(f1, "connections", (Object)a1c))
        goto error;
     printf("after positions\n"); 
   out[0] = (Object)f1;
   return OK;
 error:
   DXDelete((Object) a1);
   DXDelete((Object) a1p);
   DXDelete((Object) f1);
   return ERROR;
   
}
