#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* --------------------------------------------------------------------------
 * readraeder
 *
 * This modules reads in the compressed ascii data files created as ouput for
 * Jimmy Raeder's MHD code.
 *
 * INPUTS
 *   filename: Name of file with data
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 *
 */

#define cijk(c,i,j,k) ((c)+3*(i+iNx*((j)+iNy*(k))))
Error m_ReadRaeder(Object *in, Object *out) 

{

  /* Varible Declarations */

  /* Inputs */
  char *cpMHDName = NULL; 	/* Ptr to MHD Data file name */
  
  /* Outputs */

  Array aGridX = NULL;  	/* Array for X locations */
  float *fpGridX;		/* Ptr to X location data */
  Array aGridY = NULL;  	/* Array for Y locations */
  float *fpGridY;		/* Ptr to Y location data */
  Array aGridZ = NULL;  	/* Array for Z locations */
  float *fpGridZ;		/* Ptr to Z location data */
  Array aGrid = NULL;   	/* Array for complete Grid  locations */
  float *fpGrid;		/* Ptr to complete Grid  location data */
  Array aMHDData = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Ptr to MHD data */ 
  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  Field fMHD = NULL;		/* MHD data Field */

  /* Other varibles */
  int iNx, iNy, iNz, iNumPts, iaMHDDim[3];
  int i,j,k, iLen;
  FILE *FpMHDFile;

  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&cpMHDName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }

  FpMHDFile = fopen(cpMHDName,"r");
  if(NULL==FpMHDFile){
    DXSetError(ERROR_BAD_PARAMETER,"%s does not exist",cpMHDName);
    fclose(FpMHDFile);
    goto error;
  }
  fclose(FpMHDFile);
  
  /* First we figure out the size of the arrays in the data file */
  iLen = strlen(cpMHDName);
  getsizes_(cpMHDName,&iLen,&iNx,&iNy,&iNz);
  iNumPts = iNx*iNy*iNz;
  iaMHDDim[0] = iNz;
  iaMHDDim[1] = iNy;
  iaMHDDim[2] = iNx;

  /* Now that we know how big things are we allocate space for the grid arrays
   * and then retrive get locations from the grid data file
   */

   aGridX = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
   if(!aGridX) {
     DXSetError(ERROR_NO_MEMORY,"Creation of Grid X array failed");
     goto error;
   }
   if(!DXAddArrayData(aGridX,0,iNx,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of Grid X data array failed");
     goto error;
   }
   fpGridX = (float *)DXGetArrayData(aGridX);

   aGridY = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
   if(!aGridY) {
     DXSetError(ERROR_NO_MEMORY,"Creation of Grid Y array failed");
     goto error;
   }
   if(!DXAddArrayData(aGridY,0,iNy,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of Grid Y data array failed");
     goto error;
   }
   fpGridY = (float *)DXGetArrayData(aGridY);

   aGridZ = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
   if(!aGridZ) {
     DXSetError(ERROR_NO_MEMORY,"Creation of Grid Z array failed");
     goto error;
   }
   if(!DXAddArrayData(aGridZ,0,iNz,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of Grid Z data array failed");
     goto error;
   }
   fpGridZ = (float *)DXGetArrayData(aGridZ);

   aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
   if(!aGridZ) {
     DXSetError(ERROR_NO_MEMORY,"Creation of Grid array failed");
     goto error;
   }
   if(!DXAddArrayData(aGrid,0,iNumPts,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of Grid data array failed");
     goto error;
   }
   fpGrid = (float *)DXGetArrayData(aGrid);
   /* Now that we have space lets get the grid data */

   getgrid_(cpMHDName,&iLen,&iNx,&iNy,&iNz,fpGridX,fpGridY,fpGridZ);

   for(k=0;k<iNz;k++) {
     for(j=0;j<iNy;j++) {
       for(i=0;i<iNx;i++) {
            fpGrid[cijk(0,i,j,k)] = fpGridX[i];
            fpGrid[cijk(1,i,j,k)] = fpGridY[j];
            fpGrid[cijk(2,i,j,k)] = fpGridZ[k];
       }
     }
   }


   /* Now we allocate space for the MHD data */
   
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,14);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);


  /* Then we get the MHD data */

  getmhddata_(cpMHDName,&iLen,&iNx,&iNy,&iNz,fpMHDData);

//  for(i=0;i<iNumPts;i++) {
//   fscanf(FpMHDFile,"%f",fpMHDData+i);
//  }


  /* Now that we have all the data we can create the output field */
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aMHDConnect = DXMakeGridConnectionsV(3,iaMHDDim);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }


  /* Output the field */
  out[0] = (Object)fMHD;

  /* Clean up temporaries */
   DXDelete((Object) aGridX);
   DXDelete((Object) aGridY);
   DXDelete((Object) aGridZ);
   

  return OK;

  error:
    DXDelete((Object) fMHD);
    DXDelete((Object) aMHDData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aGridX);
    DXDelete((Object) aGridY);
    DXDelete((Object) aGridZ);
    DXDelete((Object) aMHDConnect);
    return ERROR;
}

