/* R.S. Weigel, 10/25/2004                            */
/* This is an example code that takes and input field */
/* and multiplies the data component by 2.0           */

#include <dx/dx.h>  
#include "mfhdf.h"
#include <stdio.h>  
#include <stdlib.h>
#include <stddef.h>

float *fpGetData(int32 liFileId, char *cpName);
/* cijk - indexing macro for DX Arrays */
#define cijk(c,i,j,k)  ((c)+iNumVars*((i)+nx*((j)+ny*(k))))
/* ijk - indexing macro for DX Arrays */
#define ijk(i,j,k)  ((i)+nx*((j)+ny*(k)))

Error m_ReadRCM(Object *in, Object *out)
{

  /* Input Declarations */
  char *cpFileName=NULL;

  /* Output Declarations */
  Field fOutField = NULL;
  Array aOutGrid = NULL;
  Array aOutData = NULL;
  Array aOutConnect = NULL;
  float *fpOutGrid;
  float *fpOutData;
  float *fpOutConnect;

  /* Other Variables */
  int i,j,k,iNumVars;
  int nx,ny,nz,ntot;
  char caLabel[MAX_NC_NAME];
  int32 liFileId, liIndex,liSdsId, liErr; /* HDF Access Vars */
  int32 liAttrId, liStat, liNumType, liSize, liRank, liCount;
  int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride;
  int32 liSecId;
  FILE *fptr;

  float *fpXrect, *fpYrect, *fpZrect;
  float *fpRho, *fpPress, *fpBx, *fpBy, *fpBz;
  int *ipMask;
  

  /******************************************************************/
  /* Get inputs                                                     */ 
  /******************************************************************/
  if (!in[0]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing filename");
      goto error;
   } else if (!DXExtractString(in[0], &cpFileName)) {
    DXSetError(ERROR_BAD_PARAMETER, "filename must be a string");
    goto error;
  }

  /******************************************************************/

   fptr = fopen(cpFileName,"r");
   if(fptr == NULL) {
     DXSetError(ERROR_BAD_PARAMETER,"FILE %s does not exist",cpFileName);
     goto error;
   }
   if (DFishdf(cpFileName) !=0) {
     DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpFileName);
    goto error;
   }


  /* First step is to open file and determine grid size */
   if((liFileId = SDstart(cpFileName,
                        DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;

  }
  /* Now we read the object sizes from the global attributes */
  liAttrId = SDfindattr(liFileId,"nx");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&nx);
  liAttrId = SDfindattr(liFileId,"ny");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&ny);
  liAttrId = SDfindattr(liFileId,"nz");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&nz);
  ntot=nx*ny*nz;
 
  DXMessage("Size %d %d %d",nx,ny,nz);

  /* next we get the grid information */
  fpXrect = fpGetData(liFileId,"xrect");
  fpYrect = fpGetData(liFileId,"yrect");
  fpZrect = fpGetData(liFileId,"zrect");

  iNumVars=3;
  aOutGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aOutGrid,0,ntot,NULL);
  fpOutGrid = (float*)DXGetArrayData(aOutGrid);
  for(k=0;k<nz;k++) {
    for(j=0;j<ny;j++)  {
      for(i=0;i<nx;i++)  {
        fpOutGrid[cijk(0,i,j,k)]=fpXrect[i];
        fpOutGrid[cijk(1,i,j,k)]=fpYrect[j];
        fpOutGrid[cijk(2,i,j,k)]=fpZrect[k];
      }
    }
  }

  /* then we get the variables */
  fpRho = fpGetData(liFileId,"rho");
  fpPress = fpGetData(liFileId,"press");
  fpBx = fpGetData(liFileId,"bx");
  fpBy = fpGetData(liFileId,"by");
  fpBz = fpGetData(liFileId,"bz");
  
  iNumVars=5;
  aOutData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aOutData,0,ntot,NULL);
  fpOutData = (float*)DXGetArrayData(aOutData);
  for(k=0;k<nz;k++) {
    for(j=0;j<ny;j++)  {
      for(i=0;i<nx;i++)  {
        fpOutData[cijk(0,i,j,k)]=fpRho[ijk(i,j,k)];
        fpOutData[cijk(1,i,j,k)]=fpPress[ijk(i,j,k)];
        fpOutData[cijk(2,i,j,k)]=fpBx[ijk(i,j,k)];
        fpOutData[cijk(3,i,j,k)]=fpBy[ijk(i,j,k)];
        fpOutData[cijk(4,i,j,k)]=fpBz[ijk(i,j,k)];
      }
    }
  }
 
  fOutField=DXNewField();
  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aOutData,"dep","positions")) {
    DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
    goto error;
  }

  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fOutField,"data",(Object)aOutData)) {
    DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
    goto error;
  }

  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fOutField,"positions",(Object)aOutGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }
  aOutConnect = DXMakeGridConnections(3,nz,ny,nx);

  if (!aOutConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fOutField,"connections",(Object)aOutConnect)) {
      DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
      goto error;
  }
  
  if(!DXEndField(fOutField)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }




  out[0] = (Object)fOutField;

  free(fpXrect);
  free(fpYrect);
  free(fpZrect);

  return OK;

  error:
    DXDelete((Object)fOutField);
    DXDelete((Object)aOutGrid);
    DXDelete((Object)aOutConnect);
}

float *fpGetData(int32 liFileId, char *cpName)
{
   int32 iErr, liSdsId, liIndex, liNumType, liAttr, liRank, liSize;
   int32 liaDimSize[MAX_VAR_DIMS],liaStart[MAX_VAR_DIMS],liaEnd[MAX_VAR_DIMS];
   int i, iNi, iNj,iNk;
   char caDataName[MAX_NC_NAME];
   float *fpData;

   for(i=0;i<MAX_VAR_DIMS;i++) {
     liaDimSize[i] = 0;
     liaStart[i] = 0;
     liaEnd[i] = 0;
   }
   /*Get the info about thie dataset */
   liIndex = SDnametoindex(liFileId,cpName);
   liSdsId = SDselect(liFileId,liIndex);
   iErr = SDgetinfo(liSdsId,caDataName,&liRank,liaDimSize,&liNumType,&liAttr);
   /* Create reterival information arrays and allocate space */
   liSize=1;
   for(i=0;i<liRank;i++) {
     liSize*=liaDimSize[i];
     liaStart[i]=0;
     liaEnd[i]=liaDimSize[liRank-i-1];
     liaEnd[i]=liaDimSize[i];
   }
   fpData = malloc(liSize*sizeof(float));
   /* Get dataset */
   iErr = SDreaddata(liSdsId,liaStart,NULL,liaEnd,fpData);
   SDendaccess(liSdsId);

   /* Cleanup */
   return fpData;
} 

