#include <dx/dx.h>  
#include <stdio.h>  
#include <stdlib.h>
#include <stddef.h>
#include <math.h>

#define BUFFER 256

/* This module takes the spacecraft trajectory field (out of ReadTrajectory) */
/* and the MHD Time (UT) (out of yrdayhr) and interpolates the trajectory    */
/* to find the right spacecraft position that corresponds to the MHD time.   */
/* It outputs a field of one point (the spacecraft location at that point)   */

float lininterp(float, float, float, float, float);

Error m_OrbitSync(Object *in, Object *out)
{
  /* Variable declarations */

  char message[30], *greeting;
  int i, pt, linenumber, iCount, rank, timeshape, posshape; 
  int debug, mhdDoy;   /* have no idea why i can't delete mhdDoy */
  int  *SimTime = NULL, data;
  float *fptime, *fppositions, mhd_decimalHr, startHr, endHr, startXPOS;
  float endXPOS,startYPOS, endYPOS, startZPOS, endZPOS, x, y, z, pos[3];
  float cloudradius, tmp;
  float cloud[6][3], clouddata[6];
  Array time_array, positions_array, simtime_array, aSCPosition, aSCData;
  Array aCloudPosition, aCloudData;
  Field fSCPosition = NULL;  // output field with sc position
  Field fCloudPosition = NULL;  // output field with cloud points

  debug = 0;
  cloudradius = 1.0;

  /* Get input values   */

  if (!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER, 
	       "Missing Trajectory field from ReadTrajectory");
    goto error;
  }
  else {
    time_array = (Array)DXGetComponentValue((Field)in[0],"data");
    fptime  = DXGetArrayData(time_array);
    positions_array = (Array)DXGetComponentValue((Field)in[0],"positions");
    fppositions = DXGetArrayData(positions_array);
  
    /* get number of elements (iCount) and the dimensions (time-, posshape) */
    DXGetArrayInfo(time_array, &iCount, NULL, NULL, &rank, &timeshape);
    DXGetArrayInfo(positions_array, &iCount, NULL, NULL, &rank, &posshape);
    DXMessage("did we read in the trajectory field again?");
  }

  if (!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER, "Missing UT Time from yrdayhr");
    goto error;
  }
  else {
  simtime_array = (Array) in[1];   /* get the UT time array */
  SimTime = DXGetArrayData(simtime_array);  /* put it into SimTime */
  }
  
  if (!in[2]) {
    DXMessage("Using 1 Re for the default cloud radius");
    cloudradius = 1.0;
  }
  else if (!DXExtractFloat(in[2], &cloudradius)){
    DXSetError(ERROR_BAD_PARAMETER, "Cloud Radius must be a float");
    goto error;
  }

  DXMessage("got to here in orbitsync");

  /* now convert mhdTime to YYYY DOY HH.HHHH  */
  mhd_decimalHr = (float)SimTime[2] + ((float)SimTime[3]/60) + 
    ((float)SimTime[4]/3600);
  if (debug) DXMessage("mhd_decimalHr = %f", mhd_decimalHr);

  /* make sure the MHDTime is within the bounds of the trajectory file */
  if ((fptime[0]>(float)SimTime[0]) ||(fptime[0]==(float)SimTime[0]) && ((fptime[1]>(float)SimTime[1]) 
     ||(fptime[0]==(float)SimTime[0]) && ((fptime[1]==(float)SimTime[1])  && (fptime[2]>= mhd_decimalHr)))) {
    DXSetError(ERROR_BAD_PARAMETER, 
	       "MHD Time is BEFORE all valid trajectory file times");
    goto error;
  }   
  if ((fptime[(iCount-1)*timeshape]!=(float)SimTime[0]) 
	   || ((fptime[(iCount-1)*timeshape+1]-(float)SimTime[1])*24.+ 
	   (fptime[(iCount-1)*timeshape+2]- mhd_decimalHr)<0 )) {
    DXSetError(ERROR_BAD_PARAMETER, 
	       "MHD Time is AFTER all valid trajectory file times");
    goto error;
  }

  /* now search through the time array for the right 2 entries straddling */
  /* the MHD Time   */

  if (debug) {
  DXMessage("Icount : %d,Timeshape: %d", iCount,timeshape);
  }
  for (i=0; i<iCount; i++) {
    if(fptime[i*timeshape] == (float)SimTime[0]) {
      if (fptime[(i*timeshape)+1] == (float)SimTime[1]) {
	if (fptime[(i*timeshape)+2] < mhd_decimalHr) {
	  linenumber = i;   /* this is the linenumber at the record BEFORE */
	  /* the MHD time */
	}
      }
    }
  }

  startHr = fptime[(linenumber*timeshape)+2];  /* gets the startHr  */
  endHr = fptime[((linenumber+1)*timeshape)+2];  /* gets the endHr */
  startXPOS = fppositions[(linenumber*posshape)];  /* and on.... */
  endXPOS = fppositions[((linenumber+1)*posshape)];
  startYPOS = fppositions[(linenumber*posshape)+1];
  endYPOS = fppositions[((linenumber+1)*posshape)+1];
  startZPOS = fppositions[(linenumber*posshape)+2];
  endZPOS = fppositions[((linenumber+1)*posshape)+2];

  if (debug)  DXMessage("Found correct line number at : %d", linenumber);
  if (debug)  DXMessage("startHr : %f, endHr : %f", startHr, endHr);

  x=lininterp(startHr, startXPOS, endHr, endXPOS, mhd_decimalHr);
  y=lininterp(startHr, startYPOS, endHr, endYPOS, mhd_decimalHr);
  z=lininterp(startHr, startZPOS, endHr, endZPOS, mhd_decimalHr);

  if (debug) {
  DXMessage("startXPOS : %f, endXPOS : %f, interp'd : %f", startXPOS, 
	    endXPOS, x);
  DXMessage("startYPOS : %f, endYPOS : %f, interp'd : %f", startYPOS, 
	    endYPOS, y);
  DXMessage("startZPOS : %f, endZPOS : %f, interp'd : %f", startZPOS, 
	    endZPOS, z);
  }

  pos[0] = x;   /* put these into an array to give to DXAddArrayData */
  pos[1] = y;
  pos[2] = z;

  data = 1;    /* set the 'data' at that point to be 1 */

  /* here we construct a 'Cloud' of radius 1 Re around the sc position */

/*   tmp = makecloud(pos, cloudradius);   */
  for (pt=0; pt<6; pt++) {
    for (i=0; i<3; i++) {
      
      cloud[pt][i] = pos[i];
      clouddata[pt] = 1.0;
      
      /* this if condition only adds or subtracts cloudradius to */
      if ((((i==0) && ((pt==0) || (pt==1)) 
	    || ((i==1) && ((pt==2) || (pt==3))) 
	    || ((i==2) && ((pt==4) || (pt==5)))))) {

	cloud[pt][i] = pos[i] + (pow(-1,pt)*cloudradius);

      }
    }
    if (debug) {
    DXMessage("Cloud row %d:  %f, %f, %f\n", pt, cloud[pt][0], cloud[pt][1],
	      cloud[pt][2]);
    }
  }
  
  /* end of 'makecloud' function */

  /* now we have the x, y, z interpolated (in time) s/c position */
  /* and we have to pack it into a field  */
  aSCPosition = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, 3);
  DXAddArrayData(aSCPosition, 0, 1, pos);
  aSCData = DXNewArray(TYPE_INT, CATEGORY_REAL, 1, 1);
  DXAddArrayData(aSCData, 0, 1, &data);
  
  fSCPosition = DXNewField();
  DXSetComponentValue(fSCPosition, "positions", (Object)aSCPosition);

  DXSetStringAttribute((Object)aSCData,"dep", "positions");
  DXSetComponentValue(fSCPosition, "data", (Object)aSCData);
   
  DXEndField(fSCPosition);  /* this closes the field */
  
  /* now make the field for the cloud & pack the points in */
  aCloudPosition = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, 3);
  DXAddArrayData(aCloudPosition, 0, 6, cloud);
  aCloudData = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, 1);
  DXAddArrayData(aCloudData, 0, 6, clouddata);

  fCloudPosition = DXNewField();
  DXSetComponentValue(fCloudPosition, "positions", (Object)aCloudPosition);

  DXSetStringAttribute((Object)aCloudData,"dep", "positions");
  DXSetComponentValue(fCloudPosition, "data", (Object)aCloudData);
   
  DXEndField(fCloudPosition);  /* this closes the field */

  DXMessage("closed the output field");

  out[0] = (Object)fSCPosition;  /* sends the field back to DX */
  out[1] = (Object)fCloudPosition;  /* send the cloud back */

 return OK;
 error:
 return ERROR;
}

float lininterp(float x1, float y1, float x2, float y2, float x){
  
  float m, b,y;

  m = (y2-y1)/(x2-x1);  /* compute slope  */
  b = y1-(m*x1);        /* compute intercept  */

  y=(m*x)+b;           /* compute interped value */
  return(y);
}
