#include <stdio.h>
#include <dx/dx.h>
#define MAX_NAME_SIZE 256

Error
m_SMGSE(Object *in, Object *out)
{
   /***********************/
   /*  Declare variables  */
   /***********************/
   Field fgsm,fdp;
   Array sm_array,sm_carray,a,sm_conn;
   Array fgsma,fgsmap,fdpa,fdpap,fdpac;
   
   float *sm_data,*sm_coords;
   float *fgsma_data,*fgsmap_data,*fdpa_data,*fdpap_data;

   Type     sm_data_type,sm_coord_type;
   int      sm_data_items,sm_coord_items;
   Category sm_data_cat,sm_coord_cat;
   int      sm_data_rank,sm_data_shape;
   int      sm_coord_rank,sm_coord_shape;

   char  *CoordSys = NULL;
   float  Scale;
   int Trans_Data = 1;
   int  *UT_Time = NULL;

   int *t,i,j,k,size,coord;
   float *gsm;
   float gse[3],r1[3][3], r[3],xs,ys,zs,xn,yn,zn,cj;
   float north[3], south[3];
   
   float *sm;
   char label[MAX_NAME_SIZE];
   char delim[1] = ";";
   int dims0[1];
 

   /***********************/
   /*  Get input values   */
   /***********************/
   /* Get CoordSys's value */
   if (!in[0]) {
     CoordSys = "NONE";
   }
   else if (!DXExtractString(in[0], &CoordSys)) {   
     DXSetError(ERROR_BAD_PARAMETER, "CoordSys must be a string");
     goto error;
   }

   /* Get Scale's value */
   if (!in[1]) {
     Scale = 1.0;
   }
   else if (!DXExtractFloat(in[1], &Scale)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Scale must be a float");
     goto error;
   }
   
   /* Get Time array */
   /* Get Trans_data value */
   if (!in[2]) {
     Trans_Data = 1.0;
   }
   else if (!DXExtractInteger(in[2], &Trans_Data)) {   
     DXSetError(ERROR_BAD_PARAMETER, "Trans_Data must be an integer");
     goto error;
   }
   
   if (!in[3]) {
     DXSetError(ERROR_BAD_PARAMETER,"missing UT_Time");
     goto error;
   }
   else {
     a = (Array) in[3];
     UT_Time = DXGetArrayData(a);
   }

   /* Get SmField structured mesh */
   /* Get mesh id */
   if (!in[4]) {
     DXSetError(ERROR_BAD_PARAMETER,"missing SM Field");
     goto error;
   }
   else {
     sm_array = (Array)DXGetComponentValue((Field)in[4],"data");
     sm_data  = DXGetArrayData(sm_array);
     sm_carray = (Array)DXGetComponentValue((Field)in[4],"positions");
     sm_coords = DXGetArrayData(sm_carray);
     sm_conn   = (Array)DXGetComponentValue((Field)in[4],"connections");
     DXGetArrayInfo(sm_array,&sm_data_items,&sm_data_type,
		    &sm_data_cat,&sm_data_rank,&sm_data_shape);
     DXGetArrayInfo(sm_carray,&sm_coord_items,&sm_coord_type,
		    &sm_coord_cat,&sm_coord_rank,&sm_coord_shape);
   }


   /***********************/
   /* Function's Body     */
   /***********************/

   /***********************/
   /*  Set output values  */
   /***********************/
   
   /* Set GSMField structured mesh */
   fgsm = DXNewField();
   fgsma = DXNewArray(sm_data_type,sm_data_cat,sm_data_rank,sm_data_shape);
   DXAddArrayData(fgsma,0,sm_data_items,NULL);
   fgsma_data = DXGetArrayData(fgsma);
   fgsmap = DXNewArray(sm_coord_type,sm_coord_cat,sm_coord_rank,sm_coord_shape);
   DXAddArrayData(fgsmap,0,sm_coord_items,NULL);
   fgsmap_data = DXGetArrayData(fgsmap);
   if (!DXSetStringAttribute((Object)fgsma, "dep", "positions"))
     goto error;
   if (!DXSetComponentValue(fgsm, "positions", (Object)fgsmap))
     goto error;
   if (!DXSetComponentValue(fgsm, "data", (Object)fgsma))
     goto error;
   if (!DXSetComponentValue(fgsm, "connections", (Object)sm_conn))
     goto error;


   /* Set DipoleAxis structured mesh */
   fdp = DXNewField();
   fdpa = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,0);
   DXAddArrayData(fdpa,0,2,NULL);
   fdpa_data = DXGetArrayData(fdpa);
   fdpap = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
   DXAddArrayData(fdpap,0,2,NULL);
   fdpap_data = DXGetArrayData(fdpap);
   if (!DXSetStringAttribute((Object)fdpa, "dep", "positions"))
     goto error;
   if (!DXSetComponentValue(fdp, "data", (Object)fdpa))
     goto error;
   if (!DXSetComponentValue(fdp, "positions", (Object)fdpap))
     goto error;
   fdpac = DXMakeGridConnections(1,2);
   if (!DXSetComponentValue(fdp, "connections", (Object)fdpac))
     goto error;

 
   dims0[0] = 2;
/*  FILL IN THE OUTPUT FIELD  Here!                                      */
/*                                                                       */
/* Free old field data                                                   */

   fdpa_data[0]=1.;
   fdpa_data[1]=1.;
   t = UT_Time;
   j = 0;
   north[0]=north[1]=0;
   north[2] = Scale;
   south[0]=south[1] = 0;
   south[2]=-(Scale);

   recalc_(t,t+1,t+2,t+3,t+4);
	
/* Rotate Points into selected Coord System  */
 
 
   size = sm_data_items;
   if(!strcmp(CoordSys,"NONE"))
     coord = 3;
   else if (!strcmp(CoordSys,"GSM"))
     coord = 0;
   else if (!strcmp(CoordSys,"GSE"))
     coord = 1;
   else if (!strcmp(CoordSys,"GEO"))
     coord = 2;
   /*	coord = AVSchoice_number("Coord SYS",out_sys) - 1; */

   /* Set rotation coefficients */
   j = 1;
   xn = 1.0;
   yn = 0.0;
   zn = 0.0;
   for (i=0;i<3;i++) {
     smgsm_(&xn,&yn,&zn,gse,gse+1,gse+2,&j);
     DXMessage("smgsm: %f,%f,%f,%f,%f,%f,%f",xn,yn,zn,gse[0],gse[1],gse[2]);
     if (coord ==0)  {
       r1[0][i] = gse[0];
       r1[1][i] = gse[1];
       r1[2][i] = gse[2];
     }	
     if(coord ==1) {
       j=1;
       gsmgse_(gse,gse+1,gse+2,r,r+1,r+2,&j);
       r1[0][i] = r[0];
       r1[1][i] = r[1];
       r1[2][i] = r[2];
     }
     if(coord == 2) {
       j = -1;
       geogsm_(r,r+1,r+2,gse,gse+1,gse+2,&j);	
       r1[0][i] = r[0];
       r1[1][i] = r[1];
       r1[2][i] = r[2];
     }
     k = xn;
     xn = zn;
     zn = yn; 
     yn = k; 
   }
   DXMessage("r1 = %f %f %f\n     %f %f %f\n     %f %f %f\n",
      r1[0][0],r1[1][0],r1[2][0],r1[0][1],r1[1][1],r1[1][2],r1[0][2],r1[1][2],
      r1[2][2]);


   /* No Rotation */
   
   if (coord == 3) {     
     for ( i=0;i<3*sm_coord_items;i++){
       fgsmap_data[i] = sm_coords[i];
       /* printf("%f %f\n",GSMField_coord[i],SmField_coord[i]); */
     }
     fdpap_data[0] = north[0];
     fdpap_data[1] = north[1];
     fdpap_data[2] = north[2];
     fdpap_data[3] = south[0];
     fdpap_data[4] = south[1];
     fdpap_data[5] = south[2];
   } 
   else{
     /* Do dipole axis rotation */
     for (k=0;k<3;k++) {
       fdpap_data[k] = r1[k][0]*north[0]
	               +r1[k][1]*north[1]+r1[k][2]*north[2]; 
       fdpap_data[k+3] = r1[k][0]*south[0]
			 +r1[k][1]*south[1]+r1[k][2]*south[2];
     }
   }
   
   printf("Finished Dipole Rotation:%f %f %f %f %f %f\n",
	  fdpap_data[0],fdpap_data[1],
	  fdpap_data[2],fdpap_data[3],fdpap_data[4],
	  fdpap_data[5]);

   /* Rotate data coordinates */
   if (coord != 3) {	
     for (i=0;i<sm_coord_items;i++) {
       gsm = fgsmap_data + 3*i;
       sm = sm_coords + 3*i;
  	for (k=0;k<3;k++)
	  gsm[k] = r1[k][0]*sm[0]+r1[k][1]*sm[1]+r1[k][2]*sm[2]; 
     }
   }
   /*	Now do data */
   printf("Coordinate rotation done\n");
   if(coord == 3 || !Trans_Data){
     for (i=0;i<sm_data_items*sm_data_shape;i=i+sm_data_shape) {
       gsm = fgsma_data +i;
       sm = sm_data +i;
       for (k=0;k<sm_data_shape;k++)gsm[k] = sm[k];
     }
   }
   else {
     for (i=0;i<sm_data_items*sm_data_shape;i=i+sm_data_shape) {
       gsm = fgsma_data +i;
       sm = sm_data +i;
       /*  scalar or coord ==3, no rotation */
       /*  scalar k=0 density, k=4 pressure */
       gsm[0] = sm[0];
       gsm[4] = sm[4];
       /* Vector - Do rotation */
       for (k=0;k<3;k++){
	 /* Velocity */
	 gsm[k+1] = r1[k][0]*sm[1]+r1[k][1]*sm[2]+r1[k][2]*sm[3];
	 /* B-Field */
	 gsm[k+5] = r1[k][0]*sm[5]+r1[k][1]*sm[6]+r1[k][2]*sm[7];
	 /* E-Field */
	 gsm[k+8] = r1[k][0]*sm[8]+r1[k][1]*sm[9]+r1[k][2]*sm[10];
	 /* Current */
	 gsm[k+11] = r1[k][0]*sm[11]+r1[k][1]*sm[12]+r1[k][2]*sm[13];
	 /* Position */
	 gsm[k+14] = r1[k][0]*sm[14]+r1[k][1]*sm[15]+r1[k][2]*sm[16];
       }
     }
   }
   if (!DXEndField(fgsm))
     goto error;
   if (!DXEndField(fdp))
     goto error;
 

   /*************************/
   /*  Free input variables */
   /*************************/
   out[0] = (Object) fgsm;
   out[1] = (Object) fdp;
   return OK;
error:
   return ERROR;
}
