/* R.S. Weigel, 10/25/2004 */
#include <dx/dx.h>  
#include <stdio.h>  
#include <stdlib.h>
#include <stddef.h>
#include "cxform.h"

Error m_SPTransform(Object *in, Object *out)
{

  /* Input Declarations */
  Array aDataVector;
  Array aPosVector;
  Array aConVector;
  Array aTimeVector;
  Array adT;
  float *fpDataVector;
  float *fpPosVector;
  float *fpConVector;
  int   *intTimeVector;
  int    intdT  = 0;
  int    Nsteps = 1;
  int    intNdT = 1;
  int    Ngroups = 1;
  int    verbosity=0;
  Type tDataVector;
  Type tPosVector;
  Type tConVector;
  Type tTimeVector;
  Category cDataVector;
  Category cPosVector;
  Category cConVector;
  Category cTimeVector;
  int nDataVector=0, nPosVector=0, nConVector=0, nTimeVector=1; /* Number of items */
  int rDataVector=0, rPosVector=0, rConVector=0, rTimeVector=0; /* Rank of each item */

  /* If scalar, shape value is not changed by DXArrayInfo.  So this will */
  /* make a scalar a 1-vector */
  int sDataVector = 1,sPosVector = 1,sConVector = 1,sTimeVector = 1; /* Shape of each item */

  /* Output Declarations */
  Group gOutGroup = NULL;
  Field fOutField = NULL;
  Field fOutField1 = NULL;
  Array aOutDataVector = NULL;
  Array aOutPosVector = NULL;
  Array aOutConVector = NULL;
  Array aOutTimeVector = NULL;
  float *fpOutDataVector;
  float *fpOutPosVector;
  float *fpOutConVector;

  int i,j,k;

  int Md;     /* Returned field has a data component that is a Md-vector.    */
  int Mp;     /* Returned field has a position component that is a Mp-vector.*/

  /* Declarations for cxform */
  int retVal, es, es_o, year, month, day, hour, minute, second;
  double jd,jd_o;
  Vec d_in, d_out;
  Vec p_in, p_out;
  char *inSys=NULL;
  char *outSys=NULL;
  int HEE2GSE = 0;      /* special transform b/c data is not transformed */
  int GSE2HEE = 0;      /* special transform b/c data is not transformed */
  int isScalarData = 0; /* special transform b/c data is not transformed */
  int FieldType = 0;    /* if 1, then entire field is transformed for each UT */
                    
  
  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  if (!in[7]){
    verbosity = 0;
  } 
  else {
    DXExtractInteger(in[7], &verbosity);    
  }

  if (verbosity > 0) {
    DXMessage("");
    DXMessage("Start call to SPTransform ...");
  }

  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"SPTransform: Missing Field in first input");
    return ERROR;
  }
  else {
    /* Need to apply any pending transforms existing in this object because */
    /* in OpenDX, the transforms are not applied to the data */
    /* until the rendering stage */
    in[0]         =  DXApplyTransform(in[0], NULL);
    aDataVector   = (Array) DXGetComponentValue((Field)in[0],"data");

    if (aDataVector == NULL) {
      if (verbosity > 0) 
	DXMessage("Input field does not have a data component.");
	DXMessage("Transforming only position component.");

    } else {
      fpDataVector  = DXGetArrayData(aDataVector);
      DXGetArrayInfo(aDataVector, &nDataVector, &tDataVector, &cDataVector,
		     &rDataVector, &sDataVector);
      if (verbosity > 0) {
	DXMessage("nDataVector = %d, rDataVector = %d, sDataVector = %d",
		  nDataVector,rDataVector,sDataVector);
      }
      
    }
    
    aPosVector    = (Array) DXGetComponentValue((Field)in[0],"positions");
    fpPosVector   = DXGetArrayData(aPosVector);
    
    DXGetArrayInfo(aPosVector, &nPosVector, &tPosVector, &cPosVector,
		   &rPosVector, &sPosVector);
    if (verbosity > 0) {
      DXMessage("nPosVector = %d, rPosVector  = %d, sPosVector  = %d",
		nPosVector,rPosVector,sPosVector);
    }
  }


  if (!in[1]){
    nTimeVector      = 1;
    intTimeVector    = (int*)malloc(6*sizeof(int));
    intTimeVector[0] = 2000;
    intTimeVector[1] = 1;
    intTimeVector[2] = 1;
    intTimeVector[3] = 0;
    intTimeVector[4] = 0;
    intTimeVector[5] = 0;
  } 
  else {
    aTimeVector   = (Array) in[1];
    intTimeVector = DXGetArrayData(aTimeVector);
    DXGetArrayInfo(aTimeVector, &nTimeVector, &tTimeVector, 
		   &cTimeVector, &rTimeVector, &sTimeVector);
  }
  
  if (!in[2]){
    intdT = 0;
  } 
  else {
    DXExtractInteger(in[2], &intdT);
  }


  if (!in[3]){
    inSys = "GSE";
  } 
  else {
    DXExtractString(in[3], &inSys);
  }


  if (!in[4]){
    outSys = "GSE";
  } 
  else {
    DXExtractString(in[4], &outSys);
  }

  if (!in[5]){
    FieldType = 0;
  } 
  else {
    DXExtractInteger(in[5], &FieldType);    
  }

  if (!in[6]){
    intNdT = 1;
  } 
  else {
    DXExtractInteger(in[6], &intNdT);

    if ( (intNdT < 1) ) {
      DXSetError(ERROR_BAD_PARAMETER,
		 "SPTransform: NdT must be and integer greater than zero.");
      return ERROR;
    }

    if ( (intNdT != 1) && (FieldType != 1) ) {
      DXSetError(ERROR_BAD_PARAMETER,
		 "SPTransform: FieldType must be 1 if NdT is not 1.");
      return ERROR;
    }
    if ( (nTimeVector != 1) && (intNdT != 1) ){
      DXSetError(ERROR_BAD_PARAMETER,
		 "SPTransform: If NdT is not 1, InitialTime must be a vector.");
      return ERROR;
    }

  }


  if ( (FieldType == 1) && (nTimeVector != 1) ) {
    Ngroups = nTimeVector;
  }
  if ( (FieldType == 1) && (nTimeVector == 1) ) {
    Ngroups = intNdT;
  }

  if (verbosity > 0)
    DXMessage("SPTransform: Ngroups = %d, nTimeVector = %d, intNdT = %d",
	      Ngroups,nTimeVector,intNdT);

  if (verbosity > 0) 
    DXMessage("SPTransform: InSys = %s, OutSys = %s", inSys, outSys);  

  if (nTimeVector > 1) {
    if (verbosity > 0) {
      DXMessage("SPTransform: Multiple (%d) universal times specified.  Ignoring dT parameter.", nTimeVector);
    }
    if ((nTimeVector != nPosVector) && (FieldType == 0) ){
      DXSetError(ERROR_BAD_PARAMETER,
		 "SPTransform: Number of Universal Times must be equal to number of items in input Field if FieldType = 0.");
      return ERROR;
    }
    intdT = 0;
  }
   
  if ( (sPosVector != 3) ){
    DXSetError(ERROR_BAD_PARAMETER,
	       "SPTransform: Position component must be a 3-vector.");
    return ERROR;
  }
  
    
  if ( (sDataVector != 3) && (sDataVector != 1) ){
    DXMessage("%d",sDataVector);
    DXSetError(ERROR_BAD_PARAMETER,
	       "SPTransform: Data component must be a scalar, 1-vector, or 3-vector.");
    return ERROR;
  }
  if ( (rDataVector > 1) ){
    DXSetError(ERROR_BAD_PARAMETER,
	       "SPTransform: Data component must be a scalar, 1-vector, or 3-vector.");
    return ERROR;
  }
  
  if (sDataVector == 1) {
    isScalarData = 1;
    if (verbosity > 0) 
      DXMessage("SPTransform: Input field has scalar data. Transforming only position component of field.");
  }
  else {
    isScalarData = 0;
  }
  
  if ( (nDataVector > 0) & (nDataVector != nPosVector) ) {
    DXSetError(ERROR_BAD_PARAMETER,
	       "SPTransform: If data component exists, it must have the same number of items as the position component");
  }
  
  Md      = sDataVector;
  Mp      = sPosVector;
  
  /*****************************************************************************/
  /* See if conversion between Heliocentric and Geocentric was requested       */
  
    /* If outSys started with H and inSys did not start with H */
    if ( (strncmp(outSys,"H",1) == 0) && (strncmp(inSys,"H",1) != 0) ){
      /* If outSys is HEE and inSys is GSE */
      if ( (strncmp(outSys,"HEE",3) == 0)  && (strncmp(inSys,"GSE",3) == 0) ){
	GSE2HEE = 1;
	if (verbosity > 0)
	  DXMessage("SPTransform: GSE->HEE requested.  Transforming only position component of field.");
      }
      else {
	DXSetError(ERROR_BAD_PARAMETER,
		   "SPTransform: When transforming between heliocentric and geocentric, only HEE<->GSE transformations is allowed.");
	return ERROR;
      }
    }

    /* If inSys started with H and outSys did not start with H */
    if ( (strncmp(inSys,"H",1) == 0) && (strncmp(outSys,"H",1) != 0) ){
      /* If inSys is HEE and outSys is GSE */
      if ( (strncmp(inSys,"HEE",3) == 0) && (strncmp(outSys,"GSE",3) == 0) ){
	HEE2GSE = 1;
	if (verbosity > 0)
	DXMessage("SPTransform: HEE->GSE requested.  Transforming only position component of field.");
      }
      else {
	DXSetError(ERROR_BAD_PARAMETER,
		   "SPTransform: When transforming between heliocentric and geocentric, only HEE<->GSE transformations are allowed.");
	return ERROR;
      }
    }
    /****************************************************************************/

  /******************************************************************/  
  /* Do computation and fill in the Output arrays with data         */
  /******************************************************************/  
  year   = intTimeVector[0];
  month  = intTimeVector[1];
  day    = intTimeVector[2];
  hour   = intTimeVector[3];
  minute = intTimeVector[4];
  second = intTimeVector[5];

  jd_o = gregorian_calendar_to_jd(year, month, day, hour, minute, second);
  es_o = date2es(year, month, day, hour, minute, second);

  if (verbosity > 0)
    DXMessage("SPTransform: Initial Time: %.4d/%.2d/%.2d %.2d:%.2d:%.2d  (JD %f, ES: %d)\n", 
	    year, month, day, hour, minute, second, jd_o, es_o);

  if ( ( FieldType == 1) ) {
    gOutGroup = DXNewGroup();
  }

  for (k = 0 ; k < Ngroups ; k++) {

    /******************************************************************/
    /* Allocate and declare output OpenDX arrays as M-vectors         */
    /* with nPosVector items                                          */
    /******************************************************************/
    /* If no data, no transform */
    if (nDataVector > 0) {
      aOutDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, Md);
      DXAddArrayData(aOutDataVector, 0, nDataVector, NULL);
      fpOutDataVector = (float*)DXGetArrayData(aOutDataVector);
    }    

    aOutPosVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, Mp);
    DXAddArrayData(aOutPosVector, 0, nPosVector, NULL);
    fpOutPosVector = (float*)DXGetArrayData(aOutPosVector);
    
    /******************************************************************/

    if ( FieldType == 1 ) {
      if ( nTimeVector > 1 ) {
	year   = intTimeVector[0+6*k];
	month  = intTimeVector[1+6*k];
	day    = intTimeVector[2+6*k];
	hour   = intTimeVector[3+6*k];
	minute = intTimeVector[4+6*k];
	second = intTimeVector[5+6*k];
	es     = date2es(year, month, day, hour, minute, second);
      } else {
	es     = es_o + k*intdT;
      }

      if (verbosity > 0){
	jd = gregorian_calendar_to_jd(year, month, day, hour, minute, second);
	DXMessage("SPTransform: Group %d, Current Time: %.4d/%.2d/%.2d %.2d:%.2d:%.2d  (JD %f, ES: %d)\n",k, year, month, day, hour, minute, second, jd, es);
      }

    }

    for (i=0; i<nPosVector; i++){

      if (FieldType == 0) {
	if (nTimeVector > 1) {
	  year   = intTimeVector[0+6*i];
	  month  = intTimeVector[1+6*i];
	  day    = intTimeVector[2+6*i];
	  hour   = intTimeVector[3+6*i];
	  minute = intTimeVector[4+6*i];
	  second = intTimeVector[5+6*i];
	  es     = date2es(year, month, day, hour, minute, second);
	} else {
	  es     = es_o + i*intdT;
	}
      }

      /***********************************************/
      /* Positions ***********************************/
      /***********************************************/
      for (j=0; j<Mp ; j++){
	p_in[j] = fpPosVector[j+Mp*i];
      }
      
      retVal = cxform(inSys, outSys, es, p_in, p_out);
      
      for (j=0; j<Mp ; j++){
	fpOutPosVector[j+Mp*i]  = p_out[j];
      }

      /* If no data, no transform */
      if (nDataVector > 0) {
	/***********************************************/
	/* Data      ***********************************/
	/***********************************************/
	for (j=0; j<Md ; j++){
	  d_in[j] = fpDataVector[j+Md*i];
	}
	/***********************************************/      
	/* If data is not scalar and transformation is not between heliospheric  */
	/* and geocentric systems, do computation.  Otherwise Data out = Data in. */
	if ( (isScalarData == 0) && (GSE2HEE == 0) && (HEE2GSE == 0) ) {
	  retVal = cxform(inSys, outSys, es, d_in, d_out);
	}
	
	for (j=0; j<Md ; j++){
	  if ( (isScalarData == 0) && (GSE2HEE == 0) && (HEE2GSE == 0) ) {
	    fpOutDataVector[j+Md*i]  = d_out[j];
	  }
	  else {
	    fpOutDataVector[j+Md*i]  = d_in[j];
	  }
	}
      }
      

    }
    /******************************************************************/  


    /******************************************************************/   
    /* Form output field **********************************************/
    /******************************************************************/   

    /* To keep colors and other components the same this should not be */
    /* done.  If not done for FieldType = 1, all group members are a   */
    /* copy of the last Field                                          */
    /* To prevent this, fpOutDataVector must be declalred as           */
    /* **fpOutDataVector and a malloc for Ngroups is needed along with */
    /* other required changes.                                         */
    if (FieldType == 1)
      fOutField = DXNewField();

    /* To keep colors and other components the same */
    if (FieldType == 0)
      fOutField = (Field) in[0]; 

    DXSetComponentValue(fOutField, "positions", (Object)aOutPosVector);

    /* If no data, no transform */
    if (nDataVector > 0) {
      DXSetComponentValue(fOutField, "data", (Object)aOutDataVector);    
    }

    /******************************************************************/   

    if (FieldType == 1) {
      DXSetEnumeratedMember((Group) gOutGroup, k , (Object) fOutField);
    }

  }


  if (FieldType == 1) {
    DXEndObject((Group) gOutGroup);
    out[0] = (Object)gOutGroup;
  } else {
    DXEndField((Field) fOutField);
    out[0] = (Object)fOutField;
  }

  if (verbosity > 0) {
    DXMessage("... End call to SPTransform.");
    DXMessage("");
  }

  return OK;

}

