/* R.S. Weigel, 10/25/2004                            */
/* This is an example code that takes and input field */
/* and multiplies the data component by 2.0           */

#include <dx/dx.h>  
#include <stdio.h>  
#include <stdlib.h>
#include <stddef.h>

Error m_Template(Object *in, Object *out)
{

  /* Input Declarations */
  Array aDataVector;
  Array aPosVector;
  Array aConVector;
  float *fpDataVector;
  float *fpPosVector;
  float *fpConVector;
  Type tDataVector;
  Type tPosVector;
  Type tConVector;
  Category cDataVector;
  Category cPosVector;
  Category cConVector;
  int nDataVector, nPosVector, nConVector; /* Number of items */
  int sDataVector, sPosVector, sConVector; /* Shape of each item */
  int rDataVector, rPosVector, rConVector; /* Rank of each item */

  /* Output Declarations */
  Field fOutField = NULL;
  Array aOutDataVector = NULL;
  Array aOutPosVector = NULL;
  Array aOutConVector = NULL;
  float *fpOutDataVector;
  float *fpOutPosVector;
  float *fpOutConVector;

  int i,j,k;

  int M; /* Returned field has a data component that is a M-vector.          */
  int N; /* Returned field has a data component with N M-vectors (or "items")*/

  /******************************************************************/
  /* Get inputs                                                     */
  /******************************************************************/
  if (!in[0]){
    DXSetError(ERROR_BAD_PARAMETER,"missing field");
    return ERROR;
  }
  else{

    aDataVector  = (Array) DXGetComponentValue((Field)in[0],"data");
    fpDataVector = DXGetArrayData(aDataVector);
    aPosVector   = (Array) DXGetComponentValue((Field)in[0],"positions");
    fpPosVector  = DXGetArrayData(aPosVector);
    aConVector   = (Array) DXGetComponentValue((Field)in[0],"connections");
    fpConVector  = DXGetArrayData(aConVector);

    DXGetArrayInfo(aDataVector, &nDataVector, &tDataVector, &cDataVector,
		   &rDataVector, &sDataVector);
    DXGetArrayInfo(aPosVector, &nPosVector, &tPosVector, &cPosVector,
		   &rPosVector, &sPosVector);
    DXGetArrayInfo(aConVector, &nConVector, &tConVector, &cConVector,
		   &rConVector, &sConVector);
   
    if (rPosVector > 3){
      DXSetError(ERROR_BAD_PARAMETER,
		 "Position component must be 1, 2, or 3-vector");
      return ERROR;
    }
    if (rDataVector > 3){
      DXSetError(ERROR_BAD_PARAMETER,
		 "Data component must be 1, 2, or 3-vector");
      return ERROR;
    }
    if (rDataVector != rPosVector){
      DXSetError(ERROR_BAD_PARAMETER,
		 "Data and Position component must have same rank");
      return ERROR;
    }

    if (rPosVector == 0) {
      sPosVector = 1; /* In OpenDX, shape is not returned for a 1-vector */
                      /* Define a scalar as a 1-vector for convienence   */
    }


    M = sDataVector;
    N = nDataVector;

    DXMessage("Template.c passed a Field with data component having");
    DXMessage("%d items, a rank of %d, and a shape of %d",
	      nDataVector,rDataVector,sDataVector);
  }
  /******************************************************************/

  /******************************************************************/
  /* Allocate and declare output OpenDX arrays as M-vectors         */
  /* with N elements                                                */
  /******************************************************************/
  aOutDataVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
  DXAddArrayData(aOutDataVector, 0, N, NULL);
  fpOutDataVector = (float*)DXGetArrayData(aOutDataVector);

  aOutPosVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
  DXAddArrayData(aOutPosVector, 0, N, NULL);
  fpOutPosVector = (float*)DXGetArrayData(aOutPosVector);

  aOutConVector = DXNewArray(TYPE_FLOAT, CATEGORY_REAL, 1, M);
  DXAddArrayData(aOutConVector, 0, N, NULL);
  fpOutConVector = (float*)DXGetArrayData(aOutConVector);
  /******************************************************************/

  /******************************************************************/  
  /* Multiply the input data component by 2.0                       */
  /******************************************************************/  
  for (i=0; i<N; i++){
    for (j=0; j<M ; j++){
      fpOutDataVector[j+M*i] = 2.0*fpDataVector[j+M*i];
      k = k + 1;
    }
  }
  /******************************************************************/  

  /******************************************************************/   
  /* Form output field **********************************************/
  /******************************************************************/   
  fOutField = DXNewField();
  
  DXSetComponentValue(fOutField, "data",        (Object)aOutDataVector);

  DXSetComponentValue(fOutField, "positions",   (Object)aPosVector);
  DXSetComponentValue(fOutField, "connections", (Object)aConVector);
  /* Note that we just passed the input Connection and Position components */
  /* to the output because they were not modified */

  DXSetStringAttribute((Object)aDataVector,"dep", "positions");
  
  DXEndField(fOutField);  /* this closes the field */
  /******************************************************************/   

  out[0] = (Object)fOutField;

  return OK;
}

