#include <dx/dx.h>
#include <stdio.h>
#include <math.h>


#define NUMP 72
#define NUMPP1 73
#define NUMT 36
#define NUMR 25
#define NVAR 23
/* --------------------------------------------------------------------------
 * readting
 *
 * This module reads the data files created by ting
 *
 * INPUTS
 *   filename: Name of file with data
 *
 *   model day: Day the data was taken
 *
 *   model hour: Hour the data was taken
 *
 *   model minute: Minute the data was taken
 *
 *   input variable: Determines which variable is to be included in the output
 *
 * OUTPUTS
 *   mhd: 3D field containing mhd data
 *
 *   tdmhd: 2D field containing mhd data
 */

Error m_ReadTING(Object *in, Object *out)
{
  /* Inputs */
  char *filename = NULL;
  int swapFlag;
  float mday, mhour, mmin;
  int invar;
  /* Outputs */

  Array aGrid = NULL;
  float *fpGrid;
  Array tdGrid = NULL;
  float *fptdGrid;
  Array aMHDData = NULL;
  double *fpMHDData;		/* Ptr to MHD data */ 
  Array tdMHDData = NULL;
  double *fptdMHDData;

  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int *ipMHDConnect;		/* Ptr to MHD Connection data */
  Array tdMHDConnect = NULL;    /* Array containing conncetions for MHD field */
  int *iptdMHDConnect;		/* Ptr to MHD Connection data */
  Field fMHD = NULL;
  Field gMHD = NULL;            /* MHD data Field */
  double *fpVar;	
  double *fptdVar;
  
   /* Other varibles */
  double day, hour, min;
  FILE *FpMHDFile;
  int h,i,j,k,iStatus;
  int iNumPts, iNumVars, iNx, iNy, iNz ;
  int iNumPtsm1;
  int liaEdge3d[3];             /* used in the creation of connections */
  int liaEdge3d_2d[3];
  int flag;
  

  float phi[NUMPP1], theta[NUMT];
  double rad[NUMR], height;
  double *dpr = rad;
  float fDeg2Rad;
  char *varname[NVAR]= {"T","U","V","PS","PS2","PN4S","PNO","OP","PN2D","TI","TE","E","O2P","W","Z","TNM","UNM","VNM","PSNM","PS2NM","N4SNM","PNONM","OPNM"};


  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&filename)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  } else if (strlen(filename) > 1024) {
    DXSetError(ERROR_BAD_PARAMETER,
	       "Filename must be less than 1024 characters.\nFilename=%s",
	       filename);
    goto error;
  }
  if(!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Model Day");
    goto error;
  } else if (!DXExtractFloat(in[1], &mday)){
    DXSetError(ERROR_BAD_PARAMETER, "Model day must be a float");
    goto error;
  }
 if(!in[2]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Model Hour");
    goto error;
  } else if (!DXExtractFloat(in[2], &mhour)){
    DXSetError(ERROR_BAD_PARAMETER, "Model hour must be a float");
    goto error;
  }
 if(!in[3]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Model Minute");
    goto error;
  } else if (!DXExtractFloat(in[3], &mmin)){
    DXSetError(ERROR_BAD_PARAMETER, "Model minute must be a float");
    goto error;
  }
 if(!in[4]){
   invar = -1;
 } else if (!DXExtractInteger(in[4], &invar)){
   DXSetError(ERROR_BAD_PARAMETER, "Variable position must be an integer");
   goto error;
 }

#ifdef DEBUG
 DXMessage("Filename %s",filename);
#endif
 iNz = NUMR; 
 iNy = NUMT;
 iNx = NUMPP1;
  
 liaEdge3d[0] = iNz; 
 liaEdge3d[1] = iNy;
 liaEdge3d[2] = iNx; 

 liaEdge3d_2d[0] = iNz;
 liaEdge3d_2d[1] = iNy;
 liaEdge3d_2d[2] = iNx - 1;
 
 
 iNumPts = iNz*iNy*iNx;
 iNumPtsm1 = iNz*iNy*(iNx-1);

 day = (double)mday;
 hour = (double)mhour; 
 min = (double)mmin;

 if(invar != -1) {/*import only the variable given*/
   iNumVars = 1;
   aMHDData = DXNewArray(TYPE_DOUBLE,CATEGORY_REAL,1,iNumVars);
   DXAddArrayData(aMHDData,0,iNumPts,NULL);
   fpVar = (double*)DXGetArrayData(aMHDData);
   
   tdMHDData = DXNewArray(TYPE_DOUBLE, CATEGORY_REAL,1,iNumVars);
   DXAddArrayData(tdMHDData,0,iNumPtsm1,NULL);
   fptdVar = (double*)DXGetArrayData(tdMHDData);
   
    /* call the fortran to fill the data array */
   invar = invar + 1;
   readtingvarhdf_(filename, fpVar, fptdVar, dpr, 
		   &day, &hour, &min, &flag, &invar);
 switch(flag)  {
  case 1:
    DXWarning("Data step not found in file, returning the first in the file");
    break;
  case 2:
    DXWarning("Data step not found in file, returning the last in the file");
    break;
  case -1:
   DXSetError(ERROR_BAD_PARAMETER,"Unable to open file");
   goto error;
   break;
  case -2:
   DXSetError(ERROR_BAD_PARAMETER,"Failed to find written_by field");
   goto error;
   break;
  case -3:
   DXSetError(ERROR_BAD_PARAMETER,"Unable to read C files");
   goto error;
   break;
  case -4:
   DXSetError(ERROR_BAD_PARAMETER,"Failed to determine number of steps");
   goto error;
   break;
  case -5:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading timestep info");
   goto error;
   break;
  case -6:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading time step");
   goto error;
   break;
  case -7:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing time step");
   goto error;
   break;
  case -8:
   DXSetError(ERROR_BAD_PARAMETER,"Time step not in file");
   goto error;
   break;
  case -9:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading data step info");
   goto error;
   break;
  case -10:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading data step");
   goto error;
   break;
  case -11:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing data step");
   goto error;
   break;
  case -12:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing file");
   goto error;
   break;
  }
 } else {
   /* Create the data array for the DX field */
   iNumVars = NVAR;

   aMHDData = DXNewArray(TYPE_DOUBLE,CATEGORY_REAL,1,iNumVars);
   DXAddArrayData(aMHDData,0,iNumPts,NULL);
   fpVar = (double*)DXGetArrayData(aMHDData);
      
   tdMHDData = DXNewArray(TYPE_DOUBLE, CATEGORY_REAL,1,iNumVars);
   DXAddArrayData(tdMHDData,0,iNumPtsm1,NULL);
   fptdVar = (double*)DXGetArrayData(tdMHDData);
   
   /* Get MHD parameters into the DX field */ 
   /* For MHDData we have the follow assignment of varibles
    *
    * 0 - Neutral temperature (K)
    * 1 - Neutral zonal wind (cm/s)
    * 2 - Neutral meridonial wind (cm/s)
    * 3 - O2 (molecular oxygen) mass mixing ratio
    * 4 - O (atomic oxygen) mass mixing ratio
    * 5 - N (4S) (atomic nitrogen) mass mixing ratio
    * 6 - NO mass mixing ratio
    * 7 - O+ density (cm^3)
    * 8 - N(2D) (atomic nitrogen) mass mixing ratio
    * 9 - Ion temperature (K)
    * 10 - Electron temperature (K)
    * 11 - Electron density (cm^3)
    * 12 - O2+ density (cm^3)
    * 13 - vertical wind (cm/s, needs to be multiplied by the scale height)
    * 14 - Height (cm, need to add 97km to get real height
    * 15 - Time smoothed neutral temperature (K)
    * 16 - Time smoothed neutral zonal wind
    * 17 - Time smoothed neutral meridional wind
    * 18 - Time smoothed O2
    * 19 - Time smoothed O
    * 20 - Time smoothed N (4S)
    * 21 - Time smoothed NO
    * 22 - Time smoothed O+
    */
   
   /* call the fortran to fill the data array */
   readtinghdf_(filename, fpVar, fptdVar, dpr, 
		&day, &hour, &min, &flag);
 switch(flag)  {
  case 1:
    DXWarning("Data step not found in file, returning the first in the file");
    break;
  case 2:
    DXWarning("Data step not found in file, returning the last in the file");
    break;
  case -1:
   DXSetError(ERROR_BAD_PARAMETER,"Unable to open file");
   goto error;
   break;
  case -2:
   DXSetError(ERROR_BAD_PARAMETER,"Failed to find written_by field");
   goto error;
   break;
  case -3:
   DXSetError(ERROR_BAD_PARAMETER,"Unable to read C files");
   goto error;
   break;
  case -4:
   DXSetError(ERROR_BAD_PARAMETER,"Failed to determine number of steps");
   goto error;
   break;
  case -5:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading timestep info");
   goto error;
   break;
  case -6:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading time step");
   goto error;
   break;
  case -7:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing time step");
   goto error;
   break;
  case -8:
   DXSetError(ERROR_BAD_PARAMETER,"Time step not in file");
   goto error;
   break;
  case -9:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading data step info");
   goto error;
   break;
  case -10:
   DXSetError(ERROR_BAD_PARAMETER,"Failed reading data step");
   goto error;
   break;
  case -11:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing data step");
   goto error;
   break;
  case -12:
   DXSetError(ERROR_BAD_PARAMETER,"Failed closing file");
   goto error;
   break;
  }
 }
 /* Make the positions array for the DXField */
 fDeg2Rad = M_PI/180.0;
 for(j=0;j<iNy;j++)
   theta[j] = (-87.5 + j*((2*87.5)/(iNy-1)))*fDeg2Rad;
 for(k=0;k<iNx-1;k++)
   phi[k] = k*(360.0/(iNx-1))*fDeg2Rad;
 phi[iNx-1] = phi[0];
 
 aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
 DXAddArrayData(aGrid,0,iNumPts,NULL);
 fpGrid = (float*)DXGetArrayData(aGrid);
 for(i=0;i<iNz;i++)     
   for(j=0;j<iNy;j++)  
     for(k=0;k<iNx;k++)
       {
	 fpGrid[0+3*(k+(iNx)*(j+iNy*i))]= //i;
	   rad[i]*cos(theta[j])*cos(phi[k%iNx]);
	 fpGrid[1+3*(k+(iNx)*(j+iNy*i))]= //theta[j];
	   rad[i]*cos(theta[j])*sin(phi[k%iNx]); 
	 fpGrid[2+3*(k+(iNx)*(j+iNy*i))]= //phi[k];
	   rad[i]*sin(theta[j]);
       }
 /*Make the positions array for the 2D output*/
 
 tdGrid = DXNewArray(TYPE_FLOAT, CATEGORY_REAL,1,3);
 DXAddArrayData(tdGrid,0,iNumPtsm1,NULL);
 fptdGrid = (float*)DXGetArrayData(tdGrid);
  for(i=0;i<iNz;i++)     
   for(j=0;j<iNy;j++)  
     for(k=0;k<iNx-1;k++)
       {
	 fptdGrid[0+3*(k+(iNx-1)*(j+iNy*i))]= -1;
	   //rad[i]*cos(theta[j])*cos(phi[k]);
	 fptdGrid[1+3*(k+(iNx-1)*(j+iNy*i))]= theta[j]/fDeg2Rad;
	   //rad[i]*cos(theta[j])*sin(phi[k]); 
	 fptdGrid[2+3*(k+(iNx-1)*(j+iNy*i))]= phi[k]/fDeg2Rad;
	   //rad[i]*sin(theta[j]);
       }
  
 /* Do the final steps for the DXField */
 fMHD = DXNewField();
 if(!fMHD) {
   DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
   goto error;
 }
 
 /* Set the dependence of the Data Array to be on positons*/
 if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
   goto error;
 }
 /* Set the MHD Data array as the data component of the MHD Field*/
 if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
   goto error;
 }
 /* Set the MHD Pts array as the position component of the MHD Field*/
 if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
   goto error;
 }
 aMHDConnect = DXMakeGridConnectionsV(3,liaEdge3d);
 if (!aMHDConnect) {
   DXSetError(ERROR_INTERNAL,"Couldn't Make MHD Grid Connections");
   goto error;
 }
 
 if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set MHD Grid Connections");
   goto error;
 }
 
 if(!DXEndField(fMHD)) {
   DXSetError(ERROR_INTERNAL,"Couldn't complete field");
   goto error;
 }

 /* Do the final steps for the DXField */

 gMHD = DXNewField();
 if(!gMHD) {
   DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
   goto error;
 }

 // Set the dependence of the Data Array to be on positons

 if(!DXSetStringAttribute((Object)tdMHDData,"dep","positions")) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
   goto error;
 }

 // Set the MHD Data array as the data component of the MHD Field

 if(!DXSetComponentValue(gMHD,"data",(Object)tdMHDData)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
   goto error;
 }

 // Set the MHD Pts array as the position component of the MHD Field

 if(!DXSetComponentValue(gMHD,"positions",(Object)tdGrid)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
   goto error;
 }

 tdMHDConnect = DXMakeGridConnectionsV(3,liaEdge3d_2d);
 if (!tdMHDConnect) {
   DXSetError(ERROR_INTERNAL,"Couldn't Make MHD Grid Connections");
   goto error;
 }
 
 if (!DXSetComponentValue(gMHD,"connections",(Object)tdMHDConnect)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set MHD Grid Connections");
   goto error;
 }
 
 if(!DXEndField(gMHD)) {
   DXSetError(ERROR_INTERNAL,"Couldn't complete field");
   goto error;
 }

#ifdef DEBUG
 DXMessage("After Field ");
#endif
 
 out[0] = (Object) fMHD;
 out[1] = (Object) gMHD;
 return OK;
 
 error:
 DXDelete((Object) fMHD);
 DXDelete((Object) aMHDData);
 DXDelete((Object) aGrid);
 DXDelete((Object) aMHDConnect);
 return ERROR;
}
