#include <dx/dx.h>

static char caaMonths[12][4] = {
	{'J','a','n','\0'},
	{'F','e','b','\0'},
	{'M','a','r','\0'},
	{'A','p','r','\0'},
	{'M','a','y','\0'},
	{'J','u','n','\0'},
	{'J','u','l','\0'},
	{'A','u','g','\0'},
	{'S','e','p','\0'},
	{'O','c','t','\0'},
	{'N','o','v','\0'},
	{'D','e','c','\0'}
};

/* Function Prototypes for Year/Day Manipulations */

int  iDayOfYear(int , int , int );
void MonthDay(int, int, int *, int *);

Error
m_YrDayHr(Object *in, Object *out)
{
   /***********************/
   /*  Declare variables  */
   /***********************/
   Array a=NULL;
   float  Sim_Time;
   int  Start_Year;
   int  Start_Hr;
   int  Start_Day;
   int  Start_Mon;
   char  *Date = NULL;
   char  *Date2 = NULL;
   float  Normalization;
   float  Correction;
   int  *UT_Time;

   double tout;
   int label_flags,iday,ihr,imin,isec,iYrDay,UT_Time_size,iMon;
   char newlabel[128], newlabel1[128], newlabel2[128], date[30];

   /***********************/
   /*  Get input values   */
   /***********************/
   /* Get Sim_Time's value */
   if (!in[0]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Sim_Time = 0.0;
   }
   else if (!DXExtractFloat(in[0], &Sim_Time)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Start_Year's value */ 
   if (!in[1]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Start_Year = 0;
   }
   else if (!DXExtractInteger(in[1], &Start_Year)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Start_Hr's value */ 
   if (!in[2]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Start_Hr = 0;
   }
   else if (!DXExtractInteger(in[2], &Start_Hr)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Start_Day's value */ 
   if (!in[3]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Start_Day = 0;
   }
   else if (!DXExtractInteger(in[3], &Start_Day)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Start_Mon's value */ 
   if (!in[4]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Start_Mon = 1;
   }
   else if (!DXExtractInteger(in[4], &Start_Mon)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Normalization's value */
   if (!in[5]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
      Normalization = 0.00027777777777777777;
   }
   else if (!DXExtractFloat(in[5], &Normalization)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }

   /* Get Correction's value */
   if (!in[6]) {
     /* DXSetError(ERROR_BAD_PARAMETER,"missing parameter");
	goto error; */
     Correction = 0.0;
   }
   else if (!DXExtractFloat(in[6], &Correction)) {   
     DXSetError(ERROR_BAD_PARAMETER, "parameter must be a string");
     goto error;
   }


   /***********************/
   /* Function's Body     */
   /***********************/

   /***********************/
   /*  Set output values  */
   /***********************/
   

   /* set  UT_Time  */
   a = DXNewArray(TYPE_INT,CATEGORY_REAL,1,5);
   if (!a )
     goto error;
   if (! DXAddArrayData(a, 0, 1, NULL))
    goto error;
   UT_Time = (int *)DXGetArrayData(a);
   UT_Time_size = 5;

   tout = Sim_Time*Normalization+Correction;
   /*	Assume tout is in hours at this point, output string as hr:min    */
   tout = tout + Start_Hr;
   ihr =  tout;
   iday =  ihr/24;
   imin =  (tout-ihr) *60;
   isec = 60*((tout-ihr)*60. -imin);
   ihr = ihr - 24*(iday);
   
   iday = iday+Start_Day;
   
   /* Set UT Time output Vector  - Assumes Interval does not overlap 
      Year boundary */
   iYrDay = iDayOfYear(Start_Year,Start_Mon,iday);
   UT_Time[0] = Start_Year;
   UT_Time[1] = iYrDay;
   UT_Time[2] = ihr;
   UT_Time[3] = imin;
   UT_Time[4] = isec;
   
   /* Label Vector Items */

   /* Convert iYrDay in to Mon Day pair */
        
   MonthDay(Start_Year,iYrDay,&iMon,&iday);
   sprintf(newlabel1,"%s %02d",caaMonths[iMon-1],iday);

   /* Set Date's value */
   /*OMset_name_str_val(DayHrmin_stack_id, OMstr_to_name("Date"), newlabel);*/
   /* Set Date2's value */
   sprintf(newlabel2,"%02d:%02d:%02d%s",ihr,imin,
	   isec,"UT");
   /*OMset_name_str_val(DayHrmin_stack_id, OMstr_to_name("Date2"), newlabel);*/
   out[0] = (Object) a;
   out[1] = (Object) DXMakeString(newlabel1);
   out[2] = (Object) DXMakeString(newlabel2);
      
   return OK;
 error:
   DXDelete((Object) a);
   return ERROR;
   
}


static int day_tab[2][13] = {
  {0,31,28,31,30,31,30,31,31,30,31,30,31},
  {0,31,29,31,30,31,30,31,31,30,31,30,31}
};

int iDayOfYear(int iYr,int iMon,int iDay) {
  
  int i, iLeap;
  iLeap = iYr%4 == 0 && iYr%100 !=0 || iYr%400 == 0;
  for(i=1;i<iMon;i++)
    iDay+=day_tab[iLeap][i];
  return(iDay);

}

void MonthDay(int iYear, int iYearDay, int *ipMonth, int *ipMonthDay) {
   int i, iLeap;
   
   iLeap = iYear%4 == 0 && iYear%100 !=0 || iYear%400 ==0 ;
   for(i=1;iYearDay > day_tab[iLeap][i]; i++)
     iYearDay -= day_tab[iLeap][i];
   *ipMonth = i;
   *ipMonthDay = iYearDay;
}
