%escalexy(epsFile,side,x,y,offset,angle,length,startValue,step,
%         endValue,vForm,vVisible,fontSize,lineWidth,shortTicLength,
%         longTicLength,maxValues,space,color)
% linear scaled axis
% written by Stefan Mueller stefan.mueller@fgan.de 1997

function escalexy(epsFile,side,x,y,offset,angle,elength,startValue,step,...
		  endValue,vForm,vVisible,fontSize,lineWidth,shortTicLength,...
		  longTicLength,maxValues,space,color,AxisLabels)
  if (nargin~=19)
%    usage(...
%   'escalexy(epsFile,side,x,y,offset,angle,length,startValue,step,endValue,vForm,vVisible,fontSize,lineWidth,shortTicLength,longTicLength,maxValues,space,color)');
  end

  if side=='s'
    startPos=0;
    xLength=elength;
    yLength=0;
    moveForm=sprintf('%%1.2f %1.2f moveto\n',-offset);
    ticLineForm='0 %1.2f neg rlineto\n';
    moveValueForm=sprintf('0 -%1.2f rmoveto\n',space+fontSize*0.72);
    showForm='(%s) dup stringwidth pop dup 2 div sub neg 0 rmoveto show\n';
  elseif side=='n'
    startPos=0;
    xLength=elength;
    yLength=0;
    moveForm=sprintf('%%1.2f %1.2f moveto\n',offset);
    ticLineForm='0 %1.2f rlineto\n';
    moveValueForm=sprintf('0 %1.2f rmoveto\n',space);
    showForm='(%s) dup stringwidth pop dup 2 div sub neg 0 rmoveto show\n';
  elseif side=='w'
    startPos=0;
    xLength=0;
    yLength=elength;
    moveForm=sprintf('%1.2f %%1.2f moveto\n',-offset);
    ticLineForm='%1.2f neg 0 rlineto\n';
    moveValueForm=sprintf('-%1.2f -%1.2f 0.28 mul rmoveto\n',space,fontSize);
    showForm='(%s) dup stringwidth pop neg 0 rmoveto show\n';
  elseif side=='e'
    startPos=0;
    xLength=0;
    yLength=elength;
    moveForm=sprintf('%1.2f %%1.2f moveto\n',offset);
    ticLineForm='%1.2f 0 rlineto\n';
    moveValueForm=sprintf('%1.2f -%1.2f 0.28 mul rmoveto\n',space,fontSize);
    showForm='(%s) show\n';
  end
  startEndDiff=endValue-startValue;
  NumberSubTicks = 4;
  axisFac        = elength/startEndDiff;

  if (length(step)==3) % If fixed steps
    if step(2)==0
      %autoscale
    signOfDelta=sign(startEndDiff);
    deltaLabel=eticdis(signOfDelta*startEndDiff,maxValues);
    deltaTic=signOfDelta*deltaLabel/NumberSubTicks;    
    if (isempty(AxisLabels))
	AxisLabels  = num2str([step(1):deltaLabel:step(3)]');
    end
    else
      %fixscale
      signOfDelta=sign(step(2));
      deltaLabel=signOfDelta*step(2);
      deltaTic=signOfDelta*deltaLabel/NumberSubTicks;
      if (isempty(AxisLabels))
	AxisLabels=num2str([step(1):step(2):step(3)]',2);
      end
    end
  else % If non-fixed steps
    signOfDelta=sign(step(2)-step(1));
    deltaLabel=signOfDelta*(step(2)-step(1));
    deltaTic=signOfDelta*deltaLabel/NumberSubTicks;
    if (size(step,2) > 1)
      step = step';
    end
    if (isempty(AxisLabels))
      AxisLabels=num2str(step,2);
    end
    if (~isstr(AxisLabels(1)))
      AxisLabels=num2str(step,2);
    end
  end
  if vForm==0
    expo=-log10(deltaLabel);
    if rem(expo,1)>0
      expo=expo+1;
    end
    autoForm=fix(expo);
    if autoForm>0
      vForm=autoForm;
    end
  end
  
  %start ticNo and offset
%  nShortTics=rem(startValue,deltaLabel)/deltaLabel*NumberSubTicks*signOfDelta
%  if (nShortTics<0)
%    nShortTics = NumberSubTicks+nShortTics
%  end
%  if rem(nShortTics,1)>0
%    iTic=fix(nShortTics)+1;
%    ticOffset=(iTic-nShortTics)*signOfDelta*deltaLabel/NumberSubTicks;
%  else 
%    iTic=nShortTics; 
%    ticOffset=0;
%  end
  iTic=0;
  ticOffset=0;
  
  firstTicValue=startValue+ticOffset;
  
  % start draw
  fprintf(epsFile,'%1.2f %1.2f translate\n',x,y);
  fprintf(epsFile,'%1.2f rotate\n',angle);
  fprintf(epsFile,'currentrgbcolor %1.2f %1.2f %1.2f setrgbcolor\n',...
          color(1),color(2),color(3));         
  fprintf(epsFile,'newpath\n');
  fprintf(epsFile,'/Helvetica findfont %d scalefont setfont\n',fontSize);

  %tics
  currentTic=0;
  currentValue=firstTicValue;
  k            = 1;
  j            = 1;
  L            = length(step);
  
  if (size(AxisLabels,1) == 1)
    AxisLabels = AxisLabels';
  end
  
%  while ( (k <= size(AxisLabels,1)) & ...
%	  (signOfDelta*currentValue <= signOfDelta*endValue))
  while ( k <= size(AxisLabels,1) ) 
    
    currentPos = startPos+axisFac*(currentValue-startValue);

    fprintf(epsFile,moveForm,currentPos);

    if (rem(iTic,NumberSubTicks)~=0) 
      % short tics
      fprintf(epsFile,ticLineForm,shortTicLength);
    else
      % long tics
      if (side == 's') | (side == 'w')
%fprintf('Long Ticks AxisLabels = %s, iTic = %d\n',AxisLabels(k,:),iTic);
	  L = size(AxisLabels,2);
	  if ( strncmp(AxisLabels(k,1:L),blanks(L),L) == 1)
	    % If blanks, print short tic
	    fprintf(epsFile,ticLineForm,1.0*longTicLength);
	  else 	    % If not blanks, print long tic
	    if (k == size(AxisLabels,1) | k == 1) 
	      TicFactor = 1.5;
	    else
	      TicFactor = 1.5;
	    end
	    fprintf(epsFile,ticLineForm,TicFactor*longTicLength);
	  end
      end
      
      if (side == 'w')
	%fprintf(epsFile,ticLineForm,1.5*longTicLength);
      end

      if (side == 'e')
	fprintf(epsFile,ticLineForm,1.5*longTicLength);
      end

      % value
      if vVisible & (offset>=0 | abs(currentValue)>1e-15)
	fprintf(epsFile,moveValueForm);
	if (side == 's') | (side == 'w')
	  valueForm = sprintf('%%s');
	  if (k <= size(AxisLabels,1))
	    currentStr = AxisLabels(k,:);
	    I          = find(currentStr ~= ' ');
	    currentStr = currentStr(I);
	    valueForm  = sprintf(valueForm,currentStr);
	    fprintf(epsFile,showForm,valueForm);
	  end
	end
	if side == 'e'
        fprintf(epsFile,moveValueForm);
        valueForm=sprintf('%%1.%df',vForm);
        valueForm=sprintf(valueForm,currentValue);
        fprintf(epsFile,showForm,valueForm);
	end
	k = k + 1;
      end
    
    end % If rem

    if (rem(iTic,NumberSubTicks)==0 & iTic > 0) 
      if (length(step) > 3) & (j+2 <= length(step))
	deltaLabel   = step(j+2)-step(j+1);
	deltaTic     = signOfDelta*deltaLabel/NumberSubTicks;
      end
      j = j + 1;    
    end
    
    iTic=iTic+1;
    currentTic=currentTic+1;
    currentValue=currentValue+deltaTic;

  end

  %axis
  fprintf(epsFile,moveForm,startPos);
  fprintf(epsFile,'%1.2f %1.2f rlineto\n',xLength,yLength);
  fprintf(epsFile,'%1.2f setlinewidth\n',lineWidth);
  fprintf(epsFile,'stroke\n');
  fprintf(epsFile,'setrgbcolor\n'); 
  fprintf(epsFile,'%1.2f rotate\n',-angle);
  fprintf(epsFile,'%1.2f %1.2f translate\n',-x,-y);
