function s = linetype(Linecolor,Linetype)
%LINETYPE Create a string corresponding to a numeric linetype.
%
%    LINETYPE(Linetype) returns a different line type and line color for
%    each integer and repeats when Linetype > 24.  Linetypes are
%    '-',':','-.','--' and colors are r,g,b,m,y and c.
%
%    LINETYPE(Linecolor,Linetype) returns a different line type and line
%    color.
%
%    Linecolor is an integer greater than zero.  If integer is 
%    greater than number of colors (6), colors are repeated.  
%    Available colors are r,g,b,m,y and c.
%    
%    Linetype is an integer (1-4).  '-',':','-.','--' are the choices.
%
%    See also MARKERTYPE, LINES, COLORMAP.

if (nargin == 1)
  Linetype  = 1 + mod(Linecolor - 1,4);
end

colors{1}  = ['r' ;'g' ;'b' ;'m' ;'y' ;'c' ];
colors{2}  = ['r:';'g:';'b:';'m:';'y:';'c:'];
colors{3}  = ['r-.';'g-.';'b-.';'m-.';'y-.';'c-.'];
colors{4}  = ['r--';'g--';'b--';'m--';'y--';'c--'];

lc         = 1 + mod(Linecolor - 1,6);
s          = colors{Linetype}(lc,:);

