%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot all 1-day data as time series
% Needs Matlab hdf scripts.  Tested on RHEL 3.0 with Matlab 6.5.0.  
% R.S. Weigel 08/26/2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get global variables and path setting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CISM_DX_GLOBALS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load HDF data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eps_filename = [CISM_DX_TEMPDIR,'make_plots_1_day.eps'];
Title        = 'One day data sets';
hdf_dir      = [CISM_DX_DATA_ROOT,'measurements/uniform_time_series/'];
FILES        = {[hdf_dir,'CISM_DX_1_day_data.hdf']};

make_plots_prep;
make_plots_eps;

break

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prep scales for plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eglobpar2;              % get access to global parameters
einit2;

eWinWidth          = 165; % mm  
eWinHeight         = 185; % mm

ePlotAreaPos       = [20 40]; % x y position of left bottom corner of plot area
ePlotAreaWidth     = 100;     % mm
ePlotAreaHeight    = 165;     % mm
ePlotTitleDistance = 4;

eopen2(eps_filename); % open eps-file and write eps-head

lcolors = [1 0 0;0 1 0;0 0 1;0 0 0]; % RGB colors
lcolors = repmat(lcolors,ceil(size(Z1,2)/size(lcolors,2)),1); % Repeat colors

Scale         = 2;
Ts            = 1;
L             = size(Z1,1)/Ts;
[temp,I]      = year_labels([1:L+1]',1963);
XLabels       = repmat(blanks(size(temp,2)),length(I),1);
Io            = [1:4:length(I)];
XLabels(Io,:) = temp(Io,:);
eImageLegendPos = [0 -5];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2-D time series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ePlotAreaPos    = [15 20]; % x y position of left bottom corner of plot area
ePlotAreaHeight = 50;  % mm

eaxes2(I,XLabels,[1 1 10],[]);
for i = 1:size(Z2{1},2)
  temp       = Z2{1}(:,i);
  temp       = scale_nonflag(temp,3,[],CISM_DX_FLAG);
  temp       = scale_nonflag(temp,5,[],CISM_DX_FLAG);
  Z2{1}(:,i) = reflag(temp,CISM_DX_FLAG,mean_nonflag(temp,1,CISM_DX_FLAG));
  Z2{1}(:,i) = Z2{1}(:,i) - Z2{1}(1,i);
end

% Need to change font size on colorbar text to 3
eImageLegendLabelText = Names2{1};
eimagesc_2d(flipud(Z2{1}'),ecolors2(3),'e');
eplot2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1-D time series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ePlotAreaPos    = [15 80]; % x y position of left bottom corner of plot area
ePlotAreaHeight = 85;  % mm

eaxes2(I,XLabels,[0 10 10*(1+size(Z1,2))],[]);
ePlotTitleText = 'One Day Data Sets (arb. scales)';

for j = 1:size(Z1,2)

  temp = Z1(:,j);
  temp = scale_nonflag(temp,Scale,[],CISM_DX_FLAG);
  temp = reflag(temp,CISM_DX_FLAG,mean_nonflag(temp,1,CISM_DX_FLAG));
  eplot2([1:L]',temp+10*j,'',0,lcolors(j,:));    

  x1   = L+0.05*L;
  y1   = 10*j+10;
  xpos = ePlotAreaPos(1)+(x1-ePlotAreaXValueStart)*ePlotAreaXFac/eFac;
  ypos = ePlotAreaPos(2)+(y1-ePlotAreaYValueStart)*ePlotAreaYFac/eFac;
  etext2(Names1{j},xpos,ypos,3,1,1,0,lcolors(j,:))
  
end

eplot2; 
eclose2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
