function EEPLOT(Xc,Yc,P)
%EEPLOT Easier EPLOT
%
%   EEPLOT(Y) Creates a line plot where the lines are taken as the
%   columns of matrix Y or from the elements of cell array Y.
%
%   EEPLOT(X,Y) x axis values are taken from columns of X or elements of
%   cell array X.  If X (or its subelements) are empty, x axis values are
%   the index number of the line from Y.
%
%   EEPLOT(X,Y,P) or
%   EEPLOT(Y,P) 
%
%   Sets plot parameters, where P is a strucutre with fields 
%   
%   P.FileName     = 'eeplot.eps';
%   P.Title        = [] ; 
%   P.Xlims        = [] ; A three element array, where xmin = Xlims(1), xmax =
%                         Xlims(3), and step = Xlims(2).  
%   P.Ylims        = [] ; Same form as Xlims.
%   P.Legend       = '' ; Legend is a cell array.
%   P.LineColors   = [] ; A Nx3 matrix of rgb colors, where N is the
%                         number of lines.
%   P.LineWidths   = 0.5; A vector with N elements (see eplot width parameter)
%   P.DashLengths  = 0  ; A vector with N elements (see eplot dash parameter)
% 
%   P.e*, where e* is any of the global parameters listed in the help for
%   EPLOT, EOPEN, and EAXES.
%
%   See also EEPLOT_DEMO.


if ~iscell(Yc)
  L=size(Yc,2);
  if (size(Yc,1)==1)
    L = 1;
  end
else
  L=length(Yc);
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin == 1)

  Yc = Xc; clear Xc;

  P.filename = 'eeplot.eps';

  if (size(Yc,1) == 1) & (~iscell(Yc)) % If Y is a an single-row vector.
    Yc = Yc';
  end

  if iscell(Yc)
    for i = 1:L
      Xc{i} = [];
    end
  else
    Xc = [];    
  end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin == 2)

  if isstruct(Yc) 
    P  = Yc; clear Yc;
  else
    P.filename = 'eeplot.eps';
  end

  Yc = Xc; clear Xc;

  if (size(Yc,1) == 1) & (~iscell(Yc)) % If Y is a an single-row vector.
    Yc = Yc';
  end
  
  if iscell(Yc)
    for i = 1:L
      Xc{i} = [];
    end
  else
    Xc = [];    
  end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (isfield(P,'FileName'))
  P.filename = P.FileName;
end

if (~isfield(P,'filename'))
  filename = 'eeplot.eps';
else
  filename = P.filename;
end

if (~isfield(P,'Legend'))
  for i = 1:L
    P.Legend{i} = '';
  end
end

if (~isfield(P,'LineColors'))
  LineColors = [1 0 0 ; ...
		0 1 0 ; ...
		0 0 1 ; ...
		0 1 1 ; ...
		0 0 0 ];
else
  LineColors = P.LineColors;
end

if (~isfield(P,'LineWidths'))
  LineWidths = 0.5*ones(L);
else
  LineWidths = P.LineWidths;
end

if (~isfield(P,'DashLengths'))
  DashLengths = zeros(L,1);
else
  DashLengths = P.DashLengths;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (~isfield(P,'Ylims'))

  [ymin,ystep,ymax] = eeticks(Yc,'y');
  
else

  ymin  = P.Ylims(1);
  ymax  = P.Ylims(3);
  ystep = P.Ylims(2);

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (~isfield(P,'Xlims'))

  if (isempty(Xc))
    if iscell(Yc)
      NXc = max(max(Yc{:}));
    else
      NXc = size(Yc,1);
    end
    [xmin,xstep,xmax] = eeticks(NXc,'y');  
  else
    [xmin,xstep,xmax] = eeticks(Xc,'y');  
  end

else

  xmin  = P.Xlims(1);
  xmax  = P.Xlims(3);
  xstep = P.Xlims(2);

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

eglobpar;         % get global parameters
einit;
eopen(filename);  % open eps-file and write eps-head

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (isfield(P,'eXAxisSouthLabelText'));
  eXAxisSouthLabelText = P.eXAxisSouthLabelText;
end
if (isfield(P,'eYAxisWestLabelText'));
  eYAxisWestLabelText = P.eYAxisWestLabelText;
end
if (isfield(P,'ePlotTitleText'));
  ePlotTitleText = P.ePlotTitleText;
end
if (isfield(P,'Title'));
  ePlotTitleText = P.Title;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (0)
eaxes([xmin xstep xmax],[ymin ystep ymax])
eXAxisSouthValueVisible=0;              % no values visible
valuePos=eXAxisSouthValuePos;
for posNo = 1:size(valuePos,1)
  etext(num2str(posNo),valuePos(posNo,1),...
	valuePos(posNo,2)-eAxesTicLongLength,4,0);
end
end

if (length(P.Legend) ~= L)
  error('Legend has incorrect number of elements');
end

for i = 1:L

  if iscell(Yc) & iscell(Xc)

    if (~isfield(P,'Legend'))
      P.Legend{i} = '';
    end

    if (size(Yc{i},1)==1)
      Yc{i} = Yc{i}';
      Xc{i} = Xc{i}';
    end

    % eplot can't generates bad postscript if unbalanced parenthesis
    Il = findstr('(',P.Legend{i});
    Ir = findstr(')',P.Legend{i});    
    if (length(Il) ~= length(Ir))
      error('Unbalanced parenthesis in legend string: %s',P.Legend{i});
    end

    I = find(isnan(Yc{i})==0);

    if isempty(Xc{i})
      x = [1:length(Yc{i})]';
      eplot(x(I),Yc{i}(I),P.Legend{i},DashLengths(i),LineColors(i,:));
    else
      eplot(Xc{i}(I),Yc{i}(I),P.Legend{i},DashLengths(i),LineColors(i,:));
    end
  
  end

  if ~iscell(Yc) & ~iscell(Xc)
    
    if (~isfield(P,'Legend'))
      P.Legend{i} = '';
    end

    if (size(Yc,1)==1)
      Yc = Yc';
      Xc = Xc';
    end
    
    % eplot generates bad postscript if unbalanced parenthesis
    Il = findstr('(',P.Legend{i});
    Ir = findstr(')',P.Legend{i});    
    if (length(Il) ~= length(Ir))
      error('Unbalanced parenthesis in legend string: %s',P.Legend{i});
    end

    I = find(isnan(Yc(:,i))==0);
    
    if isempty(Xc) 
      x = [1:size(Yc,1)]';
      
      eplot(x(I),Yc(I,i),P.Legend{i},DashLengths(i),...
	    LineColors(i,:),LineWidths(i));  
    elseif (size(Xc,2) == 1)
      eplot(Xc(I,1),Yc(I,i),P.Legend{i},DashLengths(i),...
	    LineColors(i,:),LineWidths(i));  
    else
      eplot(Xc(I,i),Yc(I,i),P.Legend{i},DashLengths(i),...
	    LineColors(i,:),LineWidths(i));  
    end

  
  end

end

eplot; 
eclose;
%ebbox(1,P.filename);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
