function year_doy_grid_plot(D,Year_o,EPS_FILENAME,Title,Xlabel,Ylabel)
%YEAR_DOY_GRID_PLOT Plot output from YEAR_DOY_GRID
%
%   YEAR_DOY_GRID_PLOT(D) Assumes matrix D has 365 rows corresponding to
%   day of year and an arbitrary number of columns.
%   
%   YEAR_DOY_GRID_PLOT(D,Year_o,EPS_Filename,Title,XLabel,Ylabel) where
%   EPS_FILENAME defaults to year_doy_grid_plot.eps, Title = '', Xlabel =
%   'Day of Year', and Ylabel = 'Universal Time'.
%   
%   See also YEAR_DOY_GRID, YEAR_DOY_GRID_DEMO.

% R.S. Weigel, 07/08/2004
if (nargin < 2)
  Year_o = 0;
end
if (nargin < 3)
  EPS_FILENAME = 'year_doy_grid_plot.eps';
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create a contour plot using epstk
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eglobpar2;
einit2;

eWinWidth       = 165; % mm  
eWinHeight      = 145; % mm

eopen2(EPS_FILENAME);

ePlotAreaPos    = [20 25]; % x y position of left bottom corner of plot area
ePlotAreaWidth  = 110;     % mm
ePlotAreaHeight = 110;     % mm

if (nargin < 4)
  ePlotTitleText = '';
else
  ePlotTitleText = Title;
end
if (nargin < 5)
  eXAxisSouthLabelText  = 'Year';
else
  eXAxisSouthLabelText  = Xlabel;
end
if (nargin < 6)
  eYAxisWestLabelText   = 'Day of Year';
else
  eYAxisSouthLabelText  = Ylabel;
end

ePlotTitleDistance    = 2;
eImageLegendPos       = [0 -5];

Nyr = size(D,2);
dYr = 1;
if (Nyr > 5),dYr = 2;,end
if (Nyr > 13),dYr = 4;,end
if (Nyr > 20),dYr = 5;,end
Xvalues = [Year_o:dYr:(Year_o+Nyr)];
Xlabels = num2str(Xvalues');

eaxes2(Xvalues,Xlabels,[0 30 365],[]);
eimagesc_2d(flipud(D),ecolors2(3,8),'e');
%eplot2([0 24],[79 79],'Equinoxes and Solstices',1);
%eplot2([0 24],[166 166],'',1);
%eplot2([0 24],[264 264],'',1);
%eplot2([0 24],[356 356],'',1);
eplot2;
eclose2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




