function XYZ = hdz2xyz_nonflag(HDZ,D_unit,FLAG_VALUE,EXACT)
%HDZ2XYZ Transform vector from HDZ to XYZ coordinate system.  
%
%    Y = HDZ2XYZ_NONFLAG(X)
%
%    X has rows of [H, D, Z] (Z is optional and unchanged).
%   
%    Y = HDZ2XYZ_NONFLAG(X,D_unit,FLAG_VALUE,EXACT)
%
%    D_unit is either arc-min, arc-sec, or degrees (default).
%    Default FLAG_VALUE=99999, EXACT=1.
%
%    See also *_NONFLAG.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 2)
  D_unit = 'degrees';
end
if (nargin < 3)
  FLAG_VALUE  = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

[jnk,IH] = reflag(HDZ(:,1),FLAG_VALUE,FLAG_VALUE,EXACT);
[jnk,ID] = reflag(HDZ(:,2),FLAG_VALUE,FLAG_VALUE,EXACT);
I        = union(IH,ID); % One or the other data point is FLAG.
clear jnk IH ID;

% 1 arc-min = (1/60) degree
% 1 arc-sec = (1/3600) degree

if (strmatch(D_unit,'arc-min','exact'))
  F = 60.0;
elseif (strmatch(D_unit,'arc-sec','exact'))
  F = 3600.0
elseif (strmatch(D_unit,'degrees','exact'))
  F = 1.0;
else 
  error('D_unit must be arc-min, arc-sec, or degrees');
end

XYZ      = zeros(size(HDZ));
XYZ(:,1) = HDZ(:,1).*cos((pi/180.0)*(HDZ(:,2)/F)); 

XYZ(I,1) = FLAG_VALUE;
XYZ(:,2) = HDZ(:,1).*sin((pi/180.0)*(HDZ(:,2)/F));
XYZ(I,2) = FLAG_VALUE;


