function parse_wsa_data(obsstr)
%PARSE_WSA_DATA Parse WSA solar wind velocity data file
%
% PARSE_WSA_DATA(obs_str) Parses the WSA data file.  
%
% (1) Extract the solar wind velocity predictions.
% (2) Create an archive file of the data.
%
%

% R.S. Weigel 04/01/2004
  
CISM_DX_GLOBALS;

for Ts = 1:7

clear FILE_BASE FILE_STAND FILE_DAILY obspread
% Output file from wsa
FILE_BASE=sprintf('%spred%d.dat',obsstr,Ts); 
% Output file of wsa put in standard form
FILE_STAND=sprintf('YYYY_MM_DD_HH_MM_SS_V_Polarity_Ndays_%s_%d.dat',obsstr,Ts);
% Daily average file of wsa predictions in standard form
FILE_DAILY=sprintf('YYYY_MM_DD_HH_MM_SS_V_Polarity_Ndays_%s_%d.daily.dat',obsstr,Ts);

obspred = load(FILE_BASE);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t  = obspred(:,1)-0.5;% day number. (-0.5 since Julian day starts at noon)
tf = t-floor(t);      % fraction of day.
th = floor(24*tf);    % hour of day
tf = tf*24-th;        % fraction of hour
tm = floor(60*tf);    % minute of hour
tf = tf*60-tm;        % fraction of hour
ts = round(60*tf);    % rounded second of minute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create a matrix from current files with standard time stamps.
for i = 1:size(obspred,1)
  obspred_curr(i,:) = [obspred(i,4),obspred(i,2),obspred(i,3),...
		       th(i),tm(i),ts(i),...
		       obspred(i,6),obspred(i,7),obspred(i,8)];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write standard files
fid  = fopen(FILE_STAND);
if (fid == -1)
  obspred       = [obspred_curr];
else
  fclose(fid);
  obspred_old   = load(FILE_STAND);
  obspred       = [obspred_old;obspred_curr];
end

  obspred       = unique(obspred,'rows');

fid = fopen(FILE_STAND,'w');
for i = 1:size(obspred,1)
      fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f %+8d %8d\n',...
	       obspred(i,1),obspred(i,2),obspred(i,3),...
	       obspred(i,4),obspred(i,5),obspred(i,6),...	  
	       obspred(i,7),obspred(i,8),obspred(i,9));
end
      fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create a file of daily averages
DAY_F      = 8+now;
DAY_I      = day_number(obspred(1,1:3));
DAYS_AVAIL = day_number(obspred(:,1:3));
DAYS_OUT   = [DAY_I:DAY_F];

L=length(DAYS_OUT);

for i = 1:L
  I = find(DAYS_AVAIL == DAYS_OUT(i));
  if (~isempty(I))
    Pol(i,:)   = mean(obspred(I,8:9),1);      
    V(i,1)     = mean(obspred(I,7));
    Date(i,:)  = obspred(I(end),1:6);
  else
    Pol(i,:)   = [CISM_DX_FLAG, CISM_DX_FLAG];
    V(i,1)     = CISM_DX_FLAG;
    Date(i,:)  = [ymd(DAYS_OUT(i)),23,59,59];
  end
end  

obspred = [Date V Pol];

Date = []; % Should use "clear Date V Pol", but Octave does
V    = []; % not allow "clear" to be used in function body!
Pol  = [];

fid = fopen(FILE_DAILY,'w');
for i = 1:size(obspred,1)
      fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f %+8d %8d\n',...
 	      obspred(i,1),obspred(i,2),obspred(i,3),...
	      obspred(i,4),obspred(i,5),obspred(i,6),...	  
	      obspred(i,7),obspred(i,8),obspred(i,9));
end
      fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end
