% Test forward propagating input data through an arx.
% Demonstration of using function arx.c as an AR and FIR filter.
clear;
startup;

Verbosity = 2;
model     = 'fir';      
%model     = 'ar';      % Uncomment to run AR demo.
FLAG      = 99999;
x         = [1:8]';
X         = [x,x];
L         = size(X,1);
Nin       = size(X,2)-1;
Nx        = 1;
Ts        = 1;

if (strmatch('ar',model))
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % AR model 
  Tac    = 0;
  Tc     = 0;
  T      = 1;
  P      = [0 1]'; % (persistence model)
  X(:,2) = 0;      % (This is not necessary)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % y(t) = a(0) + a(1)X(t-1) + a(2)X(t-2) + ... + a(T)X(t-T) + ...
  %        b(-Tac+1)S(t+Tac-1) + b(0)S(t-0) + ... + b(1)S(t-1) + ... b(Tc)S(t-Tc)
  
  yo(1)   = FLAG;
  yo(2:8) = X(1:7,1);
  
end
if (strmatch('fir',model))
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % FIR model is generated by setting a(i>0) = 0, and T=0.
  Tac    = 3;
  Tc     = 0;
  T      = 0;
  P      = [0 1 1 1]'; % First parameter is a(0), second is b(-1).
  X(:,1) = 0;      % (This is not necessary)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  
  % y(t) = a(0) + a(1)X(t-1) + a(2)X(t-2) + ... + a(T)X(t-T) + ...
  %        b(-Tac+1)S(t+Tac-1) + b(0)S(t-0) + ... + b(1)S(t-1) + ... b(Tc)S(t-Tc)

  yo(1)  = FLAG;
  yo(2)  = FLAG;
  yo(3)  = FLAG;
  %yo(4)  = 0 + 
  %        3*S(5) + 4*S(4) + 5*S(3) + 6*S(2) + 7*S(1)
  yo(4)  = 0  + 3*5    + 4*4    + 5*3    + 6*2  + 7*1;
  %yo(5) = 0 +
  %        3*S(6) + 4*S(5) + 5*S(4) + 6*S(3) + 7*S(2)
  yo(5)  = 0  + 3*6    + 4*5    + 5*4  + 6*3  + 7*2;
  %yo(6) = 0  +
  %           3*S(7) + 4*S(6) + 5*S(5) + 6*S(4) + 7*S(3)
  yo(6)  = 0  + 3*7    + 4*6    + 5*5  + 6*4  + 7*3;
  yo(7)  = FLAG;
  yo(8)  = FLAG;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write input files
write_files;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Execute
if (isunix == 1)
  system('./arx.o');
else
  system('./arx.bat');
end
clear T;
load T.dat;
T
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
