function h2 = horizline(X,lc,lw,lsz)
%HORIZLINE  Plot horizontal lines on current figure.
%
%    h = HORIZLINE(X) Plots a horizontal line at the positions in the
%    vector X on the current graph.  For 2D plots only.
%
%    h = HORIZLINE(X,lc,lw,ls) uses the line color specifed by the string 
%    lc, with width of lw, and style ls.
%
%    Default of horizline(X) is horizline(X,'r',1,'--').
%
%    See also VERTLINE, PLOTFORMAT.   

if (size(X,1) == 1)
  X = X';
end
  
if ~exist('lc'),lc = 'r';,end;
if ~exist('lw'),lw = 1;,end;
if ~exist('lsz'),lsz = '--';,end;


h_1  = gca;                         % Get the current axes handle
xlim = get( h_1, 'XLim' );          % Get the axes limits.
ylim = get( h_1, 'YLim' );

I    = find( X > ylim(1) & X < ylim(2) );
X    = [X(I) X(I)];                 % Only plot those within axis limits
Y    = repmat(xlim,size(X,1),1);

hold on;
h2   = line(Y',X');
set(h2, 'Color', lc );
set(h2, 'LineWidth', lw );
set(h2, 'LineStyle', lsz );
  
