function Do = block_detrend_nonflag(Di,N,FLAG,EXACT)
%BLOCK_DETREND_NONFLAG Subtracts off block mean of non-FLAG elements.
%
%   D_dt = BLOCK_DETREND_NONFLAG(D,N) Subtracts off block mean of
%   elements not equal to NaN in non-overlapping windows of width N.
%
%   D_dt = BLOCK_DETREND_NONFLAG(D,N,FLAG) uses FLAG instead of NaN.
%
%   D_dt = BLOCK_DETREND_NONFLAG(D,N,FLAG,EXACT) If EXACT = 0, substracts
%   mean of elements less than FLAG.  EXACT = 1, substracts off elements not
%   equal to flag.  EXACT = 3, subtracts off elements with absolute value
%   less than FLAG.
%
%   D is a column vector and N is the number of elements per block.  D_dt
%   will have the same size as D.  If size(D,1)/N is not an integer FLAG
%   are used to pad D_dt.
%   
%   See also BLOCK_DETREND, BLOCK_DETREND_NONFLAG_DEMO, *_NONFLAG.

% R.S. Weigel, 04/02/2004.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG  = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

for j = 1:size(Di,2)
  
  D      = Di(:,j);  

  Lo     = length(D);
  L      = floor(length(D)/N);
  tempD  = D(1:L*N);
  tempD  = reshape(tempD,N,L);

  temp   = mean_nonflag(tempD,1,FLAG);
  temp   = repmat(temp,N,1);
  I      = find( (temp == FLAG) | (tempD == FLAG));
  D      = tempD-temp;

  if (~isempty(I))
    D(I) = FLAG;
  end

  D          = D(:);
  Do(:,j)    = [D ; FLAG*ones(Lo-L*N,1)];

end
