function X = rectify_nonflag(X,FLAG,EXACT)
%RECTIFY_NONFLAG Rectify non-FLAG elements (set non-FLAG negative values to zero).
%
%   RECTIFY_NONFLAG(X,FLAG) = 0.5*(abs(X) + X). 
%
%   RECTIFY_NONFLAG(X,FLAG,EXACT) If EXACT = 0, returns mse of elements less
%   than FLAG.  EXACT = 1, returns mse of elements not equal to flag.  EXACT
%   = 2, returns mse of elements with absolute value less than FLAG.
%
%   See also ABS, *_NONFLAG.

% R.S. Weigel, 04/02/2004.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 2)
  FLAG     = NaN;
end
if (nargin < 3)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
I = [];

if (EXACT == 0)
  I = find(X >= FLAG);
end
if (EXACT == 1)
  I = find(X == FLAG);
end
if (EXACT == 2)
  I = find(abs(X) >= FLAG);
end
if (isnan(FLAG))
  I = find(isnan(X) == 1);
end

X =  0.5*(abs(X) + X);

if (~isempty(I))
  X(I) = FLAG;
end
