function [B,Ng] = sum_nonflag(varargin)
%SUM_NONFLAG Average of non-FLAG elements.
%
%   [B,Ng] = sum_nonflag(A) for 1-D array is the sum of elements not equal
%   to NaN.  Ng contains the number of elements used in the sum.
%
%   [B,Ng] = sum_nonflag(A,DIM) is sum of elements not equal to NaN.
%
%   [B,Ng] = sum_nonflag(A,DIM,FLAG) is sum of elements not equal to FLAG.
%
%   [B,Ng] = sum_nonflag(A,DIM,FLAG,COND) uses a flag condition COND
%           (see IS_FLAG for COND values).
%
%   If all elements along dimension DIM are equal to FLAG, a fill of FLAG is
%   used.  Maximum DIM is 2.
%
%   See also SUM, *_NONFLAG.

% R.S. Weigel, 04/02/2004.  

% For better accuracy, sum_nonflag should not call mean_nonflag.
% (mean_nonflag computes sum then divides by Ng and then sum_nonflag
% multiplies by Ng so multiplication by Ng is not necessary.)

[B,Ng] = mean_nonflag(varargin{:});

B = B.*Ng;