%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Creates time series and attribute cells in standard form.  The first
% example creates one time series the second creates two time series.
% TS_INSERT is called to insert the data in these files into the user's data
% base.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;

TSDS_GLOBALS; % Load global TSDS variables

Year_o = 1999;
Year_f = 2000;
V{1}   = [1:24*number_days(Year_o,Year_f)]';  % Create artificial data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example 1: Create a time series with 24 points per day and spanning 2 years.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ATTR{1}          = ts_attr([]); % Returns the TSDS attribute template.
ATTR{1}.Name     = 'My test data set';
ATTR{1}.Note     = ['Data on 1 hour time grid starting on first hour of '...
		    'Year_o and  ending on last hour of Year_f'];
ATTR{1}.Unit     = 'nT';
ATTR{1}.TimeUnit = 'hour';
ATTR{1}.PPD      = '24';    
ATTR{1}.Warning  = 'This is a test data set'; 
ATTR{1}.Year_o   = num2str(Year_o);        % All attributes are strings
ATTR{1}.Year_f   = num2str(Year_f);
ATTR{1}.FLAG     = '99999';                 % Flag value for missing
                                            % or bad data
ATTR{1}.Version  = '0';                     % TSDS uses integers for versions
ATTR{1}.Size     = size(V{1});

% Where to go for more information on the data set (also the directory where
% the original data is stored)
ATTR{1}.README    = ['tsds/original_data/mydata'];

% Important for when we want to update the data set!
ATTR{1}.Source   = 'http://www.location.of.data/source';

% Acknowledgement information
ATTR{1}.Acknowledgement = [' ']; 
ATTR{1}.CreatedWith     = ['ts_insert_demo.m'];

% Insert the data set into the data base. 
ts_insert(ATTR,V,'MyTestTimeSeries','clean');

% MyTestTimeSeries should appear if you now call ts_list
ts_list(ATTR{1}.Name);

% Compare inserted with what TS_GET returns
[I,VV{1}] = ts_get(ATTR{1}.Name);

if (sum(VV{1}-V{1}) ~= 0)
  fprintf(['ts_insert_demo: Extracted series does not match'...
	   ' inserted series.\n']);
  error(' ');
else
  fprintf(['\n\nts_insert_demo: '...
	   'Success! Created first file!\n\n\n']);  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example 2: Put two time series into one file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ATTR{1}          = ATTR{1};
V{1}             = V{1}*10;
ATTR{1}.Name     = 'My test data times 10';
ATTR{1}.Size     = size(V{1});

V{2}             = [1:number_days(Year_o,Year_f)]';
ATTR{2}          = ATTR{1}; 
ATTR{2}.Size     = size(V{2});
%ATTR{2}.Name     = 'Wind solar wind Vy (CDAWeb)';
ATTR{2}.Name     = 'More test data';
ATTR{2}.TimeUnit = 'day';
ATTR{2}.PPD      = '1';

% Insert the data set into the data base
% My2ndTestTimeSeries should appear if you now call ts_list
ts_insert(ATTR,V,'My2ndTestTimeSeries','clean')

clear VV

% Compare inserted with what TS_GET returns
[I,VV{2}] = ts_get(ATTR{2}.Name);
[I,VV{1}] = ts_get(ATTR{1}.Name);

if (sum(VV{1}-V{1}) ~= 0) | (sum(VV{2}-V{2}) ~= 0)
  fprintf(['ts_insert_demo: Extracted series does not match'...
	   ' inserted series.\n']);
  error(' ');
else
  fprintf(['\n\nts_insert_demo: '...
	   'Success! Created second file!\n\n\n']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
