function Bc = B_wire(z1,z2,xo,I)
%B_WIRE Compute analytic value of magnetic field due to straight wire
%
%   B = B_WIRE(z1,z2,xo,I)
%
%   where z1 and z2 are the end positions of the wire.  Current of magnitude
%   I flows from z1 to z2.  xo are observation points.
%   
%   To put B in MKS units, multiply by mu_o/(4*pi).
% 
%   See also BIOT_SAVART_DEMO, B_RING.
  
z1 = z1-xo(:,3);
z2 = z2-xo(:,3);

Bc = zeros(size(xo));
for i = 1:size(xo,1)
  ro      = (xo(i,1).^2 + xo(i,2).^2).^0.5;
  R1      = ((z1(i))^2 + ro^2).^0.5;
  R2      = ((z2(i))^2 + ro^2).^0.5;
  Bc(i,1) = -(1/ro)*(z2(i)/R2-z1(i)/R1)*(xo(i,2)/ro);
  Bc(i,2) =  (1/ro)*(z2(i)/R2-z1(i)/R1)*(xo(i,1)/ro);
end

Bc = I*Bc;