%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate the circumference of a circle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt = 0.1*(pi/2);
t  = [0:dt:2*pi]';
if (t(end)==2*pi),t=t(1:end-1);,end % To prevent x from having double points
x  = [ cos(t) , sin(t) , zeros(length(t),1)];

% Using method 1
L = arc_length(x);

fprintf(['arc_length_demo: Error in circum. of circle method 1, '...
	 'dL(1) unknown %.2e\n'],abs(1-L/(2*pi)));

% Using method 2
[L,dL] = arc_length(x,2);

fprintf(['arc_length_demo: Error in circum. of circle method 2, '...
	 'dL(1) unknown %.2e\n'],abs(1-L/(2*pi)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate the exact perimeter of box with segement centers given by x
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt = 0.5*pi/2;
t  = [0:dt:2*pi]';
if (t(end)==2*pi),t=t(1:end-1);,end % To prevent x from having double points
N = length(t);
x = [ cos(t) , sin(t) , zeros(length(t),1)];

% For regular polygon formulas, see
% http://www.math.rutgers.edu/~erowland/polygons.html
a  = 1;             % apothem (radius of inscribed circle)
r  = a/cos(pi/N);   % radius of circumscribed circle
s  = 2*r*sin(pi/N); % length of each side
Le = s*N;

% Using method 2 with correct value of initial dL.
[L,dL] = arc_length(x,2,[0 s 0]);
fprintf(['arc_length_demo: Error in box perimeter method 2, '...
	 'dL(1) known %.2e\n'],abs(1-L/Le));

% Using method 2 and its guess value of dL(1,:).
[L,dL] = arc_length(x,2);
fprintf(['arc_length_demo: Error in box perimeter method 2, '...
	'dL(1) unknown %.2e\n'],abs(1-L/Le));

% Using method 1 and its guess value of dL(1,:).
[L,dL] = arc_length(x);
fprintf(['arc_length_demo: Error in box perimeter method 1, '...
	 'dL(1) unknown %.2e\n'],abs(1-L/Le));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%