%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test mex function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%system('rm -f biot_savart_mex.mexglx libsp.o');

coordsys = 1;

for i = 1:3
for n = 3:3

  N        = 10^n;
  xo       = [1 pi/2 0];
  x        = zeros(N,3);
  x(:,1)   = [0:N-1]';
  IdL      = zeros(N,3);
  IdL(:,1) = 1;

  % IdL(1:end/2,1) = 1;
  
  fprintf('n = 10^%d\n',n);

  tic; use_script = 1;
  Bo2 = biot_savart(x,IdL,xo,coordsys,use_script);
  t2(i,n) = toc;
  fprintf('Time for shared library      = %f\n',t2(i,n));

  tic; use_script = 0;
  Bo1 = biot_savart(x,IdL,xo,coordsys,use_script); 
  t1(i,n) = toc;
  fprintf('Time for script              = %f\n',t1(i,n));

  Rt(i,n) = t1(i,n)/t2(i,n);
  fprintf('script/library time ratio    = %f\n',Rt(i,n));

  Da(i,n) = (mag(Bo2)-mag(Bo1))/eps;
  fprintf('script-library answer/eps    = %f\n',Da(i,n));

end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test Fukushima's Theorem case 2 (Here the field inside the sphere should be
% zero.).  Note that the closeness to zero is most sensitive to both dz of the
% wire and the length of the wire.
% References:
% Fukushima, N., Radio Science, Volume 6, Number 2, pages 269-275, 1971.
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  Nos. 1/2, pp. 35-40, (1976).
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  pp. 113-117, (1976).
% Qihou, Eur. J. Phys. 8 (1987), 128-130.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xo  = [0.3 pi/4 0];
Nph = 100;
Nth = 99;    % Keep odd for symmetry around theta = pi/2
Io  = 1/Nph;

[th,ph] = meshgrid([pi/Nth:pi/Nth:pi-pi/Nth],[0:2*pi/Nph:2*pi-2*pi/Nph]);
r       = ones(size(th));

Ir      = zeros(size(r));
Iph     = zeros(size(r));
Ith     = Io*ones(size(r));

dth     = repmat(th(1,2)-th(1,1),size(r));
dL      = r.*dth;

xc  = transform('Spherical','Cartesian',[r(:) th(:) ph(:)]);
xoc = transform('Spherical','Cartesian',xo);
Ic  = transform('Spherical2','Cartesian',...
		[Ir(:) Ith(:) Iph(:)],[r(:) th(:) ph(:)]);

IdLc = Ic.*repmat(dL(:),1,3);
Bc   = biot_savart(xc,IdLc,xoc,0,1);

%Bo = transform('Cartesian','Spherical2',Bc,xo)

dz = 0.001;
z  = [1:dz:100]';
Nz = length(z);
N  = length(z);
x  = zeros(N,1);
y  = x;
Ix = x;
Iy = x;
Iz = -ones(N,1);

IdL = [Ix(:) Iy(:) Iz(:)].*repmat(dz,N,3);

Bu  = biot_savart([x(:) y(:) z(:)],IdL,xoc);
Buc = B_wire(1,max(z),xoc,-1); % Exact solution

Bl  = biot_savart([x(:) y(:) -z(:)],IdL,xoc);
Blc = B_wire(-1,-max(z),xoc,1); % Exact solution

Bt  = Bu+Bl+Bc;
Btc = Buc+Blc+Bc;

fprintf('biot_savart_demo:\n');
fprintf('biot_savart_demo: Current into/out of poles and along meridians.\n');
fprintf('biot_savart_demo: Magnitude inside sphere (should be zero)\n');
fprintf('biot_savart_demo: using Biot-Savart on all currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nth = %d, Nz = %d: %.2e\n'],...
	Nph,Nth,Nz,mag(Bt));
Error(1) = mag(Bt);
fprintf('biot_savart_demo: using Biot-Savart on sphere currents\n');
fprintf('biot_savart_demo: and analytic for abs(z)>=1 currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nth = %d, Nz = %d: %.2e\n'],...
	Nph,Nth,Nz,mag(Btc));
Error(2) = mag(Btc);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test Fukushima's Theorem case 1 (For any x,y and z<0, field created by
% current flowing into z=0 in a wire extending from z=0 to Inf is canceled
% by a uniformly diverging current at z=0.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xo  = [8 2 -1];  % Point where B is computed.

% Create diverging current in x-y plane emanating from x,y,z = 0.
Nph = 4^3;   % Number of grid points in phi
Nr  = 100;   % Number of grid point in r
Io  = 1/Nph; % Total current of 1 is shared by each wire.
dr  = 0.05;

[r,ph] = meshgrid([dr:dr:100],[0:2*pi/Nph:2*pi-2*pi/Nph]);

x  = r.*cos(ph);   % Convert to cartesian
y  = r.*sin(ph);
z  = zeros(size(r));

% Create current
Ix = Io.*cos(ph);  % Convert to cartesian
Iy = Io.*sin(ph);
Iz = zeros(size(r));

% Compute the length of each current element
dr = repmat(r(1,2)-r(1,1),length(r(:)),3); 

% Compute the field from current in z=0 plane.
IdL = [Ix(:) Iy(:) Iz(:)].*dr;
Bop = biot_savart([x(:) y(:) z(:)],IdL,xo);

% Create wire from z=0 to Inf.  Current of 1 flows into z=0.
Io = 1;  % Total current is 1.
dz = 0.05;
z  = [0:dz:200]';
x  = zeros(size(z));
y  = zeros(size(z));

% Create current
Ix = zeros(size(z));
Iy = zeros(size(z));
Iz = -Io*ones(size(z));

% Compute the field of wire along z-axis.
IdL  = [Ix(:) Iy(:) Iz(:)].*repmat(dz,length(z),3);
Bow  = biot_savart([x(:) y(:) z(:)],IdL,xo);
Bowc = B_wire(z(1),z(end),xo,-1);

fprintf('biot_savart_demo:\n');
fprintf('biot_savart_demo: Current along z-axis for z>0 in -z direction\n');
fprintf('biot_savart_demo: + diverging current in z=0 plane.\n');
fprintf('biot_savart_demo: Magnitude below plane (should be zero)\n');
fprintf('biot_savart_demo: using Biot-Savart on all currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nr = %d, Nz = %d: %.2e\n'],...
	Nph,Nr,Nz,mag(Bop+Bow));
Error(3) = mag(Bop+Bow);
fprintf('biot_savart_demo: Magnitude below plane (should be zero)\n');
fprintf('biot_savart_demo: using Biot-Savart on currents in plane\n');
fprintf('biot_savart_demo: and analytic for z>0 currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nr = %d, Nz = %d: %.2e\n'],...
	Nph,Nr,Nz,mag(Bop+Bowc));
Error(4) = mag(Bop+Bowc);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wire loop of radius 1 in the x-y plane with center at the origin.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('biot_savart_demo: \n');
fprintf('biot_savart_demo: Wire loop: on axis\n');

Ns = 100;                         % Number of segments
b  = 1;                          % Wire radius
Io = 1;                          % Current in wire
xo = [0 0 0];                    % Observation point

x      = zeros(Ns,3);
th     = 2*pi*([0:Ns-1]')/Ns;
x(:,1) = cos(th);
x(:,2) = sin(th);

[L,dL] = arc_length(x); 
IodL   = Io.*dL;

Bd    = biot_savart(x,IodL,xo);    % Discrete solution
Bc(3) = B_ring(xo(3),b,Io);      % Continuous solution

fprintf(['biot_savart_demo: Percent error from exact solution, '...
	 'Ns = %d: %.2e %%\n'],Ns,100*(1-Bd(3)/Bc(3)));
Error(5) = 1-Bd(3)/Bc(3);

% Test scaling of error with Ns
for k = 1:5
  Ns = 10^k;

  x      = zeros(Ns,3);
  th     = 2*pi*([0:Ns-1]')/Ns;
  x(:,1) = cos(th);
  x(:,2) = sin(th);

  [L,dL] = arc_length(x); 
  IodL = Io.*dL;

  Bd = biot_savart(x,IodL,xo);

  E(k) = 1-Bd(3)/Bc(3);

  fprintf(['biot_savart_demo: Percent error from exact solution, '...
	   'Ns = 10^%d: %.2e %%\n'],k,100*(1-Bd(3)/Bc(3)));
end

% Test off-axis
xo = [1 0 1];
[L,dL] = arc_length(x); 
IodL = Io.*dL;
Bd = biot_savart(x,IodL,xo);
z  = xo(3);
r  = max(xo(:,1:2));

[Bzc,Brc] = B_ring([z r],b,Io);

Bc = [Brc 0 Bzc];

fprintf('biot_savart_demo:\n');
fprintf('biot_savart_demo: Wire loop: off axis\n');
fprintf(['biot_savart_demo: Percent error from exact solution, '...
	 'Ns = 10^%d: %.2e %%\n'],k,100*(1-mag(Bd)/mag(Bc)));
Error(6) = 1-Bd(3)/Bc(3);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wire loop of radius 1 in the x-y plane with center at the origin.  Use
% alternative method (where J is given in different coordinate system) to
% compute.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('biot_savart_demo: \n');
fprintf('biot_savart_demo: Wire loop: on axis using alternative method\n');

% with N>=1.
b   = 1;                 % Wire radius
xo  = [1 0 0];           % Observation point (spherical)

N   = 4;                 % Number of segments
Ir  = zeros(N,1);        % Current in radial direction
Ith = zeros(N,1);        % Current in theta direction
Iph = ones(N,1);         % Current phi direction
r   = b*ones(N,1); 
th  = (pi/2)*ones(N,1);
ph  = [0:pi/(N/2):2*pi-pi/(N/2)]';
dL  = (2*pi*b/N)*ones(N,1);

IdL = [Ir(:) Ith(:) Iph(:)].*repmat(dL(:),1,3);
Bda = biot_savart([r(:) th(:) ph(:)],IdL,xo,1);
% This is in [r, theta, ph] coordinates

Bc = B_ring(xo(1),b,Iph(1));
% This is in cartesian

fprintf('biot_savart_demo: Percent error from exact solution %.2e %%\n',...
	(1-Bda(1)/Bc)*100);
Error(7) = 1-Bda(1)/Bc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Continuous wire that carries a unit current along z-axis between z1 and z2. 
% Bc = continuous (exact) solution 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('biot_savart_demo: \n');
fprintf('biot_savart_demo: Wire segment\n');
Ns = 10;                   % Number of segments
I  = 1;                    % Current in wire
xo = [1.0 1.0 1.0];        % Observation point

x      = zeros(Ns,3);
x(:,3) = [1:Ns]'/Ns;       % Location of centers of current segments

[L,dL] = arc_length(x); 
IdL = I.*dL;

Bd = biot_savart(x,IdL,xo);  % Bd = discrete solution

dz = x(2,3)-x(1,3);
z2 = x(end,3) + dz/2;      % +/- dz/2 because segments are centered on 
z1 = x(1,3)   - dz/2;      % each z value

Bc = B_wire(z1,z2,xo,I);

fprintf(['biot_savart_demo: Percent error from exact solution, '...
	'Ns = %d  : %.2e %%\n'],Ns,100*(1-mag(Bd)/mag(Bc)));
Error(8) = 1-mag(Bd)/mag(Bc);

% Test scaling of error with Ns
for k = 2:5
  Ns = 10^k;

  x      = zeros(Ns,3);
  x(:,3) = [1:Ns]'/Ns;  

  [L,dL] = arc_length(x); 
  IdL = I.*dL;

  Bd = biot_savart(x,IdL,xo);

  dz = x(2,3)-x(1,3);
  z2 = x(end,3) + dz/2;      % +/- dz/2 because segments are centered on 
  z1 = x(1,3)   - dz/2;      % each z value

  Bc = B_wire(z1,z2,xo,I);
  
  E(k) = 1-mag(Bd)/mag(Bc);

  fprintf(['biot_savart_demo: Percent error from exact solution, '...
	   'Ns = 10^%d: %.2e %%\n'],k,100*(1-mag(Bd)/mag(Bc)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute using alternative method (where J is given in different coordinate
% system).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('biot_savart_demo: \n');
fprintf('biot_savart_demo: Wire segment using alternative method\n');

Io  = 1;
xo  = [1 1 1];           % Observation point (cartesian)

Ns     = 10;             % Number of segments
x      = zeros(Ns,3);
x(:,3) = [1:Ns]'/Ns;     % Location of centers of current segments

Ir  = Io*ones(Ns,1);     % Current in radial direction
Ith = zeros(Ns,1);       % Current in theta direction
Iph = zeros(Ns,1);       % Current phi direction
r   = x(:,3);            % 
th  = zeros(Ns,1);
ph  = zeros(Ns,1);
dL  = repmat(x(2,3)-x(1,3),Ns,3);
IdL = [Ir(:) Ith(:) Iph(:)].*dL;
% Current segments are located at [r,th,ph] and current vector is specified
% in a spherical coordinate system with origin at [r,th,ph].
xo_s  = transform('Cartesian','Spherical',xo); % Put xo in spherical
Bda_s = biot_savart([r(:) th(:) ph(:)],IdL,xo_s,1);
% Output is in the spherical coordinate system with origin at [r,th,ph].
% Transform this to cartesian
Bda = transform('Spherical2','Cartesian',Bda_s,xo_s);

dz = x(2,3)-x(1,3);
z2 = x(end,3) + dz/2;      % +/- dz/2 because segments are centered on 
z1 = x(1,3)   - dz/2;      % each z value

Bc = B_wire(z1,z2,xo,Io);
fprintf(['biot_savart_demo: Percent error from exact solution, '...
	 'Ns = %d  : %.2e %%\n'],Ns,100*(1-mag(Bda)/mag(Bc)));
Error(9) = 1-mag(Bda)/mag(Bc);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
