clear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test Fukushima's Theorem case 2 (Here the field inside the sphere should be
% zero.).  Note that the closeness to zero is most sensitive to both dz of the
% wire and the length of the wire.
% References:
% Fukushima, N., Radio Science, Volume 6, Number 2, pages 269-275, 1971.
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  Nos. 1/2, pp. 35-40, (1976).
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  pp. 113-117, (1976).
% Qihou, Eur. J. Phys. 8 (1987), 128-130.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
show_plot = 0;

a = 6378.137e3; % Earth radius in meters
h = a/60.0;
It = 8e3; % Total amps in/out

xo  = [a pi/4 0];
xos = xo;

Nph = 64;
Nth = 63;  
tho = 0.1473*180/pi;
thf = 180-tho;

thd = [tho:(thf-tho)/Nth:thf];
thx = (pi/180)*thd;
Nth = length(thx);

%Io  = It/Nph;
Io = It

[th,ph] = meshgrid(thx,[0:2*pi/Nph:2*pi-2*pi/Nph]);

r       = ones(size(th));

Ir      = zeros(size(r));
Ith     = Io*ones(size(r));
Iph     = zeros(size(r));

dth     = repmat(th(1,2)-th(1,1),size(r));
dL      = r.*dth;

IdL = [Ir(:),Ith(:),Iph(:)].*repmat(dth(:),1,3);
Bc  = biot_savart([r(:),th(:),ph(:)],IdL,xo,1)

if (show_plot)
  clf
  vout=transform('Spherical','Cartesian',[r(:),th(:),ph(:)]);
  x=vout(:,1);
  y=vout(:,2);
  z=vout(:,3);

  plot3(x,y,z,'r.','LineWidth',3);hold on;grid on;
  xlabel('x');ylabel('y');zlabel('z')
end


r  = (a+h)*[1:0.1:5];
dr = r(2)-r(1);
Nr = length(r);

th  = repmat((pi/180)*tho,size(th,1),Nr);
ph  = repmat(ph(:,1),1,Nr);
r   = repmat(r,size(th,1),1);

if (show_plot)
  vout=transform('Spherical','Cartesian',[r(:),th(:),ph(:)]);
  x=vout(:,1);
  y=vout(:,2);
  z=vout(:,3);

  plot3(x,y,z,'g.','LineWidth',3);hold on;grid on;
  xlabel('x');ylabel('y');zlabel('z')
end

Ix  = zeros(size(th));;
Iy  = zeros(size(th));;
Ir  = -Io*ones(size(ph));

IdL = [Ir(:) Ix(:) Iy(:)].*repmat(dr,length(Ir(:)),3);
Bu  = biot_savart([r(:) th(:) ph(:)],IdL,xos,1)

Ir  = -Ir;
th  = repmat((pi/180)*thf,size(th,1),Nr);

if (show_plot)
  vout=transform('Spherical','Cartesian',[r(:),th(:),ph(:)]);
  x=vout(:,1);
  y=vout(:,2);
  z=vout(:,3);

  plot3(x,y,z,'b.','LineWidth',3);hold on;grid on;
  xlabel('x');ylabel('y');zlabel('z')
end


IdL = [Ir(:) Ix(:) Iy(:)].*repmat(dr,length(Ir(:)),3);
Bl  = biot_savart([r(:) th(:) ph(:)],IdL,xos,1)

Bt  = Bu+Bl+Bc

fprintf('biot_savart_demo:\n');
fprintf('biot_savart_demo: Current into/out of poles and along meridians.\n');
fprintf('biot_savart_demo: Magnitude inside sphere (should be zero)\n');
fprintf('biot_savart_demo: using Biot-Savart on all currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nth = %d, Nz = %d: %.1e [nT]\n'],...
	Nph,Nth,Nr,(mu_o/4/pi)*mag(Bt)/1e-9);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
