clear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test Fukushima's Theorem case 2 (Here the field inside the sphere should be
% zero.).  Note that the closeness to zero is most sensitive to both dz of the
% wire and the length of the wire.
% References:
% Fukushima, N., Radio Science, Volume 6, Number 2, pages 269-275, 1971.
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  Nos. 1/2, pp. 35-40, (1976).
% Fukushima, N., Rep. of Ionosphere and Space Research in Japan, Vol. 30,
%  pp. 113-117, (1976).
% Qihou, Eur. J. Phys. 8 (1987), 128-130.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xo  = [0.1 pi/2 0];
xos = xo;
Nph = 100;
Nth = 10;  
Io  = 1/Nph;
tho = 89.9;
thf = 180-tho;

thd = [tho:(thf-tho)/Nth:thf];
thx = (pi/180)*thd;
Nth = length(thx);

[th,ph] = meshgrid(thx,[0:2*pi/Nph:2*pi-2*pi/Nph]);

r       = ones(size(th));

Ir      = zeros(size(r));
Ith     = Io*ones(size(r));
Iph     = zeros(size(r));

dth     = repmat(th(1,2)-th(1,1),size(r));
dL      = r.*dth;

IdL = [Ir(:),Ith(:),Iph(:)].*repmat(dth(:),1,3);
Bc  = biot_savart([r(:),th(:),ph(:)],IdL,xo,1)

sum(Ith(1,:))

break

dr = 0.0001;
r  = [1:dr:100]';
Nr = length(r);
N  = length(r);
x  = th(1)*ones(N,1);
y  = zeros(N,1);
Ix = zeros(N,1);;
Iy = zeros(N,1);;
Ir = -(1/Nph)*ones(N,1);


IdL = [Ir(:) Ix(:) Iy(:)].*repmat(dr,N,3);
Bu  = biot_savart([r(:) x(:) y(:)],IdL,xos,1)

x  = (pi-th(1))*ones(N,1);
y  = zeros(N,1);
Ix = zeros(N,1);;
Iy = zeros(N,1);;
Ir = (1/Nph)*ones(N,1);
IdL = [Ir(:) Ix(:) Iy(:)].*repmat(dr,N,3);
Bl  = biot_savart([r(:) x(:) y(:)],IdL,xos,1)

Bt  = Bu+Bl+Bc;

[Bt(3)/Bu(3),Bt(3)/Bu(3),Bt(3)/Bc(3)]

fprintf('biot_savart_demo:\n');
fprintf('biot_savart_demo: Current into/out of poles and along meridians.\n');
fprintf('biot_savart_demo: Magnitude inside sphere (should be zero)\n');
fprintf('biot_savart_demo: using Biot-Savart on all currents with\n');
fprintf(['biot_savart_demo: Nph = %d, Nth = %d, Nz = %d: %.2e\n'],...
	Nph,Nth,Nr,mag(Bt));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
