function get_cdaweb_data(SATINFO)
%GET_CDAWEB_DATA Use wget to download CDAWeb data 
%
%   GET_CDAWEB_DATA(Satellite) Uses wget program to download data required
%   for the CISM_DX_DATA/make_uniform_time_series/make_* functions that use
%   CDAWeb data.  Data files already in directory are not re-downloaded.
% 
%   DATA_STR can currently take the values of ACE, IMP8, GEOTAIL, WIND,
%   or POLAR.
%   Data are put in one of the following directories
%   [CISM_DX_DATA_FULL_ROOT,'measurements/original_data/ace/']
%   [CISM_DX_DATA_FULL_ROOT,'measurements/original_data/imp8/']
%   [CISM_DX_DATA_FULL_ROOT,'measurements/original_data/geotail/']
%   [CISM_DX_DATA_FULL_ROOT,'measurements/original_data/wind/'].
%   [CISM_DX_DATA_FULL_ROOT,'measurements/original_data/polar/'].
%
%   See also GET_ORIGINAL_DATA.

% R.S. Weigel 04/01/2004  
  
CISM_DX_GLOBALS;

satellite = SATINFO.satellite;

% Kludge because goesX data is not stored in goesX subdirectory
% as is convention for satellites.  This cuts off the X.
if (strmatch('goes',satellite))
  satellite = 'goes';
end

base_path = sprintf('%s%s',...
		    CISM_DX_DATA_FULL_ROOT,...
		    'measurements/original_data/CDAWeb/')

data_root = sprintf('%s%s',base_path,satellite);

% If directory does not exist, error
if (~exist(data_root)) 
  error(sprintf('Could not find directory %s\n. Aborting.\n',data_root));
end

cwd = pwd;
cd(data_root);

fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
fprintf('... using wget to get %s .cdf files\n',satellite);
fflush(stdout);

CISM_DX_FTP_STR     = ['ftp://cdaweb.gsfc.nasa.gov/pub/istp/',SATINFO.DATA_DIR];

command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
system(command_str);
fprintf('... done downloading %s .cdf files.\n',satellite);fflush(stdout);


cd(cwd);

if (0) % Old code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (strmatch(upper(DATA_STR),'POLAR'))  

  cwd = pwd;
  cd([CISM_DX_DATA_FULL_ROOT,'measurements/original_data/ace/']);

  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get POLAR mfe k0 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/polar/mfe';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading POLAR mfe k0 .cdf files.\n');fflush(stdout);

  if (0) % This will give total ion not Np as is case for others
  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get POLAR tid h1 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/po/tid_h1';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading POLAR tid h1 .cdf files.\n');fflush(stdout);
  end
  
  cd(cwd);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (strmatch(upper(DATA_STR),'ACE'))  

  cwd = pwd;
  cd([CISM_DX_DATA_FULL_ROOT,'measurements/original_data/ace/']);

  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get ACE mfi h0 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/ace/mfi_h0';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading ACE mfi h0 .cdf files.\n');fflush(stdout);
  
  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get ACE swe h0 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/ace/swe_h0';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading ACE swe h0 .cdf files.\n');fflush(stdout);

  cd(cwd);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (strmatch(upper(DATA_STR),'IMP8'))  

  cwd = pwd;
  cd([CISM_DX_DATA_FULL_ROOT,'measurements/original_data/imp8/']);

  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get IMP8 mag .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/imp8/mag';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading IMP8 mag .cdf files.\n');fflush(stdout);
  
  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get IMP8 pla .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/imp8/pla';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading IMP8 pla .cdf files.\n');fflush(stdout);

  cd(cwd);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (strmatch(upper(DATA_STR),'WIND'))  

  cwd = pwd;
  cd([CISM_DX_DATA_FULL_ROOT,'measurements/original_data/wind/']);

  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get WIND swe_k0 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/wind/swe';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading WIND swe_k0 .cdf files.\n');fflush(stdout);
  
  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get WIND mfi_h0 .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/wind/mfi_h0';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading WIND mfi_h0.cdf files.\n');fflush(stdout);

  cd(cwd);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (strmatch(upper(DATA_STR),'GEOTAIL'))  

  cwd = pwd;
  cd([CISM_DX_DATA_FULL_ROOT,'measurements/original_data/geotail/']);

  fprintf('Program get_cdaweb_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to get GEOTAIL mgf .cdf files\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/geotail/mgf';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading GEOTAIL mgf .cdf files.\n');fflush(stdout);

  CISM_DX_FTP_STR     = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/geotail/cpi_h0';
  command_str = sprintf('%s -m -nH --cut-dirs=3 %s',CISM_DX_WGET_PATH,CISM_DX_FTP_STR);
  system(command_str);
  fprintf('... done downloading GEOTAIL cpi_h0 .cdf files.\n');fflush(stdout);
  
  cd(cwd);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
