function get_real_time_data(DATA_STR)
%GET_REAL_TIME_DATA Downloads and prepares r.t. data for FM
%
%   GET_REAL_TIME_DATA(DATA_STR)   
%  
%   DATA_STR = 'ace','Ap', 'synoptic', or 'ss_maps'.
%   
%   See models/Ap/main.m, models/Solar_Wind_Velocity/main.m, and
%   models/Electron/main.m for example usage.
%
%   See also GET_*.

% R.S. Weigel 05/01/2004  

CISM_DX_GLOBALS;

curr_dir = pwd;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (strmatch(DATA_STR,'ace'))  
  
  cd(CISM_DX_DATA_DIR_ACE)

  fprintf('Program get_real_time_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to download ACE swepam data files to directory\n');
  fprintf('... %s\n',CISM_DX_DATA_DIR_ACE);fflush(stdout);

  command_str = sprintf('%s -nd -m -a wget_logfile \"%s\"',...
			CISM_DX_WGET_PATH,CISM_DX_FTP_STR_ACE);
  system(command_str);
  fprintf('... done downloading ACE swepam data.\n');fflush(stdout);

  fprintf('... calling cat_swepam_data.pl to concatenate ACE swepam files\n');
  fflush(stdout);
  command_str = sprintf('%s %s/cat_swepam_files.pl',...
			CISM_DX_PERL_PATH,CISM_DX_PERLFILES_ROOT);
  system(command_str);
  fprintf('... done calling cat_swepam_data.pl.\n');fflush(stdout);

  fprintf('... calling parse_swepam_data.m to put data in standard format\n');
  fflush(stdout);
  parse_swepam_data;
  fprintf('... done calling parse_swepam_data.m.\n');fflush(stdout);
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (strmatch(DATA_STR,'Ap'))  

  cd(CISM_DX_DATA_DIR_Ap)  

  fprintf('Program get_real_time_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to download latest Ap data to directory\n');
  fprintf('... %s\n',CISM_DX_DATA_DIR_Ap);fflush(stdout);
  command_str = sprintf('%s -nd -m -a wget_logfile \"%s\"',...
			CISM_DX_WGET_PATH,CISM_DX_FTP_STR_Ap);
  system(command_str);
  fprintf('... done downloading latest Ap data.\n');fflush(stdout);

  fprintf('... using parse_Ap_data.pl to parse downloaded Ap data\n');
  fflush(stdout);
  command_str = sprintf('%s %s/parse_Ap_data.pl',...
			CISM_DX_PERL_PATH,CISM_DX_PERLFILES_ROOT);
  system(command_str);
  fprintf('... done parsing downloaded Ap data.\n');fflush(stdout);

  fprintf('... using unique_Ap_data.m parse downloaded Ap data\n');
  fflush(stdout);
  unique_Ap_data;
  fprintf('... done parsing downloaded Ap data.\n');fflush(stdout);
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (strmatch(DATA_STR,'synoptic'))  

  cd(CISM_DX_DATA_DIR_SYN)
   
  % Leslie pushes the last 6 mwo*** files into the directories
  % fusion.sec.noaa.gov:/ftp/dist/narge/synoptic/{MWO,WSO}/last6

fprintf('Program get_real_time_data.m is ...\n');fflush(stdout);
  fprintf('... using wget to download latest 6 synoptic maps to directory\n');
  fprintf('... %s\n',CISM_DX_DATA_DIR_SYN);fflush(stdout);
  fflush(stdout);
  
  cd('./MWO')
  command_str = ...
      sprintf('%s -m -np -nH --cut-dirs=6 -a wget_logfile \"%s/MWO/last6/*\"',...
	      CISM_DX_WGET_PATH,CISM_DX_FTP_STR_SYN);
  system(command_str);

  cd('../WSO')
  command_str = ...
      sprintf('%s -m -np -nH --cut-dirs=6 -a wget_logfile \"%s/WSO/last6/*\"',...
	      CISM_DX_WGET_PATH,CISM_DX_FTP_STR_SYN);
  system(command_str);

  fprintf('... done using wget to download latest synoptic maps.\n');
  fflush(stdout);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (strmatch(DATA_STR,'ss_maps'))  

  cd(CISM_DX_DATA_DIR_WSA)
  
  % Leslie pushes the last 6 cr*** files into the directories
  % fusion.sec.noaa.gov:/ftp/dist/ss_maps/{MWO,WSO}/CS/last6
  
  fprintf('Program get_real_time_data.m is ...\n');
  fprintf('... using wget to download latest 6 solar surface maps to dir\n');
  fprintf('... %s\n',CISM_DX_DATA_DIR_SYN);fflush(stdout);

  fflush(stdout);  

  cd('./ss_maps/MWO')
  command_str = ...
  sprintf('%s -m -np -nH --cut-dirs=6 -a wget_logfile \"%s/MWO/CS/last6/*\"',...
	  CISM_DX_WGET_PATH,CISM_DX_FTP_STR_SSM); 
  system(command_str);

  cd('../WSO')
  command_str = ...
  sprintf('%s -m -np -nH --cut-dirs=6 -a wget_logfile \"%s/WSO/CS/last6/*\"',...
	  CISM_DX_WGET_PATH,CISM_DX_FTP_STR_SSM); 
  system(command_str);
  cd('./../..')
  fprintf('... done using wget to download last 6 ss_maps.\n');
  fflush(stdout);
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cd(curr_dir);