function [DT,J1,J2] = bdl(R,Thresh)
%BDL Burst distribution lifetime.  
%    [DT,J1,J2] = BDL(X,THRESH) returns J1, the times at which the vector 
%    X crosses THRESH upwards, J2, the time of downward crossings, and DT,
%    the amount of time spent above THRESH.  X must be a vector.
%
%    J1 is floor(t_up) and J2 is floor(t_down), where t_up is actual time of
%    upward crossing and t_down is actual time of downward crossing.
% 
%    Example: If X = [0;0;1;0;0] and THRESH = 0.1, the upward crossing time
%    must be between 2 and 3 and the downward crossing between 3 and 4.
%    BDL(X,THRESH) returns DT = 1, J1 = 2 and J3 = 3;
%    
%    See also CHUNK, QQCHUNK, DEMOBDL.

J1         = find((R(1:end-1) <= Thresh) & (R(2:end) >  Thresh));
J2         = find((R(1:end-1) >  Thresh) & (R(2:end) <= Thresh));

if ((isempty(J1)) | (isempty(J2)))
  J1 = [];
  J2 = [];
  DT = [];
else	
  if (R(1) <= Thresh)
    if (size(J2,1) < size(J1,1))
      DT  = J2(1:end) - J1(1:end-1);
      J1  = J1(1:end-1);
      J2  = J2(1:end);
    end
    if (size(J2,1) == size(J1,1))
      DT  = J2 - J1;
    end
  end
  if (R(1) > Thresh)
    if (size(J1,1) < size(J2,1))
      DT  = J2(2:end) - J1(1:end);
      J1  = J1(1:end);
      J2  = J2(2:end);
    end
    if (size(J2,1) == size(J1,1))
      if (length(J2) > 1)
	DT  = J2(2:end) - J1(1:end-1);
	J1  = J1(1:end-1);
	J2  = J2(2:end);
      else 
	DT  = [];
	J1  = [];
	J2  = [];
      end
    end
  end
end

