function Y = qqchunk3(X,T_quiet,Thresh_A,T_surround)
%QQCHUNK3 Returns inital and final positions above threshold.
%
%    Y = QQCHUNK3(X,T_above,THRESH) where X is a matrix, THRESH is the
%    value above which A = X(:,1) is considered active, and T_above is the
%    length of time above THRESH to be considered an active time.
%
%    Y.chunks a cell matrix.  Each cell contains a chunk of X.
%    Y.mean   a matrix that contains the mean of time series in Y.chunks.
%    Y.max    a matrix that contains the maximum value of t.s. in Y.chunks.
%    Y.area   a matrix that contains the area under t.s. in Y.chunks.
%    Y.j1     a matrix.  Each element contains left boundaries J1.
%    Y.j2     a matrix.  Each element contains right boundaries J2.
%   
%    See also QQCHUNK, BDL, CHUNK, QQCHUNK_DEMO, QQCHUNK_PLOT.

if (nargin < 4),
  T_surround = [0 0];
end

T_before   = T_surround(1);
T_after    = T_surround(2);

A          = X(:,1);

Mark       = [];
zz         = 0;

J          = find(A > Thresh_A);
J2         = find((J(2:end,1) - J(1:end-1)) > 1);

if (~isempty(J2))
  J3       = find((J2(2:end,1) - J2(1:end-1,1)) > T_quiet);
  if (~isempty(J3))
    Mark     = J(J2(J3)+1);
    Mark2    = J(J2(J3+1));
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~isempty(Mark))
for k = 1:length(Mark)
for i = 1:size(X,2)
    y{k}(:,i)    = X(Mark(k)             : Mark2(k),i);
    ybefore(:,i) = X(Mark(k)  - T_before : Mark(k)  - 1      ,i);
    yafter(:,i)  = X(Mark2(k) + 1        : Mark2(k) + T_after,i);
    yall{k}(:,i) = X(Mark(k)  - T_before : Mark2(k) + T_after,i);
    ymean(k,i)   = mean(y{k}(:,i));
    ymax(k,i)    = max(y{k}(:,i));
    yarea(k,i)   = sum(y{k}(:,i));
  end
end

if (~isempty(y))
  Y.chunk  = y;
  Y.mean   = ymean;
  Y.max    = ymax;
  Y.area   = yarea;
  Y.j1     = Mark;
  Y.j2     = Mark2;
  Y.before = ybefore;
  Y.after  = yafter;
  Y.all    = yall;
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


