function [Ybefore,Yafter,Jbefore,Jafter] = qqchunk4(X,Tbelow,Tabove,THRESH)
%QQCHUNK4 Data surrounding threshold crossing event.
%
%    [Ybefore,Yafter,Jbefore,Jafter] = QQCHUNK4(X,Tbelow,Tabove,THRESH) where X is a
%    matrix, THRESH(1) is the value above which X(:,1) is considered active
%    and   , THRESH(2) is the value below which X(:,1) is considered quiet.
%
%    Define T is the index of the threshold crossing, where X(T-1,1) < THRESH(1),
%    and X(T,1) >= THRESH(2).  T values are returned when
%
%    X is below THRESH(1) at T-Tbelow(1),..., T-1 and
%    X is above THRESH(2) at T, ..., T+Tabove(1)-1.
%
%    Ybefore has Tbelow(2) rows containing data before each event and has
%    columns of events.  
%    Yafter has Tabove(2) rows containing data after each event, and 
%    has columns of events.
%
%    If Tbelow, Tabove, or THRESH is scalar, Tbelow(2) = Tbelow(1), 
%    Tabove(2) = Tabove(1), or THRESH(2) = THRESH(1).  
%
%    If X has more than one column, the third dimension of Ybefore and
%    Yafter corresponds to the columns in X, where the events are defined by
%    the behavior of the first column of X only.
%   
%    See also QQCHUNK, BDL, CHUNK, QQCHUNK_DEMO, QQCHUNK_PLOT.

% R.S. Weigel, 12/1/2002.  
  
T_above    = Tabove(1);
T_below    = Tbelow(1);

if (length(Tabove) == 1)
  T_after    = Tabove;
else
  T_after    = Tabove(2);
end

if (length(Tabove) == 1)
  T_before   = Tbelow;
else
  T_before   = Tbelow(2);
end

if (length(THRESH) == 1)
  THRESH(2) = THRESH(1);
end

J          = 1 + find(X(1:end-1,1) < THRESH(1) & X(2:end,1) >= THRESH(2)); 
J          = J(find(  J                      > (T_below + T_before) )); 
J          = J(find( (J + T_above + T_after) <  length(X(:,1))           ));
Jo         = J;

for i = 1:T_below
  J = J(find(X(J-i,1) < THRESH(1)));
end

%fprintf('\n')

for i = 1:T_above-1
  J = J(find(X(J+i,1) >= THRESH(2)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~isempty(J))
  for k = 1:length(J)
    Jbefore(:,k)   = [J(k)  - T_before  : J(k)  - 1]';
    Jafter(:,k)    = [J(k)              : J(k) + T_after-1]';
    for i = 1:size(X,2)
      Ybefore(:,k,i) = X(J(k)  - T_before : J(k)  - 1      ,i);
      Yafter(:,k,i)  = X(J(k)             : J(k) + T_after-1,i);
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

