function Y = qqchunk5(xin,thresh1,thresh2,qtime1,plot_it)
%QQCHUNK5 Left boundary set by column 1, right boundary by column 2.
%
%    Y = QQCHUNK5(A,thresh1,thresh2,qtime1,plot_it)
%
%    Start time when A(:,1) crosses above thresh1.  End time is when A(:,2)
%    crosses below thresh2 (and after A(:,1) has crossed below thresh1).  
%    This is nearly the same as qqchunk2, except right boundary is changed.
%    
%    See also QQCHUNK, BDL, CHUNK, QQCHUNK_DEMO, QQCHUNK_PLOT.

% See end of program for difference between this and qqchunk2.
  
qtime  = qtime1;
thresh = thresh1;

if (nargin < 4)
  plot_it = 0;
end

Y      = [];
y      = [];

x      = [thresh;xin(:,1);thresh];
J1     = find(x >= thresh);
D1     = diff(J1);
Id1    = find(D1 > qtime);
bound  = J1(Id1) + 1;

if (length(bound) < 2)
  return;
else 
  if (mod(length(bound),2) == 1)
    B = bound(1:end-1);
  else 
    B = bound;
  end
end

B1 = B;
B2 = B;

for i = 0:length(B)/2 - 1
  k       = 1+2*i;
  I       = find(x(B(k):end) >= thresh);
  B1(k)   = B(k) + I(1) - 2;
  I       = find(x(B(k+1):end) >= thresh);
  if (isempty(I)),k,B(k+1),end;
  B2(k+1) = B(k+1) + I(1) - 2; 
end

if (mod(length(bound),2) == 1)
  B2 = [B2(2:end); bound(end)];
else
  B2 = B2(2:end-1);
end

%B1'
%B2'
B3 = sort([B1;B2]);

keven = [1:2:length(B3)-1];
a     = B3(keven);
b     = B3(keven + 1) - 2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that no below thresh crossings
% between a and b.
kk = 1;
for k = 1:length(a)
  I      = find(xin(a(k):b(k),1) < thresh);
  if (isempty(I))
    aa(kk,1) = a(k); 
    bb(kk,1) = b(k);
    kk       = kk+1;
  end
end
if (~isempty(aa))
  a = aa;
  b = bb;
end

% This should be vectorized and put earlier.
for i = 1:length(a)
  I = find(xin(b(i):end,2) < thresh2);
  if (isempty(I))
    I(1)    = b(i);
  else
    bnew(i) = b(i) + I(1);
  end
end

b = bnew';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k = 1:length(a)
  for i = 1:size(xin,2)
    y{k}(:,i)    = xin(a(k):b(k),i);
    ymean(k,i)   = mean(y{k}(:,i));
    ymax(k,i)    = max(y{k}(:,i));
    yarea(k,i)   = sum(y{k}(:,i));
  end
end

if (~isempty(y))
  Y.chunk = y;
  Y.mean  = ymean;
  Y.max   = ymax;
  Y.area  = yarea;
  Y.j1    = a;
  Y.j2    = b;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if (plot_it == 1)
  qqchunk_plot(xin,Y,thresh,qtime)
end
  



