function Y = qqchunk7(xin,thresh1,thresh2,qtime1,plot_it)
%QQCHUNK7 Left boundary set by column 1, right boundary by column 2.
%
%    Y = QQCHUNK7(A,thresh1,thresh2,qtime1,plot_it)
%
%    Start time when A(:,1) crosses above thresh1.  End time is when A(:,2)
%    crosses below thresh2 (and after A(:,1) has crossed below thresh1).  
%    Also, if A(to,2) >= thresh2, chunk is rejected (second column must
%    initially be quiet.
%
%    This is nearly the same as qqchunk5, except for above restriction.
%
%    Also, Area for 2 is determined by subtracting off initial value of A(To,2).    
%
%    See also QQCHUNK, BDL, CHUNK, QQCHUNK_DEMO, QQCHUNK_PLOT.

% See end of program for difference between this and qqchunk5.
  
qtime  = qtime1;
thresh = thresh1;

if (nargin < 4)
  plot_it = 0;
end

Y      = [];
y      = [];

x      = [thresh;xin(:,1);thresh];
J1     = find(x >= thresh);
D1     = diff(J1);
Id1    = find(D1 > qtime);
bound  = J1(Id1) + 1;

if (length(bound) < 2)
  return;
else 
  if (mod(length(bound),2) == 1)
    B = bound(1:end-1);
  else 
    B = bound;
  end
end

B1 = B;
B2 = B;

for i = 0:length(B)/2 - 1
  k       = 1+2*i;
  I       = find(x(B(k):end) >= thresh);
  B1(k)   = B(k) + I(1) - 2;
  I       = find(x(B(k+1):end) >= thresh);
  if (isempty(I)),k,B(k+1),end;
  B2(k+1) = B(k+1) + I(1) - 2; 
end

if (mod(length(bound),2) == 1)
  B2 = [B2(2:end); bound(end)];
else
  B2 = B2(2:end-1);
end

%B1'
%B2'
B3 = sort([B1;B2]);

keven = [1:2:length(B3)-1];
a     = B3(keven);
b     = B3(keven + 1) - 2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that no below thresh crossings
% between a and b.
kk = 1;
for k = 1:length(a)
  I      = find(xin(a(k):b(k),1) < thresh);
  if (isempty(I))
    aa(kk,1) = a(k); 
    bb(kk,1) = b(k);
    kk       = kk+1;
  end
end
if (~isempty(aa))
  a = aa;
  b = bb;
end

% This should be vectorized and put earlier.
for i = 1:length(a)
%  I = find(xin(b(i):end,2) < thresh2);
  I = find(xin(b(i):end,2) < a(i));
  if (isempty(I))
    I(1)    = b(i);
  else
    bnew(i) = b(i) + I(1);
  end
end

b = bnew';
z = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k = 1:length(a)
  z = 1;
  if (xin(a(k),2) < thresh2)
    for i = 1:size(xin,2)
      y{z}(:,i)    = xin(a(k):b(k),i);
      ymean(z,i)   = mean(y{z}(:,i));
      ymax(z,i)    = max(y{z}(:,i));
      yarea(z,i) = sum( xin(a(k):b(k),i) );
      yarea(z,i)   = sum(y{z}(:,i));
      a2(z)        = a(k);
      b2(z)        = b(k);
    end
    z            = z+1;
  end
end

if (~isempty(y))
  Y.chunk = y;
  Y.mean  = ymean;
  Y.max   = ymax;
  Y.area  = yarea;
  Y.j1    = a2;
  Y.j2    = b2;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if (plot_it == 1)
  qqchunk_plot(xin,Y,thresh,qtime)
end
  



