function plot_qqchunk(x,Y,thresh,qtime,Names,Nperpage,Npages,E)
%QQCHUNK_PLOT Plot output of QQCHUNK function.
% 
%     qqchunk_plot(x,Y,thresh,qtime) plots the output  
%     Y = qqchunk(x,thresh,qtime).
%
%     qqchunk_plot(x,Y,thresh,qtime,Names,Nperpage,Npages) attaches a legend
%     for each column of x and plots Nperpage and a total of Npages.
%
%
% See also QQCHUNK, QQCHUNK_PLOT.

if (nargin < 5) | isempty(Names) 
  Names    = num2str([1:size(x,2)]');
end

if (1)
figure
setpos
orient landscape;
hold on;
grid on;
zoom on;
t  = [1:size(x,1)];

h=plot(x);
legend(h,Names);
plot(t,t*0 + thresh,'b-')
if (~isempty(Y))
  vertline(Y.j1,'g',1,'--');
  vertline(Y.j2,'r',1,'--');
end

figure
setpos;
orient landscape;
hold on;
grid on;
subplot(2,2,1)
 logloghist(Y.mean);
 xlabel('Output Mean')
 ylabel('Number')
 grid on;
subplot(2,2,2)
 logloghist(Y.max); 
 xlabel('Output Max')
 ylabel('Number')
 grid on
subplot(2,2,3)
 logloghist(Y.area); 
 xlabel('Output Area')
 ylabel('Number')
 grid on
subplot(2,2,4)
 logloghist(Y.j2-Y.j1); 
 xlabel('Length of event')
 ylabel('Number')
 grid on
end
 
% For plotting all chunks

if (exist('Nperpage'))
if (~exist('Npages')),Npages = 1000000;,end;
if (~exist('E')),E = [1:length(Y.chunk)];,end;
Np = 0;

for i = 1:length(E)
  if (Np <= Npages)
    k = mod(i-1,Nperpage);
    if (k == 0),figure;setpos;Np = Np+1;,end;
    %I = [1 3 5 7 2 4 6 8];
    I  = [1:Nperpage];
    subplot(Nperpage/2,2,I(k+1))
    t  = [Y.j1(E(i)):Y.j1(E(i))+size(Y.chunk{E(i)},1)-1];
    plot(t-Y.j1(E(i)),Y.chunk{E(i)})
    grid on;axis tight
  end
end
close
end




