function D = mag_H(MAGNAME,SOURCE,D_NUM,IN_YEARS)
%MAG_H Return horizontal component of ground magnetic field
%
%  H = mag_H(MAGNAME,SOURCE);
%
%  H = mag_H(MAGNAME,SOURCE,D_NUM,IN_YEARS)
%
%  D_NUM = 0 means H data that is not detrended.
%
%
  
mag_data_info;

LTb           = 30;
DOYb          = 30;
MAGNAME       = lower(MAGNAME);

if (SOURCE == 'fmi')
  I         = strmatch(MAGNAME,lower(MAGLIST_FMI));
  COORDSYS  = COORDSYS_FMI;
  YEARS     = YEARS_FMI;
  MAGLIST   = MAGLIST_FMI;
  directory = directory_fmi;
  FLAG      = FLAG_FMI;
end
if (SOURCE == 'dmi')
  I         = strmatch(MAGNAME,lower(MAGLIST_DMI));
  COORDSYS  = COORDSYS_DMI;
  YEARS     = YEARS_DMI;
  MAGLIST   = MAGLIST_DMI;
  directory = directory_dmi;
  FLAG      = FLAG_DMI;
end
if (SOURCE == 'spi')
  I         = strmatch(MAGNAME,lower(MAGLIST_SPI));
  COORDSYS  = COORDSYS_SPI;
  YEARS     = YEARS_SPI;
  MAGLIST   = MAGLIST_SPI;
  directory = directory_spi;
  FLAG      = FLAG_SPI;
end

if (nargin == 4) 
  YEARS(I,1) = IN_YEARS(1);
  YEARS(I,2) = IN_YEARS(end);
end

if isempty(I)
  error(sprintf('Could not find data for %s',MAGNAME));
end

if (nargin < 3) 
  D_NUM = 1;
end

i = 1;
D = [];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for years = [YEARS(I,1):YEARS(I,2)];

  if (COORDSYS(I,:) == 'HDZ')

    tempD = mag_data_load(MAGLIST(I,1:3),years,COORDSYS(I,1),directory,FLAG(I,1));
    if (D_NUM > 0)
      tempD = block_detrend_nonflag(tempD,1440*365,FLAG(I,1),FLAG(I,1));
    end
  end 
  if (COORDSYS(I,:) == 'XYZ') 

    Dx     = mag_data_load(MAGLIST(I,1:3),years,COORDSYS(I,1),directory,FLAG(I,1));
    IDx    = find(abs(Dx)>=FLAG(I,1));
    Dy     = mag_data_load(MAGLIST(I,1:3),years,COORDSYS(I,2),directory,FLAG(I,2));
    IDy    = find(abs(Dy)>=FLAG(I,2));
    tempD  = sqrt(Dx.^2 + Dy.^2);
  
    tempD(IDx) = FLAG(I,1);
    tempD(IDy) = FLAG(I,1);
    if (D_NUM > 0)    
      tempD      = block_detrend_nonflag(tempD,1440*365,FLAG(I,1),FLAG(I,1));
    end
    clear Dx Dy IDx IDy
  end  

  D = [D;tempD];

end

% Columns of D are years, rows are first 24*365*60 minutes of year.
% Make D a column vector.
D   = D(:);
F   = length(find(abs(D)<FLAG(I,1)))/length(D);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot time series to eps file
if (0)
  Ig     = find(D<99999);
  Ib     = find(D>=99999);
  meanIg = mean(D(Ig));
  Dtemp(Ib)  = meanIg;

  setpos;
  plot(blockave(Dtemp,60),'r');hold on;
  title(sprintf('%s %d-%d H, mean = %.1e, fraction good = %.2f',...
		MAGLIST(I,:),YEARS(I,1),YEARS(I,2),meanIg,F));

  if (YEARS(I,2)-YEARS(I,1) < 15)
    year_ticklabels(YEARS(I,1),60,2);
  end
  if (YEARS(I,2)-YEARS(I,1) < 8)
    year_ticklabels(YEARS(I,1),60,1);
  end
  if (YEARS(I,2)-YEARS(I,1) > 14)
    year_ticklabels(YEARS(I,1),60,4);
  end

  grid on;
  axis tight;
  drawnow;
  eps_write(sprintf('%s_%d_%d',MAGLIST(I,1:3),YEARS(I,1),YEARS(I,2)),2);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
