function [X,Y] = mag_rotate(X,Y,FLAGVAL)
%MAG_ROTATE Rotates into magnetic N-S and E-W direction.
%
%   [X,Y] = MAG_ROTATE(X,Y) Transforms so that average of Y is
%   zero.  Input is uniform matrix obtained from DMI_UNIFORM.
%
%   theta  = atan2(meanX,meanY);
%    xx    =  x*sin(theta) + y*cos(theta);
%    yy    = -x*cos(theta) + y*sin(theta);
%
%   [X,Y] = DMI_UNIFORM_ROTATE(X,Y,FLAGVAL)
%
%   Default FLAGVAL is 99999. 
%
%   See also DMI_DEMO, DMI_DOWNLOAD, DMI_PLOT.

if (~exist('FLAGVAL')),FLAGVAL=99999;,end  

n_years = size(X,3);

for j = 1:size(X,4)
  
    x      = X(:);
    y      = Y(:);
    
    Igx    = find(x(:)<FLAGVAL);   % Good indices for x
    Ibx    = find(x(:)>=FLAGVAL);  % Bad indices for x
    mx     = mean(x(Igx));      
    
    Igy    = find(y(:)<FLAGVAL);   % Good indices for y
    Iby    = find(y(:)>=FLAGVAL);  % Bad indices for y
    my     = mean(y(Igy));      
    
    theta  = atan2(mx,my);

    fprintf('Number of minutes (hrs) used for angle calculation = %d (%d)\n',...
	    min(length(Igy),length(Igx)),floor(min(length(Igy),length(Igx))/60))   
    fprintf('Angle for magnetometer %d = %.2f degrees\n',j,theta*(180/pi))
	
    xn    =  x*sin(theta) + y*cos(theta);
    y     = -x*cos(theta) + y*sin(theta);
    
    xn(Iby) = FLAGVAL;
    xn(Ibx) = FLAGVAL;

    y(Iby) = FLAGVAL;
    y(Ibx) = FLAGVAL;

end
