function COORDS = station_name2geographic(Name)
%STATION_NAME2GEOGRAPHIC Get geographic coordinates of station.
%
%    COORDS = STATION_NAME2GEOGRAPHIC(MAG_NAMES)
%
%    COORDS will have the same number of columns as MAG_NAMES has rows.
%    MAG_NAMES should be a Nx3 string matrix of 3 letter station names
%    (e.g., ['NAL';'SOD']).
%    
%    COORDS(1,:) is geographic latitude.
%    COORDS(2,:) is geographic east longitude.
%
%    See also STATION_NAME2GEOMAG, UT2MLT, UT2LT.

REGEXP  = sprintf('=~ m/.*/');  
fname   = which('station_coordinates.dat');
[I,A]   = perlop(fname,REGEXP,[],[],'file');
Adata   = str2num(A(:,5:end));
Aname   = A(:,1:4);

for i = 1:size(Name,1)  

  name    = upper(Name(i,:)); % Change to upper case
  I       = strmatch(name,upper(Aname));
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if (isempty(I)) % If station data does not exist in station_coordinates.dat

    error(sprintf('I do not have data for %s.  Inserting zeros for coordinates\n',Name(i,:)));
    error(sprintf('Manually add data to station_coordinates.dat. Aborting.\n'));
    COORDS(i,:) = [0 ; 0];
    
  else % If station data does exist in station_coordinates.dat
    
    COORDS(i,:) = Adata(I(1),1:2);      

  end  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
end
