function MLTMN = station_name2mltmn(Name)
%STATION_NAME2GMLTMN Get MLTMN for a list of magnetometers.
%
%    MLTMN = STATION_NAME2MLTMN(Names)
%
%    MLTMN is obtained from NSSDC website using the year 1998.  MLTMN is UT
%    that gives a MLT of midnight.  Values taken from NSSDC are stored
%    in station_coordinates.dat.  
%
%    See also STATION_NAME2GEOGRAPHIC, UT2MLT, UT2LT.

REGEXP  = sprintf('=~ m/.*/');  
fname   = which('station_coordinates.dat');
%[I,A]   = perlop(fname,REGEXP,[],[],'file');

Adata   = str2num(A(:,5:end));
Aname   = A(:,1:4);

for i = 1:size(Name,1)  

  name    = upper(Name(i,:)); % Change to upper case
  I       = strmatch(name,upper(Aname));
  
  if (isempty(I)) % If station data does not exist in station_coordinates.dat

    error(...
	sprintf('I do not have data for %s in station_coordinates.dat.'...
		'Returning NaNs.\n',Name(i,:)));
    COORDS(i,:) = [NaN ; NaN];
    
  else % If station data does exist in station_coordinates.dat
    
    MLTMN(i,1) = Adata(I(1),end);
    
  end  
  
end
