function LT = ut2lt(UT,E_LONG)
%UT2LT Convert UT to local time.
%
%    LT = UT2LT(UT,E_LONG) UT [hours] and geographic E_LONG [degrees] will
%    have same number of rows.  UT should be in the range [0,24].  E_LONG
%    must be a single-row vector and UT must be a single-column vector.  LT
%    will have size(LT) = [length(UT) length(E_LONG)];
%
%    LT = UT + (24 - LTMN), where LTMN is obtained from geographic latitude.
%    LTMN is UT that gives a LT of midnight.  
%
%    See also, UT2MLT.
%    MLT = UT + (24 - MLTMN), where MLTMN is obtained from NSSDC website.
%    MLTMN is UT that gives a MLT of midnight.  For example, for KEV
%    Geographic Longitude = 27.0, Geomagnetic Longitude = 110.1 in 1993,
%    MLTMN = 20:51, so MLT = UT + (24 - 20:51) = UT + 3:09, which agrees
%    with Engebretson et al., JGR 103, A11, 1998.
%
  
  
LTMN   = repmat([24],1,length(E_LONG)) - E_LONG/15.0; % In hours;

UT     = repmat(UT,1,length(E_LONG)); % Make UT have same number columns
                                      % as E_LONG.
				      
LTMN   = repmat(LTMN,size(UT,1),1);   % Make LTMN have same number of rows
                                      % as UT.


LT     = UT + 24.0 - LTMN;            % Transformation is LT   = UT + 24 - LTMN;
LT     = mod(LT,24);
