function timelabels = year_ticklabels(year_o,dT,dyr)
%YEAR_TICKLABELS Put year labels on x-axes.
%
%   years = YEAR_TICKLABELS(YEAR), where YEAR is the year for the first data
%   point, which is assumed to correspond to Jan 1.  Assumes data is 1
%   minute samples.
%
%   IMPORTANT: If plotting multiple time series, put this command after first
%   plot command.  This restriction should be removed.
%
%   Assumes that the most recent number of y data points plotted is the
%   number of minutes.  
%  
%   years = YEAR_TICKLABEL(year_o,dT) specifies that time between data
%   points is dT minutes.
%
%   years = YEAR_TICKLABEL(year_o,dT,dyr) puts year labels with spacing of
%   dyr.  Default is dyr = 1.
%
%   See also IS_LEAP_YEAR.

%   R.S. Weigel, December 22, 2003.  

%   To do: Find method for removing need for temp variable.
  
if (nargin == 1)
  dyr  = 1;
  dT   = 1;
end

if (nargin == 2)
  dyr  = 1;
end

Npd        = 24*60/dT; % Number of points per day.

if (length(year_o)==2)
  L          = year_o(2);
  year_o     = year_o(1);
else
  if (iscell(get(get(gca,'Children'),'YData')))
    % This is a memory sink.
    temp     = get(get(gca,'Children'),'YData');
    L        = length(temp{end});
    clear temp;
  else
    L        = length(get(get(gca,'Children'),'YData'));
  end
end

F          = floor(L/(365.242190*Npd));
Xt(1)      = 1;

for i = 2:F+1
  if (is_leap_year(year_o-2+i))
    Ndays = 366;
  else
    Ndays = 365;
  end
  Xt(i) = Xt(i-1) + Ndays*Npd;
end  

Xt         = Xt([1:dyr:length(Xt)]);
             set(gca,'XTick',Xt);

year_f     = year_o+floor(L/(365.242190*Npd));
timelabels = [year_o:dyr:year_f];
             set(gca,'XTickLabel',timelabels);