/* R.S. Weigel, 09/02/2004 */
#include <octave/config.h>

#include <iostream>

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>

#include <octave/Cell.h>
#include <octave/ov-list.h>
#include <octave/ov-struct.h>

#include "octave/config.h"

#include "octave.h"
#include "symtab.h"
#include "parse.h"
#include "unwind-prot.h"
#include "toplev.h"
#include "error.h" 
#include "quit.h"
#include "variables.h"
#include "sighandlers.h"
#include "sysdep.h"

#include "read_hdf_sd.h"

extern "C" int free_SDS(SDS_structure *SDS, int return_data);
extern "C" int read_hdf_sd(SDS_structure *SDS, int return_data,
			   int verbosity);

DEFUN_DLD (load_hdf, args, nargout, 
"LOAD_HDF Extract scientific data from HDF (.hdf) file                  \n\
                                                                        \n\
   n_datasets  = LOAD_HDF(fname) Returns the number of scientific       \n\
                 data sets in file fname.                               \n\
                                                                        \n\
   Info        = LOAD_HDF(filename,sds_index) Returns information about \n\
                 data set number sds_index = (0, ..., n_datasets).      \n\
                                                                        \n\
   [Info,data] = LOAD_HDF(filename,sds_index) Returns data from data set\n\
                 number sds_index.                                      \n\
                                                                        \n\
   [Info,data] = LOAD_HDF(filename,sds_index,1) Prints debug info.      \n\
                                                                        \n\
   Assumes that the HDF file contains only scientific data sets.        \n\
                                                                        \n\
   Note that Octave's LOAD can load HDF5 (.h5), which is a different    \n\
   file format than HDF (.hdf).  (An alternative to using LOAD_HDF      \n\
   .hdf files can be converted to .h5 with h4h5tools available          \n\
   from NCSA.)                                                          \n\
                                                                        \n\
   See also LOAD.")
{

  int i,j,row=0,col=0;
  int verbosity = 1;
  int return_data = 0;

  octave_value_list retval;
  //octave_value_list tmp = eval_string ("a=1",true,verbosity);
  //octave_value tmp = eval_string ("a=1",true,verbosity);
  //printf("%.2f\n",tmp.double_value ());

  static SDS_structure SDS;

  if (args.length () == 4){  
    octave_value arg3 = args(3);
    verbosity         = (int) arg3.double_value ();
  }

  /* Get file name ***********************************************/  
  if (args.length () > 0){

    octave_value arg0 = args(0);
    charMatrix cm     = arg0.char_matrix_value();
    row               = arg0.rows();
    col               = arg0.columns();

    SDS.sd_filename   = (char *)calloc((col+1),sizeof(char));
    //printf("col = %d\n",col);
    for(j=0;j<row;j++){
      strncpy(SDS.sd_filename,(cm.extract(j,0,j,col-1).data()),col);
    }
  }  
  /****************************************************************/
  /* First get info about file by passing return_data = 0 *********/
  read_hdf_sd(&SDS, 0, verbosity);
  /****************************************************************/
  if (args.length () > 2) {
    octave_value arg2 = args(2);
    Matrix Index    = arg2.matrix_value ();
    SDS.dims.start  = (int32*)calloc(1+Index.cols (),sizeof(int32));
    SDS.dims.stride = (int32*)calloc(1+Index.cols (),sizeof(int32));
    SDS.dims.edges  = (int32*)calloc(1+Index.cols (),sizeof(int32));
    SDS.dims.Npts   = (int32*)calloc(1+Index.cols (),sizeof(int32));
    for (i = 0 ; i < Index.cols () ; i ++) {
      SDS.dims.start[i]  = (int) Index(0,i);
      SDS.dims.stride[i] = (int) Index(1,i);
      SDS.dims.edges[i]  = (int) Index(2,i);
      SDS.dims.Npts[i]   =  SDS.dims.edges[i]/SDS.dims.stride[i];
      if (verbosity > 0) {
	printf("SDS.dims.start[%d]=%d\n",i,SDS.dims.start[i]);
	printf("SDS.dims.stride[%d]=%d\n",i,SDS.dims.stride[i]);
	printf("SDS.dims.edges[%d]=%d\n",i,SDS.dims.edges[i]);
	printf("SDS.dims.Npts[%d]=%d\n",i,SDS.dims.Npts[i]);
      }
    }

  } else {
    SDS.dims.start  = (int32*)calloc(1,sizeof(int32));
    SDS.dims.stride = (int32*)calloc(1,sizeof(int32));
    SDS.dims.edges  = (int32*)calloc(1,sizeof(int32));
    SDS.dims.Npts   = (int32*)calloc(1,sizeof(int32));
    SDS.dims.start[0]  = -1;
    SDS.dims.stride[0] = -1;
    SDS.dims.edges[0]  = -1;
    SDS.dims.Npts[0]   = -1;
    if (verbosity > 0) {
      printf("SDS.dims.start[0]=%d\n",SDS.dims.start[0]);
      printf("SDS.dims.stride[0]=%d\n",SDS.dims.stride[0]);
      printf("SDS.dims.edges[0]=%d\n",SDS.dims.edges[0]);
    }
  }

  /* If only filename was passed, just return number of datasets **/
  if (args.length () == 1){
    retval(0) = SDS.n_datasets;
    return retval;
  }
  /****************************************************************/

  /* If second argument was given, it is SDS.sds_index ************/
  if (args.length () > 1){

    /* If only one output, return only file info */
    if (nargout == 2)
      return_data =  1;
    if (nargout == 1)
      return_data =  0;    

    octave_value arg1 = args(1);
    double tmp        = arg1.double_value ();
    SDS.sds_index     = (int) tmp;
  }
  /***************************************************************/
  
  /* Return sds_info if sds_index was passed *********************/
  /* (also return data if nargout == 1) **************************/
  read_hdf_sd(&SDS, return_data, verbosity);
  /***************************************************************/  

  if (return_data == 1) { 
    /* Copy array_1D to Octave matrix. *****************************/
    /* This assumes number of dimensions is 1 or 2.                */
    /* (It should not.  Need to find documentation for N-D arrays  */
    /* in Octave ...)                                              */
    /* Should remove this step and return array_1D directly.       */
    /* (Need to find reshape documentation for Octave              */

    Matrix a_matrix = Matrix((int) SDS.dims.Npts[0],
			     (int) SDS.dims.Npts[1]);

    for (col = 0; col < (int) SDS.dims.Npts[1]; col++){
      for (row = 0; row < (int) SDS.dims.Npts[0]; row++){
	a_matrix(row, col) = SDS.float_array_1D[row + col*SDS.dims.Npts[1]];
      }
    }
    retval(1) = a_matrix;
    /***************************************************************/  
  }

  //string_vector sds_dim_value (4);

  //sds_dim_value(0)  (SDS.dims.value[0]);
  //sds_dim_value(1)  (SDS.dims.value[1]);
  //sds_dim_value(2)  (SDS.dims.value[2]);
  //sds_dim_value(3)  (SDS.dims.value[3]);

  string_vector sds_attr_names  (SDS.n_attributes);
  string_vector sds_attr_values (SDS.n_attributes);
  for (i = 0; i < SDS.n_attributes; i++) {
    sds_attr_names(i)   = SDS.attributes.attr_names[i];
    sds_attr_values(i)  = SDS.attributes.attr_values[i];
  }

  Matrix dims = Matrix (1,2);
  dims(0,0) = SDS.dim_sizes[0];
  dims(0,1) = SDS.dim_sizes[1];

  Cell Names  = Cell (sds_attr_names);
  Cell Values = Cell (sds_attr_values);

  Octave_map map;
  map.assign ("name",octave_value (SDS.name));
  map.assign ("label",octave_value (SDS.label));
  map.assign ("unit",octave_value (SDS.unit));
  map.assign ("format",octave_value (SDS.format));
  map.assign ("coord_system",octave_value (SDS.coord_system));
  map.assign ("n_attributes",octave_value (SDS.n_attributes));
  map.assign ("rank",octave_value (SDS.rank));
  map.assign ("dim_sizes",octave_value (dims));
  map.assign ("AttrNames" , octave_value (sds_attr_names));
  map.assign ("AttrValues" , octave_value (sds_attr_values));

  retval(0) = map;
  //return retval;
  //return octave_value (map);

  free_SDS(&SDS,return_data);

  return retval;

}
