## Copyright (C) 2006 R.S Weigel
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## -*- texinfo -*-
## @deftypefn {Function File} {} cleanpath (@var{p})
##
## Puts path string into standard format. If @var{p} is a string,
## first make @var{p} begin and end with ":", then
## replace any number of neigboring "/"s with "/" (except "//:" or ":///:"), 
## replace any number of neigboring ":"s with ":",
## and replace ":./:" or ":.:" with [].  
##
## cleanpath(':.') =>  '::'
##
## cleanpath(':./') =>  '::'
##
## cleanpath('///') => ':///:'
##
## cleanpath('/dir//') => ':/dir//:'
##
## cleanpath('/dir///') => ':/dir//:'
##
## cleanpath('/dir1/:/dir2//:') => ':/dir1:/dir2//:'
##
## If @var{p} is a cell, return a cell with one cleaned path
## per element.  
## cleanpath(@{'/abc:/def','/geh'@}) => @{':/abc:',':/def:',':/geh:'@}
##
## The function checkpath can be used to determine if the paths are fit to be added to LOADPATH.
##
## @seealso{LOADPATH, rmpath, savepath, setpath}
##
## @end deftypefn

## Author: R.S. Weigel <rweige@gmu.edu>

function p = cleanpath(p)


  if (isempty(p))
    if (nargout)
      p = [];
    end
    return;
  end

if (iscell(p))

  ## Split up compound paths
  ## {'abc:def','geh'} => {'abc','def','geh'}
  ## {':abc:def','geh'} => {'abc','def','geh'}
  ## {'abc:def:','geh'} => {'abc','def','geh'}
  k = 1;
  
  for i = 1:length(p)
    p{i} = cleanpath(p{i});
    if (~isempty(p{i}))
    I = find(p{i}==":");
    if (length (I) == 1)
      np{k} = p{i}(1:I(1)-1);
      k = k+1;
      np{k} = p{i}(I(1)+1:end);
      k = k+1;      
    elseif (length (I) > 1)
      for j = 2:length(I)
	np{k} = p{i}(I(j-1)+1:I(j)-1);
	np{k} = cleanpath(np{k});
	k = k+1;
      end
    else
      tmp = cleanpath(p{i});
      if (~isempty(tmp))
	np{k} = tmp;
	k = k+1;
      end
    end
    end

  end
  if (k>1)
    p = np; ## One path per cell
  else
    p = [];
  end

  return;
  
end

if (length (p) == 1)
  if (! isempty(strmatch(p(1),".")))
    p = [];
    return;
  end
end

I = find(p==":");
if (isempty(I)) && length(p)>0
  p=[":",p,":"];
  I = [1,length(p)+2];
end

if (I(1)!=":") 
  p = [':',p];
end

if (I(end)!=":"); 
  p = [p,':'];
end

## Any number of neigboring ":"s => ":"
pold = [];
while length(p) != length(pold)
  pold = p;
  p = strrep (p, "::", ":");
end

## Any number of neigboring "/"s => "/"
pold = [];
while length(p) != length(pold)
  pold = p;
  p = strrep (p, "//:", "///:");
  p = strrep (p, "//", "/");
end

if (! isempty (strmatch(p,"//"))) && (warn_matlab_incompatible)
  warning ("cleanpath: Double forward slashes found at end of path");
end

## remove any :.: or :./:
p = strrep (p,":.:",":");
p = strrep (p,":./:",":");
p = strrep (p,"://:",":///:");

if (length(p)==1) && (p(1) == ":")
  p = [];
end



