function plotformat(h,lw,ms,ti,tb,xl,xb,yl,yb,ns,nb)
%PLOTFORMAT Format fonts and attributes for readability.
%
%   PLOTFORMAT(h,lw,ms,ti,tb,xl,xb,yl,yb,ns,nb) where h is the plot handle.  
%
%   If h is a vector, LineWidth and MarkerSize is changed for all lines
%   and markers specified by h on current figure.
%
%   lw is line width
%   ms is marker size
%   ti is title size
%   tb is 1 for bold 0 for regular
%   xl is xlabel size
%   xb is 1 for bold 0 for regular
%   yl is ylabel size
%   yb is 1 for bold 0 for regular
%   ns is number size
%   nb is 1 for bold 0 for regular
%
%   Note: Because of error in matlab, set all bolds to 0 if you want greek in any of
%   the labels.
%
%   PLOTFORMAT(h) is the same as PLOFORMAT(h,3 ,10,18,1 ,18,1 ,18,1 ,18,1);
%   PLOTFORMAT(h,'latex') is the same as PLOFORMAT(h,3 ,10,18,0 ,18,0 ,18,0 ,18,0);
%
%   See also LAPRINT.

if ( (nargin == 1) )
  lw = 3;
  ms = 10;
  ti = 18;
  tb = 1;
  xl = 18;
  xb = 1;
  yl = 18;
  yb = 1;
  ns = 18;
  nb = 1;
end

if (nargin == 2)
  ms = 20;
  ti = 18;
  tb = 1;
  xl = 18;
  xb = 1;
  yl = 18;
  yb = 1;
  ns = 18;
  nb = 1;
  if strmatch(lw,'latex','exact');
    nb = 0;
    yb = 0;
    xb = 0;
    tb = 0;
    lw = 3;
  end
end


% Change marker and line size
for i = 1:length(h)  
  set(h(i),'LineWidth',lw)
  set(h(i),'MarkerSize',ms)
end

a    = get(gcf,'CurrentAxes');
bx   = get(a,'Xlabel');
by   = get(a,'Ylabel');
titl = get(a,'Title');

% Change FontSize on title, x and yaxis
set(titl,'FontSize',ti)
if ti==1,set(titl,'FontWeight','bold'),end;
set(bx,'FontSize',xl)
if xb==1,set(bx,'FontWeight','bold'),end;
set(by,'FontSize',yl)
if yb==1,set(by,'FontWeight','bold'),end;


% Change numbers
set(a,'FontSize',ns)
if nb==1,set(a,'FontWeight','bold'),end;





