function Do = block_detrend(Di,N,FLAG)
%BLOCK_DETREND_NONFLAG Subtract off block mean.
%
%   D_dt = BLOCK_DETREND(D,N) Subtracts off block mean of elements in
%   non-overlapping windows of width N.  Operation is done columnwise.
%   size(D_dt) = size(D).  If size(D,1)/N is not an integer FLAG is used used
%   to pad D_dt.  Use BLOCK_DETREND(D,N,FLAG) to insert FLAG instead of NaN.
%   
%   See also BLOCK_DETREND_NONFLAG.

% R.S. Weigel, 04/02/2004.

if (nargin < 3)
  FLAG = NaN;
end
  
for j = 1:size(Di,2)
  
  D      = Di(:,j);  

  Lo     = length(D);
  L      = floor(length(D)/N);
  tempD  = D(1:L*N);
  tempD  = reshape(tempD,N,L);

  temp   = mean(tempD,1);
  temp   = repmat(temp,N,1);
  D      = tempD-temp;

  D       = D(:);
  Do(:,j) = [D ; FLAG*ones(Lo-L*N,1)];

end
