function Z = corrcoef_nonflag(X,Y,FLAG_VAL,EXACT)
%CORRCOEF_NONFLAG Correlation coefficient of non-FLAG elements.
%
%   Z = corrcoef_nonflag(X,Y) is std of elements in 1-D arrays X and Y not
%   equal to NaN.
%
%   Z = corrcoef_nonflag(X,Y,FLAG,COND) See IS_FLAG for definitions of
%   FLAG and COND.
%
%   See also CORRCOEF, *_NONFLAG.

% R.S. Weigel, 04/02/2004.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG_VAL = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
M = size(X,1);
N = size(X,2);

if (N > M)
  X = X';
  Y = Y';
  M = size(X,1);
  N = size(X,2);
  if (N > 1)
    error('X and Y must be 1-D arrays');
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

[jnk,IX,IgX] = reflag(X,FLAG_VAL,FLAG_VAL,EXACT)

[jnk,IY,IgY] = reflag(Y,FLAG_VAL,FLAG_VAL,EXACT)

I = intersect(IgX,IgY);

if (length(I)>1)
  Z   = corrcoef(X(I),Y(I));
else
  Z   = FLAG_VAL;
end

Z = Z(1);

