function I = is_flag(x,FLAG,COND)
%IS_FLAG Tests elements to determine if they qualify as a flagged value   
%
%    I = IS_FLAG(x) Same as isnan(x)
%  
%    I = IS_FLAG(x,FLAG,COND)
%
%    Default FLAG is NaN.
%
%    COND = -1 flag condition is equivalent to isnan(x); FLAG must be [] or NaN.
%    COND =  0 flag condition is x >= FLAG.
%    COND =  1 flag condition is x == FLAG (default).  
%    COND =  2 flag condition is abs(x) >= FLAG.
%
%    See also *_NONFLAG.

if (nargin < 2)
  FLAG = NaN;
  COND = 1;
end
if (nargin < 3)
  COND = 1;
end

if (COND == -1)
  if (~isnan(FLAG))
    error('For COND = -1, FLAG must be [] or NaN');
  end
end

I = zeros(size(x));

if (COND == -1)
  I = isnan(x);
end

if (COND == 0)
  I = (x >= FLAG);
end

if (COND == 1)
  if isnan(FLAG)
    I = find(isnan(x) == 1);
  else
    I = (x == FLAG);
  end
end

if (COND == 2)
  I = (abs(x) >= FLAG);
end

