function [X,I,Ig] = reflag(X,FLAG_o,FLAG_f,EXACT)
%REFLAG Change flag values.
%
%  X = REFLAG(X,FLAG_o,FLAG_f) Replaces all instances of FLAG_o in X with
%  FLAG_f.
%
%  X = REFLAG(X,FLAG_o,FLAG_f,EXACT) If EXACT = 0, replaces elements greater
%  than or equal to FLAG_o with FLAG_f.  EXACT = 1, replaces elements equal
%  to FLAG_o with FLAG_f.  EXACT = 2, replaces elements with absolute value
%  greater than or equal to FLAG_o with FLAG_f.  Default is EXACT = 1.
%
%  [X,I]    = REFLAG(...) Returns I, the indices of the reflagged elements.
%  [X,I,Ig] = REFLAG(...) Returns Ig, the indices of the nonflagged elements.
%
%  See also *_NONFLAG.

% R.S. Weigel, 04/02/2004.

if (nargin < 4)
  EXACT = 1;
end

if (EXACT == 0)
  I = find(X >= FLAG_o);
  if (nargout == 3)
    Ig = find(X < FLAG_o);
  end
end

if (EXACT == 1)

  if (isnan(FLAG_o))
    I = find(isnan(X) == 1);  
  else
    I = find(X == FLAG_o);  
  end

  if (nargout == 3)
    if (isnan(FLAG_o))
      Ig = find(isnan(X) ~= 1);  
    else
      Ig = find(X ~= FLAG_o);  
    end
  end
end

if (EXACT == 2)
  if (FLAG_o < 0)
    warning(['With EXACT = 2 and FLAG_o < 0, all data points '...
	     'will be reflaged as bad']);
  end
  
  I = find(abs(X) >= FLAG_o);

  if (nargout == 3)
    Ig = find(abs(X) < FLAG_o);
  end

end

if (~isempty(I))
  X(I) = FLAG_f*ones(length(I),1);
end


