# Usage: perlop.pl filename a b pattern maxlength write_dat
# filename  = $ARGV[0] Filename
# a         = $ARGV[1] First line to output
# b         = $ARGV[2] Last line to output
# pattern   = $ARGV[3] Line is output if it matches pattern
# left      = $ARGV[4] Left boundary of line
# right     = $ARGV[5] Right boundary of line
# write_dat = $ARGV[6] If $ARGV[6] exists and is 1, padded data file with beginning
#                      whitespace removed is written.  If $ARGV[6] is 2, nonpadded 
#                      data file is written.  If $ARGV[6] = 0 or is not specified,
#                      no data file is written (only index data is written).

# Output files are named "filename_index" "filename_processed" All
# lines in output file have the same number of columns if $ARGV[6] == 1.
# Blanks are added to lines shorter than the longest line.

# To do:
# Need to tell perlop.pl if writing A file is necessary.
# Need have perlop.pl determine if padding A is necessary.
# note that matlab knows 1E-5 notation, etc.
# how to determine how many columns?

# R.S. Weigel 04/01/2002.  Many times I have disowned this file
# slicing and dicing program and the perlop.m program that calls it.
# Instead of using the perlop.m/perlop.pl programs to parse a data
# file I suggest that you just modify one of the other .pl files in
# this directory.

$nargin = @ARGV;                        # number of input arguments.
if ($nargin < 6) {$ARGV[6] = 0};

$COM     = "\$line " . $ARGV[3] . "o";  # Prepare pattern for eval function
                                        # Also add the "o" option for the pattern,
                                        # which tells perl that pattern will never
                                        # change and tells perl to compile pattern.

#print $ARGV[3];
######################################################################################
$Nlong = 0; # largest number of columns in file
$k     = 1;
$x     = 1;

open(DATA,$ARGV[0]) or die "Can't open $ARGV[0] file: $!\n";
open(DATAi,">$ARGV[0]_index") or die "Can't open $ARGV[0]_index file: $!\n";

# Put while loop in one big string and then eval the string.  This
# is considerably faster than perlopv0.1.pl which calls eval
# for each line of file read.  See evaltest.pl for comparison.
# Note the use of \$ and \\s \\n, etc which means that "\\n" will
# be \n.  See the perl man page on "".

#print $ARGV[6];

if ($ARGV[6] == 0 | $ARGV[6] == 1) {

   $com = "
      while (\$line = <DATA>){
        if ( (\$x >= $ARGV[1]) & (\$x <= $ARGV[2]) )  {
           if ($COM) {         
           \$line      =~ s/^\\s+//;
           \$line      =~ s/\\s+\$//;
           \$N         = length(\$line);
           if (\$N > \$Nlong) {\$Nlong = \$N};
           \$Line[\$k] = \$line;
           \$k         = \$k + 1;
           printf(DATAi \"\$x\\n\");
         };
       };
       \$x = \$x + 1;
      }";

}

if ($ARGV[6] == 2) {
  $com = "
  while (\$line = <DATA>){
    if ( (\$x >= $ARGV[1]) & (\$x <= $ARGV[2]) )  {
      if ($COM) {         
         chomp(\$line);
         \$N         = length(\$line);
         if (\$N > \$Nlong) {\$Nlong = \$N};
         \$Line[\$k] = \$line;
         \$k         = \$k + 1;
         printf(DATAi \"\$x\\n\");
      };
   };
  \$x = \$x + 1;
  }";
}

#print $ARGV[4];

if ($ARGV[6] == 3) {
  $com = "
  while (\$line = <DATA>){
    if ( (\$x >= $ARGV[1]) & (\$x <= $ARGV[2]) )  {
       \$line = substr(\$line,$ARGV[4]-1,$ARGV[5]-$ARGV[4]+1); 
        if ($COM){
         printf(DATAi \"\$x\\n\");
         chomp(\$line);
         \$Line[\$k] = \$line;
         \$N         = length(\$line);
         if (\$N > \$Nlong) {\$Nlong = \$N};
         \$k         = \$k + 1;
         }
   };
  \$x = \$x + 1;
  }";
}

# Make the $com string into one line by removing newlines.
# If $ARGV[3] contains a \n, it is not removed!  This method is
# better than placing commas at the end of each line in $com
# and then removing commas since the commas in $ARGV[3] will 
# also be removed.

$com =~ s/\n/ /g;
#print "$com\n"; 
eval $com;

close(DATA);
close(DATAi);


if ($ARGV[6] > 0) {
################################################################
# Create a blank line and add blanks so all lines have same length 
# and output to file
$blnks = ' ';
$blnks =~ s/ /' 'x($Nlong)/e; # Creates Nmax blanks
################################################################
open(DATAw,">$ARGV[0]_processed") or die "Can't open $ARGV[0]_processed output file: $!\n";
for ($i = 1;$i < $k;$i++){
    $N         = length($Line[$i]); 
    $Line[$i]  = $Line[$i] . substr($blnks,0,$Nlong-$N);  # Add Nmax - N blanks to line

    if ( ($ARGV[5]-$ARGV[4] >= 0) ){
	#print "$Line[$i]\n";
	#$Line[$i] = substr($Line[$i],$ARGV[4]-1,$ARGV[5]-$ARGV[4]+1); 
	#print "$Line[$i]\n";
        #Output only first $ARGV[5]-$ARGV[4] non-newlines
    }
    
    printf(DATAw "%s\n",substr($Line[$i],$ARGV[4],$ARGV[5]));
    # Output to file named $ARGV[0]_processed

}
close (DATAw);
}


