/* R.S. Weigel, 06/21/2005 */
#include <octave/oct.h>
#include <lo-mappers.h>

#ifdef __cplusplus    /* make it C if in C++ compiler */
  extern "C" {
#endif

#include "cxform.h"

#ifdef __cplusplus  /* closing } for C in C++ compiler  */
  }
#endif



DEFUN_DLD (sptransform_oct, args, ,"SPTRANSFORM_OCT Called by SPTRANSFORM. \
Do not call directly.")
{

  octave_value_list retval;

  int i,j,k=0;
  int debug;
  
  int year, month, day, hour=0, minute=0, second=0;
  int ret;
  int es;
  double jd;
  int inSysNumber, outSysNumber;
  Vec v_in, v_out;

  static char *Systems[11];

  Systems[0] = "GEI";
  Systems[1] = "J2000";
  Systems[2] = "GEO";
  Systems[3] = "MAG";
  Systems[4] = "GSE";
  Systems[5] = "GSM";
  Systems[6] = "SM";
  Systems[7] = "GSEQ";
  Systems[8] = "HEE";
  Systems[9] = "HAE";
  Systems[10] = "HEEQ";

  //  printf("%s\n",Systems[9]);
  
  inSysNumber  = int ( args(0).double_value () ) ;
  outSysNumber = int ( args(1).double_value () );

  debug = int ( args(4).double_value () );

  Matrix   UTov(args(2).matrix_value());
  Matrix   V_in_ov(args(3).matrix_value());

  int mrowsV_in  = V_in_ov.rows();
  int mrowsUT    = UTov.rows();
  int mcolsUT    = UTov.cols();

  Matrix   V_out_ov(mrowsV_in*mrowsUT,3);

  for (j=0;j<mrowsUT;j++){

    year   = int ( UTov(j,0) );
    month  = int ( UTov(j,1) );
    day    = int ( UTov(j,2) );
    if (mcolsUT > 3) { hour   = int ( UTov(j,3) );};
    if (mcolsUT > 4) { minute = int ( UTov(j,4) );};
    if (mcolsUT > 5) { second = int ( UTov(j,5) );};
  
    //if ( j%1000 == 0)
    //printf("j=%d/%d\n",j,mrowsUT);

    if (debug == 1) {  
      jd = gregorian_calendar_to_jd(year, month, day, hour, minute, second);
    }

    es = date2es(year, month, day, hour, minute, second);
    
    for (i=0;i<mrowsV_in;i++){
      
      v_in[0] = V_in_ov(i,0);
      v_in[1] = V_in_ov(i,1);
      v_in[2] = V_in_ov(i,2);
      
      ret = cxform(Systems[inSysNumber],Systems[outSysNumber],es,v_in,v_out);
      
      if (debug == 1) {
	printf("Time: %.4d/%.2d/%.2d %.2d:%.2d:%.2d (JD %f, ES: %d)\n", 
	       year, month, day, hour, minute, second, jd, es);

  
	if (ret == 0)  {
	  printf("Input Vector (%s): \t%f\t%f\t%f\n",
		 Systems[inSysNumber], v_in[0], v_in[1], v_in[2]);
	    
	  printf("Output Vector (%s):\t%f\t%f\t%f\n",
		 Systems[outSysNumber], v_out[0], v_out[1], v_out[2]);
	} else {
	  printf("Error during call to cxform\n");
	}  
      
	fflush(stdout);

      }
    
      V_out_ov(k,0) = v_out[0];
      V_out_ov(k,1) = v_out[1];
      V_out_ov(k,2) = v_out[2];

      k = k + 1;
      
    }
    
  }

  retval(0) = V_out_ov;
  
  return retval; 

}


