/*
    WebStats Scripts
 
    Original Version Copyright(c) 2006, Jeff Schiller (http://blog.codedread.com/).
    
    Several enhancements provided by Johan Sundstrom (http://ecmanaut.blogspot.com/)
    and the full article about how this came to be is available at
    http://ecmanaut.blogspot.com/2006/01/svg-challenge-taken.html
    
    BIZARRE NOTE: ASV's scripting engine does not like the way Johan's last name is 
    spelled so I have modified it.

    Permission is hereby granted to use and/or modify this software as long as 
    credits are given to the creators listed above.

    Reference: http://www.rad-e8.com/log/i/0912diag_ua.jpg

    Enhancements since the last iteration:
    
    - Added all of Johan's enhancements (including the CSS, the better text, more labels)
    - Made the legend draggable (bug: in IE after you drag, it requires 2 clicks to move the horiz cursor)
    - Fixed colour scheme for all browsers
    - Made the horizontal cursor draggable, pie chart updates as you drag (slowly at this time)
    - Moved time-based chart drawing to client side
    - Made grippies part of a scrollbar that allows panning (slick but slow)
*/


// for some reason, Opera seems not to apply CSS styles to created nodes. :-(
function forceStyles( attr )
{
    var all = attr["id"]+' '+attr["class"], id, type, styles, selector = /\S+/g;
    while( id = selector.exec( all ) )
    {
        if( (styles = css[id[0]]) )
        for( type in styles ) // (attr[type] ? attr[type]+';' : '' ) + 
            attr[type] = styles[type];
    }
  return attr;
}

function $( id )
{
  return document.getElementById( id );
}

function make( name, attr, text, url )
{
    var node = document.createElementNS( ns.svg, name );
    var name = '', atns;
    if(attr) {
        attr = forceStyles( attr );
        for( var i in attr )
        {
            atns = (/^(.*):/.exec( i )||[,null])[1];
            node.setAttributeNS( atns && ns[atns], i, attr[i] );
        }
    }
    if( text )
    {
        text = document.createTextNode( text );
        node.appendChild( text );
        if( url )
        {
            var a = make( 'a', { 'xlink:href':url } );
            a.appendChild( node );
            return a;
        }
    }
    return node;
}

function removeChildren( node )
{
  while( node.hasChildNodes() )
  node.removeChild( node.firstChild );
}

function cssname( name )
{
  return name.replace( /[. /]/g, '' );
}

function draggingLegend() {
    bDraggingLegend = true;
}

// SVG file will have the raw data in:
// - NumDataPoints is the number of data points
// - BrowserData maps series names to arrays of values
// - Colors maps series names to RGB color strings
function init() 
{

    var svgnode = $("svgsvg");
    if(svgnode) {

	if (INTERACTIVE==1) {
          svgnode.addEventListener("click", clickGraph, false);
          svgnode.addEventListener("keydown", getKey, false);
       
  	  if (SHOW_LEGEND==1) {
            var legend = $("browser_legend");
            if(legend) {
                legend.addEventListener("mousedown", draggingLegend, false);
            }
          }
        
          // set up a listener for the horizontal cursor being dragged
          addDragEventListener("dragstart", dragStart);
          addDragEventListener("dragmove", dragMove);
          addDragEventListener("dragdrop", dragDrop);
        }


      var g = $( 'plot_background' );
      if( !g ) return;
      g.appendChild( make( 'rect', { "x":PLOT_ULx, "y":PLOT_ULy, "width":PLOT_WIDTH, "height":PLOT_HEIGHT, "fill":"url(#g1)", "stroke":"black", "stroke-width":"2px"} ) );
      var legend = $("plot_background");
      if(legend) {
          legend.setAttributeNS(null, "display", "inline");
      }

      var g = $( 'window_background' );
      if( !g ) return;
      g.appendChild( make( 'rect', { "x":WINDOW_ULx, "y":WINDOW_ULy, "width":WINDOW_WIDTH, "height":WINDOW_HEIGHT, "fill":"yellow" } ) );
      var backgnd = $("window_background");
      if(backgnd) {
          legend.setAttributeNS(null, "display", "inline");
      }



      var x = 10, y = 20, h = 11, w = 30, dy = 0, c; 
      var g = $( 'browser_legend2' );
      if( !g ) return;
      g.appendChild( make( 'rect', { "x":x, "y":y, "width":w, "height":h, "class":c } ) );
      g.appendChild( make( 'text', { "x":x+w+x, "y":y+8, "font-size":'10', "fill":'#DEDEDE' }, "zz" ) );
      var legend = $("browser_legend2");
      if(legend) {
          legend.setAttributeNS(null, "display", "inline");
      }


        createTimeBasedChart();

        // set up pie chart elements
	if (SHOW_PIE==1) {        
          BrowserPieElements = new PieElementsContainer();
	  drawPieChart();
	}

	if (SHOW_LEGEND==1) {
          drawLegend();
          showLegend();
	}	

    } // if(svgnode)

} //init()

// TO DO: KEY_UP and KEY_DOWN changes the currently selected stat
function getKey(evt) {
    switch(evt.keyCode) {
    case KEY_LEFT:
        evt.preventDefault();
        if(SHOW_PIE && gCursorX > browserStartPoint) {
            --gCursorX;
            // update cursor
            var cur = $("browser_horizcurs");
            if(cur) {
                var x = PLOT_WIDTH*(gCursorX-browserStartPoint)/(browserEndPoint-browserStartPoint);
                cur.setAttributeNS(null, "transform", "translate(" + x + ",0)");
                cur.setAttributeNS(ns.drag, "x", x);
            }

	    // update pie chart
            drawPieChart();
        }
        break;
    case KEY_RIGHT:
        evt.preventDefault();
        if(SHOW_PIE && gCursorX < browserEndPoint) {
            ++gCursorX;
            // update cursor
            var cur = $("browser_horizcurs");
            if(cur) {
                var x = PLOT_WIDTH*(gCursorX-browserStartPoint)/(browserEndPoint-browserStartPoint);
                cur.setAttributeNS(null, "transform", "translate(" + x + ",0)");
                cur.setAttributeNS(ns.drag, "x", x);
            }

            // update pie chart
	    drawPieChart();
        }
        break;
    }
}

function dragMove(evt) {
    if(!bDraggingLatch) {
        bDraggingLatch = true;
        
        if(SHOW_PIE && evt.dragEnt == $('browser_horizcurs')) {
            var newCursorX = browserStartPoint + Math.round( (evt.userx/PLOT_WIDTH) * (browserEndPoint-browserStartPoint) );
            if(newCursorX >= browserStartPoint && newCursorX <= browserEndPoint) {
                gCursorX = newCursorX;
                drawPieChart();
            }
        }
        else if(evt.dragEnt == $('browser_leftgrip') || evt.dragEnt == $('browser_rightgrip')) {
            var left = parseFloat($('browser_leftgrip').getAttributeNS(ns.drag,"x"));
            var right = PLOT_WIDTH + parseFloat($('browser_rightgrip').getAttributeNS(ns.drag,"x"));
            changeGlobalXAxisBounds(left,right);
        }
        else if(evt.dragEnt == $('browser_scrollbar') ) {
            var left = parseFloat($('browser_scrollbar').getAttributeNS(ns.drag,"x"));
            var right = left + parseFloat($('browser_scrollbar').getAttributeNS(null, "width"));
            changeGlobalXAxisBounds(left,right);
        }
        else {
            // moving legend - who cares
        }
        
        bDraggingLatch = false;
    }
}

function dragStart(evt) {
    if( evt.dragEnt == $('browser_leftgrip') || 
        evt.dragEnt == $('browser_rightgrip') ||
        evt.dragEnt == $('browser_scrollbar')
        ) 
    {
        // remove axes
        var axes = $('browser_timechartaxes');
        removeChildren(axes);
    }
}

// recreate axes
function dragDrop(evt) {
    if( evt.dragEnt == $('browser_leftgrip') || 
        evt.dragEnt == $('browser_rightgrip') ||
        evt.dragEnt == $('browser_scrollbar')
        ) 
    {
        // this will change the MaxY and the gCursorX
        reflowTimeBasedAxesAndPie();

        // this will potentially rescale in the y-direction
        var left = parseFloat($('browser_leftgrip').getAttributeNS(ns.drag,"x"));
        var right = PLOT_WIDTH + parseFloat($('browser_rightgrip').getAttributeNS(ns.drag,"x"));
        changeGlobalXAxisBounds(left,right);
    }
}

function clickGraph(evt) {
    // only process if the click was on the graph (not the legend)
    if(bDraggingLegend) { 
        bDraggingLegend = false;
        return;
    }
            
    var p = document.documentElement.createSVGPoint();
    p.x = evt.clientX;
    p.y = evt.clientY;

    var screenCTMInv = getScreenCTM().inverse();
    p = p.matrixTransform(screenCTMInv);
    
    if(SHOW_PIE && p.x >= 0 && p.y >= 0 && p.x <= PLOT_WIDTH && p.y <= PLOT_HEIGHT) {
        // determine the data point to use
        gCursorX = browserStartPoint + Math.round( (p.x/PLOT_WIDTH) * (browserEndPoint-browserStartPoint) );

        // actual X will be aligned with the data points
        var actualX = (PLOT_WIDTH * (gCursorX-browserStartPoint)) / (browserEndPoint-browserStartPoint);
        
        // update cursor
        var cur = $('browser_horizcurs');
        if(cur) {
            cur.setAttributeNS(null, "transform", "translate(" + actualX + ",0)");
            cur.setAttributeNS(ns.drag, "x", actualX);
        }
        
        // update pie chart
        drawPieChart();
    }
}

function changeGlobalXAxisBounds(left, right) {
    // modify the constraints of each grip
    var leftgrip = $('browser_leftgrip');
    var rightgrip = $('browser_rightgrip');
    var scrollBar = $('browser_scrollbar');

    if(!leftgrip || !rightgrip || !scrollBar) { return; }

    leftgrip.setAttributeNS(null, "transform", "translate(" + left + ")");
    leftgrip.setAttributeNS(ns.drag, "x", left);
    leftgrip.setAttributeNS(ns.drag,"constraintRight",(right - (10 * PLOT_WIDTH / NumDataPoints)));
    
    rightgrip.setAttributeNS(null, "transform", "translate(" + (right - PLOT_WIDTH) + ")");
    rightgrip.setAttributeNS(ns.drag, "x", (right - PLOT_WIDTH));
    rightgrip.setAttributeNS(ns.drag,"constraintLeft",(-PLOT_WIDTH + left + (10 * PLOT_WIDTH / NumDataPoints)));
    
    // change the scrollbar dimensions:
    var w = (right - left);

    scrollBar.setAttributeNS(null, "width", w);
    scrollBar.setAttributeNS(null, "transform", "translate(" + left + ")");

    scrollBar.setAttributeNS(ns.drag, "x", left);
    scrollBar.setAttributeNS(ns.drag, "constraintLeft",0);
    scrollBar.setAttributeNS(ns.drag, "constraintRight", PLOT_WIDTH-w);
    
    reflowTimeBasedGraph(left,right);
}

function reflowTimeBasedAxesAndPie() 
{
    // next, run through the raw data, determine the max Y (find the y-limit)
    // it's too bad there is no way around this 
    var maxY = 0;
    for(var day = browserStartPoint; day <= browserEndPoint; ++day) {
        var thisDaySumY = 0;
        for(browser in BrowserData) {
            thisDaySumY += BrowserData[browser][day];
        }

        // determine maximum Y
        var max = parseInt(thisDaySumY/100 + 1) * 100;
        if(max > maxY) { maxY = max; }
    }
    BrowserCurrentMaxY = maxY;

    // next, re-create the x and y axes elements
    var axes = $('browser_timechartaxes');
    var yaxis = drawTimeBasedYAxis(maxY);
    var xaxis = drawTimeBasedXAxis(browserStartPoint,browserEndPoint);

    // next, determine if the horizontal cursor needs to be shifted
    var bNeedToRedrawPieChart = false;
    if(gCursorX < browserStartPoint) { 
        gCursorX = browserStartPoint; 
        bNeedToRedrawPieChart = true;
    }
    else if(gCursorX > browserEndPoint) { 
        gCursorX = browserEndPoint; 
        bNeedToRedrawPieChart = true;
    }

    // update horizontal cursor
    var cur = $("browser_horizcurs");
    if(cur) {
        var x = PLOT_WIDTH*(gCursorX-browserStartPoint)/(browserEndPoint-browserStartPoint);
        cur.setAttributeNS(null, "transform", "translate(" + x + ",0)");
        cur.setAttributeNS(ns.drag, "x", x);
    }
    if(SHOW_PIE && bNeedToRedrawPieChart) {
//        setTimeout("drawPieChart()", 1);
        drawPieChart();
    }
    
    // next, remove the x and y axes
    removeChildren(axes);
    // next, add in the new x- and y- axes elements
    axes.appendChild(yaxis);
    axes.appendChild(xaxis);
}

function reflowTimeBasedGraph(left,right)
{
    // first, determine the data points that the grippies map to
    browserStartPoint = Math.round( (left/PLOT_WIDTH) * (NumDataPoints-1) );
    browserEndPoint = Math.round( (right/PLOT_WIDTH) * (NumDataPoints-1) );

    // next, re-transform the paths
    var numPts = (browserEndPoint - browserStartPoint);
    var scaleX = PLOT_WIDTH / numPts;
    var scaleY = PLOT_HEIGHT / BrowserCurrentMaxY;
    // translateX by PLOT_WIDTH will always align the right-most point (most recent) with
    // the right edge of the graph - we need to shove it further right by 
    // (NumDataPoints - 1 - browserEndPoint) points
    var nudge = (NumDataPoints - 1 - browserEndPoint)*PLOT_WIDTH/numPts;
    var translateX = PLOT_WIDTH + nudge;
    var translateY = PLOT_HEIGHT;
    var graphUnscaled = $('browser_timechartseries');
    graphUnscaled.setAttributeNS(null, "transform", "translate(" + translateX + "," + translateY + ")  scale(-1,-1)");
    var graph = $('browser_timechartseries_scaled');
    graph.setAttributeNS(null, "transform", "scale(" + scaleX + "," + scaleY + ") ");
}

