function S = CDAWeb_info(NAME,ins_no,var_no,Attr_in)
%CDAWEB_INFO Returns info on variables to be extracted from CDAWeb files
%
%   NAMES = CDAWeb_info returns cell array of available satellite names.
%
%   S = CDAWeb_info(satellite) returns structure with fields
%
% satellite    =  % Satellite name
% DATA_DIR     =  % Directory on ftp site where data is 
%                 % (you need to look this up)
% fname_base   =  % Base of the file name (can usually be derived from 
%                 % directory name above)
% timename     =  % Always time variable name 
%                   (you need to find this using from CDFINFO on a file)
% varname      =  % Variable name
% unitfactor   =  % Unit conversion factor for variable
%
%    Attr = CDAWeb_info(satellite,Attr) returns structure Attr
  
% R.S. Weigel 12/23/2004

if (nargin == 0)  
  S = {'doublestar1','doublestar2','cluster','goes8','goes10',...
       'ulysses','ace','wind','polar','imp8','geotail'};
  return;
end

if (strmatch('doublestar1',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'doublestar1';            % Satellite name
S(1).DATA_DIR       = 'doublestar/d1/sp/aux';   % Directory on ftp site
                                                % a guess since not on
                                                % NSSDC yet
S(1).fname_base     = 'T1_SP_AUX';              % Base of the file name	
S(1).timename       = 'Epoch__T1_SP_AUX';       % Time variable name
S(1).varname        = {'sc_r_xyz_gse__T1_SP_AUX'};% Variable 1
S(1).unitfactor     = 1; % Unit conversion factor for Variable 1
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'doublestar.tgz';
 
S(1).Attr(1).Name{1}  = 'Doublestar 1 X position (German Cluster Data Centre)';
S(1).Attr(1).Name{2}  = 'Doublestar 1 Y position (German Cluster Data Centre)';
S(1).Attr(1).Name{3}  = 'Doublestar 1 Z position (German Cluster Data Centre)';
S(1).Attr(1).CoordinateSystem = 'GSE';
S(1).Attr(1).Unit     = 'km';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).Source   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(1).Attr(1).README   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(1).Attr(1).Acknowledgement = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';


S(2).satellite      = 'doublestar1';             
S(2).DATA_DIR       = 'doublestar/d1/sp/fgm';    
S(2).fname_base     = 'T1_SP_FGM';             
S(2).timename       = 'Epoch__T1_SP_FGM';    
S(2).varname        = {'B_xyz_gse__T1_SP_FGM'}; 
S(2).unitfactor     = 1;
S(2).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(2).DATA_PACKAGE_NAME = 'doublestar.tgz';

S(2).Attr(1).Name{1}  = 'Doublestar 1 Bx (German Cluster Data Centre)';
S(2).Attr(1).Name{2}  = 'Doublestar 1 By (German Cluster Data Centre)';
S(2).Attr(1).Name{3}  = 'Doublestar 1 Bz (German Cluster Data Centre)';
S(2).Attr(1).CoordinateSystem = 'GSE';
S(2).Attr(1).Unit     = 'nT';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).Source   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(2).Attr(1).README   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(2).Attr(1).Acknowledgement = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('doublestar2',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'doublestar2';             % Satellite name
S(1).DATA_DIR       = 'doublestar/d2/sp/aux';    % Directory on ftp site
                                                 % a guess since not on
                                                 % NSSDC yet
S(1).fname_base     = 'T2_SP_AUX';               % Base of the file name
S(1).timename       = {'Epoch__T2_SP_AUX'};      % Time variable name
S(1).varname        = {'sc_r_xyz_gse__T2_SP_AUX'}; % Variable 1
S(1).unitfactor     = 1; % Unit conversion factor for variable 1
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'doublestar.tgz';

S(1).Attr(1).Name{1}  = 'Doublestar 2 X position (German Cluster Data Centre)';
S(1).Attr(1).Name{2}  = 'Doublestar 2 Y position (German Cluster Data Centre)';
S(1).Attr(1).Name{3}  = 'Doublestar 2 Z position (German Cluster Data Centre)';
S(1).Attr(1).CoordinateSystem = 'GSE';
S(1).Attr(1).Unit     = 'km';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).Source   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(1).Attr(1).README   = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
S(1).Attr(1).Acknowledgement = 'http://cl1.plasma.mpe-garching.mpg.de/cdms/';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
  
if (strmatch('cluster',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'cluster';               
S(1).DATA_DIR       = 'cluster/cl/sp/aux';     
S(1).fname_base     = 'cl_sp_aux';             
S(1).timename       = {'Epoch__CL_SP_AUX'};    
S(1).varname        = {'sc_r_xyz_gse__CL_SP_AUX'};
S(1).unitfactor     = 1;
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'cluster.tgz';

S(1).Attr(1).Name{1}  = 'Cluster X position (CDAWeb)';
S(1).Attr(1).Name{2}  = 'Cluster Y position (CDAWeb)';
S(1).Attr(1).Name{3}  = 'Cluster Z position (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSE';
S(1).Attr(1).Unit     = 'km';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesC.html#CL_SP_AUX'];

S(2).satellite      = 'cluster';
S(2).DATA_DIR       = 'cluster/cl/sp/fgm';
S(2).fname_base     = 'cl_sp_fgm';            
S(2).timename       = {'Epoch__CL_SP_FGM'};   
S(2).varname        = {'B_xyz_gse__CL_SP_FGM'}; 
S(2).unitfactor     = 1;
S(2).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(2).DATA_PACKAGE_NAME = 'cluster.tgz';

S(2).Attr(1).Name{1}  = 'Cluster Bx';
S(2).Attr(1).Name{2}  = 'Cluster By';
S(2).Attr(1).Name{3}  = 'Cluster Bz';
S(2).Attr(1).CoordinateSystem = 'GSE';
S(2).Attr(1).Unit     = 'nT';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesC.html#CL_SP_FGM'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

if (strmatch('goes10',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'goes10';
S(1).DATA_DIR       = 'goes/0_mag';
S(1).fname_base     = 'g0_k0_mag';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'B_GSM_c';
S(1).varname{2}     = 'SC_pos_sm';
S(1).unitfactor(1)  = 1; 
S(1).unitfactor(2)  = 1; 
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'goes.tgz';

S(1).Attr(1).Name{1}  = 'GOES 10 Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'GOES 10 By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'GOES 10 Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    

S(1).Attr(2).Name{1}  = 'GOES 10 X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'GOES 10 Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'GOES 10 Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('goes8',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'goes8';
S(1).DATA_DIR       = 'goes/8_mag/';
S(1).fname_base     = 'g8_k0_mag';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'B_GSM_c';
S(1).varname{2}     = 'SC_pos_sm';
S(1).unitfactor(1)  = 1; 
S(1).unitfactor(2)  = 1; 
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'goes.tgz';

S(1).Attr(1).Name{1}  = 'GOES 8 Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'GOES 8 By (CDAWeb)';
pS(1).Attr(1).Name{3}  = 'GOES 8 Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#G8_k0_MAG'];

S(1).Attr(2).Name{1}  = 'GOES 8 X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'GOES 8 Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'GOES 8 Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#G8_k0_MAG'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('ulysses',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'ulysses';
S(1).DATA_DIR       = 'ulysses/vhm_1min';
S(1).fname_base     = 'uy_1min_vhm';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'B_RTN';
S(1).unitfactor(1)  = 1; 
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = 'ulysses.tgz';

S(1).Attr(1).Name{1}  = 'Ulysses Br (CDAWeb)';
S(1).Attr(1).Name{2}  = 'Ulysses Bt (CDAWeb)';
S(1).Attr(1).Name{3}  = 'Ulysses Bn (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'RTN';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesU.html#UY_M1_VHM'];

% No position data on CDAWeb as of 2005.06.23
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('ace',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'ace';
S(1).DATA_DIR       = 'ace/mfi_h0';
S(1).fname_base     = 'ac_h0_mfi';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'BGSM';
S(1).varname{2}     = 'SC_pos_GSM';
S(1).unitfactor(1)  = 1; 
S(1).unitfactor(2)  = 1; 
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp/ace';
S(1).DATA_PACKAGE_NAME = 'mfi_h0.tgz';

S(1).Attr(1).Name{1}  = 'ACE Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'ACE By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'ACE Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesA.html#AC_H0_MFI'];

S(1).Attr(2).Name{1}  = 'ACE X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'ACE Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'ACE Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesA.html#AC_H0_MFI'];

S(2).satellite      = 'ace';
S(2).DATA_DIR       = 'ace/swe_h0/';
S(2).fname_base     = 'ac_h0_swe';
S(2).timename       = {'Epoch'};
S(2).varname{1}     = 'V_GSM';
S(2).varname{2}     = 'Np';
S(2).unitfactor(1)  = 1; 
S(2).unitfactor(2)  = 1; 
S(2).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp/ace';
S(2).DATA_PACKAGE_NAME = 'swe_h0.tgz';

S(2).Attr(1).Name{1}  = 'ACE solar wind Vx (CDAWeb)';
S(2).Attr(1).Name{2}  = 'ACE solar wind Vy (CDAWeb)';
S(2).Attr(1).Name{3}  = 'ACE solar wind Vz (CDAWeb)';
S(2).Attr(1).CoordinateSystem = 'GSM';
S(2).Attr(1).Unit     = 'km/s';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesA.html#AC_H0_SWE'];

S(2).Attr(2).Name{1}  = 'ACE proton number density (CDAWeb)';
S(2).Attr(2).Unit     = '#/cc';
S(2).Attr(2).PPD      = '1440';		    
S(2).Attr(2).TimeUnit = 'minute';		    
S(2).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesA.html#AC_H0_SWE'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('wind',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'wind';
S(1).DATA_DIR       = 'wind/mfi_h0';
S(1).fname_base     = 'wi_h0_mfi';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'BGSM';
S(1).varname{2}     = 'PGSM';
S(1).unitfactor(1)  = 1;
S(1).unitfactor(2)  = 6378; % Unit conversion factor for variable 2 (From Re);
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp/wind/mfi_h0';
S(1).DATA_PACKAGE_NAME = ['1994.tgz';...
  		    '1995.tgz';...
  		    '1996.tgz';...
  		    '1997.tgz';...
  		    '1998.tgz';...
  		    '1999.tgz';...
  		    '2000.tgz';...
  		    '2001.tgz';...
  		    '2002.tgz';...
  		    '2003.tgz';...
    		    '2004.tgz'];
      		    

S(1).Attr(1).Name{1}  = 'Wind Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'Wind By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'Wind Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesW.html#WI_H0_MFI'];

S(1).Attr(2).Name{1}  = 'Wind X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'Wind Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'Wind Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesW.html#WI_H0_MFI'];

%%

S(2).satellite      = 'wind';
S(2).DATA_DIR       = 'wind/swe';
S(2).fname_base     = 'wi_k0_swe';
S(2).timename       = {'Epoch'};
S(2).varname{1}     = 'V_GSM';
S(2).unitfactor     = 1; 
S(2).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp/wind';
S(2).DATA_PACKAGE_NAME = 'swe.tgz';

S(2).Attr(1).Name{1}  = 'Wind solar wind Vx (CDAWeb)';
S(2).Attr(1).Name{2}  = 'Wind solar wind Vy (CDAWeb)';
S(2).Attr(1).Name{3}  = 'Wind solar wind Vz (CDAWeb)';
S(2).Attr(1).CoordinateSystem = 'GSM';
S(2).Attr(1).Unit     = 'km/s';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesW.html#WI_K0_SWE'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('polar',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'polar';
S(1).DATA_DIR       = 'polar/mfe';
S(1).fname_base     = 'po_k0_mfe';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'B_GSM';
S(1).varname{2}     = 'POS_GSM';    
S(1).unitfactor(1)  = 1;
S(1).unitfactor(2)  = 6378; % Unit conversion factor for variable 2 (From Re);
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = ['polar.tgz'];


S(1).Attr(1).Name{1}  = 'Polar Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'Polar By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'Polar Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesP.html#PO_K0_MFE'];

S(1).Attr(2).Name{1}  = 'Polar X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'Polar Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'Polar Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesP.html#PO_K0_MFE'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('imp8',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'imp8';
S(1).DATA_DIR       = 'imp8/mag';
S(1).fname_base     = 'i8_k0_mag';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'B_GSM_c';
S(1).varname{2}     = 'SC_pos_sm';
S(1).unitfactor(1)  = 1;
S(1).unitfactor(2)  = 1;
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = ['imp8.tgz'];

S(1).Attr(1).Name{1}  = 'IMP 8 Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'IMP 8 By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'IMP 8 Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSM';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesI.html#I8_K0_MAG'];

S(1).Attr(2).Name{1}  = 'IMP 8 X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'IMP 8 Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'IMP 8 Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSM';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesI.html#I8_K0_MAG'];

S(2).satellite      = 'imp8';
S(2).DATA_DIR       = 'imp8/pla';
S(2).fname_base     = 'i8_k0_pla';
S(2).timename       = {'Epoch'};
S(2).varname{1}     = 'V_GSE';
S(2).varname{2}     = 'Np';    
S(2).unitfactor(1)  = 1;
S(2).unitfactor(2)  = 1;
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = ['imp8.tgz'];

S(2).Attr(1).Name{1}  = 'IMP 8 solar wind Vx (CDAWeb)';
S(2).Attr(1).Name{2}  = 'IMP 8 solar wind Vy (CDAWeb)';
S(2).Attr(1).Name{3}  = 'IMP 8 solar wind Vz (CDAWeb)';
S(2).Attr(1).CoordinateSystem = 'GSM';
S(2).Attr(1).Unit     = 'km/s';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesI.html#I8_K0_PLA'];

S(2).Attr(2).Name{1}  = 'IMP 8 Np (CDAWeb)';
S(2).Attr(2).Unit     = '#/cc';
S(2).Attr(2).PPD      = '1440';		    
S(2).Attr(2).TimeUnit = 'minute';		    
S(2).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesI.html#I8_K0_PLA'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (strmatch('geotail',NAME,'exact'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S(1).satellite      = 'geotail';
S(1).DATA_DIR       = 'geotail/mgf';
S(1).fname_base     = 'ge_k0_mgf';
S(1).timename       = {'Epoch'};
S(1).varname{1}     = 'IB_vector';
S(1).varname{2}     = 'POS';
S(1).unitfactor(1)  = 1;
S(1).unitfactor(2)  = 1;
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = ['geotail.tgz'];

S(1).Attr(1).Name{1}  = 'Geotail Bx (CDAWeb)';
S(1).Attr(1).Name{2}  = 'Geotail By (CDAWeb)';
S(1).Attr(1).Name{3}  = 'Geotail Bz (CDAWeb)';
S(1).Attr(1).CoordinateSystem = 'GSE';
S(1).Attr(1).Unit     = 'nT';
S(1).Attr(1).PPD      = '1440';		    
S(1).Attr(1).TimeUnit = 'minute';		    
S(1).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#GE_K0_MGF'];

S(1).Attr(2).Name{1}  = 'Geotail X position (CDAWeb)';
S(1).Attr(2).Name{2}  = 'Geotail Y position (CDAWeb)';
S(1).Attr(2).Name{3}  = 'Geotail Z position (CDAWeb)';
S(1).Attr(2).CoordinateSystem = 'GSE';
S(1).Attr(2).Unit     = 'km';
S(1).Attr(2).PPD      = '1440';		    
S(1).Attr(2).TimeUnit = 'minute';		    
S(1).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#GE_K0_MGF'];

S(2).satellite      = 'geotail';
S(2).DATA_DIR       = 'geotail/cpi_h0';
S(2).fname_base     = 'ge_h0_cpi';
S(2).timename       = {'Epoch'};
S(2).varname{1}     = 'SW_Vc';
S(2).varname{2}     = 'SW_P_Den';    
S(2).unitfactor(1)  = 1;
S(2).unitfactor(2)  = 1;
S(1).DATA_PACKAGE_DIR = 'cdaweb.gsfc.nasa.gov/pub/istp';
S(1).DATA_PACKAGE_NAME = ['geotail.tgz'];

S(2).Attr(1).Name{1}  = 'Geotail ion Vx (CDAWeb)';
S(2).Attr(1).Name{2}  = 'Geotail ion Vy (CDAWeb)';
S(2).Attr(1).Name{3}  = 'Geotail ion Vz (CDAWeb)';
S(2).Attr(1).CoordinateSystem = 'GSE';
S(2).Attr(1).Unit     = 'km/s';
S(2).Attr(1).PPD      = '1440';		    
S(2).Attr(1).TimeUnit = 'minute';		    
S(2).Attr(1).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#GE_H0_CPI'];

S(2).Attr(2).Name{1}  = 'Geotail Ni (CDAWeb)';
S(2).Attr(2).Unit     = '#/cc';
S(2).Attr(2).PPD      = '1440';		    
S(2).Attr(2).TimeUnit = 'minute';		    
S(2).Attr(2).README   = ['http://cdaweb.gsfc.nasa.gov/cdaweb/misc/' ...
		         'NotesG.html#GE_H0_CPI'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

if (nargin == 4)
  for i = 1:length(Attr_in)
    Attr{i}          = ts_attr;
    Attr{i}.Year_o   = Attr_in{i}.Year_o;
    Attr{i}.Year_f   = Attr_in{i}.Year_f;
    Attr{i}.Size     = Attr_in{i}.Size;
    Attr{i}.Name     = S(ins_no).Attr(var_no).Name{i};
    Attr{i}.Unit     = S(ins_no).Attr(var_no).Unit;
    Attr{i}.PPD      = S(ins_no).Attr(var_no).PPD;
    Attr{i}.TimeUnit = S(ins_no).Attr(var_no).TimeUnit;
    Attr{i}.CoordinateSystem = S(ins_no).Attr(var_no).CoordinateSystem;
    
    Attr{i}.Source = 'ftp://cdaweb.gsfc.nasa.gov/istp/pub';
    Attr{i}.Acknowledgement = ...
	'CDAWeb as data provider and README for instrument PI';
    
    Attr{i}.Label   = sprintf('%s %s (CDAWeb %s)',...
			     S(ins_no).satellite,...
			     S(ins_no).varname{var_no},...
			     S(ins_no).fname_base);
  end
  clear S;
  S = Attr;
end