function [Datai,Ni] = interp1g(T,Data,Ti,method)
%INTERP1g INTERP1 with limits on gaps
%
%   [Datai,Ni] = interp1g(T,Data,Ti,method)
%   
%   Does INTERP1 for method = 'nearest' or 'linear'.
%
%   If method = 'nearest' and Ni(j) < 1, Datai(j) is set to NaN.
%   If method = 'nearest' and Ni(j) < 2, Datai(j) is set to NaN.
%   
%   See also INTERP1, INTERP1G_DEMO.
  
if (size(Data,1) == 1)
  Data = Data';
  T = T';
  Ti = Ti';
  flip = 1;
else
  flip = 0;
end

for i = 1:size(Data,2)

  I    = find(isnan(Data(:,i))==0);
  data = Data(I,i);
  t    = T(I);

  Ni = hist(t,Ti)'; % N has one row even if T and Ti have one column ...

  datai = interp1(t,data,Ti,method);

  Datai(:,i) = datai;

  if strmatch(method,'nearest','exact')
    I = find(Ni<1);
    Datai(I,i) = NaN;
  elseif strmatch(method,'linear','exact')
    I = find(Ni<2);
    Datai(I,i) = NaN;
  else
    error('method must be "nearest" or "exact"');
  end
end

if (flip == 1)
  Datai = Datai';
  Ni = Ni';
end


