function [V,ATTR] = COHOWeb_extract(DATA_DIR,SATINFO)
  
%  R.S Weigel, November 2003, December 2004.

% Get global variables and path setting
TSDS_GLOBALS;

fname    = sprintf('%s/%s.cdf',DATA_DIR,SATINFO.fname_base);
xInfo    = cdfinfo(fname);

% CDFREAD can read multiple records at once, so the loop
% can be eliminated if desired.

fprintf('Extracting data from %s\n',fname);
temp = cdfread(fname,'Variable',SATINFO.timename);
  
if (~isempty(temp)) 
  Time   = (todatenum(cat(2,temp{:})))';
else
  Time = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for var_no = 1:length(SATINFO.varname)

  % Get flag value
  tempcell = getfield(xInfo.VariableAttributes,'FILLVAL');
  ind      = strmatch(SATINFO.varname{var_no},tempcell(:,1),'exact');
  
  if (isempty(ind))
    error('Did not find variable %s in CDF file\n', SATINFO.varname{var_no});
  else
    FLAGV2(var_no) = tempcell{ind,2};
  end
  
  % Get variable
  temp           = cdfread(fname,'Variable',SATINFO.varname{var_no});
  Data(:,var_no) = SATINFO.unitfactor(var_no)*double(cat(2,temp{:})');

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FLAGV2  = unique(FLAGV2);
FLAGval = max(abs(FLAGV2));
Nf      = length(FLAGV2);   % Number of unique flag values

if (isinteger(Data)) % This happens for GEOTAIL magnetic field and causes
		     % problems with interp1
  Data = double(Data);
end

if (Nf > 1)
  warning(['Flag value is not unique. Reflagging using' ...
	   ' max(abs(FLAGS)).']);
  for i = 1:Nf
    Data = reflag(Data,FLAGV2(i),FLAGval,1);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sometimes there are repeats in time value which causes 
% interpolation to fail.  This removes repeats.
LTime     = length(Time);
[Time,I1] = unique(Time);

if (LTime > length(Time))
  Data = Data(I1,:);
  fprintf('Removed %d repeated time values (%.4f percent).\n',...
	  LTime-length(Time),100.0*(LTime-length(Time))/length(Time));
end

Year_o  = datevec(Time(1));
Year_f  = datevec(Time(end));

Year_o  = Year_o(1);
Year_f  = Year_f(1);

% Data in merged cdf files is on an hourly time grid

% DN is day number with Jan. 1, Year_o having DN = 0
DN = datenum(Time) - datenum([Year_o,1,1]);
% HN is hour number 
HN = 1 + round(24*DN);

Nhrs = 24*number_days(Year_o,Year_f);

Datai     = repmat(NaN,Nhrs,size(Data,2));
Datai(HN,:) = Data;

% Replace values greater than 0.99*abs(FLAGval) with TSDS_FLAG.
Datai    = reflag(Datai,abs(0.99*FLAGval),NaN,2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:size(Datai,2)
  V{i}            = Datai(:,i);
  ATTR{i}.Year_o  = num2str(Year_o);
  ATTR{i}.Year_f  = num2str(Year_f);
  ATTR{i}.Size    = size(V{i});
end
