function [V,ATTR] = DMI_1_hour_extract2(Files)


TSDS_GLOBALS;  

TEMPDIR = [TSDS_DATA_TEMP,'/wdc1hr/temp'];

system(sprintf('rm -rf %s ; mkdir -p %s',TEMPDIR,TEMPDIR)); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load .mat files created by make_DMI_1_hour_extract.m, convert FLAGS, fill
% missing hours, and do tabular base conversion.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ntseries = 1;

EL_NAME  = ['D','H','X','Y','Z','F'];

for k = 1:length(Files)

  fprintf('\nLoading %s\n',Files{k});
  varname = Files{k}(end-9:end-4);
  mag = varname(1:3);
  com   = sprintf('load(''%s''); X=%s ; clear %s ; ',Files{k},varname,varname);
  eval(com);
  
  YR  = X(:,1); YR  = repmat(YR',24,1); YR  = YR(:);
  MO  = X(:,2); MO  = repmat(MO',24,1); MO  = MO(:);
  DY  = X(:,3); DY  = repmat(DY',24,1); DY  = DY(:);
  EL  = X(:,4); EL  = repmat(EL',24,1); EL  = EL(:);
  QD  = X(:,5); QD  = repmat(QD',24,1); QD  = QD(:);
  TB  = X(:,6); TB  = repmat(TB',24,1); TB  = TB(:);
  
  HR  = repmat([0:23]',size(X,1),1);
  HR  = HR(:);
  
  DATA = X(:,7:30)'; % 24 data points per day
  AVE  = X(:,31);    % Will be ignored.
  DATA = DATA(:);    % make DATA a single column array.
  
  % Change TB FLAG from 9999 to TSDS_FLAG
  TB   = reflag(TB,9999,NaN);
  DATA = reflag(DATA,9999,NaN);  

  for i = 1:5
  
    I = find((EL+1) == i); 
  
    if (~isempty(I))
    
      if (i == 1) % If D component
	TBfac = 600.0; % Tabular base for D component.
		       % See file wdcHR2dat.format.
      else % If not D component
	TBfac = 100.0; % Tabular base for intensity elements.
		       % See file wdcHR2dat.format.
      end
      
      % Time vector
      tempT = [YR(I,:) MO(I,:) DY(I,:) HR(I,:)];
      
      % Original data
      tempDATAo   = DATA(I);
      % Offset is product of TBfac and TB
      tempOFFSET  = prod_nonflag(TBfac,TB(I),NaN,1);
      
      % Put data in correct order
      Dn          = day_number(tempT);
      [Dn,Is]     = sort(Dn);
      tempT       = tempT(Is,:);
      tempDATAo   = tempDATAo(Is,:);
      tempOFFSET  = tempOFFSET(Is,:);
      
      tempDATA    = tempOFFSET+tempDATAo;
      
      Year_o = tempT(1,1);
      Year_f = tempT(end,1);

      fprintf('%s component, ',EL_NAME(i));
      fprintf('Yo = %6d, Yf = %6d ',Year_o,Year_f);
      
      % Number of days expected
      Nd = number_days(Year_o,Year_f); 
      Nh = 24*Nd; % Number of hours expected
      
      if (Nh == length(tempDATA))
	fprintf(['\nNumber of minutes of data same as number minutes in \n'...
		 'given year interval. No filling necessary.\n']);
	Var = tempDATA;
      else
	warnstr = sprintf('Number of data points is %d (%d expected)',...
			  length(tempDATA),Nh);
	warnstr = sprintf('%s. Filling missing hours with %d',...
			  warnstr,NaN);
	fprintf('%s\n',warnstr);
	
	Hn = 24*day_number(tempT(:,1:3)) + tempT(:,4);
	Hn = Hn - Hn(1) + 1;
	
	Var     = NaN*ones(Nh,1);
	Var(Hn) = tempDATA; 	
	
	% Sanity check that the number of intervals matches what is expected.
	if (~isempty(Var))
	  L_diff  = Nh - length(Var);
	  if (L_diff ~= 0)
	    error('Time series length is incorrect');
	  end
	end

      end % if Nh == 

      Attr.Size   = size(Var);
      Attr.Name   = sprintf('%s %s',mag,EL_NAME(i));
      Attr.Year_o = num2str(Year_o);
      Attr.Year_f = num2str(Year_f);

      V{Ntseries}    = sprintf('%s/temp%d.mat',TEMPDIR,Ntseries);
      ATTR{Ntseries} = Attr;
      
      
      eval(sprintf('save %s Var Attr',V{Ntseries}));
	
      Ntseries = Ntseries + 1;

    end % ~isempty(I)
  
  end % for i

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
