function [V,ATTR] = DMI_1_hour_extract2(Files)


TSDS_GLOBALS;  

TEMPDIR = [TSDS_DATA_TEMP,'/wdc1min/temp'];

system(sprintf('rm -rf %s ; mkdir -p %s',TEMPDIR,TEMPDIR)); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load .mat files created by make_DMI_1_min_extract.m, convert FLAGS, fill
% missing hours.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ntseries = 1;

EL_NAME  = ['D','H','X','Y','Z','F'];

for k = 1:length(Files)

  fprintf('\nLoading %s\n',Files{k});
  
  varname = Files{k}(end-13:end-5);
  mag = varname(1:3);

  com   = sprintf('load(''%s''); X=%s ; clear %s ; ',...
		  deblank(Files{k}),varname,varname);
  fprintf('Evaluating %s\n',com);
  eval(com);

  fprintf('Checking time series length\n');

  Year_o = min(X(:,1));
  Year_f = max(X(:,1));

  fprintf('Sorting data\n');
  % Put data in correct order if not already
  Hn      = 24*day_number(X(:,1:3)) + X(:,4); 
  Hn      = Hn - 24*day_number([Year_o,1,1]);
  [Hn,Is] = sort(Hn); 
  X       = X(Is,:); clear Is;

  fprintf('Creating time vector\n');

  fprintf('Computing minute number\n');
  Mn = cumsum([60*(Hn') + 1 ; ones(59,length(Hn))]); clear Hn;
  
  EL  = X(1,5); 
  AVE = X(:,66);    % Will be ignored.

  fprintf('Reshaping.\n');
  X  = reshape(X(:,6:65)', 60*size(X,1),1 ); % 60 data points per day
  X  = reflag(X,999999,NaN);  
  
  fprintf('%s component, ',EL_NAME(EL+1));
  fprintf('Yo = %6d, Yf = %6d ',Year_o,Year_f);
  
  % Number of days possible
  Nd = number_days(Year_o,Year_f); 
  Nm = 1440*Nd; % Number of minutes expected
  
  if (Nm == length(X))
    fprintf(['\nNumber of minutes of data same as number minutes in \n'...
	     'interval. No filling necessary.\n']);
    Var = X ; clear Mn X; 	
  else
    warnstr = sprintf('Number of data points is %d (%d expected)',...
		      length(X),Nm);
    warnstr = sprintf('%s. Filling missing hours with %d',...
		      warnstr,NaN);
    fprintf('%s\n',warnstr);

    fprintf('Inserting data.\n');    
    Var     = repmat(NaN,Nm,1);
    Var(Mn) = X; clear Mn Is X; 	
    
  end % if Nm
  
  Attr.Size   = size(Var);
  Attr.Name   = sprintf('%s %s',mag,EL_NAME(EL+1));
  Attr.Year_o = num2str(Year_o);
  Attr.Year_f = num2str(Year_f);
  
  % Sanity check that the number of intervals matches what is expected.
  if (~isempty(Var))
    L_diff  = Nm - length(Var);
    if (L_diff ~= 0)
      error('Time series length is incorrect');
    end
  end

  V{Ntseries}    = sprintf('%s/temp%d.mat',TEMPDIR,Ntseries);
  ATTR{Ntseries} = Attr;
      
  Var = single(Var);
  
  if (is_octave==0)
    com = sprintf('save %s Var Attr',V{Ntseries});
  else
    com = sprintf('save -V6 %s Var Attr',V{Ntseries});
  end
  
  fprintf('%s\n',com);      
  eval(com);

  clear Var;
  
  Ntseries = Ntseries + 1;

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
