function [Files_good,Attr] = FMI_1_min_extract(DATA_DIR,parse_files)
%FMI_1_MIN_EXTRACT
%
%   Files = FMI_1_MIN_EXTRACT(Data_dir)
%
%   Transform and concatanate all 1-min .wdc files into .dat files (with no
%   strings) using a Perl script.  Load this .dat file and save as a .mat
%   file.
%
%   ASCII file abc_MIN.dat has columns YEAR MONTH DAY HOUR ELEMENT DATA AVE
%
%   abc is the magnetometer 3-letter code.
%   ELEMENT is 0,1,2,3,4,5 for D,H,X,Y,Z,F
%   DATA is 60 columns corresponding to 60 mins in hour
%
%   .mat file abc_min.mat has matrix variable named abc_hr with contents of
%   .dat file abc_min.dat.
%
%   Files = FMI_1_MIN_EXTRACT(Data_dir,parse_files) If parse_files = 0, do
%   not call Perl script and use existing .dat files to create .mat files.
%
%
  
if (nargin < 2)
  parse_files = 1;
end
  
TSDS_GLOBALS;  

TEMPDIR = [TSDS_DATA_TEMP,'/fmi1min'];
today_yr = date;
today_yr = str2num(today_yr(8:end));

if (parse_files)

com = sprintf('rm -rf %s ; mkdir -p %s',TEMPDIR,TEMPDIR);
system(com); 
fprintf('Evaluating %s\n',com)
com = sprintf('cd %s ; %s %s/FMI_1_min_unwrap.pl %s ; ',...
	       DATA_DIR,TSDS_PERL,TSDS_PERLFILES,TEMPDIR);

% Don't need this anymore.  Original files can be found in
% original-wrapped.tgz.
%system(com); % Call the Perl program.

com = sprintf('cd %s ; %s %s/FMI_1_min_extract.pl %s ; ',...
	       DATA_DIR,TSDS_PERL,TSDS_PERLFILES,TEMPDIR);
system(com); % Call the Perl program.

end

% All possible magnetometers that may show up in a data file.  If a new
% magnetometer shows up in a later year, add to list.  If data for a
% magnetometer is missing on a given day, it is filled with FLAG.  This
% method should be changed.

FLAG_old = 99999.9;

Files = readdir(TEMPDIR);

if (1) % Create .mat files

  k = 1;
  for i = 1:size(Files,1)

    if ~isempty(findstr(Files(i,:),'.dat')) % If file has extension .col2

      mag = Files(i,1:3);
      var = Files(i,1:5);

      fname = sprintf('%s/%s',TEMPDIR,Files(i,:));

      com = sprintf('load %s ; temp = %s ; clear %s;',fname,var,var);
      fprintf('Evaluating command: %s\n',com);
      eval(com);

      DN     = day_number(temp(:,1:3));
      [DN,I] = sort(DN);
      temp   = temp(I,:);

      DN     = DN-DN(1);
      MN     = 1 + 24*60*DN + 60*temp(:,4) + temp(:,5);

      [MN,I] = sort(MN);
      temp   = temp(I,:);
      
      Year_f = max(temp(:,1));
      Year_o = min(temp(:,1));
      Nd     = 1440*number_days(Year_o,Year_f);
      
      T   = [1:Nd];
      Var = repmat(NaN,Nd,1);

      temp(:,6) = reflag(temp(:,6),FLAG_old,NaN);
      Var(MN)   = single(temp(:,6));
      
      % Save all data for given mag in an individual .mat binary file.
      fnamemat = sprintf('%s/%s.mat',TEMPDIR,var);

      if (is_octave)
	com=sprintf('save -mat-binary %s Var Attr ; ',fnamemat);
      else
	com=sprintf('save %s Var Attr ; ',fnamemat);
      end

      Files_good{k}  = fnamemat;
      Attr{k}.Size   = size(Var);
      Attr{k}.Year_o = num2str(Year_o);
      Attr{k}.Year_f = num2str(Year_f);
      Attr{k}.Name   = sprintf('%s %s',var(1:3),var(5));
      
      fprintf('Evaluating command: %s\n',com);
      eval(com);

      k = k + 1;
      
    end
  
  end

end  