function [V,Attr] = NGDC_Ap_extract(DATADIR)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code has been tested in Matlab 6.5 on Linux.
% It needs to be made to work in Octave by making PERLOP Octave-compatable.
%
% Original data at 3-hour resolution is resampled to 1-hour resolution
% and averaged to 1-day resolution.
%
% Data is extracted from file spidr_Ap_Apstar_Kp.txt which was downloaded
% from NSSDC SPIDR.
%
% Note that the time stamps in above .txt file should be interpreted as
% 00:00 -> value for measurements in range 00:00-03:00
% This can be verified by reading
% ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/MONTHLY.FMT
% (SPIDR data is apparently derived from these data files.  In the
% future, derive time series directly from these data files instead of
% through the SPIDR interface.
%
% Created on 04/03/2004 by R.S. Weigel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get global variables and path setting
TSDS_GLOBALS;

Year_f        = 2003;
Year_o        = 1932;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Note that Matlab's LOAD cannot deal with hyphens in filenames, but it
% can deal with them in parts of the path.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = [DATADIR,'/spidr_Ap_Apstar_Kp.txt'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find bounds of first chunk of data which is Ap
REGEXP = '=~ m/yyyy/';
Ia     = perlop(fname,REGEXP,[],[],'file');
a      = Ia(1) + 1;

REGEXP = '=~ m/Element/';
Ib     = perlop(fname,REGEXP,[],[],'file');
b      = Ib(2) - 2;

height = [a b];
REGEXP = '=~ s/\-|\:|\"/ /g';
[I,A]  = perlop(fname,REGEXP,[],height,'file');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T      = A(:,1:5);
ap     = A(:,6);

I04    = find(T(:,1) == (Year_f+1));

ap     = ap(1:I04(1)-1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make sure there are expected number of data points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ( 8*number_days(1932,Year_f) ~= length(ap) )
  error('Wrong number of data points. Aborting\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Put data at 1 hour resolution from 3-hour resolution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ap = repmat(ap',3,1);
Ap = ap(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Attr.Year_o   = num2str(Year_o);
Attr.Year_f   = num2str(Year_f);
Attr.Size     = size(ap);
V             = ap;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
