function [V,ATTR] = NGDC_PC_extract(DATA_DIR)

% Get global variables and path setting
TSDS_GLOBALS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the data after transforming original data to a matrix data file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
TEMPDIR = [TSDS_DATA_TEMP,'/ngdc'];
com=sprintf(...
    'rm -f %s/*; mkdir -p %s;cd %s ; %s %s/NGDC_extract_PC.pl %s %s',...
    TEMPDIR,TEMPDIR,DATA_DIR,TSDS_PERL,TSDS_PERLFILES,TEMPDIR,DATA_DIR);
system(com);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1-minute files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% File with 1440 points per line
Xmn1 = load(sprintf('%s/NGDC_extract_PC_mn.txt',TEMPDIR));

Yr   = Xmn1(:,1);
Mon  = Xmn1(:,2);
Day  = Xmn1(:,3);
Type = Xmn1(:,4);
Name = Xmn1(:,5);

YMD1 = [Yr Mon Day];
X1   = Xmn1(:,6:end);

[X1u,T1] = time_union2([YMD1 X1]);
% One day per line

% Remove data after 1998-9-9 (should all be NaN)
I = find( (T1(:,1)==1998) & (T1(:,2)==9) & (T1(:,3)==9) );
X1u = X1u(1:I,:);
T1  = T1(1:I,:);

% File with 60 points per line
Xmn2 = load(sprintf('%s/NGDC_extract_PC_mn2.txt',TEMPDIR));

Yr   = Xmn2(:,1);
Mon  = Xmn2(:,2);
Day  = Xmn2(:,3);
Hr   = Xmn2(:,4);
Type = Xmn2(:,5);
Name = Xmn2(:,6);

X2 = Xmn2(:,7:end);

[X2u,T2] = time_union2([Yr Mon Day Hr X2],'hour');
% One hour per line

X2u = reshape(X2u',1440,size(X2u,1)/24)';
% One day per line
[jnk,T2] = time_union2([Yr([1,end]) Mon([1,end]) Day([1,end])]);

% Remove before 1998-11-2 (should all be NaN)
I = find( (T2(:,1)==1998) & (T2(:,2)==11) & (T2(:,3)==2) );
X2u = X2u(I:end,:);
T2  = T2(I:end,:);

T = [T1 ; T2];
X = [X1u ; X2u];

% If duplicate times, picks later data point, which is what we want.
Xu = time_union2([T X]); 
Xu = Xu';
Xu = Xu(:);

V{3} = reflag(Xu,99999,NaN);

ATTR{3}.Year_o = num2str(T(1,1));
ATTR{3}.Year_f = num2str(T(end,1));
ATTR{3}.Size = size(V{3});
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X = load(sprintf('%s/NGDC_extract_PC.txt',TEMPDIR));
Yr   = X(:,1);
Mon  = X(:,2);
Day  = X(:,3);
Type = X(:,4);
Name = X(:,5);

I    = find(X==9999.0);
X(I) = NaN;

I1   = find(Name == 1); % Vostok
I0   = find(Name == 0); % Thule
X0   = X(I0,6:end);
X1   = X(I1,6:end);

YMDx = [Yr Mon Day];

YMD1    = YMDx(I1,:);
DN1     = day_number(YMD1);        % Convert to day number
[DN1,I] = sort(DN1);
YMD1    = YMD1(I,:);
X1      = X1(I,:);
[X1u,T1] = time_union2([YMD1 X1]);
Year_o1 = T1(1,1);
Year_f1 = T1(end,1);

YMD0    = YMDx(I0,:);
DN0     = day_number(YMD0);        % Convert to day number
[DN0,I] = sort(DN0);
YMD0    = YMD0(I,:);
X0      = X0(I,:);
[X0u,T0] = time_union2([YMD0 X0]);
Year_o0 = T0(1,1);
Year_f0 = T0(end,1);

X0u = reflag(X0u,NaN,99999);
X1u = reflag(X1u,NaN,99999);

% Resample to 1-minute time grid
X0u = X0u';
X0u = repmat(X0u(:),1,15)';
X0u = X0u(:);

X1u = X1u';
X1u = repmat(X1u(:),1,15)';
X1u = X1u(:);

V{1} = reflag(X0u,99999,NaN);
V{2} = reflag(X1u,99999,NaN);

ATTR{1}.Year_o = num2str(Year_o0);
ATTR{1}.Year_f = num2str(Year_f0);
ATTR{2}.Year_o = num2str(Year_o1);
ATTR{2}.Year_f = num2str(Year_f1);

ATTR{1}.Size = size(V{1});
ATTR{2}.Size = size(V{2});
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

