function [ATTR,SAT,ATTR_NAMES] = NSSDC_mpause_cross_info(Attr)
  
SAT = {'ampte_cce','ampte_irm','explorer33','geotail',...
       'hawkeye','heos2','imp1','imp3','imp4','imp6','imp8',...
       'isee','ogo5','prognoz10','prognoz7','prognoz8'};

ATTR_NAMES = {...
    'Satellite Number',...
    'Spacecraft position Xgse, Re',...
    'Spacecraft position Ygse, Re',...
    'Spacecraft position Zgse, Re',...
    'Spacecraft position Ygsm, Re',...
    'Spacecraft position Ygsz, Re',...
    'Upstream magnitude Avg (scalar), nT',...
    'Upstream magnetic field intensity, nT, see footnote 2',...
    'Xgse component of upstream magnetic field vector',...
    'Ygse component of upstream magnetic field vector',...
    'Zgse component of upstream magnetic field vector',...
    'Ygsm component of upstream magnetic field vector',...
    'Zgsm component of upstream magnetic field vector',...
    'Upstream flow speed, km/s, see footnote 7',...
    '-Xgse component of upstream sw velocity, km/s, see footnote 3',...
    '-Ygse component of upstream sw velocity, km/s, see footnote 3',...
    '-Zgse component of upstream sw velocity, km/s, see footnote 3',...
    'Upstream proton temperature (deg K)',...
    'Upstream solar wind proton density (cm**-3)',...
    'Upstream flow pressure, nPa, see footnote 4',...
    'Electric Field, mV/m footnote 5',...
    'Upstream plasma beta, see footnote 6',...
    'Upstream Alfven speed, km/s, see footnote 7',...
    'Upstream sound speed, km/s, see footnote 8',...
    'Upstream magnetosonic speed, km/s, see footnote 9',...
    'Upstream Alfven Mach number',...
    'Upstream sonic Mach number',...
    'Upstream magnetosonic Mach number',...
    'IMF cone angle, deg, see footnote 10',...
    'IMF clock angle, deg, see footnote 11',...
    '3-hr Kp (Kp*10)index from NGDC',... 	  
    'Daily Sunspot Number from NGDC',... 	  
    'Hourly Dst-index, nT, from NGDC provisional when?',...
    'Hourly AE-index, nT, provisional when?',...
    'Source of code',...
       };

ack = ['Use of these data in publications should be accompanied by '...
      'acknowledgements to Space Physics Data Facility(SPDF/GSFC)'];

note = ['Creation of this data set and the FTPBrowser interface '...
	'were supported by the NASA LWS TR&T  Research Program. '...
	'[Grant Principal Investigator: Ramona Kessel '...
	'<ramona.l.kessel@nasa.gov>]'];

FOOTNOTE{1} = [' ... footnote 1. In general, the minute given is that in/during which the crossing is observed.  For extended crossings, the minute given generally corresponds to the beginning of the magnetopause interval. For multiple crossings in the early IMP data submitted by Fairfield, sometimes an ''average position'' was selected by visually determining the point that equalized the magnetosphere time outside the point with the magnetosheath time inside the point; the minute given corresponds to the ''average position''.'];

FOOTNOTE{2} = 'footnote 2. The field intensity is [Bx**2+By**2+Bz**2]**0.5';

FOOTNOTE{3} = ['footnote 3. The Xgse component of the solar wind velocity vector is negative.  To avoid negative values for this component, we have chosen to give components for the vector anti-aligned with the velocity vector.'];

FOOTNOTE{4} = ['footnote 4. Flow pressure = (2.0/10**6) * N * V**2 (note that this 2.0 includes a 5% alpha contribution)'];

FOOTNOTE{5} = 'footnote 5.  Electric field= V(km/s) * Bz (nT; GSM) * 10**-3';

FOOTNOTE{6} = 'footnote 6.  Plasma beta = [(T*4.16/10**5) + 5.34] * N / B**2';

FOOTNOTE{7} = 'footnote 7.  The Alfven speed = 20. * B / N**0.5';

FOOTNOTE{8} = 'footnote 8.  The sound speed = 0.12 * [T + 1.28*10**5]**0.5';

FOOTNOTE{9} = 'footnote 9.  Magnetosonic speed = [(sound speed)**2 + (Alfv speed)**2]**0.5 ';

FOOTNOTE{10} = 'footnote 10.  IMF cone angle = 57.3 * arc cos [|Bx|/Bt]';

FOOTNOTE{11} = 'footnote 11.  IMF clock angle = 57.3 * arc tan [|Bygsm|/Bzgsm]';

source = ['1 D. Fairfield'...
	  ' 2 N. Crooker, G. Paschman, G. Haerendel'...
	  ' 3 J. Slavin'...
	  ' 4 T. Eastman'...
	  ' 5 A. T. Y. Lui'...
	  ' 6 R. Rijnbeck'...
	  ' 7 G. Zastenker'...
	  ' 8 T. Eastman, S. Christon'...
	  ' 9 H. Hasegawa'...
	  ' 10 D. G. Sibeck, A. Davis'...
	  ' 11 no source given'...
	  ' 12 S. Boardsen, T. Eastman'];

if ~isempty(Attr)

k = 1;
  for i = 1:length(Attr)

  ATTR{k}      = ts_attr;
  ATTR{k}.Name = sprintf(['Kessel magnetopause crossings '...
 		          'data set %s (NSSDC 1-min)'],ATTR_NAMES{i});
  ATTR{k}.Size   = Attr{k}.Size;
  ATTR{k}.Year_o = Attr{k}.Year_o;
  ATTR{k}.Year_f = Attr{k}.Year_f;
  ATTR{k}.Source = ...
      'ftp://nssdcftp.gsfc.nasa.gov/spacecraft_data/magnetopause_crossings';
  
  ATTR{k}.Acknowledgement = [ack,' ',note];;
  ATTR{k}.PPD = '1440';
  ATTR{k}.TimeUnit = 'minute';
  ATTR{k}.Note = [FOOTNOTE{1}, ...
		  sprintf(['For name of scientist who provided source code,'...
		    ' see time series: Kessel magnetopause crossings data'...
		     'set %s'],ATTR_NAMES{35})];

  if (i == 1)
    ATTR{k}.Note = ...
	['Key for time series: 1=ampte_cce,','2=ampte_irm,','3=explorer33,',...
	 '4=geotail,',...
	 '5=hawkeye,','6=heos2,','7=imp1,','8=imp3,','9=imp4,',...
	 '10=imp6,','11=imp8,',...
	 '12=isee,','13=ogo5,','14=prognoz10,','15=prognoz7,','16=prognoz8'];
  end
  
  footstr = ('footnote ');
  I = findstr(footstr,ATTR_NAMES{i});
  if ~isempty(I)
    a  = I(1) + length(footstr);
    if ( a == length(ATTR_NAMES{i}) )
     b = a;
    end
    if ( (a+1) == length(ATTR_NAMES{i}) )
      b = a+1;
    end

    fn = str2num(ATTR_NAMES{i}([a:b]));
    ATTR{k}.README = [FOOTNOTE{fn}];
      
  end

  if (i == 35)
    ATTR{35}.README = source;    
  end

  ATTR{k}.README = [ATTR{k}.README,...
      'ftp://nssdcftp.gsfc.nasa.gov/spacecraft_data/magnetopause_crossings',...
       '00readme.txt'];
  
  k = k+1;
end

else
ATTR=[];
end