function ATTR = Nagoya_mm210_info(Attr_in,version)

TSDS_GLOBALS;  

URL = 'ftp://stdb2.stelab.nagoya-u.ac.jp/pub/mm210';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = 1:length(Attr_in)

Attr_in{i}.Version = num2str(version);

Attr          = ts_attr;
Attr.Year_o   = Attr_in{i}.Year_o;
Attr.Year_f   = Attr_in{i}.Year_f;
Attr.Size     = Attr_in{i}.Size;
Attr.PPD      = 1440;
Attr.Dt       = 1;
Attr.TimeUnit = 'min';
Attr.Category1 = 'Ground Magnetometer';

Attr.Acknowledge  = ...
    ['From ',URL,...
     ': \"Please contact Dr. K. Yumoto (Principal Investigator)' ...
     ' before using the data at any presentation or publication. In case '...
     'of publication, please acknowledge the Solar-Terrestrial'...
     ' Environment Laboratory, Nagoya University for managing '...
     'the database of the 210 MM magnetic observations.\"  (See README.txt'...
     ' for contact inforamation.)'];

Attr.CreatedWith  = ['Nagoya_mm210.m'];
Attr.Source       = URL;
Attr.README       = [URL,'/README.txt'];
Attr.NOTE         = ['From README.txt: !!!!!!! Please note that X is '...
		    'H-component and Y '...
		    'is D-component in the 210 MM data. !!!!!!!!!!!!!! '...
		    '!!!!!!!!!!!!!!!!!!!!!'];
Attr.Name         = sprintf('%s (Nagoya mm210 1-min)',Attr_in{i}.Name);
Attr.Unit         = 'nT';
Attr.Warning      = sprintf(['Always spot check this data with data from'...
		    'source: %s.  Content should be identical except for '...
		    'change in unit from 0.1nT to nT.'],Attr.Source);

%Attr.Warning      = sprintf(['In TSDS metadata, H in original data file = X '...
%		    'in TSDS metadata and D in original data file = Y in '...
%		    'TSDS metadata. Some other corrections specified in README '...
%		    'have been applied.  Always spot check this data '...
%		    'with data from source: %s.  Content should be '...
%		    'identical except for changes noted in metadata'],...
%			    Attr.Source);

  nm = Attr.Name;
  if strmatch(nm,'MSR') | strmatch(nm,'ONW') | strmatch(nm,'KAG') | ... 
	strmatch(nm,'EWA') | strmatch(nm,'GUA') | strmatch(nm,'TIK') | ...
	strmatch(nm,'CHD') | strmatch(nm,'MGD') | strmatch(nm,'PTK') | ...
	strmatch(nm,'YAP') | strmatch(nm,'KOR') | strmatch(nm,'WEP') | ...
	strmatch(nm,'ADL')
    Attr.Warning = [...
	'(1) for all data from MSR, ONW, KAG, EWA, GUA, TIK, CHD, MGD, PTK, '
	'YAP, and KOR, for data from WEP before 05UT on Jan. 31, 1991, '
	'and for data from ADL before July 14, 1993 '
	'[H:+ = northward]  [D:+ = eastward]  [Z:+ = downward] ',...
	Attr.Warining];
  end

  if strmatch(nm,'BSV') | strmatch(nm,'LMT') | strmatch(nm,'DAW') | ...
     strmatch(nm,'BIK') | strmatch(nm,'MCQ') | strmatch(nm,'CAN') | ...
     strmatch(nm,'DAL') | strmatch(nm,'WEP') | strmatch(nm,'ADL')
    % (2) for all data from BSV, LMT, DAW, BIK, MCQ, CAN, and DAL, 
    %     for data from WEP after 05UT on Jan. 31, 1991, and for data from 
    %    ADL after Aug.12, 1993
    %    [H:+ = northward]  [D:+ = eastward]  [Z:+ = upward]
    Attr.Warning = [...
	'(2) for all data from BSV, LMT, DAW, BIK, MCQ, CAN, and DAL, '... 
	'for data from WEP after 05UT on Jan. 31, 1991, and for data from '...
	'ADL after Aug.12, 1993 '...
	'[H:+ = northward]  [D:+ = eastward]  [Z:+ = upward]',...
	Attr.Warning];
  end
  
  if strmatch(Attr.Name,'CBI D')
     %  (3) for data from CBI 
     %      [H:+ = northward]  [D:+ = westward]  [Z:+ = downward]
     Attr.Warning = [...
	 '(3) for data from CBI [H:+ = northward]  [D:+ = westward] '
	 '[Z:+ = downward]',Attr.Warning];
	 
  end

  if strmatch(Attr.Name,'WEW D')    
    % (4) for data from WEW 
    %    [H:+ = northward]  [D:+ = westward]  [Z:+ = upward]
    Attr.Warning = [...
	'(4) for data from WEW [H:+ = northward]  [D:+ = westward] '...
	'[Z:+ = upward]',Attr.Warning];
	
  end

  if strmatch(Attr.Name,'BSV')
    % (1) The time of the data from BSV for Nov. 24, 1992 - Dec. 7, 1992 
    % is 10 hour behind the correct time.
    % (2) The Z-component of BSV data sometimes becomes zero due to a
    % temporal disconnection of cable.
    % (7) The time of the data from BSV for Nov. 2, 1991 - Nov. 16, 1991 
    %  is 1 hour faster than the correct time.
    Attr.Warning = ['(1) The time of the data from BSV for Nov. 24, 1992'...
		    '- Dec. 7, 1992 is 10 hour behind the correct time. '...
		    '(2) The Z-component of BSV data sometimes becomes '...
		    'zero due to a temporal disconnection of cable. '...
		    '(7) The time of the data from BSV for Nov. 2, 1991'...
		    '- Nov. 16, 1991 is 1 hour faster than the correct time.'...
		    Attr.Warning];
  end
  if strmatch(Attr.Name,'BSV')  
    % (4) The Z-component of DAL data is meaningless after Jan. 1, 1993.
    % (9) The D-component of DAL data satulate for April 13, 1992 - July 
    % 10, 1992
    Attr.Warning = ['(4) The Z-component of DAL data is meaningless ' ...
		    'after Jan. 1, 1993. (9) The D-component ' ...
		    ' of DAL data satulate for April 13, 1992 - July' ...
		    ' 10, 1992',Attr.Warning];
  end    
  if strmatch(Attr.Name,'ADL')  
    % (3) The Z-component of ADL data sometimes shows noisy fluctuations 
    %     due to an equipment problem. 
    % (6) ADL is the Adelaide station (ADE).
    Attr.Warning = ['(3) The Z-component of ADL data sometimes shows noisy ' ...
		    'fluctuations due to an equipment problem. (6) ADL '...
		    'is the Adelaide station (ADE).',Attr.Warning];
  end
  if strmatch(Attr.Name,'EWA')  
    % (8) The H-component of EWA data sometimes shows noisy fluctuations 
    %  due to an equipment problem
    Attr.Warning = ['(8) The H-component of EWA data sometimes shows noisy' ...
		    ' fluctuations due to an equipment problem.',Attr.Warning];
  end
  if strmatch(Attr.Name,'MSR')  
    % (5) The D- and Z-components of MSR data have a DC-level, artificial 
    % fluctuation on a few days/year.
    Attr.Warning = ['(5) The D- and Z-components of MSR data have a' ...
		    ' DC-level, artificial fluctuation on a few days/' ...
		    ' year.',Attr.Warning];
  end  
  
  ATTR{i} = Attr;

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



