function [V,ATTR] = SEC_ACErt_extract(DATA_DIR,var)

% R.S. Weigel 04/04/2005

TSDS_GLOBALS;

if (var == 'swe')
  
com = sprintf('cd %s ; %s %s/cat_ace_rt_files.pl all swe',...
	      DATA_DIR,TSDS_PERL,TSDS_PERLFILES);
system(com);

fprintf('Loading swepam.dat\n');
SWEPAM = load(sprintf('%s/swepam.dat',DATA_DIR)); 

% Sample data file header:

%:Data_list: 20050101_ace_swepam_1m.txt
%:Created: 2005 Jan 02 0008 UT
%# Prepared by the U.S. Dept. of Commerce, NOAA, Space Environment Center.
% Please send comments and suggestions to SEC.Webmaster@noaa.gov 
% 
% Units: Proton density p/cc
% Units: Bulk speed km/s
% Units: Ion tempeture degrees K
% Status(S): 0 = nominal data, 1 to 8 = bad data record, 9 = no data
% Missing data values: Density and Speed = -9999.9, Temp. = -1.00e+05
% Source: ACE Satellite - Solar Wind Electron Proton Alpha Monitor
%
%   1-minute averaged Real-time Bulk Parameters of the Solar Wind Plasma
% 
%                Modified Seconds   -------------  Solar Wind  -----------
% UT Date   Time  Julian  of the          Proton      Bulk         Ion
% YR MO DA  HHMM    Day     Day     S    Density     Speed     Temperature
%-------------------------------------------------------------------------

% Sort the data.
DN          = day_number(SWEPAM(:,1:3));
[jnk,I]     = sort(DN);
SWEPAM      = SWEPAM(I,:);
Year_o      = SWEPAM(1,1);
Year_f      = SWEPAM(end,1);

% Extract time info
YYYY        = SWEPAM(:,1);
MO          = SWEPAM(:,2);
DD          = SWEPAM(:,3);
HH          = floor(SWEPAM(:,4)/100);
MM          = SWEPAM(:,4)-100*HH;

% Reflag
X1          = [YYYY MO DD HH MM SWEPAM(:,8:10)];
X1          = reflag(X1,-9999.9,NaN,1);
X1          = reflag(X1,-1.00e5,NaN,1);

% Put on uniform time grid
X1u         = time_union2(X1,X1,NaN,'min');

% Extract variables
Ni          = X1u(:,1);
Vsw         = X1u(:,2);
Ti          = X1u(:,3);

Attr.Year_o = num2str(Year_o);
Attr.Year_f = num2str(Year_f);
Attr.Size   = size(Ni);

V    = {Ni,Vsw,Ti};
ATTR = {Attr,Attr,Attr};

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (var == 'mag')

com = sprintf('cd %s ; %s %s/cat_ace_rt_files.pl all mag',...
	      DATA_DIR,TSDS_PERL,TSDS_PERLFILES);
system(com);

fprintf('Loading mag.dat\n');
MAG    = load(sprintf('%s/mag.dat',DATA_DIR)); 

% Sample data file header:

%Data_list: 20000101_ace_mag_1m.txt
%:Created: 2000 Jan 02 0009 UT
% Prepared by the U.S. Dept. of Commerce, NOAA, Space Environment Center.
% Please send comments and suggestions to sec@sec.noaa.gov
%
% Magnetometer values are in GSM coordinates.
%
% Units: Bx, By, Bz, Bt in nT
% Units: Latitude  degrees +/-  90.0
% Units: Longitude degrees 0.0 - 360.0
% Status(S): 0 = nominal data, 1 to 8 = bad data record, 9 = no data
% Missing data values: -999.9
% Source: ACE Satellite - Magnetometer
%
%              1-minute averaged Real-time Interplanetary Magnetic Field Values
%
%                 Modified Seconds
% UT Date   Time  Julian   of the   ----------  GSM Coordinates ---------------
% YR MO DA  HHMM    Day      Day S  Bx      By      Bz      Bt     Lat.   Long.
%------------------------------------------------------------------------------

% Sort the data.
DN        = day_number(MAG(:,1:3));
[jnk,I]   = sort(DN);
MAG       = MAG(I,:);
Year_o    = MAG(1,1);
Year_f    = MAG(end,1);

% Extract time info
YYYY        = MAG(:,1);
MO          = MAG(:,2);
DD          = MAG(:,3);
HH          = floor(MAG(:,4)/100);
MM          = MAG(:,4)-100*HH;

% Reflag
X1          = [YYYY MO DD HH MM MAG(:,8:11)];
X1          = reflag(X1,-999.9,NaN,1);

% Put on uniform time grid
X1u         = time_union2(X1,X1,TSDS_FLAG,'min');

% Extract variables
Bx         = X1u(:,1);
By         = X1u(:,2);
Bz         = X1u(:,3);

Attr.Year_o = num2str(Year_o);
Attr.Year_f = num2str(Year_f);
Attr.Size   = size(Bx);

V    = {Bx,By,Bz};
ATTR = {Attr,Attr,Attr};

end