function [V,ATTR] = USAF_Ap_extract(DATADIR)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code has been tested in Matlab 6.5 on RHEL 3.0.
% data from two files are transformed to uniform format
%
% USAF_Ap/apUSAF86-02.txt; USAF 1-3 day predictions and measured Ap
% USAF_Ap/ap_7dUSAF.txt; USAF 1-7 day predictions and measured Ap
%
% R.S. Weigel 04/03/2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get global variables and path setting
TSDS_GLOBALS;

Year_f = 2003;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First data file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname    = [DATADIR,'/ap_7dUSAF.txt'];
temp_Ap  = load(fname);
time     = temp_Ap(:,1:3);
temp_Ap  = temp_Ap(:,5:end);

% Put data in standard format
% First three columns of ap_7dUSAF are Year, Month, Day
doy_o    = doy(time(1,:));             % Find doy of first day of data
doy_f    = doy(time(end,:));           % Find doy of last day of data
Year_o   = time(1,1);
Year_f   = time(end,1);

if (doy_o ~= 1)
  pad_o = NaN*ones(doy_o-1,size(temp_Ap,2));
else
  pad_o = [];
end

Nd_f = number_days(time(1,1));
if (doy_f ~= Nd_f)
  pad_f = TSDS_FLAG*ones(Nd_f-doy_f,size(temp_Ap,2));  
else
  pad_f = [];
end

% Add flags so data starts on beginning of first year when data is available
% and ends on last day of last year when data is available.
temp_Ap  = [pad_o; temp_Ap ; pad_f]; 

Ap       = temp_Ap(:,1);      % Actual Ap
Ap_pred  = temp_Ap(:,2:end);  % 1-7 day lead time forecasted Ap

% Change flag from -99999 to 99999
Ap      = reflag(Ap,-TSDS_FLAG,TSDS_FLAG);
Ap_pred = reflag(Ap_pred,-TSDS_FLAG,TSDS_FLAG);

if ( number_days(Year_o,Year_f) ~= size(Ap,1) )
  error('Wrong number of data points. Aborting.');
end

Attr.Year_o = num2str(Year_o);
Attr.Year_f = num2str(Year_f);
Attr.Size   = size(Ap);

V{1}     = Ap;  clear Ap;
ATTR{1}  = Attr;

for i = 1:7
  Attr.Name = sprintf('Ap index %d-day lead time prediction (USAF,v0)',i);
  ATTR{1+i} = Attr;
  V{1+i}    = Ap_pred(:,i);
end

clear Ap Ap_pred Attr;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second data file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fname = [DATADIR,'/apUSAF86-02.txt'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the historical Ap data from USAF
% Format of datafile
% ap_USAF_86-02						
% Year -- Month -- DayOfMonth -- Aobs -- Ap1 -- Ap2 -- Ap3
% First data point is 07/01/1986 = day 182.
% First data point is 12/31/2002
Year_o = 1986;
Year_f = 2002;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X     = load(fname);
X     = [TSDS_FLAG*ones(181,7) ; X]; % Make first data point Jan 1, 1986
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make sure there are expected number of data points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ap  = X(:,4); % Ap observed
Ap1 = X(:,5); % 1-day lead time predicted Ap
Ap2 = X(:,6); % 2-day lead time predicted Ap
Ap3 = X(:,7); % 3-day lead time predicted Ap

if ( number_days(Year_o,Year_f) ~= size(Ap,1) )
  error('Wrong number of data points. Aborting.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Attr.Year_o = num2str(Year_o);
Attr.Year_f = num2str(Year_f);
Attr.Size   = size(Ap);

ATTR{9}   = Attr;
V{9}      = Ap;

ATTR{10}   = Attr;
V{10}      = Ap1;

ATTR{11}   = Attr;
V{11}      = Ap2;

ATTR{12}  = Attr;
V{12}     = Ap3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
