function make_1_min_hdf_ground_magnetometer(mag_fns_dir)

% These functions are not currently in the TSDS distribution
% addpath ~/Programs/TSDS_To_Add/magnetometer_data_functions
%
% R.S. Weigel 04/17/2005
  
addpath(mag_fns_dir)  

if ~exist(mag_data_info)
  error(['Some mfiles need to run this script are not available in your' ...
	 ' TSDS distribution. Contact R.S. Weigel for these functions.']);
end

more off;

for insert_flags = 0:1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get global variables and path setting
TSDS_GLOBALS;
if (is_octave)
  error('Only works in Matlab.')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

descr = sprintf('Part of the TSDS uniform time series ');
descr = [descr ,  sprintf('database version ')];
descr = [descr ,  sprintf('%.2f.\nCreated %s.\n',VERSION,datestr(now,1))];

mag_data_info;

MAGLIST_c   = {MAGLIST_DMI,MAGLIST_FMI,MAGLIST_SPI};
COORDSYS_c  = {COORDSYS_DMI,COORDSYS_FMI,COORDSYS_SPI};
YEARS_c     = {YEARS_DMI,YEARS_FMI,YEARS_SPI};
DIRECTORY_c = {directory_dmi,directory_fmi,directory_spi};
SOURCE_c    = {SOURCE_DMI,SOURCE_FMI,SOURCE_SPI};
WEBSOURCE_c = {'http://web.dmi.dk/fsweb/Projects/wdcc1/homepage.html',...
	       'http://www.geo.fmi.fi/image/',...
	       'http://spidr.ngdc.noaa.gov/'};

ACKNOWLEDGE_c = {'See http://web.dmi.dk/fsweb/Projects/wdcc1/homepage.html',...
		 'See http://www.geo.fmi.fi/image/',...
		 'See spidr.ngdc.noaa.gov/'};
		 
k    = 1;	       
part = 1;
for z = 1:3
  
  MAGLIST   = MAGLIST_c{z};
  COORDSYS  = COORDSYS_c{z};
  YEARS     = YEARS_c{z};
  DIRECTORY = DIRECTORY_c{z};
  SOURCE    = SOURCE_c{z};
  WEBSOURCE = WEBSOURCE_c{z};
  SDS.Note  = ['Data on 1 minute time grid starting on first hour of Year_o' ...
	       ' and ending on last hour of Year_f'];
  
  SDS.timeunit = 'minute';
  SDS.dt       = '1';
  SDS.Source   = WEBSOURCE;
  SDS.Ack      = ACKNOWLEDGE_c{z};
  SDS.Warn     = '';
  
  SDS.FLAG     = TSDS_FLAG;
  SDS.README   = ['see make_',SOURCE,'*.m'];
  SDS.Created_With  = ['make_',SOURCE,'*.m in TSDS-',VERSION];
  SDS.Format   = '';
  SDS.Label    = '';  
  SDS.Note     = '';
  SDS.Created_With  = ['make_*.m in TSDS-',VERSION];
  SDS.PPD      = '1440';
    
  for i = 1:size(MAGLIST,1)

    if (mod(k-1,0) == 0)

      hdf_file = ...
	  sprintf('TSDS_1_min_ground_magnetometer_data.part_%d.hdf',...
			 part);
      if (exist(hdf_file))
	eval(sprintf('!rm %s',hdf_file));,end
      if (insert_flags == 1)
	hdf_file_meta = ...
	    sprintf('TSDS_1_min_ground_magnetometer_data-metadataonly.part_%d.hdf',part);
      if (exist(hdf_file_meta))
	eval(sprintf('!rm %s',hdf_file_meta));,end
      end

      part = part + 1;

      fprintf('k = %2d. Creating and opening %s\n',k,hdf_file);
      eval(sprintf('!rm -f %s',hdf_file));
      
      sd_id   = hdfsd('start',hdf_file,'DFACC_CREATE');
      status  = hdfsd('setattr',sd_id,'FileDescription',descr);
    end
    
    % Need to fix this:
    SDS.Unit     = ['nT for H, X, Y and Z.  Varies for D; see original ',...
	           'data source for details'];
    SDS.cordsys  = COORDSYS(i,:);
    SDS.Year_o   = num2str(YEARS(i,1));
    SDS.Year_f   = num2str(YEARS(i,2));
    SDS.LatLong  = [num2str(station_name2geographic(MAGLIST(i,1:3))),...
		    ' (Degrees Geographic)'];

    LatLong      = station_name2geomagnetic(MAGLIST(i,1:3));
    MLTMN        = ut2mlt(0,MAGLIST(i,1:3));
    SDS.LatLongGeomag = ...
	sprintf('%.2f,%.2f,%.2f (Lat,Long in degrees, and MLTMN in hour)',...
		LatLong(1),LatLong(2),MLTMN);
    years = [YEARS(i,1):YEARS(i,2)];
    
    for j = 1:3
      % Load the magnetometer data and write to hdf file
      if (insert_flags == 1)
      	temp   = TSDS_FLAG*ones(1440*number_days(years(1),years(end)),1);
      	fprintf('%s\n',upper(MAGLIST(i,1:3)));
      else
	temp    = ...
	    mag_data_load(MAGLIST(i,1:3),years,COORDSYS(i,j),...
			  DIRECTORY,TSDS_FLAG);
      end
      SDS.Name  = sprintf('%s %s (%s, nT for H,X,Y,and Z)',...
			  upper(MAGLIST(i,1:3)),COORDSYS(i,j),COORDSYS(i,:));
      insert_sds(sd_id,temp',SDS,insert_flags);
    end
  
    if (mod(k,10) == 0) | (i == size(MAGLIST,1))
      fprintf('k = %d. Closing file %s\n',k,hdf_file);
      stat = hdfsd('end',sd_id);
      if (insert_flags == 1)
	fprintf('Renaming %s\n',hdf_file);
	fprintf('to       %s\n',hdf_file_meta);
	system(sprintf('mv %s %s',hdf_file , hdf_file_meta));;
      end
      k    = 1;
    else
      k = k+1;
    end
    
  end
  
end % z

end % for insert_flags = 0:1