function make_1_min_hdf_satellite(satellite)
%
% Need to derive more tag info using this function so this file does not
% need to be hand edited when a new satellite or instrument is added.
%

% R.S. Weigel 01/12/2004  
  
more off;

% insert_flags = 0 creates data file with data + metadata

% insert_flag = 1 creates data file with data=TSDS_FLAG + metadata (files
% are small for this case because hdf compresses data streams.)  Should
% consider changing this to use the HDF functionality that this is essentially
% emulating.

for insert_flags = 0:1

SATINFO = CDAWeb_info(satellite);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get global variables and path setting
TSDS_GLOBALS;
if (is_octave)
  error('Only works in Matlab.')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SDS.Name     = '';
SDS.Format   = '';
SDS.PPD      = '1440';
SDS.Label    = '';
SDS.timeunit = 'min';
SDS.Warn     = '';
SDS.dt       = '1';
SDS.cordsys  = '';
SDS.Unit     = 'nT';
SDS.Note     = ['Data on 1 minute time grid starting on first hour of Year_o' ...
		' and  ending on last hour of Year_f. Data put on 1-minute' ...
		' time grid with nearest neighbor interpolation. '];
SDS.FLAG     = TSDS_FLAG;
SDS.Source   = 'ftp://cdaweb.gsfc.nasa.gov/pub/istp/';
SDS.Source   = [SDS.Source,lower(satellite)];

SDS.Ack      = 'See http://cdaweb.gsfc.nasa.gov/ for info on acknowledements';
SDS.README   = 'see make_CDAWeb.m';
SDS.LatLong  = 'NA';
SDS.Created_With  = ['make_CDAWeb.m in TSDS-',VERSION];

descr = sprintf('Part of the TSDS uniform time series ');
descr = [descr ,  sprintf('database, version ')];
descr = [descr ,  sprintf('%.2f.\nCreated %s.\n',VERSION,datestr(now,1))];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hdf_file = sprintf('TSDS_1_min_%s_data.hdf',upper(satellite));
if (exist(hdf_file))
  eval(sprintf('!rm %s',hdf_file));,end

if (insert_flags == 1)
  hdf_file_meta = sprintf('TSDS_1_min_%s_data-metadataonly.hdf',...
			  upper(satellite));
  if (exist(hdf_file_meta))
    eval(sprintf('!rm %s',hdf_file_meta));,end
end

sd_id    = hdfsd('start',hdf_file,'DFACC_CREATE');
status   = hdfsd('setattr',sd_id,'FileDescription',descr);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (strmatch('goes10',satellite,'exact'))
  SDS.cordsys  = 'GSM';

  load('GOES10_B_Pos_GSM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'nT';
  SDS.Name     = ['GOES10 Bx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['GOES10 By (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['GOES10 Bz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.Unit     = 'km';
  SDS.Name     = ['GOES10 X Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['GOES10 Y Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['GOES10 Z Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3

end

if (strmatch('goes8',satellite,'exact'))
  SDS.cordsys  = 'GSM';

  load('GOES8_B_Pos_GSM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'nT';
  SDS.Name     = ['GOES8 Bx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['GOES8 By (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['GOES8 Bz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.Unit     = 'km';
  SDS.Name     = ['GOES8 X Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['GOES8 Y Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['GOES8 Z Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3

end

if (satellite(1:2) == 'ul')

  SDS.cordsys  = 'RTN';
  
  load(sprintf('%sULYSSES_B_RTN_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'nT';
  SDS.Name     = ['ULYSSES Br (CDAWeb,RTN,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['ULYSSES Bt (CDAWeb,RTN,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['ULYSSES Bn (CDAWeb,RTN,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  LV2 = size(V2,1);
  clear V2

  load('ULYSSES_Pos_HSE_1_day.mat'); 
  
  Pz = repmat(Pos(:,3)',1440,1);
  Pz = Pz(:);
  Py = repmat(Pos(:,2)',1440,1);
  Py = Py(:);
  Px = repmat(Pos(:,1)',1440,1);
  Px = Px(:);
  
  if (str2num(SDS.Year_o) == Year_o)
    if (LV2 < length(Px))
      Px = Px(1:LV2,:);
      Py = Py(1:LV2,:);
      Pz = Pz(1:LV2,:);
    else
      % Should really code this case so it is not an error ...
      error('Position vector is shorter than data vector');
    end
  else
    error('Start years are not the same');
  end

  SDS.Unit     = 'AU';
  SDS.Name     = sprintf('ULYSSES Radius (SSCWeb,HSE,AU)');
  insert_sds(sd_id,Px',SDS,insert_flags);
  SDS.Unit     = 'Degrees';
  SDS.Name     = sprintf('ULYSSES Latitude (SSCWeb,HSE,deg)');
  insert_sds(sd_id,Py',SDS,insert_flags);
  SDS.Name     = sprintf('ULYSSES Longitude (SSCWeb,HSE,deg)');
  insert_sds(sd_id,Pz',SDS,insert_flags);

end

if (satellite(1:2) == 'po')
  SDS.cordsys  = 'GSM';
  
  load('POLAR_B_Pos_GSM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0

  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'nT';
  SDS.Name     = ['POLAR B (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  clear V2

  SDS.Unit     = 'km';
  SDS.Name     = ['POLAR X Position (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['POLAR Y Position (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['POLAR Z Position (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3
end

if (satellite(1:2) == 'ac')
  SDS.cordsys  = 'GSM';

  load('ACE_B_Pos_GSM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'nT';
  SDS.Name     = ['ACE Bx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['ACE By (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['ACE Bz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.Unit     = 'km';
  SDS.Name     = ['ACE X Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['ACE Y Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['ACE Z Pos (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3
  
  load('ACE_V_Np_GSM_1_min.mat');
  ftag         = SATINFO(2).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'km/s';
  SDS.Name     = ['ACE Vx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['ACE Vy (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['ACE Vz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);

  SDS.coordsys = '';
  SDS.Unit     = '#/cc';
  SDS.Name     = ['ACE Np (CDAWeb,,#/cc,',ftag,')'];
  insert_sds(sd_id,V3',SDS,insert_flags)
end

if (strmatch('imp8',satellite,'exact'))

  load('IMP8_B_Pos_GSM_SM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.cordsys  = 'GSM';
  SDS.Unit     = 'nT';
  SDS.Name     = ['IMP8 Bx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['IMP8 By (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['IMP8 Bz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.cordsys  = 'SM';
  SDS.Unit     = 'km';
  SDS.Name     = ['IMP8 X Pos (CDAWeb,SM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['IMP8 Y Pos (CDAWeb,SM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['IMP8 Z Pos (CDAWeb,SM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3

  load('IMP8_V_Np_GSE_1_min.mat');
  ftag         = SATINFO(2).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.cordsys  = 'GSE';
  SDS.Unit     = 'km/s';
  SDS.Name     = ['IMP8 Vx (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['IMP8 Vy (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['IMP8 Vz (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);

  SDS.coordsys = '';
  SDS.Unit     = '#/cc';
  SDS.Name     = ['IMP8 Np (CDAWeb,GSE,#/cc,',ftag,')'];
  insert_sds(sd_id,V3',SDS,insert_flags)
end

if (satellite(1:2) == 'ge')
  SDS.Source   = [SDS.Source,lower(satellite)];

  load('GEOTAIL_B_Pos_GSE_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.cordsys  = 'GSE';
  SDS.Unit     = 'nT';
  SDS.Name     = ['GEOTAIL Bx (CDAWeb,GSE,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL By (CDAWeb,GSE,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL Bz (CDAWeb,GSE,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.coordsys = 'GSE';
  SDS.Unit     = 'km';
  SDS.Name     = ['GEOTAIL X Pos (CDAWeb,GSE,km,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL Y Pos (CDAWeb,GSE,km,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL Z Pos (CDAWeb,GSE,km,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3

  load('GEOTAIL_V_Np_GSE_1_min.mat');
  ftag         = SATINFO(2).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'km/s';
  SDS.Name     = ['GEOTAIL Vx (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL Vy (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['GEOTAIL Vz (CDAWeb,GSE,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.coordsys = '';
  SDS.Unit     = '#/cc';
  SDS.Name     = ['GEOTAIL Np (CDAWeb,GSE,#/cc',ftag,')'];
  insert_sds(sd_id,V3',SDS,insert_flags);
  clear V3
end

if (satellite(1:2) == 'wi')

  load('WIND_B_Pos_GSM_1_min.mat');
  ftag         = SATINFO(1).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.cordsys  = 'GSM';
  SDS.Unit     = 'nT';
  SDS.Name     = ['WIND Bx (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['WIND By (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['WIND Bz (CDAWeb,GSM,nT,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.Unit      = 'km';
  SDS.Name      = ['WIND X Pos (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,1)',SDS,insert_flags);
  SDS.Name      = ['WIND Y Pos (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,2)',SDS,insert_flags);
  SDS.Name      = ['WIND Z Pos (CDAWeb,GSM,km,',ftag,')'];
  insert_sds(sd_id,V3(:,3)',SDS,insert_flags);
  clear V3

  load('WIND_V_Np_GSM_1_min.mat');
  ftag         = SATINFO(2).fname_base(4:end); % e.g., mfi_h0
  SDS.Year_o   = num2str(Year_o);
  SDS.Year_f   = num2str(Year_f);

  SDS.Unit     = 'km/s';
  SDS.Name     = ['WIND Vx (CDAWeb,GSM,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,1)',SDS,insert_flags);
  SDS.Name     = ['WIND Vy (CDAWeb,GSM,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,2)',SDS,insert_flags);
  SDS.Name     = ['WIND Vz (CDAWeb,GSM,km/s,',ftag,')'];
  insert_sds(sd_id,V2(:,3)',SDS,insert_flags);
  clear V2

  SDS.Name     = ['WIND Np (CDAWeb,GSM,#/cc,',ftag,')'];
  SDS.coordsys = '';
  SDS.Unit     = '#/cc';
  insert_sds(sd_id,V3',SDS,insert_flags);
  clear V3
end

stat       = hdfsd('end',sd_id);

if (insert_flags == 1)
  system(sprintf('mv %s %s',hdf_file,hdf_file_meta));;
end

end % for insert_flags = 0:1