clear;
TSDS_GLOBALS;

SAVE_LOCATION = [TSDS_DATA_LOCAL_WRITE,'measurements/uniform_time_series/'];
end_year      = 2002;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create an historical data input files for input to models
% Use Ap USAF as input for Autoregressive and Persistence models since
% this is what we will be comparing to with ARV.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
HIST_Ap_DIR = [TSDS_DATA_ROOT,'measurements/original_data/SEC_Ap/'];
temp_Ap     = load([HIST_Ap_DIR,'ap_7dUSAF.txt']);

% Put data in standard format
time              = temp_Ap(:,1:3); % First three columns of ap_7dUSAF are
				    % Year, Month, Day
% Put data in standard format by including Hour, Min, Sec.			      
time              = [time , 23*ones(size(temp_Ap,1),1) 59*ones(size(temp_Ap,1),2)];
% Change flag from -99999 to 99999
USAF_Ap           = [time , reflag(temp_Ap(:,5),-FLAG,FLAG)];
% Create input file in starndard format
HIST_Ap           = [time , USAF_Ap(:,7)];
% Create USAF prediction matrix in standard format
USAF              = [time , reflag(temp_Ap(:,6:12),-FLAG,FLAG)];

% Write the Ap data to a file that models will read.
fid = fopen('./YYYY_MM_DD_HH_MM_SS_Ap.daily.dat','w');
fprintf(fid,'%4d %2d %2d %2d %2d %2d %4d\n',HIST_Ap');
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%

HIST_V_DIR  = [TSDS_DATA_ROOT,'measurements/uniform_time_series/matfiles/'];
              load([HIST_V_DIR,'OMNIWeb_B_V_Np_1_day_1963_2002.mat']);
N           = number_days(1963,1988);
temp_Vx     = X(N+1:end,4); % Grab Vx
time        = ymd([1:length(temp_Vx)]',1989);
Vx          = [time,temp_Vx];

fid = fopen('./YYYY_MM_DD_HH_MM_SS_V.daily.dat','w');
fprintf(fid,'%4d %2d %2d %2d %2d %2d %4d\n',Vx');
fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run the models using above-created historical data as input.
% Compare below with what is done in ../main.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('--Start running models.\n');fflush(stdout);
if (1)
curr_dir = pwd;

clear run_* pre_* post_* create_*
cd([TSDS_ROOT,'models/Ap/Persistence/']);
  pre_persistence([curr_dir,'/YYYY_MM_DD_HH_MM_SS_Ap.daily.dat']);
  run_persistence
 post_persistence

clear run_* pre_* post_* create_*
cd([TSDS_ROOT,'models/Ap/Autoregressive/']);
  pre_autoregressive([curr_dir,'/YYYY_MM_DD_HH_MM_SS_Ap.daily.dat']);
  run_autoregressive
 post_autoregressive

clear run_* pre_* post_* create_*
cd([TSDS_ROOT,'models/Ap/ARX/']);
  pre_arx([curr_dir,'/YYYY_MM_DD_HH_MM_SS_Ap.daily.dat'],...
	   [curr_dir,'/YYYY_MM_DD_HH_MM_SS_V.daily.dat']);
  run_arx
 post_arx

 cd(cwd);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the output data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CISM_Per    = load([TSDS_ROOT,'models/Ap/Persistence/T_combined.dat']);
CISM_Auto   = load([TSDS_ROOT,'models/Ap/Autoregressive/T_combined.dat']);
CISM_ARX    = load([TSDS_ROOT,'models/Ap/ARX/T_combined.dat']);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Put data on common time grid that starts on first day of
% earliest year and ends on last day of latest year.
X      = {USAF_Ap , USAF , CISM_Per , CISM_Auto , CISM_ARX};
[Xu,T] = time_union(X,FLAG);
L      = size(T,1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

I      = find(T(:,1) == (end_year+1));
T      = T(1:I(1)-1,:);         % Keep only data through end_year

Nd        = number_days(1963,T(1,1)-1);
fill      = FLAG*ones(Nd,7);
CISM_Per  = [fill;Xu{3}(1:I(1)-1,:)];
CISM_Auto = [fill;Xu{4}(1:I(1)-1,:)];
CISM_ARX  = [fill;Xu{5}(1:I(1)-1,:)];

com=sprintf('save %smatfiles/CISM_FM_Ap_1_day_1963_%d.mat CISM_Per CISM_Auto CISM_ARX',...
	    SAVE_LOCATION,end_year);
eval(com);


