function TS_DBs = ts_db(dirtype,regen);
%TS_DB Create time series data base file
%
%   TS_DB is generally only called from TS_LIST.
%
%   Recreate all files: ts_db('all')
%
%   Read existing data base files
%   ts_db('user'); ts_db('base'); ts_db('combined');
%
%   Regenerate data base from existing *metadata*.mat in ~/.tsds/data and
%   tsds/data/final-s0 directories:
%   ts_db('user',1);
%   ts_db('base',1);
%   
%   Combine existing user and base ts_db_base.mat and ts_db_user.mat files:
%   ts_db('combined',1);
%
%   See also: TS_LIST.
  
TSDS_GLOBALS;

if (nargin < 1)
  dirtype = 'base';
end
if (nargin < 2)
  regen = 0;
end

mat_db_file_base = [TSDS_DATA_LOCAL,'/ts_db.mat'];
mat_db_file_user = [TSDS_DATA_LOCAL_WRITE,'/ts_db.mat'];
mat_db_file_comb = [TSDS_DATA_LOCAL_WRITE,'/ts_db_comb.mat'];

if (is_unix==0)
  fs = '\\';
  I = find(mat_db_file_base=='/');
  mat_db_file_base(I) = fs(1);
  I = find(mat_db_file_user=='/');
  mat_db_file_user(I) = fs(1);
  I = find(mat_db_file_comb=='/');
  mat_db_file_comb(I) = fs(1);
end
if (is_octave == 0)
  ml_ver=version;
  if (str2num(ml_ver(1))>6)
   SAVE_OPT = '-V6';
  else
   SAVE_OPT = '';
  end
else
  SAVE_OPT = '';
end

if strmatch(dirtype,'all')
  ts_db('base',1);ts_db('user',1);ts_db('combined',1);
  return;
end

if strmatch(dirtype,'combined') & (regen == 0)
  if exist(mat_db_file_comb)
    fprintf80(...
	sprintf('ts_db         : Reading names in combined data base. '));
    load(mat_db_file_comb); % Load combined TS_DSs structure if exists
  else % No combined file found or dirtype=0
    fprintf80(sprintf(['ts_db        : Combined data base file not found.'...
		       ' Attempting to generate.']));
    ts_db('combined',1);
    TS_DBs=ts_db(dirtype);
  end
  return;
end

if strmatch(dirtype,'user') & (regen == 0)
  if exist(mat_db_file_user)
    fprintf80(...
	sprintf('ts_db        : Reading names in user data base. '));
    load(mat_db_file_user); % Load combined TS_DSs structure if exists
  else
    fprintf80(sprintf(['ts_db        : User data base file not found.',...
		       ' Attempting to generate.']));
    ts_db('user',1);
    if ~exist(mat_db_file_user)
      TS_DBs=[];
    end
  end
  return;
end

if strmatch(dirtype,'base') & (regen == 0)
  if exist(mat_db_file_base)
    fprintf80(...
	sprintf('ts_db        : Reading names in base data base. '));
    load(mat_db_file_base); % Load combined TS_DSs structure if exists
  else
    fprintf80(sprintf(['ts_db        : Base data base file not found',...
		       ' Attempting to generate.']));
    ts_db('base',1);
    TS_DBs=ts_db(dirtype);
  end
  return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strmatch(dirtype,'combined') & (regen == 1)

  TS_DBs0 = ts_db('user');
  TS_DBs  = ts_db('base');
  
  if (isempty(TS_DBs0))

    load(mat_db_file_base);
    if (is_octave==1)
      eval(sprintf('save %s TS_DBs',mat_db_file_comb));
    else
      tmpVER = ver;
      if (str2num(tmpVER(1).Version) < 7)
	whos TS_DBs
	eval(sprintf('save %s TS_DBs',mat_db_file_comb));
	sprintf('save %s TS_DBs',mat_db_file_comb)
      else
	if ~isempty(SAVE_OPT)
	  eval(sprintf('save ''%s'' ''%s'' TS_DBs',SAVE_OPT,mat_db_file_comb)); 
	else
	  save(mat_db_file_comb,'TS_DBs');
        end
      end
    end

    fprintf80(...
	sprintf('ts_db        : Saved data base information in %s.\n',...
		mat_db_file_comb));

  else

    % Cat user data with base data
    Lu = length(TS_DBs0.FILENAMES);
    Lb = length(TS_DBs.FILENAMES);
    FILENAMES = TS_DBs.FILENAMES;
    for i = 1:Lu
      FILENAMES{Lb+i} = TS_DBs0.FILENAMES{i};
    end
    SET_NAMES = strvcat(TS_DBs.SET_NAMES,TS_DBs0.SET_NAMES);
    SET_FILE_NUM = [TS_DBs.SET_FILE_NUM  ; TS_DBs0.SET_FILE_NUM + Lb];
    SET_DIR_NUM = [TS_DBs.SET_DIR_NUM  ; TS_DBs0.SET_DIR_NUM];
    
    SET_YEAR_O = [TS_DBs.SET_YEAR_O ; TS_DBs0.SET_YEAR_O];
    SET_YEAR_F = [TS_DBs.SET_YEAR_F ; TS_DBs0.SET_YEAR_F];
    SET_NUM_INFILE = [TS_DBs.SET_NUM_INFILE ; TS_DBs0.SET_NUM_INFILE];
    
    TS_DBs.SET_NAMES   = SET_NAMES;
    TS_DBs.SET_YEAR_O  = SET_YEAR_O; 
    TS_DBs.SET_YEAR_F  = SET_YEAR_F;
    TS_DBs.SET_NUM_INFILE = SET_NUM_INFILE;	  
    TS_DBs.SET_FILE_NUM = SET_FILE_NUM;
    TS_DBs.SET_DIR_NUM = SET_DIR_NUM;
    TS_DBs.FILENAMES   = FILENAMES;

    if (is_octave==1)
      eval(sprintf('save %s TS_DBs',mat_db_file_comb));
    else
      tmpVER = ver;
      if (str2num(tmpVER(1).Version) < 7)
	eval(sprintf('save %s TS_DBs',mat_db_file_comb));
      else
	if ~isempty(SAVE_OPT)
	  eval(sprintf('save ''%s'' ''%s'' TS_DBs',SAVE_OPT,mat_db_file_comb)); 
	else
	  save(mat_db_file_comb,'TS_DBs');
        end
      end
    end

    fprintf80(...
	sprintf('ts_db        : Saved data base information in %s.\n',...
		mat_db_file_comb));
  end
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(strmatch(dirtype,'combined'))

if strmatch(dirtype,'base')
  data_dir = TSDS_DATA_LOCAL;
  mat_db_file = mat_db_file_base;
  set_dir_num = '0';
end
if strmatch(dirtype,'user')
  data_dir = TSDS_DATA_LOCAL_WRITE;
  mat_db_file = mat_db_file_user;
  set_dir_num = '1';
end

fprintf80(...
    sprintf(...
	['ts_db        : Generating time series name list '...
	 'from *meta* files in directory %s'],...
	data_dir));

D = readdir(data_dir); % Get names of all hdf files  
% Octave's readdir function changed sometime after 2.1.50 (output is cell).
% instead of char array.  This will break on Octave versions after 2.1.50
% that output a cell and if strvcat function cannot handle cell arrays.
if (iscell(D))
  if (is_octave)
  Dm = [];
  for i = 1:length(D)
    Dm = strvcat(Dm,D{i});
  end
  D = Dm;
  else
    D = strvcat(D);
  end
end

k = 1;
z = 1;

% Look for files with names containing string "metadataonly".
for i = 1:size(D,1)
  if (~isempty(findstr('metadataonly',D(i,:))))

    fname = deblank(D(i,:));
%    fprintf('ts_db        : Found file %s\n',fname);
    FILENAMES{k,1} = fname;
    k = k+1;
  end
end

if (~exist('FILENAMES')) 
  str=sprintf(['ts_db        : Note: no user-generated *metadata* files'...
	       ' found in %s.'],data_dir);
  fprintf80(str);
  rm(mat_db_file_user); % Delete any existing ts_db mat file
  return;
end

% Extract attribute information from all metadata found
SET_NAMES  = '';
SET_YEAR_O = '';
SET_YEAR_F = '';

for i = 1:length(FILENAMES)  

  fprintf80( sprintf('ts_db        : Reading metadata in %s',...
		     FILENAMES{i}));

  INFO = ts_dfinfo( [data_dir,'/',FILENAMES{i}] );

  for j = 1:length(INFO)

    SET_NAMES  = strvcat(SET_NAMES,INFO{j}.Name);
    SET_YEAR_O = strvcat(SET_YEAR_O,num2str(INFO{j}.Year_o));
    SET_YEAR_F = strvcat(SET_YEAR_F,num2str(INFO{j}.Year_f));

    
    SET_NUM_INFILE(z,1)     = j;
    SET_FILE_NUM(z,1) = i;
    z = z+1;
  
  end

end

TS_DBs.SET_NAMES   = SET_NAMES;
TS_DBs.SET_YEAR_O  = SET_YEAR_O; 
TS_DBs.SET_YEAR_F  = SET_YEAR_F;
TS_DBs.SET_NUM_INFILE = SET_NUM_INFILE;	  
TS_DBs.SET_FILE_NUM = SET_FILE_NUM;
TS_DBs.SET_DIR_NUM = repmat(set_dir_num,size(SET_NAMES,1),1);
TS_DBs.FILENAMES   = FILENAMES;

if (is_octave==1)
  eval(sprintf('save %s TS_DBs',mat_db_file));
else
  tmpVER = ver;
  if (str2num(tmpVER(1).Version) < 7)
    eval(sprintf('save %s TS_DBs',mat_db_file));
  else
    if ~isempty(SAVE_OPT)
      eval(sprintf('save ''%s'' ''%s'' TS_DBs',SAVE_OPT,mat_db_file));
    else
      save(mat_db_file,'TS_DBs');
    end
  end
end

fprintf80(...
    sprintf('ts_db        : Saved data base information in %s.\n',...
	      mat_db_file));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
